package com.hjy.business.appuser.controller;

import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.text.Convert;
import com.hjy.common.utils.StringUtils;
import com.hjy.common.utils.spring.SpringUtils;
import com.hjy.module.service.gather.ICbGatherStatisticsWbService;
import com.hjy.module.service.rank.ICbRankService;
import com.hjy.module.statistics.service.GxjckzbnzbService;
import com.hjy.module.vo.analysis.YearPercentage;
import com.hjy.module.vo.gather.CbGatherTmpVo;
import com.hjy.module.vo.statistics.NormCollValueVo;
import com.hjy.system.mapper.SysDeptMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author yjh
 * @ClassName APPwbController
 * @description: TODO
 * @date 2023年09月13日
 * @version: 1.0
 */
@Api(tags = "移动端-湾办图表Controller")
@RestController
@RequestMapping("/app_api/wb")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class APPwbController {

    private final ICbGatherStatisticsWbService cbGatherStatisticsWbService;

    private final GxjckzbnzbService gxjckzbnzbService;

    private final ICbRankService rankService;

    @ApiOperation("移动端-广西本年进出口总值趋势图(亿元)")
    @PostMapping("/comprehensive/total/value/year/listAll")
    public AjaxResult getAllTotalValue() {
        return cbGatherStatisticsWbService.getAllTotalValue();
    }

    @ApiOperation("移动端-各项进出口值本年占比(亿元)")
    @PostMapping("/comprehensive/selectGxjckzbnzbVoList")
    public AjaxResult selectGxjckzbnzbVoList() {
        // 获取当前年份有数据的最新月份
        CbGatherTmpVo vo1 = cbGatherStatisticsWbService.selectTmp(null);
        Map<Integer, Map<String, List<NormCollValueVo>>> vo = cbGatherStatisticsWbService.getTotalValue(null,vo1.getYear(), vo1.getMonth());
        // 组装数据返回
        Map<String, Object> objectObjectHashMap = new HashMap<>();
        objectObjectHashMap.put("year",vo1.getYear());
        objectObjectHashMap.put("month",vo1.getMonth());
        objectObjectHashMap.put("data",vo);
        return AjaxResult.success(objectObjectHashMap);
    }



    @ApiOperation("广西综保区近年进出口总值对比情况")
    @ApiResponse(code = 200, message = "成功; imExport为进出口总值，cityTotal为市进出口总值，percentage为年增长率，growthRate为同比增长率", response = YearPercentage.class)
    @GetMapping("/inout/year/percentage")
    public AjaxResult getInoutPercentageList() {
        // 获取当前年份有数据的最新月份
        CbGatherTmpVo vo = cbGatherStatisticsWbService.selectTmp(null);
        List<YearPercentage> inoutPercentageList = gxjckzbnzbService.getInoutPercentageList(vo.getYear(), vo.getMonth());
        // 组装数据返回
        Map<String, Object> objectObjectHashMap = new HashMap<>();
        objectObjectHashMap.put("year",vo.getYear());
        objectObjectHashMap.put("month",vo.getMonth());
        objectObjectHashMap.put("date",inoutPercentageList);
        return AjaxResult.success(objectObjectHashMap);
    }

    @ApiOperation("广西各综保区进出口总值近年排名变化")
    @ApiResponse(code = 200, message = "返回一个Map集合，key是年份，value是保税区进出口排名对象")
    @GetMapping("/inout/ranking/year/list")
    public AjaxResult getDeptRank() {
        return AjaxResult.success(rankService.queryInOutRanking());
    }

    /**
     * 当前默认查询六个月数据
     * @param deptId 保税区id
     * @param normCode 指标代码
     * @param year 年份
     * @param month 月份
     * @param selMonthSum 查询月份数量
     * @return
     */
    @ApiOperation("获取各项指标数据(当前默认查询六个月数据)")
    @PostMapping(value = "/normHis")
    public AjaxResult normHis(@RequestParam(value = "deptId", required = false) String deptId,
                              @RequestParam("normCode") String[] normCode,
                              @RequestParam("year") Long year,
                              @RequestParam("month") Long month,
                              @RequestParam("selMonthSum") Long selMonthSum,
                              @RequestParam("isTB")  boolean isTB) {
        Map<String,List<NormCollValueVo>> result = gxjckzbnzbService.normHis(deptId,normCode,year,month,selMonthSum,isTB);
        return AjaxResult.success(result);
    }

    @ApiOperation("全区进出口总值对比报表数据")
    @GetMapping("/jckzz")
    public AjaxResult getjckzzList() {
        // 获取当前年份有数据的最新月份
        CbGatherTmpVo vo = cbGatherStatisticsWbService.selectTmp(null);
        Map<String, List<NormCollValueVo>> result = new HashMap<>();
        String[] codeArr = new String[]{"A001","F004"};
        for (String code:codeArr){
            if (code.equals("A001")){
                Map<String, List<NormCollValueVo>> temp =  gxjckzbnzbService.normHisZS((String) null,
                        StringUtils.split(code,"##"),
                        Convert.toLong(vo.getYear()),
                        Convert.toLong(vo.getMonth().replace("月",""))
                        , 6L,
                        true);
                result = putAllItem(result,temp);
            }else if (code.equals("F004")){
                SysDept dept = new SysDept();
                dept.setType(2);
                dept.setDelFlag("0");
                List<String> deptList = SpringUtils.getBean(SysDeptMapper.class).selectDeptList(dept).stream().map(d -> String.valueOf(d.getDeptId())).collect(Collectors.toList());
                Map<String, List<NormCollValueVo>> temp =  gxjckzbnzbService.normHisF004(deptList,
                        StringUtils.split(code,"##"),
                        Convert.toLong(vo.getYear()),
                        Convert.toLong(vo.getMonth().replace("月",""))
                        , 6L,
                        true);
                result = putAllItem(result,temp);
            }

        }
        // 组装数据返回
        Map<String, Object> objectObjectHashMap = new HashMap<>();
        objectObjectHashMap.put("year",vo.getYear());
        objectObjectHashMap.put("month",vo.getMonth());
        objectObjectHashMap.put("data",result);
        return AjaxResult.success(objectObjectHashMap);
    }

    private Map<String, List<NormCollValueVo>> putAllItem(Map<String, List<NormCollValueVo>> result,Map<String, List<NormCollValueVo>> temp){
        if (CollectionUtils.isEmpty(result)) return temp;

        if (CollectionUtils.isEmpty(temp)) return result;

        for (String item1:temp.keySet()){
            for (String item2:result.keySet()){
                if (item1.equals(item2)) result.get(item2).addAll(temp.get(item2));
            }
        }
        return result;
    }
}
