package com.hjy.business.appuser.controller;

import com.hjy.business.appuser.service.DefaultLoginService;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.model.LoginBody;
import com.hjy.common.core.sm2.Sm2;
import com.hjy.common.utils.Sm2Utils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.bouncycastle.crypto.InvalidCipherTextException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.security.NoSuchAlgorithmException;

/**
 * 默认登录
 *
 * @author Tellsea
 * @date 2022/9/2
 */
@Api(tags = "移动端登录")
@RestController
@RequestMapping("/au/defaultLogin")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class DefaultLoginController {

    private final DefaultLoginService defaultLoginService;

    private String privateKeyHex = null;
//    @ApiOperation("登录检查")
//    @GetMapping("beforeLogin")
//    public AjaxResult beforeLogin() {
//        return AjaxResult.success("检查成功");
//    }

    @ApiOperation("账号密码登录")
    @PostMapping("login")
    public AjaxResult login(@RequestBody LoginBody entity) throws InvalidCipherTextException {
        String decrypt = Sm2Utils.decrypt(entity.getPassword(), privateKeyHex);
        entity.setPassword(decrypt);
        return defaultLoginService.login(entity);
    }
    /**
     * 获取公密接口
     */
    @PostMapping("/genPUPRkey")
    public AjaxResult genPUPRkey() throws NoSuchAlgorithmException {
        Sm2 sm2 = Sm2Utils.genPUPRkey();
        privateKeyHex = sm2.getPrivateKeyHex();
        return AjaxResult.success(sm2.getPublicKeyHex());
    }
    @ApiOperation("获取用户信息")
    @GetMapping("getInfo")
    public AjaxResult getInfo() {
        return defaultLoginService.getInfo();
    }





//    @ApiOperation("获取验证码")
//    @GetMapping("getCode/{username}")
//    public AjaxResult getCode(@PathVariable String username) {
//        return defaultLoginService.getCode(username);
//    }

//    @ApiOperation("账号验证码登录")
//    @PostMapping("loginByCode")
//    public AjaxResult loginByCode(@RequestBody LoginBody entity) {
//        return defaultLoginService.loginByCode(entity);
//    }

//    @ApiOperation("注册验证码登录")
//    @PostMapping("register")
//    public AjaxResult register(@RequestBody LoginBody entity) {
//        return defaultLoginService.register(entity);
//    }
}
