package com.hjy.modlue.activecompanies;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;

import com.hjy.module.service.activecompanies.IActiveCompaniesService;
import com.hjy.module.vo.activecompanies.ActiveCompaniesVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import com.hjy.common.core.page.TableDataInfo;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 历年活跃企业数Controller
 *
 * @author Tellsea
 * @date 2024-05-28
 */
@Api(tags = "历年活跃企业数Controller")
@RestController
@RequestMapping("/business/ACTIVECOMPANIES")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class ActiveCompaniesController extends BaseController {

    private final IActiveCompaniesService activeCompaniesService;

    @ApiOperation("查询历年活跃企业数列表")
    @PreAuthorize("@ss.hasPermi('business:ACTIVECOMPANIES:list')")
    @GetMapping("/list")
    public TableDataInfo<ActiveCompaniesVo> list(ActiveCompaniesVo entity) {
        return activeCompaniesService.queryList(entity);
    }

    @ApiOperation("查询历年活跃企业数所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(ActiveCompaniesVo entity) {
        return AjaxResult.success("查询成功", activeCompaniesService.queryAll(entity));
    }

    @ApiOperation("导出历年活跃企业数列表")
    @PreAuthorize("@ss.hasPermi('business:ACTIVECOMPANIES:export')")
    @Log(title = "历年活跃企业数", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ActiveCompaniesVo entity) {
        List<ActiveCompaniesVo> list = activeCompaniesService.queryAll(entity);
        ExcelUtil<ActiveCompaniesVo> util = new ExcelUtil<>(ActiveCompaniesVo.class);
        util.exportExcel(response, list, "历年活跃企业数数据");
    }

    @ApiOperation("获取历年活跃企业数详细信息")
    @PreAuthorize("@ss.hasPermi('business:ACTIVECOMPANIES:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", activeCompaniesService.queryById(id));
    }

    @ApiOperation("新增历年活跃企业数")
    @PreAuthorize("@ss.hasPermi('business:ACTIVECOMPANIES:add')")
    @Log(title = "历年活跃企业数", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody ActiveCompaniesVo entity) {
        return toAjax(activeCompaniesService.save(entity));
    }

    @ApiOperation("修改历年活跃企业数")
    @PreAuthorize("@ss.hasPermi('business:ACTIVECOMPANIES:edit')")
    @Log(title = "历年活跃企业数", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody ActiveCompaniesVo entity) {
        return toAjax(activeCompaniesService.updateById(entity));
    }

    @ApiOperation("删除历年活跃企业数")
    @PreAuthorize("@ss.hasPermi('business:ACTIVECOMPANIES:remove')")
    @Log(title = "历年活跃企业数", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(activeCompaniesService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
