package com.hjy.modlue.analysis;

import com.hjy.common.core.domain.AjaxResult;
import com.hjy.module.service.gather.ICbGatherStatisticsWbService;
import com.hjy.module.service.rank.ICbRankService;
import com.hjy.module.service.totexp.ICbTotexpService;
import com.hjy.module.vo.gather.CbGatherTmpVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author sakura
 * @date 2023/10/10 16:16:03 周二
 */
@Api(tags = "湾办大屏统计Controller")
@RestController
@RequestMapping("/gather/statistics/analysis")
public class DataAnalysisController {
    private final ICbGatherStatisticsWbService cbGatherStatisticsWbService;

    private final ICbTotexpService cbTotexpService;
    private final ICbRankService rankService;

    public DataAnalysisController(ICbTotexpService cbTotexpService, ICbRankService rankService, ICbGatherStatisticsWbService cbGatherStatisticsWbService) {
        this.cbTotexpService = cbTotexpService;
        this.rankService = rankService;
        this.cbGatherStatisticsWbService = cbGatherStatisticsWbService;
    }

    @ApiOperation("广西各综保区进出口总值近年排名变化")
    @ApiResponse(code = 200, message = "返回一个Map集合，key是年份，value是保税区进出口排名对象")
    @GetMapping("/inout/ranking/year/list")
    public AjaxResult getDeptRank() {
        return AjaxResult.success(rankService.queryInOutRanking());
    }


    @ApiOperation("广西各综保区近年进出口总值(亿元)")
    @ApiResponse(code = 200, message = "返回一个Map集合，key是年份，value是保税区进出口总值的对象")
    @GetMapping("/inout/year/list")
    public AjaxResult getInOutYearList() {
        // 获取当前年份有数据的最新月份
        CbGatherTmpVo vo = cbGatherStatisticsWbService.selectTmp(null);
        HashMap<String, List<BigDecimal>> inoutPercentageList = cbTotexpService.queryInOutYear(vo.getYear(), vo.getMonth());
        // 组装数据返回
        Map<String, Object> objectObjectHashMap = new HashMap<>();
        objectObjectHashMap.put("year",vo.getYear());
        objectObjectHashMap.put("month",vo.getMonth());
        objectObjectHashMap.put("date",inoutPercentageList);
        return AjaxResult.success(objectObjectHashMap);
    }
}
