package com.hjy.modlue.apiconfig;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.apiconfig.ICbApiConfigService;
import com.hjy.module.vo.apiconfig.CbApiConfigVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 数据接口api配置Controller
 *
 * @author Tellsea
 * @date 2023-09-01
 */
@Api(tags = "数据接口api配置Controller")
@RestController
@RequestMapping("/APICONFIG")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbApiConfigController extends BaseController {

    private final ICbApiConfigService cbApiConfigService;

    @ApiOperation("查询数据接口api配置列表")
    @PreAuthorize("@ss.hasPermi('apiconfig:APICONFIG:list')")
    @GetMapping("/list")
    public TableDataInfo<CbApiConfigVo> list(CbApiConfigVo entity) {
        return cbApiConfigService.queryList(entity);
    }

    @ApiOperation("查询数据接口api配置所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbApiConfigVo entity) {
        return AjaxResult.success("查询成功", cbApiConfigService.queryAll(entity));
    }

    @ApiOperation("导出数据接口api配置列表")
    @PreAuthorize("@ss.hasPermi('apiconfig:APICONFIG:export')")
    @Log(title = "数据接口api配置", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbApiConfigVo entity) {
        List<CbApiConfigVo> list = cbApiConfigService.queryAll(entity);
        ExcelUtil<CbApiConfigVo> util = new ExcelUtil<>(CbApiConfigVo.class);
        util.exportExcel(response, list, "数据接口api配置数据");
    }

    @ApiOperation("导入数据接口api配置列表")
    @PreAuthorize("@ss.hasPermi('apiconfig:APICONFIG:upload')")
    @Log(title = "数据接口api配置", businessType = BusinessType.OTHER)
    @PostMapping("/upload")
    public AjaxResult upload(MultipartFile file) throws Exception {
        ExcelUtil<CbApiConfigVo> util = new ExcelUtil<>(CbApiConfigVo.class);
        List<CbApiConfigVo> cbApiConfigVoList = util.importExcel(file.getInputStream());
        if(cbApiConfigVoList.isEmpty()){
            return AjaxResult.error("上传的文件内容有误!");
        }else{
            for (CbApiConfigVo cbApiConfigVo : cbApiConfigVoList) {
                cbApiConfigService.save(cbApiConfigVo);
            }
            return AjaxResult.success("保存成功");
        }

    }

    @ApiOperation("获取数据接口api配置详细信息")
    @PreAuthorize("@ss.hasPermi('apiconfig:APICONFIG:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbApiConfigService.queryById(id));
    }

    @ApiOperation("新增数据接口api配置")
    @PreAuthorize("@ss.hasPermi('apiconfig:APICONFIG:add')")
    @Log(title = "数据接口api配置", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbApiConfigVo entity) {
        return toAjax(cbApiConfigService.save(entity));
    }

    @ApiOperation("修改数据接口api配置")
    @PreAuthorize("@ss.hasPermi('apiconfig:APICONFIG:edit')")
    @Log(title = "数据接口api配置", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbApiConfigVo entity) {
        return toAjax(cbApiConfigService.updateById(entity));
    }

    @ApiOperation("删除数据接口api配置")
    @PreAuthorize("@ss.hasPermi('apiconfig:APICONFIG:remove')")
    @Log(title = "数据接口api配置", businessType = BusinessType.DELETE)
    @GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbApiConfigService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
