package com.hjy.modlue.article;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.domain.model.LoginUser;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.core.text.Convert;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.enums.WorkFlowServiceEnums;
import com.hjy.common.utils.SecurityUtils;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.domain.article.CbArticleDate;
import com.hjy.module.service.article.ICbArticleDateService;
import com.hjy.module.service.article.ICbArticleService;
import com.hjy.module.service.portal.ICbDataFillingService;
import com.hjy.module.service.wf.IWfInstActionTaskService;
import com.hjy.module.vo.article.CbArticleDateVo;
import com.hjy.module.vo.article.CbArticleVo;
import com.hjy.module.vo.portal.CbDataFillingVo;
import com.hjy.system.service.ISysDeptService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.*;

/**
 * 重点项目管理Controller
 *
 * @author Tellsea
 * @date 2023-09-19
 */
@Api(tags = "重点项目管理Controller")
@RestController
@RequestMapping("/ARTICLE")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbArticleController extends BaseController {
    private final ICbDataFillingService cbDataFillingService;


    private final ICbArticleService cbArticleService;

    private final ISysDeptService deptService;

    private final ICbArticleDateService articleDateService;

    private final IWfInstActionTaskService wfInstActionTaskService;


    @ApiOperation("查询重点项目管理列表")
    @PreAuthorize("@ss.hasPermi('business:ARTICLE:list')")
    @GetMapping("/list")
    public TableDataInfo<CbArticleVo> list(CbArticleVo entity) {
        return cbArticleService.queryList(entity);
    }

    @ApiOperation("查询重点项目管理所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbArticleVo entity) {
        return AjaxResult.success("查询成功", cbArticleService.queryAll(entity));
    }

    @ApiOperation("导出重点项目管理列表")
    @PreAuthorize("@ss.hasPermi('business:ARTICLE:export')")
    @Log(title = "重点项目管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbArticleVo entity) {
        List<CbArticleVo> list = cbArticleService.queryAll(entity);
        ExcelUtil<CbArticleVo> util = new ExcelUtil<>(CbArticleVo.class);
        util.exportExcel(response, list, "重点项目管理数据");
    }

    @ApiOperation("获取重点项目管理详细信息")
    @PreAuthorize("@ss.hasPermi('business:ARTICLE:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbArticleService.queryById(id));
    }

    @ApiOperation("获取重点项目管理实际投资金额详细信息")
    @PreAuthorize("@ss.hasPermi('business:ARTICLE:query')")
    @GetMapping(value = "/getPriceInfo/{id}")
    public AjaxResult getPriceInfo(@PathVariable("id") String id) {
        CbArticleVo vo = cbArticleService.queryById(id);
        CbDataFillingVo cbDataFillingVo = new CbDataFillingVo();
        cbDataFillingVo.setArticleId(id);
        List<CbDataFillingVo> list = cbDataFillingService.queryAll(cbDataFillingVo);
        Map<String, Object> result = new HashMap<>();
        result.put("article", vo);
        result.put("dataFillingList", list);
        return AjaxResult.success("查询成功", result);
    }

    @ApiOperation("新增重点项目管理")
    @PreAuthorize("@ss.hasPermi('business:ARTICLE:add')")
    @Log(title = "重点项目管理", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbArticleVo entity) {
            CbArticleDate cbArticleDate = cbArticleService.queryArticleDate();
            Date date = new Date();
            if (date.compareTo(cbArticleDate.getBeginTime()) > 0 &&
                    date.compareTo(cbArticleDate.getEndTime()) < 0) {
                LoginUser user = SecurityUtils.getLoginUser();
                SysDept sysDept = deptService.selectDeptById(user.getDeptId());
                entity.setDeptName(sysDept.getDeptName());
                cbArticleService.save(entity);
                if (1 == entity.getStatus()) {
                    wfInstActionTaskService.statrWfqueue(WorkFlowServiceEnums.ARTICLESERVICE.getService(), entity.getId(), entity.getAtvName(), String.valueOf(user.getUserId()));
                    return AjaxResult.success("审核发起");
                }
                return AjaxResult.success("保存成功");
            }
            return AjaxResult.success("当前不在开放时间内");
    }

    @ApiOperation("修改重点项目管理")
    @PreAuthorize("@ss.hasPermi('business:ARTICLE:edit')")
    @Log(title = "重点项目管理", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbArticleVo entity) {
        return toAjax(cbArticleService.updateById(entity));
    }

    @ApiOperation("删除重点项目管理")
    @PreAuthorize("@ss.hasPermi('business:ARTICLE:remove')")
    @Log(title = "重点项目管理", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbArticleService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }

    @ApiOperation("修改提交审核接口")
    @Log(title = "提交审核", businessType = BusinessType.UPDATE)
    @PostMapping("/audit")
    public AjaxResult audit(@RequestBody CbArticleVo entity) {
        entity.setContent(entity.getContent().replace("/dev-api", "").replace("/hjy-api", ""));
        cbArticleService.updateById(entity);
        wfInstActionTaskService.statrWfqueue(WorkFlowServiceEnums.MODIFYARTICLESERVICE.getService(), entity.getId(), entity.getAtvName());
        return AjaxResult.success("审核发起");
    }

    @ApiOperation("设置重点项目录入开放时间")
    @PreAuthorize("@ss.hasPermi('business:ARTICLE:insert')")
    @Log(title = "重点项目管理", businessType = BusinessType.INSERT)
    @PostMapping("/articleDate/{time}")
    public AjaxResult articleDate(@PathVariable("time") Long[] time) {
        Date[] dates = Arrays.stream(time)
                .map(Date::new)
                .toArray(Date[]::new);

        Date beginTime = dates[0];
        Date endTime = dates[1];
        CbArticleDate articleDate = cbArticleService.queryArticleDate();
        if (Objects.isNull(articleDate)){
            CbArticleDate saveEny = new CbArticleDateVo();
            saveEny.setBeginTime(beginTime);
            saveEny.setEndTime(endTime);
            saveEny.setVersion("1");
            saveEny.setStatus("0");
            return toAjax(articleDateService.save(saveEny));
        }
        articleDate.setStatus("1");
        articleDateService.updateById(articleDate);
        // 写入 新数据
        articleDate.setId(null);
        articleDate.setBeginTime(beginTime);
        articleDate.setEndTime(endTime);
        articleDate.setVersion((Convert.toInt(articleDate.getVersion(),1)+1)+"");
        articleDate.setStatus("0");
        return toAjax(articleDateService.save(articleDate));
    }

    @ApiOperation("获取重点项目开放时间")
    @PreAuthorize("@ss.hasPermi('business:ARTICLE:query')")
    @GetMapping("/queryArticleDate")
    public AjaxResult queryArticleDate() {
        return AjaxResult.success("获取成功",cbArticleService.queryArticleDate());
    }

    @ApiOperation("获取配置时间历史记录")
    @PreAuthorize("@ss.hasPermi('business:ARTICLE:ArticleHis')")
    @GetMapping("/ArticleHis")
    public AjaxResult getArticDateList(){
        CbArticleDateVo sel = new CbArticleDateVo();
        sel.setStatus("1");
        return AjaxResult.success(articleDateService.queryAll(sel));
    }
}
