package com.hjy.modlue.auxiliary;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;

import com.hjy.module.service.auxiliary.IAuxiliaryVideoService;
import com.hjy.module.vo.auxiliary.AuxiliaryVideoVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import com.hjy.common.core.page.TableDataInfo;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 通关视频Controller
 *
 * @author Tellsea
 * @date 2024-06-18
 */
@Api(tags = "通关视频Controller")
@RestController
@RequestMapping("/video/AUXILIARYVIDEO")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class AuxiliaryVideoController extends BaseController {

    private final IAuxiliaryVideoService auxiliaryVideoService;

    @ApiOperation("查询通关视频列表")
    @PreAuthorize("@ss.hasPermi('video:AUXILIARYVIDEO:list')")
    @GetMapping("/list")
    public TableDataInfo<AuxiliaryVideoVo> list(AuxiliaryVideoVo entity) {
        return auxiliaryVideoService.queryList(entity);
    }

    @ApiOperation("查询通关视频所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(AuxiliaryVideoVo entity) {
        return AjaxResult.success("查询成功", auxiliaryVideoService.queryAll(entity));
    }

    @ApiOperation("导出通关视频列表")
    @PreAuthorize("@ss.hasPermi('video:AUXILIARYVIDEO:export')")
    @Log(title = "通关视频", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, AuxiliaryVideoVo entity) {
        List<AuxiliaryVideoVo> list = auxiliaryVideoService.queryAll(entity);
        ExcelUtil<AuxiliaryVideoVo> util = new ExcelUtil<>(AuxiliaryVideoVo.class);
        util.exportExcel(response, list, "通关视频数据");
    }

    @ApiOperation("获取通关视频详细信息")
    @PreAuthorize("@ss.hasPermi('video:AUXILIARYVIDEO:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", auxiliaryVideoService.queryById(id));
    }

    @ApiOperation("新增通关视频")
    @PreAuthorize("@ss.hasPermi('video:AUXILIARYVIDEO:add')")
    @Log(title = "通关视频", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody AuxiliaryVideoVo entity) {
        return toAjax(auxiliaryVideoService.save(entity));
    }

    @ApiOperation("修改通关视频")
    @PreAuthorize("@ss.hasPermi('video:AUXILIARYVIDEO:edit')")
    @Log(title = "通关视频", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody AuxiliaryVideoVo entity) {
        return toAjax(auxiliaryVideoService.updateById(entity));
    }

    @ApiOperation("删除通关视频")
    @PreAuthorize("@ss.hasPermi('video:AUXILIARYVIDEO:remove')")
    @Log(title = "通关视频", businessType = BusinessType.DELETE)
    @GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(auxiliaryVideoService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
