package com.hjy.modlue.garden;


import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.garden.ICbGardenDataService;
import com.hjy.module.vo.garden.CbGardenDataVo;
import com.hjy.system.service.ISysDeptService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 企业园区大屏数据Controller
 *
 * @author Tellsea
 * @date 2023-10-11
 */
//@Api(tags = "企业园区大屏数据Controller")
@RestController
@RequestMapping("/analysis/GARDENDATA")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbGardenDataController extends BaseController {

    private final ICbGardenDataService cbGardenDataService;
    ISysDeptService sysDeptService;
    @Autowired
    private ISysDeptService sysDeptService(ISysDeptService sysDeptService){
        return this.sysDeptService = sysDeptService;
    }

    //    @ApiOperation("查询企业园区大屏数据列表")
    @PreAuthorize("@ss.hasPermi('analysis:GARDENDATA:list')")
    @GetMapping("/list")
    public TableDataInfo<CbGardenDataVo> list(CbGardenDataVo entity) {
        return cbGardenDataService.queryList(entity);
    }

    //    @ApiOperation("查询企业园区大屏数据所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbGardenDataVo entity) {
        return AjaxResult.success("查询成功", cbGardenDataService.queryAll(entity));
    }

    //    @ApiOperation("导出企业园区大屏数据列表")
    @PreAuthorize("@ss.hasPermi('analysis:GARDENDATA:export')")
    @Log(title = "企业园区大屏数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbGardenDataVo entity) {
        List<CbGardenDataVo> list = cbGardenDataService.queryAll(entity);
        ExcelUtil<CbGardenDataVo> util = new ExcelUtil<>(CbGardenDataVo.class);
        util.exportExcel(response, list, "企业园区大屏数据数据");
    }

    //    @ApiOperation("获取企业园区大屏数据详细信息")
    @PreAuthorize("@ss.hasPermi('analysis:GARDENDATA:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbGardenDataService.queryById(id));
    }

    //    @ApiOperation("新增企业园区大屏数据")
    @PreAuthorize("@ss.hasPermi('analysis:GARDENDATA:add')")
    @Log(title = "企业园区大屏数据", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbGardenDataVo entity) {
        SysDept sysDept = sysDeptService.selectDeptById(Long.valueOf(entity.getDeptId()));
        if(sysDept != null){
            entity.setDeptName(sysDept.getDeptName());
        }
        return toAjax(cbGardenDataService.save(entity));
    }

    //    @ApiOperation("修改企业园区大屏数据")
    @PreAuthorize("@ss.hasPermi('analysis:GARDENDATA:edit')")
    @Log(title = "企业园区大屏数据", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbGardenDataVo entity) {
        return toAjax(cbGardenDataService.updateById(entity));
    }

    //    @ApiOperation("删除企业园区大屏数据")
    @PreAuthorize("@ss.hasPermi('analysis:GARDENDATA:remove')")
    @Log(title = "企业园区大屏数据", businessType = BusinessType.DELETE)
    @GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbGardenDataService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
