package com.hjy.modlue.garden;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.garden.ITbBondedScreenDataService;
import com.hjy.module.vo.garden.TbBondedScreenDataVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

/**
 * 保税区大屏所需数据Controller
 *
 * @author Mechrevo
 * @date 2023-09-28
 */
@Api(tags = "保税区大屏所需数据Controller")
@RestController
@RequestMapping("/BONDEDSCREENDATA")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class TbBondedScreenDataController extends BaseController {

    private final ITbBondedScreenDataService tbBondedScreenDataService;

    @ApiOperation("查询保税区大屏所需数据列表")
//    @PreAuthorize("@ss.hasPermi('business:BONDEDSCREENDATA:list')")
    @GetMapping("/list")
    public TableDataInfo<TbBondedScreenDataVo> list(TbBondedScreenDataVo entity) {
        return tbBondedScreenDataService.queryList(entity);
    }

    @ApiOperation("查询保税区大屏所需数据所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(TbBondedScreenDataVo entity) {
        return AjaxResult.success("查询成功", tbBondedScreenDataService.queryAll(entity));
    }

    @ApiOperation("导出保税区大屏所需数据列表")
//    @PreAuthorize("@ss.hasPermi('business:BONDEDSCREENDATA:export')")
    @Log(title = "保税区大屏所需数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TbBondedScreenDataVo entity) {
        List<TbBondedScreenDataVo> list = tbBondedScreenDataService.queryAll(entity);
        ExcelUtil<TbBondedScreenDataVo> util = new ExcelUtil<>(TbBondedScreenDataVo.class);
        util.exportExcel(response, list, "保税区大屏所需数据数据");
    }

    @ApiOperation("获取保税区大屏所需数据详细信息")
//    @PreAuthorize("@ss.hasPermi('business:BONDEDSCREENDATA:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", tbBondedScreenDataService.queryById(id));
    }

    @ApiOperation("新增保税区大屏所需数据")
//    @PreAuthorize("@ss.hasPermi('business:BONDEDSCREENDATA:add')")
    @Log(title = "保税区大屏所需数据", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody TbBondedScreenDataVo entity) {
        return toAjax(tbBondedScreenDataService.save(entity));
    }

    @ApiOperation("修改保税区大屏所需数据")
//    @PreAuthorize("@ss.hasPermi('business:BONDEDSCREENDATA:edit')")
    @Log(title = "保税区大屏所需数据", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody TbBondedScreenDataVo entity) {
        return toAjax(tbBondedScreenDataService.updateById(entity));
    }

    @ApiOperation("删除保税区大屏所需数据")
//    @PreAuthorize("@ss.hasPermi('business:BONDEDSCREENDATA:remove')")
    @Log(title = "保税区大屏所需数据", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(tbBondedScreenDataService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }


    @ApiOperation("导入湾办大屏所需数据")
//    @PreAuthorize("@ss.hasPermi('business:BONDEDSCREENDATA:edit')")
    @Log(title = "导入湾办大屏所需数据")
    @PostMapping("importData")
    public AjaxResult importData(@RequestParam("file")MultipartFile file) throws IOException {
        String string = tbBondedScreenDataService.importData(file);
        return AjaxResult.success(string);
    }

}
