package com.hjy.modlue.garden;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.garden.ITbBondedUpplyService;
import com.hjy.module.vo.garden.TbBondedUpplyVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

/**
 * 保税区供应链数据Controller
 *
 * @author Tellsea
 * @date 2023-09-28
 */
@Api(tags = "保税区供应链数据Controller")
@RestController
@RequestMapping("/BONDEDUPPLY")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class TbBondedUpplyController extends BaseController {

    private final ITbBondedUpplyService tbBondedUpplyService;

    @ApiOperation("查询保税区供应链数据列表")
//    @PreAuthorize("@ss.hasPermi('business:BONDEDUPPLY:list')")
    @GetMapping("/list")
    public TableDataInfo<TbBondedUpplyVo> list(TbBondedUpplyVo entity) {
        return tbBondedUpplyService.queryList(entity);
    }

    @ApiOperation("查询保税区供应链数据所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(TbBondedUpplyVo entity) {
        return AjaxResult.success("查询成功", tbBondedUpplyService.queryAll(entity));
    }

    @ApiOperation("导出保税区供应链数据列表")
//    @PreAuthorize("@ss.hasPermi('business:BONDEDUPPLY:export')")
    @Log(title = "保税区供应链数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TbBondedUpplyVo entity) {
        List<TbBondedUpplyVo> list = tbBondedUpplyService.queryAll(entity);
        ExcelUtil<TbBondedUpplyVo> util = new ExcelUtil<>(TbBondedUpplyVo.class);
        util.exportExcel(response, list, "保税区供应链数据数据");
    }

    @ApiOperation("获取保税区供应链数据详细信息")
//    @PreAuthorize("@ss.hasPermi('business:BONDEDUPPLY:query')")
    @GetMapping(value = "/getInfo/{lddwmc}")
    public AjaxResult getInfo(@PathVariable("lddwmc") String lddwmc) {
        return AjaxResult.success("查询成功", tbBondedUpplyService.queryById(lddwmc));
    }

    @ApiOperation("新增保税区供应链数据")
//    @PreAuthorize("@ss.hasPermi('business:BONDEDUPPLY:add')")
    @Log(title = "保税区供应链数据", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody TbBondedUpplyVo entity) {
        return toAjax(tbBondedUpplyService.save(entity));
    }

    @ApiOperation("修改保税区供应链数据")
//    @PreAuthorize("@ss.hasPermi('business:BONDEDUPPLY:edit')")
    @Log(title = "保税区供应链数据", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody TbBondedUpplyVo entity) {
        return toAjax(tbBondedUpplyService.updateById(entity));
    }

    @ApiOperation("删除保税区供应链数据")
//    @PreAuthorize("@ss.hasPermi('business:BONDEDUPPLY:remove')")
    @Log(title = "保税区供应链数据", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(tbBondedUpplyService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }

    @ApiOperation("园区供应链数据 -> 导入")
    @Log(title = "园区供应链数据 -> 导入")
    @PostMapping("/importData")
    public AjaxResult importData(@RequestParam("file") MultipartFile file) throws IOException {
        String string = tbBondedUpplyService.importData(file);
        return AjaxResult.success(string);
    }
}
