package com.hjy.modlue.garden;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.garden.ITbEconomicsIndexService;
import com.hjy.module.vo.garden.TbEconomicsIndexVo;
import com.hjy.system.service.ISysDeptService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 综合保税区经济运行主要指标Controller
 *
 * @author Tellsea
 * @date 2023-09-27
 */
@Api(tags = "综合保税区经济运行主要指标Controller")
@RestController
@RequestMapping("/ECONOMICSINDEX")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class TbEconomicsIndexController extends BaseController {

    private final ITbEconomicsIndexService tbEconomicsIndexService;
    ISysDeptService sysDeptService;
    @Autowired
    private ISysDeptService sysDeptService(ISysDeptService sysDeptService){
        return this.sysDeptService = sysDeptService;
    }

    @ApiOperation("查询综合保税区经济运行主要指标列表")
//    @PreAuthorize("@ss.hasPermi('business:ECONOMICSINDEX:list')")
    @GetMapping("/list")
    public TableDataInfo<TbEconomicsIndexVo> list(TbEconomicsIndexVo entity) {
        return tbEconomicsIndexService.queryList(entity);
    }

    @ApiOperation("查询综合保税区经济运行主要指标所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(TbEconomicsIndexVo entity) {
        return AjaxResult.success("查询成功", tbEconomicsIndexService.queryAll(entity));
    }

    @ApiOperation("导出综合保税区经济运行主要指标列表")
//    @PreAuthorize("@ss.hasPermi('business:ECONOMICSINDEX:export')")
    @Log(title = "综合保税区经济运行主要指标", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TbEconomicsIndexVo entity) {
        List<TbEconomicsIndexVo> list = tbEconomicsIndexService.queryAll(entity);
        ExcelUtil<TbEconomicsIndexVo> util = new ExcelUtil<>(TbEconomicsIndexVo.class);
        util.exportExcel(response, list, "综合保税区经济运行主要指标数据");
    }

    @ApiOperation("获取综合保税区经济运行主要指标详细信息")
//    @PreAuthorize("@ss.hasPermi('business:ECONOMICSINDEX:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", tbEconomicsIndexService.queryById(id));
    }

    @ApiOperation("新增综合保税区经济运行主要指标")
//    @PreAuthorize("@ss.hasPermi('business:ECONOMICSINDEX:add')")
    @Log(title = "综合保税区经济运行主要指标", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody TbEconomicsIndexVo entity) {
        SysDept sysDept = sysDeptService.selectDeptById(Long.valueOf(entity.getDeptId()));
        if(sysDept != null){
            entity.setDeptName(sysDept.getDeptName());
        }
        return toAjax(tbEconomicsIndexService.save(entity));
    }

    @ApiOperation("修改综合保税区经济运行主要指标")
//    @PreAuthorize("@ss.hasPermi('business:ECONOMICSINDEX:edit')")
    @Log(title = "综合保税区经济运行主要指标", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody TbEconomicsIndexVo entity) {
        return toAjax(tbEconomicsIndexService.updateById(entity));
    }

    @ApiOperation("删除综合保税区经济运行主要指标")
//    @PreAuthorize("@ss.hasPermi('business:ECONOMICSINDEX:remove')")
    @Log(title = "综合保税区经济运行主要指标", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(tbEconomicsIndexService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }

    @ApiOperation("综合保税区发展绩效指标汇总统计表 -> 数据导入")
    @Log(title = "综合保税区发展绩效指标汇总统计表 -> 数据导入")
    @PostMapping("/transpose")
    public String transposeExcel(@RequestPart("file") MultipartFile excelFile) throws Exception {
//        tbEconomicsIndexService.rowChangeLine(excelFile);
        tbEconomicsIndexService.readLine(excelFile);
        return "excel数据导入完成";
    }
}
