package com.hjy.modlue.garden;

import com.hjy.common.annotation.DataScope;
import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.domain.garden.TbEnterpriseScreenData;
import com.hjy.module.service.garden.ITbEnterpriseScreenDataService;
import com.hjy.module.vo.garden.TbEnterpriseScreenDataVo;
import com.hjy.system.service.ISysDeptService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 企业大屏所需供应商数据Controller
 *
 * @author Tellsea
 * @date 2023-10-11
 */
@Api(tags = "企业大屏所需供应商数据Controller")
@RestController
@RequestMapping("/ENTERPRISESCREENDATA")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class TbEnterpriseScreenDataController extends BaseController {

    private final ITbEnterpriseScreenDataService tbEnterpriseScreenDataService;
    ISysDeptService sysDeptService;
    @Autowired
    private ISysDeptService sysDeptService(ISysDeptService sysDeptService){
        return this.sysDeptService = sysDeptService;
    }

    @ApiOperation("查询企业大屏所需供应商数据列表")
    @PreAuthorize("@ss.hasPermi('business:ENTERPRISESCREENDATA:list')")
    @DataScope(deptAlias = "A")
    @GetMapping("/list")
    public TableDataInfo<TbEnterpriseScreenDataVo> list(TbEnterpriseScreenDataVo entity) {
        return tbEnterpriseScreenDataService.queryList(entity);
    }

    @ApiOperation("查询企业大屏所需供应商数据所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(TbEnterpriseScreenDataVo entity) {
        return AjaxResult.success("查询成功", tbEnterpriseScreenDataService.queryAll(entity));
    }

    @ApiOperation("导出企业大屏所需供应商数据列表")
    @PreAuthorize("@ss.hasPermi('business:ENTERPRISESCREENDATA:export')")
    @Log(title = "企业大屏所需供应商数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TbEnterpriseScreenDataVo entity) {
        List<TbEnterpriseScreenDataVo> list = tbEnterpriseScreenDataService.queryAll(entity);
        ExcelUtil<TbEnterpriseScreenDataVo> util = new ExcelUtil<>(TbEnterpriseScreenDataVo.class);
        util.exportExcel(response, list, "企业大屏所需供应商数据数据");
    }

    @ApiOperation("获取企业大屏所需供应商数据详细信息")
    @PreAuthorize("@ss.hasPermi('business:ENTERPRISESCREENDATA:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", tbEnterpriseScreenDataService.queryById(id));
    }

    @ApiOperation("新增企业大屏所需供应商数据")
    @PreAuthorize("@ss.hasPermi('business:ENTERPRISESCREENDATA:add')")
    @Log(title = "企业大屏所需供应商数据", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody TbEnterpriseScreenData entity) {
        SysDept sysDept = sysDeptService.selectDeptByName(entity.getEnterpriseName());
        if(sysDept != null){
            entity.setEnterpriseName(String.valueOf(sysDept.getDeptName()));
        }
        return toAjax(tbEnterpriseScreenDataService.save(entity));
    }

    @ApiOperation("修改企业大屏所需供应商数据")
    @PreAuthorize("@ss.hasPermi('business:ENTERPRISESCREENDATA:edit')")
    @Log(title = "企业大屏所需供应商数据", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody TbEnterpriseScreenData entity) {
        return toAjax(tbEnterpriseScreenDataService.updateById(entity));
    }

    @ApiOperation("删除企业大屏所需供应商数据")
    @PreAuthorize("@ss.hasPermi('business:ENTERPRISESCREENDATA:remove')")
    @Log(title = "企业大屏所需供应商数据", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(tbEnterpriseScreenDataService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
