package com.hjy.modlue.garden;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.domain.garden.TbEnterpriseTradePerformance;
import com.hjy.module.service.garden.ITbEnterpriseTradePerformanceService;
import com.hjy.module.vo.garden.TbEnterpriseTradePerformanceVo;
import com.hjy.system.service.ISysDeptService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 企业贸易绩效数据（每个月填报）Controller
 *
 * @author Tellsea
 * @date 2023-10-11
 */
@Api(tags = "企业贸易绩效数据（每个月填报）Controller")
@RestController
@RequestMapping("/ENTERPRISETRADEPERFORMANCE")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class TbEnterpriseTradePerformanceController extends BaseController {

    private final ITbEnterpriseTradePerformanceService tbEnterpriseTradePerformanceService;
    ISysDeptService sysDeptService;
    @Autowired
    private ISysDeptService sysDeptService(ISysDeptService sysDeptService){
        return this.sysDeptService = sysDeptService;
    }

    @ApiOperation("查询 列表")
//    @PreAuthorize("@ss.hasPermi('business:ENTERPRISETRADEPERFORMANCE:list')")
    @GetMapping("/list")
    public TableDataInfo<TbEnterpriseTradePerformanceVo> list(TbEnterpriseTradePerformanceVo entity) {
        TableDataInfo<TbEnterpriseTradePerformanceVo> queriedList = tbEnterpriseTradePerformanceService.queryList(entity);
        return queriedList;
    }

    @ApiOperation("查询  所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(TbEnterpriseTradePerformanceVo entity) {
        return AjaxResult.success("查询成功", tbEnterpriseTradePerformanceService.queryAll(entity));
    }

    @ApiOperation("导出  列表")
//    @PreAuthorize("@ss.hasPermi('business:ENTERPRISETRADEPERFORMANCE:export')")
    @Log(title = "企业贸易绩效数据（每个月填报）", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TbEnterpriseTradePerformanceVo entity) {
        List<TbEnterpriseTradePerformanceVo> list = tbEnterpriseTradePerformanceService.queryAll(entity);
        ExcelUtil<TbEnterpriseTradePerformanceVo> util = new ExcelUtil<>(TbEnterpriseTradePerformanceVo.class);
        util.exportExcel(response, list, "企业贸易绩效数据（每个月填报）数据");
    }

    @ApiOperation("获取 详细信息")
//    @PreAuthorize("@ss.hasPermi('business:ENTERPRISETRADEPERFORMANCE:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", tbEnterpriseTradePerformanceService.queryById(id));
    }

    @ApiOperation("新增 ")
//    @PreAuthorize("@ss.hasPermi('business:ENTERPRISETRADEPERFORMANCE:add')")
    @Log(title = "企业贸易绩效数据（每个月填报）", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody TbEnterpriseTradePerformance entity) {
        int check = tbEnterpriseTradePerformanceService.lambdaQuery()
                .eq(TbEnterpriseTradePerformance::getDeptId,entity.getDeptId())
                .eq(TbEnterpriseTradePerformance::getYear,entity.getYear())
                .eq(TbEnterpriseTradePerformance::getMonth,entity.getMonth()).count();
        if (check>0){
            return error("该月数据已存在，无法再次填报！");
        }
        SysDept sysDept = sysDeptService.selectDeptById(Long.valueOf(entity.getDeptId()));
        if(sysDept != null){
            if(sysDept.getType() == 3){
                entity.setDeptName(String.valueOf(sysDept.getDeptName()));
                entity.setBsqDeptId(String.valueOf(sysDept.getParentId()));
                return toAjax(tbEnterpriseTradePerformanceService.save(entity));
            }else {
                return AjaxResult.error("当前不属于企业!");
            }
        }
        return AjaxResult.error("新增失败!");
    }

    @ApiOperation("修改 ")
//    @PreAuthorize("@ss.hasPermi('business:ENTERPRISETRADEPERFORMANCE:edit')")
    @Log(title = "企业贸易绩效数据（每个月填报）", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody TbEnterpriseTradePerformance entity) {
        return toAjax(tbEnterpriseTradePerformanceService.updateById(entity));
    }

    @ApiOperation("删除")
//    @PreAuthorize("@ss.hasPermi('business:ENTERPRISETRADEPERFORMANCE:remove')")
    @Log(title = "企业贸易绩效数据（每个月填报）", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(tbEnterpriseTradePerformanceService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
