package com.hjy.modlue.garden;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.enums.CellExtraTypeEnum;
import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.file.FileUtils;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.EasyExcel.GardenBaseDataListener;
import com.hjy.module.domain.garden.TbGardenBaseData;
import com.hjy.module.service.garden.ITbGardenBaseDataService;
import com.hjy.module.vo.garden.GardenBaseDataEasyexcelVo;
import com.hjy.module.vo.garden.TbGardenBaseDataVo;
import com.hjy.system.service.ISysDeptService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Arrays;
import java.util.List;

/**
 * 园区内企业基础数据Controller
 *
 * @author Tellsea
 * @date 2023-10-11
 */
@Api(tags = "园区内企业基础数据Controller")
@RestController
@RequestMapping("/GARDENBASEDATA")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class TbGardenBaseDataController extends BaseController {

    private final ITbGardenBaseDataService tbGardenBaseDataService;
    ISysDeptService sysDeptService;

    @Autowired
    private ISysDeptService sysDeptService(ISysDeptService sysDeptService) {
        return this.sysDeptService = sysDeptService;
    }

    @ApiOperation("查询园区内企业基础数据列表")
    @GetMapping("/list")
    public TableDataInfo<TbGardenBaseDataVo> list(TbGardenBaseDataVo entity) {
        return tbGardenBaseDataService.queryList(entity);
    }

    @ApiOperation("查询园区内企业基础数据所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(TbGardenBaseDataVo entity) {
        return AjaxResult.success("查询成功", tbGardenBaseDataService.queryAll(entity));
    }

    @ApiOperation("导出园区内企业基础数据列表")
    @Log(title = "园区内企业基础数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TbGardenBaseDataVo entity) {
        List<TbGardenBaseDataVo> list = tbGardenBaseDataService.queryAll(entity);
        ExcelUtil<TbGardenBaseDataVo> util = new ExcelUtil<>(TbGardenBaseDataVo.class);
        util.exportExcel(response, list, "园区内企业基础数据数据");
    }

    @ApiOperation("获取园区内企业基础数据详细信息")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", tbGardenBaseDataService.queryById(id));
    }

    @ApiOperation("新增园区内企业基础数据")
    @Log(title = "园区内企业基础数据", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody TbGardenBaseDataVo entity) {
        SysDept sysDept = sysDeptService.selectDeptById(Long.valueOf(entity.getDeptId()));
        if (sysDept != null) {
            entity.setDeptId(String.valueOf(sysDept.getDeptId()));
            entity.setParentId(String.valueOf(sysDept.getParentId()));
            entity.setDeptName(String.valueOf(sysDept.getDeptName()));
        }
        if (tbGardenBaseDataService.lambdaQuery().eq(TbGardenBaseData::getDeptId, entity.getDeptId()).count() > 0) {
            return error("企业信息已存在，无法再次填写！");
        }
        return toAjax(tbGardenBaseDataService.save(entity));
    }

    @ApiOperation("修改园区内企业基础数据")
    @Log(title = "园区内企业基础数据", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody TbGardenBaseDataVo entity) {
        return toAjax(tbGardenBaseDataService.updateById(entity));
    }

    @ApiOperation("删除园区内企业基础数据")
    @Log(title = "园区内企业基础数据", businessType = BusinessType.DELETE)
    @GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        Arrays.stream(ids).forEach(item -> {
            TbGardenBaseData baseData = tbGardenBaseDataService.getById(item);
            if (ObjectUtil.isNotNull(baseData)) {
                tbGardenBaseDataService.removeById(item);
                sysDeptService.deleteBydeptNameAndPraentId(baseData.getDeptName(), baseData.getParentId());
            }
        });
        return AjaxResult.success();
    }

    @ApiOperation("导入企业基础数据")
    @Log(title = "导入企业基础数据", businessType = BusinessType.INSERT)
    @PostMapping("/baseData/import")
    public AjaxResult baseDataImport(@RequestParam("file") MultipartFile file, @RequestParam("deptId") String deptId) {
        File failFile = FileUtils.multipartFileToFile(file);

        EasyExcel.read(failFile,
                        GardenBaseDataEasyexcelVo.class,
                        new GardenBaseDataListener(deptId))
                .extraRead(CellExtraTypeEnum.MERGE)
                .sheet()
                .headRowNumber(1)
                .doRead();
        FileUtils.deleteFile(failFile.getPath());
        return success();
    }


    @ApiOperation("下载导入模板")
    @Log(title = "下载导入模板", businessType = BusinessType.DOWNLOAD)
    @GetMapping("/download")
    public void downloadModlue(HttpServletResponse response) throws IOException {
        FileUtils.setAttachmentResponseHeader(response, "企业基础数据导入模板.xlsx");
        InputStream fis = Thread.currentThread().getContextClassLoader().getResourceAsStream("mouldFile/企业基础数据导入模板.xlsx");
        OutputStream os = response.getOutputStream();
        byte[] b = new byte[1024];
        int length;
        while ((length = fis.read(b)) > 0) {
            os.write(b, 0, length);
        }
        IOUtils.close(os);
        IOUtils.close(fis);
    }

}
