package com.hjy.modlue.garden;

import com.hjy.common.annotation.DataScope;
import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.domain.garden.TbGardenTradeData;
import com.hjy.module.service.garden.TbGardenTradeDataService;
import com.hjy.module.vo.garden.TbGardenTradeDataVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 园区贸易数据Controller
 *
 * @author Mechrevo
 * @date 2023-09-26
 */
@Api(tags = "园区贸易数据Controller")
@RestController
@RequestMapping("/gardentradedata")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class TbGardenTradeDataController extends BaseController {
    @Autowired
    private TbGardenTradeDataService gardenTradeDataService;



    @ApiOperation("查询园区贸易数据列表")
    @GetMapping("/list")
    @DataScope(deptAlias = "A")
    public TableDataInfo<TbGardenTradeDataVo> list(TbGardenTradeDataVo entity) {
        return gardenTradeDataService.queryList(entity);
    }

    @Log(title = "导入园区信息", businessType = BusinessType.IMPORT)
    @PostMapping("/importData")
    @ApiModelProperty("导入园区信息")
    public AjaxResult importData(@RequestParam("file")MultipartFile file) throws Exception {
        String string = gardenTradeDataService.importData(file);
        return AjaxResult.success(string);
    }


    @ApiOperation("查询园区贸易数据所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(TbGardenTradeDataVo entity) {
        return AjaxResult.success("查询成功", gardenTradeDataService.queryAll(entity));
    }

    @ApiOperation("导出园区贸易数据列表")
    @Log(title = "园区贸易数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TbGardenTradeDataVo entity) {
        List<TbGardenTradeDataVo> list = gardenTradeDataService.queryAll(entity);
        ExcelUtil<TbGardenTradeDataVo> util = new ExcelUtil<>(TbGardenTradeDataVo.class);
        util.exportExcel(response, list, "园区贸易数据数据");
    }

    @ApiOperation("获取园区贸易数据详细信息")
    @GetMapping(value = "/getInfo/{gardenId}")
    public AjaxResult getInfo(@PathVariable("gardenId") String gardenId) {
        TbGardenTradeDataVo tbGardenTradeDataVo = gardenTradeDataService.queryById(gardenId);
        return AjaxResult.success("查询成功", tbGardenTradeDataVo);
    }

    @ApiOperation("新增园区贸易数据")
    @Log(title = "园区贸易数据", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody TbGardenTradeDataVo entity) {
        int check = gardenTradeDataService.lambdaQuery()
                .eq(TbGardenTradeData::getCbId,entity.getCbId())
                .eq(TbGardenTradeData::getYear,entity.getYear())
                .eq(TbGardenTradeData::getMonth,entity.getMonth()).count();
        if (check > 0) return error("该年月数据已存在，无法再次添加");
        entity.setDeptId(entity.getCbId());
        return toAjax(gardenTradeDataService.save(entity));
    }

    @ApiOperation("修改园区贸易数据")
    @Log(title = "园区贸易数据", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody TbGardenTradeDataVo entity) {
        return toAjax(gardenTradeDataService.updateById(entity));
    }

    @ApiOperation("删除园区贸易数据")
    @Log(title = "园区贸易数据", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{gardenIds}")
    public AjaxResult remove(@PathVariable String[] gardenIds) {
        return toAjax(gardenTradeDataService.removeByIds(Arrays.asList(gardenIds)) ? 1 : 0);
    }
}
