package com.hjy.modlue.garden;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.garden.ITbImportantProjectService;
import com.hjy.module.vo.garden.TbImportantProjectVo;
import com.hjy.system.service.ISysDeptService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 重点项目数据Controller
 *
 * @author Tellsea
 * @date 2023-10-11
 */
@Api(tags = "重点项目数据Controller")
@RestController
@RequestMapping("/business/IMPORTANTPROJECT")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class TbImportantProjectController extends BaseController {

    private final ITbImportantProjectService tbImportantProjectService;
    ISysDeptService sysDeptService;
    @Autowired
    private ISysDeptService sysDeptService(ISysDeptService sysDeptService){
        return this.sysDeptService = sysDeptService;
    }
    @ApiOperation("查询重点项目数据列表")
//    @PreAuthorize("@ss.hasPermi('business:IMPORTANTPROJECT:list')")
    @GetMapping("/list")
    public TableDataInfo<TbImportantProjectVo> list(TbImportantProjectVo entity) {
        return tbImportantProjectService.queryList(entity);
    }

    @ApiOperation("查询重点项目数据所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(TbImportantProjectVo entity) {
        return AjaxResult.success("查询成功", tbImportantProjectService.queryAll(entity));
    }

    @ApiOperation("导出重点项目数据列表")
//    @PreAuthorize("@ss.hasPermi('business:IMPORTANTPROJECT:export')")
    @Log(title = "重点项目数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TbImportantProjectVo entity) {
        List<TbImportantProjectVo> list = tbImportantProjectService.queryAll(entity);
        ExcelUtil<TbImportantProjectVo> util = new ExcelUtil<>(TbImportantProjectVo.class);
        util.exportExcel(response, list, "重点项目数据数据");
    }

    @ApiOperation("获取重点项目数据详细信息")
//    @PreAuthorize("@ss.hasPermi('business:IMPORTANTPROJECT:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", tbImportantProjectService.queryById(id));
    }

    @ApiOperation("新增重点项目数据")
//    @PreAuthorize("@ss.hasPermi('business:IMPORTANTPROJECT:add')")
    @Log(title = "重点项目数据", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody TbImportantProjectVo entity) {
        SysDept sysDept = sysDeptService.selectDeptById(Long.valueOf(entity.getDeptId()));
        if(sysDept == null || sysDept.getType() != 3){
            return AjaxResult.error("当前企业信息错误!");
        }
        return toAjax(tbImportantProjectService.save(entity));
    }

    @ApiOperation("修改重点项目数据")
//    @PreAuthorize("@ss.hasPermi('business:IMPORTANTPROJECT:edit')")
    @Log(title = "重点项目数据", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody TbImportantProjectVo entity) {
        return toAjax(tbImportantProjectService.updateById(entity));
    }

    @ApiOperation("删除重点项目数据")
//    @PreAuthorize("@ss.hasPermi('business:IMPORTANTPROJECT:remove')")
    @Log(title = "重点项目数据", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(tbImportantProjectService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
