package com.hjy.modlue.gather;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.enums.CellExtraTypeEnum;
import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.model.LoginUser;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.core.text.Convert;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.enums.WorkFlowServiceEnums;
import com.hjy.common.utils.SecurityUtils;
import com.hjy.common.utils.file.FileUtils;
import com.hjy.module.EasyExcel.GatherDataListener;
import com.hjy.module.domain.gether.CbGather;
import com.hjy.module.service.gather.ICbGatherService;
import com.hjy.module.service.wf.IWfInstActionTaskService;
import com.hjy.module.vo.gather.CbGatherVo;
import com.hjy.module.vo.gather.GatherImportVo;
import com.hjy.module.vo.gather.vo.TaxTableVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.Date;

/**
 * 数据采集Controller
 *
 * @author Tellsea
 * @date 2023-08-30
 */
@Api(tags = "数据采集Controller")
@RestController
@RequestMapping("/gather")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbGatherController extends BaseController {

    private final ICbGatherService cbGatherService;

    private final IWfInstActionTaskService wfInstActionTaskService;

    @ApiOperation("查询数据采集列表")
    @GetMapping("/list")
    public TableDataInfo<CbGatherVo> list(CbGatherVo entity) {
        return cbGatherService.queryList(entity);
    }

    @ApiOperation("查询数据采集所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbGatherVo entity) {
        return AjaxResult.success("查询成功", cbGatherService.queryAll(entity));
    }

    @ApiOperation("获取可采集的数据")
    @GetMapping(value = "/getInfoByColl/{id}")
    public AjaxResult getInfoByColl(@PathVariable("id") String id) {
        return AjaxResult.success(cbGatherService.getInfoByColl(id));
    }

    @ApiOperation("获取数据采集详细信息")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        CbGatherVo cbGatherVo = cbGatherService.queryById(id);
        return AjaxResult.success(cbGatherVo);
    }

    @ApiOperation("数据采集录入")
    @Log(title = "数据采集", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbGatherVo entity) {
        return toAjax(cbGatherService.saveEntity(entity));
    }

    @ApiOperation("数据采集确认")
    @Log(title = "数据采集确认", businessType = BusinessType.INSERT)
    @PostMapping("confirm/{id}")
    public AjaxResult confirm(@PathVariable String id) {
        CbGather gather = cbGatherService.getById(id);
        if (ObjectUtils.isEmpty(gather)) return error("数据不存在");
        if (!gather.getCollStatus().equals("2")) return error("当前数据非草稿，无法确认！");
        gather.setCollStatus("1");
        return toAjax(cbGatherService.updateById(gather));
    }


    @ApiOperation("提交审核")
    @Log(title = "提交审核", businessType = BusinessType.UPDATE)
    @PostMapping("audit")
    public AjaxResult audit(String gatherId) {
        CbGather entity = cbGatherService.getById(gatherId);
        if (entity != null && entity.getCollStatus().equals("1")) {
            wfInstActionTaskService.statrWfqueue(WorkFlowServiceEnums.GATHERSERSERVICE.getService(),
                    gatherId,
                    entity.getYear()+"-"+entity.getMonth()+"数据采集",
                    Convert.toStr(SecurityUtils.getUserId()));
            entity.setApproveStatus("1");
            cbGatherService.updateById(entity);
            return success();
        }
        return AjaxResult.error("当前数据未采集!");
    }


    @ApiOperation("数据上报")
    @Log(title = "数据上报", businessType = BusinessType.UPDATE)
    @PostMapping("report")
    public AjaxResult report(@RequestBody CbGatherVo entity) {
        if (!entity.getApproveStatus().equals("2")) {
            return AjaxResult.error("当前数据未经管理员审核!");
        }
        cbGatherService.insertGatherStatisData(entity); // 生成报表积分数据
        entity.setReportStatus("1");
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String nickName = loginUser.getUser().getNickName();
        entity.setReportName(nickName);
        // 上报时间
        entity.setReportTime(new Date());

        // 在中间表插入该数据的年份月份和id
        cbGatherService.updateTmp(entity.getYear(), entity.getMonth(), entity.getDeptId());
        return toAjax(cbGatherService.updateById(entity));
    }


    @ApiOperation("数据采集修改")
    @Log(title = "数据采集", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbGatherVo entity) {
        if (entity.getApproveStatus().equals("1") || entity.getApproveStatus().equals("2")) {
            return AjaxResult.error("当前数据已提交审核,无法修改");
        }
        return toAjax(cbGatherService.updateEntity(entity));
    }

    @ApiOperation("湾办修改接口")
    @Log(title = "数据采集", businessType = BusinessType.UPDATE)
    @PostMapping("wbEdit")
    public AjaxResult wbEdit(@RequestBody CbGatherVo entity) {
        return toAjax(cbGatherService.updateEntity(entity));
    }


    @ApiOperation("下载导入模板")
    @PreAuthorize("@ss.hasPermi('gather:GATHER:download')")
    @Log(title = "下载导入模板", businessType = BusinessType.DOWNLOAD)
    @GetMapping("/download/{id}")
    public void downloadModlue(HttpServletResponse response, @PathVariable String id) throws IOException {
        String fileName = "导入模板";
        response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-disposition", "attachment;filename=" + URLEncoder.encode(fileName + ".xlsx", "UTF-8"));
        response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");

        cbGatherService.downloadModlue(response, id);
    }

    /**
     * 数据类型(0=量化指标数据,1=月报表数据)
     * 代码过于臃肿，执行代码损耗性能过高，需优化
     *
     * @param file
     * @return
     */
    @ApiOperation("上传采集附件-历史数据")
    @PreAuthorize("@ss.hasPermi('gather:GATHER:upload')")
    @Log(title = "上传采集附件", businessType = BusinessType.OTHER)
    @PostMapping("/upload")
    public TaxTableVo upload(@RequestParam("file") MultipartFile file) {
        TaxTableVo taxTableVo = cbGatherService.readExcelData(file, "0");
        int size = taxTableVo.getList().size();
        return taxTableVo;
    }

    /**
     * 数据类型(0=量化指标数据,1=月报表数据)
     * 代码过于臃肿，执行代码损耗性能过高，需优化
     *
     * @param file
     * @param type
     * @return
     */
    @ApiOperation("上传采集附件-新数据")
    @PreAuthorize("@ss.hasPermi('gather:GATHER:uploadnew')")
    @Log(title = "上传采集附件-新数据", businessType = BusinessType.OTHER)
    @PostMapping("/uploadnew")
    public AjaxResult uploadNew(@RequestParam("file") MultipartFile file, @RequestParam("type") String type) {
        File failFile = FileUtils.multipartFileToFile(file);
        EasyExcel.read(failFile ,
                GatherImportVo.class, new GatherDataListener("0"))
                .extraRead(CellExtraTypeEnum.MERGE)
                .sheet()
                .headRowNumber(2)
                .doRead();
        FileUtils.deleteFile(failFile.getPath());
        return AjaxResult.success("上传附件成功!");
    }

    /**
     * 数据类型(0=量化指标数据,1=月报表数据)
     *
     * @param file
     * @param type
     * @param mark 标识类型(1:海关部分贸易报表,2:海关进出口总值报表)
     * @return
     */
    @ApiOperation("海关报表上传采集附件")
    @Log(title = "海关报表上传采集附件", businessType = BusinessType.OTHER)
    @PostMapping("/customs/upload")
    public AjaxResult customsUpload(@RequestParam("file") MultipartFile file,
                                    @RequestParam("type") String type,
                                    @RequestParam("mark") String mark) throws IOException {
        this.cbGatherService.customsUpload(file, "0", mark);
        return AjaxResult.success("上传采集附件成功!");
    }

}
