package com.hjy.modlue.gather;

import com.alibaba.fastjson2.JSONObject;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.module.domain.garden.TbGardenBaseData;
import com.hjy.module.service.gather.ICbGatherStatisticsBsqService;
import com.hjy.module.service.gather.ICbGatherStatisticsWbService;
import com.hjy.module.service.rank.ICbRankService;
import com.hjy.module.statistics.service.GxjckzbnzbService;
import com.hjy.module.statistics.vo.*;
import com.hjy.module.vo.article.CbArticleVo;
import com.hjy.module.vo.gather.CbGatherTmpVo;
import com.hjy.module.vo.rank.CbRankVo;
import com.hjy.module.vo.statistics.NormCollValueParentVo;
import com.hjy.module.vo.statistics.NormCollValueVo;
import com.hjy.system.mapper.SysDeptMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;


/**
 * @author 12635
 */
@Api(tags = "各保税区大屏统计Controller")
@RestController
@RequestMapping("/gather/statistics/bsq")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbGatherStatisticsBsq extends BaseController {

    private final ICbGatherStatisticsBsqService cbGatherStatisticsBsqService;

    private final ICbGatherStatisticsWbService cbGatherStatisticsWbService;

    private final GxjckzbnzbService gxjckzbnzbService;

    private final SysDeptMapper sysDeptMapper;

    private final ICbRankService cbRankService;



    /**以下是待删除代码（接口更新重写）**/

//    @ApiOperation("各个保税区本年各项进出口值占比")
//    @GetMapping("/comprehensive/total/value/year")
//    public AjaxResult getYearTotalValue(Integer deptId, String year) {
//        return AjaxResult.success("查询成功", cbGatherStatisticsBsqService.getTotalValue(deptId, year, null));
//    } 待删除

//    @ApiOperation("各个保税区本年每月各项进出口值经营总收入")
//    @GetMapping("/comprehensive/total/value/month/{deptId}")
//    public AjaxResult getMonthTotalValue(@PathVariable("deptId") Integer deptId) {
//        return AjaxResult.success("查询成功", cbGatherStatisticsBsqService.getMonthTotalValue(deptId));
//    } 待删除

    @ApiOperation("园区进出口总值与广西进出口总值 (亿元)")
    @PostMapping("/comprehensive/getOneDeptSumData/{deptId}")
    public AjaxResult getOneDeptSumData(@PathVariable("deptId") String deptId) {
        //北海保税区主键,后期可根据保税区灵活更换
        List<ItemList> oneDeptSumData = gxjckzbnzbService.getOneDeptSumData(deptId);
        return AjaxResult.success(oneDeptSumData);
    }

    @ApiOperation("查询广西保税区排名所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbRankVo entity) {
        return AjaxResult.success("查询成功", cbRankService.queryAll(entity));
    }


    @ApiOperation("园区与所在市进出口总值")
    @PostMapping("/comprehensive/selectGardenEnterpriseType")
    public AjaxResult selectGardenEnterpriseType(String deptId) {
        // 获取当前年份有数据的最新月份
        CbGatherTmpVo vo = cbGatherStatisticsWbService.selectTmp(null);
        List<CityAndBsqVo> result = gxjckzbnzbService.selectGardenEnterpriseType(deptId,vo.getYear(), vo.getMonth());
        // 组装数据返回
        Map<String, Object> objectObjectHashMap = new HashMap<>();
        objectObjectHashMap.put("year",vo.getYear());
        objectObjectHashMap.put("month",vo.getMonth());
        objectObjectHashMap.put("date",result);
        return AjaxResult.success(objectObjectHashMap);
    }



    @ApiOperation("仓库面积和产业定位等信息")
    @GetMapping(value = "/cangKuArea/{deptId}")
    public AjaxResult cangKuArea(@PathVariable("deptId") String deptId) {
        SysDept sysDept = sysDeptMapper.selectCangKuArea(deptId);
        return AjaxResult.success(sysDept);
    }


    @ApiOperation("园区企业和账册分类")
    @GetMapping(value = "/enterpriseType/{deptId}")
    public AjaxResult enterpriseType(@PathVariable("deptId") String deptId) {
        List<EnterpriseTypeVo> enterpriseTypeVos = gxjckzbnzbService.enterpriseType(Long.valueOf(deptId));
        return AjaxResult.success(enterpriseTypeVos);
    }

    @ApiOperation("企业结构占比")
    @GetMapping(value = "/enterpriseStructure/{deptId}")
    public AjaxResult enterpriseStructure(@PathVariable("deptId") String deptId) {
        return cbGatherStatisticsBsqService.getEnterpriseStructure(deptId);
    }

    @ApiOperation("企业业务统计")
    @GetMapping(value = "/businessEvents/{deptId}")
    public AjaxResult businessEvents(@PathVariable("deptId") String deptId) {
        return cbGatherStatisticsBsqService.getBusinessEvents(deptId);
    }
    /**以上是待删除代码（接口更新重写）**/

    /**
     * 当前默认查询六个月数据
     * @param deptId 保税区id
     * @param normCode 指标代码
     * @param year 年份
     * @param month 月份
     * @param selMonthSum 查询月份数量
     * @return
     */
    @ApiOperation("获取各项指标数据(当前默认查询六个月数据)")
    @PostMapping(value = "/normHis")
    public AjaxResult normHis(@RequestParam(value = "deptId", required = false) String deptId,
                              @RequestParam("normCode") String[] normCode,
                              @RequestParam("year") Long year,
                              @RequestParam("month") Long month,
                              @RequestParam("selMonthSum") Long selMonthSum,
                              @RequestParam("isTB")  boolean isTB) {
        Map<String, List<NormCollValueVo>> result;
        // 获取当前年份有数据的最新月份
        CbGatherTmpVo vo = cbGatherStatisticsWbService.selectTmp(null);
        if (normCode[0].equals("A001")){
            result = gxjckzbnzbService.normHisZS(deptId,normCode,Long.valueOf(vo.getYear()),Long.valueOf(vo.getMonth()),selMonthSum,isTB);
        }else {
            result = gxjckzbnzbService.normHis(deptId, normCode, Long.valueOf(vo.getYear()), Long.valueOf(vo.getMonth()), selMonthSum, isTB);
        }
        // 组装数据返回
        Map<String, Object> objectObjectHashMap = new HashMap<>();
        objectObjectHashMap.put("year",vo.getYear());
        objectObjectHashMap.put("month",vo.getMonth());
        objectObjectHashMap.put("date",result);
        return AjaxResult.success(objectObjectHashMap);
    }

    /**
     * 当前默认擦查询当月数据
     * @param deptId 保税区id
     * @param normCode 指标代码
     * @param year 年份
     * @param month 月份
     * @return
     */
    @ApiOperation("获取各项指标数据(当前默认查询当月数据)")
    @PostMapping(value = "/norNowSia")
    public AjaxResult norNowSia(@RequestParam("deptId") String deptId,
                              @RequestParam("normCode") String[] normCode,
                              @RequestParam("year") Long year,
                              @RequestParam("month") Long month) {
        List<NormCollValueParentVo> result = gxjckzbnzbService.norNowSia(deptId,normCode,year,month);
        return AjaxResult.success(result);
    }



    /**
     * 获取指标年数据
     * @param deptId 保税区id
     * @param normCode 指标代码
     * @param year 年份
     * @return
     */
    @ApiOperation("获取指标年数据(当前默认查询5年数据)")
    @PostMapping(value = "/normHisByyear")
    public AjaxResult normHisByyear(@RequestParam("deptId") String deptId,
                              @RequestParam("normCode") String[] normCode,
                              @RequestParam("year") Long year,
                              @RequestParam("isTB")  boolean isTB) {

        Map<String,List<NormCollValueVo>> result = gxjckzbnzbService.normHisByyear(deptId,normCode,year,isTB);
        return AjaxResult.success(result);
    }

    /**
     * 多指标占比数据
     * @param deptId 保税区id
     * @param normCode 指标代码
     * @param year 年份
     * @return
     */
    @ApiOperation("多指标占比数据")
    @PostMapping(value = "/normzbSource")
    public AjaxResult normzbSource(@RequestParam("deptId") String deptId,
                                    @RequestParam("normCode") String[] normCode,
                                    @RequestParam("year") Long year) {

        List<NormCollValueVo> result = gxjckzbnzbService.normzbSource(deptId,normCode,year);
        return AjaxResult.success(result);
    }

    @ApiOperation("重点项目情况")
    @GetMapping("/beihai/comprehensive/majorProject/{deptId}")
    public List<CbArticleVo> getInOutHalfaYearList(@PathVariable("deptId") String deptId) {
        return cbGatherStatisticsBsqService.getMajorProject(deptId);
    }

    @ApiOperation("企业名录")
    @GetMapping(value = "/enterpriseDirectories/{deptId}")
    public AjaxResult enterpriseDirectories(@PathVariable("deptId") String deptId) {
        List<TbGardenBaseData> sysDepts = cbGatherStatisticsBsqService.enterpriseDirectories(deptId);
        return AjaxResult.success(sysDepts);
    }


    @ApiOperation("园区产业结构和分布")
    @PostMapping("/comprehensive/gardenIndustryDistribute")
    public AjaxResult gardenIndustryDistribute(String deptId) {
        List<YqcyjghfbVo> result = gxjckzbnzbService.gardenIndustryDistribute(deptId);
        return AjaxResult.success(result);
    }
    @ApiOperation("园区入驻企业账册分类情况")
    @GetMapping(value = "/yqqyrzqk/{deptId}")
    public AjaxResult yqqyrzqk(@PathVariable("deptId") String deptId) {
        List<YqqerzqkVo> yqqyrzqk = gxjckzbnzbService.yqqyrzqk(deptId);
        return AjaxResult.success(yqqyrzqk);
    }

    @ApiOperation("获取园区指定相关数据")
    @GetMapping(value = "/dept/infoValue/{deptId}")
    public AjaxResult deptInfoValue(@PathVariable("deptId") String deptId) {
        JSONObject result = gxjckzbnzbService.deptInfoValue(deptId);
        return AjaxResult.success(result);
    }

}
