package com.hjy.modlue.gather;


import com.hjy.common.core.domain.AjaxResult;
import com.hjy.module.mapper.garden.TbEnterpriseTradePerformanceMapper;
import com.hjy.module.service.garden.ICbGardenDataService;
import com.hjy.module.service.gather.ICbGatherStatisticsQyService;
import com.hjy.module.vo.analysis.GardenAnalysisVo;
import com.hjy.module.vo.gather.EnterpriseDataVo;
import com.hjy.module.vo.gather.statistics.comprehensive.DeptVO;
import com.hjy.module.vo.gather.statistics.comprehensive.GylgxVO;
import com.hjy.module.vo.gather.statistics.comprehensive.QyzyzbVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Objects;

/**
 * @author 12635
 */
@Api(tags = "企业大屏统计Controller")
@RestController
@RequestMapping("/gather/statistics/qy")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbGatherStatisticsQyController {

    private final ICbGatherStatisticsQyService cbGatherStatisticsQyService;

    private final ICbGardenDataService gardenDataService;
    ICbGardenDataService cbGardenDataService;

    TbEnterpriseTradePerformanceMapper tbEnterpriseTradePerformanceMapper;

    @Autowired
    private ICbGardenDataService cbGardenDataService(ICbGardenDataService cbGardenDataService) {
        return this.cbGardenDataService = cbGardenDataService;
    }


    @ApiOperation("获取企业进出口总值历年大屏数据")
    @ApiResponse(code = 200, message = "响应成功; 返回有两个数据；‘Inout’代表是进出口，‘operate’代表经营收入", response = GardenAnalysisVo.class)
    @GetMapping("/inout/year/total/valur/{deptId}")
    public AjaxResult getInoutYearTotalValue(@PathVariable("deptId") String deptId) {
        return AjaxResult.success(gardenDataService.queryInoutYearTotalValueByDeptId(deptId));
    }

    @ApiOperation("企业基本信息")
    @GetMapping("/beihai/comprehensive/qyjbxx/{deptId}")
    public AjaxResult getqyjbxx(@PathVariable("deptId") Integer deptId) {
        DeptVO deptVO = cbGatherStatisticsQyService.getqyjbxx(deptId);
        if (Objects.nonNull(deptVO.getSwbmss())) {
            deptVO.setSwbmss(deptVO.getSwbmss() * 10000);
        }
        if (Objects.nonNull(deptVO.getSjsywzje())) {
            deptVO.setSjsywzje(deptVO.getSjsywzje() * 10000);
        }
        if (Objects.nonNull(deptVO.getHgss())) {
            deptVO.setHgss(deptVO.getHgss() * 10000);
        }
        if (Objects.nonNull(deptVO.getJysr())) {
            deptVO.setJysr(deptVO.getJysr() * 10000);
        }
        if (Objects.nonNull(deptVO.getJckzz())) {
            deptVO.setJckzz(deptVO.getJckzz() * 10000);
        }
        List<GylgxVO> list = cbGatherStatisticsQyService.getgylgxgs(deptId);
        List<QyzyzbVO> qyzyzbVOList = cbGatherStatisticsQyService.getqyzyzb(deptId);
        deptVO.setGylgxVOS(list);
        deptVO.setQyzyzbVOS(qyzyzbVOList);
        return AjaxResult.success(deptVO);
    }


    @ApiOperation("企业进出口总值,经营收入占比")
    @GetMapping(value = "/jckzzyqjnzb/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        List<EnterpriseDataVo> jckzzyqjnzb = cbGatherStatisticsQyService.jckzzyqjnzb(id);
        return AjaxResult.success(jckzzyqjnzb);
    }

    @ApiOperation("企业基本信息环比  --  已废弃")
    @GetMapping(value = "/qyjbxxhb/{deptId}")
    public AjaxResult getInfo(@PathVariable("deptId") Integer deptId) {
        return AjaxResult.success(cbGatherStatisticsQyService.qyjbxxhb(deptId));
    }

    @ApiOperation("企业的主要指标项")
    @GetMapping(value = "/enterpriseMainIndex")
    public AjaxResult enterpriseMainIndex(String deptId) {
        return AjaxResult.success(cbGatherStatisticsQyService.enterpriseMainIndex(deptId));
    }
    @ApiOperation("企业基本信息和其他指标项")
    @GetMapping(value = "/abc/{deptId}")
    public AjaxResult abc(@PathVariable("deptId") String deptId) {
        return AjaxResult.success(cbGatherStatisticsQyService.abc(deptId));
    }

    @ApiOperation("企业各统计图接口")
    @GetMapping(value = "/year")
    public AjaxResult month(String deptId) {
        return AjaxResult.success(cbGatherStatisticsQyService.queryList(deptId));
    }

    @ApiOperation("供应链关系接口")
    @GetMapping(value = "/screenData/{deptId}")
    public AjaxResult screenData(@PathVariable("deptId") String deptId) {
        return AjaxResult.success(cbGatherStatisticsQyService.screenData(deptId));
    }
}
