package com.hjy.modlue.gather;


import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.text.Convert;
import com.hjy.common.utils.StringUtils;
import com.hjy.common.utils.spring.SpringUtils;
import com.hjy.module.service.gather.ICbGatherStatisticsWbService;
import com.hjy.module.statistics.service.GxjckzbnzbService;
import com.hjy.module.vo.analysis.YearPercentage;
import com.hjy.module.vo.gather.CbGatherTmpVo;
import com.hjy.module.vo.gather.statistics.comprehensive.InAndOutVo;
import com.hjy.module.vo.statistics.NormCollValueVo;
import com.hjy.system.mapper.SysDeptMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * @author 12635
 */
@Api(tags = "湾办大屏统计Controller")
@RestController
@RequestMapping("/gather/statistics/wb")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbGatherStatisticsWbController extends BaseController {

    private final ICbGatherStatisticsWbService cbGatherStatisticsWbService;


    private final GxjckzbnzbService gxjckzbnzbService;


    @ApiOperation("近年进出口总值-百分比")
    @GetMapping("/comprehensive/inout/halfa/list")
    public List<InAndOutVo> getInOutHalfaYearList() {
        return cbGatherStatisticsWbService.getInOutHalfaYearList();
    }

    @ApiOperation("广西本年进出口总值趋势图(亿元)")
    @PostMapping("/comprehensive/total/value/year/listAll")
    public AjaxResult getAllTotalValue() {
        return cbGatherStatisticsWbService.getAllTotalValue();
    }

//    @ApiOperation("广西综保区本年各项进出口值占比2")
//    @PostMapping("/comprehensive/total/value/year/list")
//    public AjaxResult getTotalValueYear() {
//        ArrayList<Qyzyzb2VO> vo = cbGatherStatisticsWbService.getTotalValue(null, null, null);
//        return AjaxResult.success("查询成功", vo);
//    }

    @ApiOperation("广西综保区本年各项进出口值占比1")
    @PostMapping("/comprehensive/total/value/deptId/list")
    public AjaxResult getTotalValueBsq() {
        // 获取最新年份月份
        CbGatherTmpVo vo1 = cbGatherStatisticsWbService.selectTmp(null);
        Map<Integer, Map<String, List<NormCollValueVo>>> vo = cbGatherStatisticsWbService.getTotalValueBsq(vo1.getYear(), vo1.getMonth());
        // 组装数据返回
        Map<String, Object> objectObjectHashMap = new HashMap<>();
        objectObjectHashMap.put("year",vo1.getYear());
        objectObjectHashMap.put("month",vo1.getMonth());
        objectObjectHashMap.put("date",vo);
        return AjaxResult.success(objectObjectHashMap);
    }

    @ApiOperation("各项进出口值本年占比(亿元)")
    @PostMapping ("/comprehensive/selectGxjckzbnzbVoList")
    public AjaxResult selectGxjckzbnzbVoList() {
        // 获取最新年份月份
        CbGatherTmpVo vo1 = cbGatherStatisticsWbService.selectTmp(null);
        Map<Integer, Map<String, List<NormCollValueVo>>> stringListMap = gxjckzbnzbService.selectGxjckzbnzbVoList(vo1.getYear(), vo1.getMonth());
        // 组装数据返回
        Map<String, Object> objectObjectHashMap = new HashMap<>();
        objectObjectHashMap.put("year",vo1.getYear());
        objectObjectHashMap.put("month",vo1.getMonth());
        objectObjectHashMap.put("data",stringListMap);
        return AjaxResult.success(objectObjectHashMap);
    }

    @ApiOperation("广西本年规模效益")
    @PostMapping("/comprehensive/sumStatistics")
    public AjaxResult sumStatistics() {
        // 获取最新年份月份
        CbGatherTmpVo vo1 = cbGatherStatisticsWbService.selectTmp(null);
        Map<String, List<NormCollValueVo>> gxbngmxyVos = gxjckzbnzbService.sumStatistics(vo1.getYear(),vo1.getMonth());
        // 组装数据返回
        Map<String, Object> objectObjectHashMap = new HashMap<>();
        objectObjectHashMap.put("year",vo1.getYear());
        objectObjectHashMap.put("month",vo1.getMonth());
        objectObjectHashMap.put("date",gxbngmxyVos);
        return AjaxResult.success(objectObjectHashMap);
    }

    @ApiOperation("广西综合保税区三大进出口值占比")
    @PostMapping("/comprehensive/GxzhbsqsdzzjckzbVo")
    public AjaxResult gxzhbsqsdzzjckzbVo() {
        // 获取最新年份月份
        CbGatherTmpVo vo = cbGatherStatisticsWbService.selectTmp(null);
        Map<Integer,Map<String, List<NormCollValueVo>>> gxzhbsqsdzzjckzbVos = gxjckzbnzbService.GxzhbsqsdzzjckzbVo(vo.getYear(), vo.getMonth());
        // 组装数据返回
        Map<String, Object> objectObjectHashMap = new HashMap<>();
        objectObjectHashMap.put("year",vo.getYear());
        objectObjectHashMap.put("month",vo.getMonth());
        objectObjectHashMap.put("date",gxzhbsqsdzzjckzbVos);
        return AjaxResult.success(objectObjectHashMap);
    }

    @ApiOperation("广西综保区近年进出口总值对比情况")
    @ApiResponse(code = 200, message = "成功; imExport为进出口总值，cityTotal为市进出口总值，percentage为年增长率，growthRate为同比增长率", response = YearPercentage.class)
    @GetMapping("/inout/year/percentage")
    public AjaxResult getInoutPercentageList() {
        // 获取当前年份有数据的最新月份
        CbGatherTmpVo vo = cbGatherStatisticsWbService.selectTmp(null);
        List<YearPercentage> inoutPercentageList = gxjckzbnzbService.getInoutPercentageList(vo.getYear(), vo.getMonth());
        // 组装数据返回
        Map<String, Object> objectObjectHashMap = new HashMap<>();
        objectObjectHashMap.put("year",vo.getYear());
        objectObjectHashMap.put("month",vo.getMonth());
        objectObjectHashMap.put("date",inoutPercentageList);
        return AjaxResult.success(objectObjectHashMap);
    }

    @ApiOperation("全区进出口总值对比报表数据")
    @GetMapping("/jckzz")
    public AjaxResult getjckzzList() {

        // 获取当前年份有数据的最新月份
        CbGatherTmpVo vo = cbGatherStatisticsWbService.selectTmp(null);

        // 获取最新月份的进出口总值数据
        Map<String, List<NormCollValueVo>> result = new HashMap<>();
        String[] codeArr = new String[]{"A001","F004"};
        for (String code:codeArr){
            if (code.equals("A001")){
                Map<String, List<NormCollValueVo>> temp =  gxjckzbnzbService.normHisZS(null,
                        StringUtils.split(code,"##"),
                        Convert.toLong(vo.getYear()),
                        Convert.toLong(vo.getMonth().replace("月",""))
                        , 6L,
                        true);
                result = putAllItem(result,temp);
            }else if (code.equals("F004")){
                SysDept dept = new SysDept();
                dept.setType(2);
                dept.setDelFlag("0");
                List<String> deptList = SpringUtils.getBean(SysDeptMapper.class).selectDeptList(dept).stream().map(d -> String.valueOf(d.getDeptId())).collect(Collectors.toList());
                Map<String, List<NormCollValueVo>> temp =  gxjckzbnzbService.normHisF004(deptList,
                        StringUtils.split(code,"##"),
                        Convert.toLong(vo.getYear()),
                        Convert.toLong(vo.getMonth().replace("月",""))
                        , 6L,
                        true);
                result = putAllItem(result,temp);
            }
        }

        // 组装数据返回
        Map<String, Object> objectObjectHashMap = new HashMap<>();
        objectObjectHashMap.put("year",vo.getYear());
        objectObjectHashMap.put("month",vo.getMonth());
        objectObjectHashMap.put("date",result);
        return AjaxResult.success(objectObjectHashMap);
    }

    private Map<String, List<NormCollValueVo>> putAllItem(Map<String, List<NormCollValueVo>> result,Map<String, List<NormCollValueVo>> temp){
        if (CollectionUtils.isEmpty(result)) return temp;

        if (CollectionUtils.isEmpty(temp)) return result;

       for (String item1:temp.keySet()){
           for (String item2:result.keySet()){
               if (item1.equals(item2)) result.get(item2).addAll(temp.get(item2));
           }
       }
       return result;
    }


}
