package com.hjy.modlue.gather;

import com.hjy.common.annotation.DataScope;
import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.core.text.Convert;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.enums.WorkFlowServiceEnums;
import com.hjy.common.utils.SecurityUtils;
import com.hjy.module.domain.norm.CbQuotaBonded;
import com.hjy.module.service.norm.ICbQuotaBondedService;
import com.hjy.module.service.wf.IWfInstActionTaskService;
import com.hjy.module.vo.norm.CbQuotaBondedVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.Date;

/**
 * 辅助指标-综保区Controller
 *
 * @author lbl
 * @date 2023-10-09
 */
@Api(tags = "辅助指标-综保区Controller")
@RestController
@RequestMapping("/business/QUOTABONDED")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbQuotaBondedController extends BaseController {

    private final ICbQuotaBondedService cbQuotaBondedService;

    private final IWfInstActionTaskService actionTaskService;

    @ApiOperation("查询辅助指标-综保区列表")
    @PreAuthorize("@ss.hasPermi('business:QUOTABONDED:list')")
    @DataScope(deptAlias="A")
    @GetMapping("/list")
    public TableDataInfo<CbQuotaBondedVo> list(CbQuotaBondedVo entity) {
        return cbQuotaBondedService.queryList(entity);
    }

    @ApiOperation("查询辅助指标-综保区所有列表")
    @DataScope(deptAlias="A")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbQuotaBondedVo entity) {
        return AjaxResult.success("查询成功", cbQuotaBondedService.queryAll(entity));
    }

    @ApiOperation("获取辅助指标-综保区详细信息")
     @PreAuthorize("@ss.hasPermi('business:QUOTABONDED:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbQuotaBondedService.queryById(id));
    }

    @ApiOperation("新增辅助指标-综保区")
     @PreAuthorize("@ss.hasPermi('business:QUOTABONDED:add')")
    @Log(title = "辅助指标-综保区", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbQuotaBondedVo entity) {
        return toAjax(cbQuotaBondedService.saveFee(entity));
    }

    @ApiOperation("修改辅助指标-综保区")
     @PreAuthorize("@ss.hasPermi('business:QUOTABONDED:edit')")
    @Log(title = "辅助指标-综保区", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbQuotaBondedVo entity) {
        return toAjax(cbQuotaBondedService.updateFeeById(entity));
    }

    @ApiOperation("删除辅助指标-综保区")
     @PreAuthorize("@ss.hasPermi('business:QUOTABONDED:remove')")
    @Log(title = "辅助指标-综保区", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbQuotaBondedService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }

    /*-------------------------------------------------------------*/
    @ApiOperation("辅助指标提交审核")
    @PreAuthorize("@ss.hasPermi('business:QUOTABONDED:audit')")
    @Log(title = "辅助指标提交审核", businessType = BusinessType.UPDATE)
    @PostMapping("audit")
    public AjaxResult audit(String quotoBondedId) {

        CbQuotaBonded bonded = cbQuotaBondedService.getById(quotoBondedId);
        bonded.setAuditTime(new Date());
        bonded.setAuditName(SecurityUtils.getLoginUser().getUser().getNickName());
        actionTaskService.statrWfqueue(WorkFlowServiceEnums.QUOTABONDEDSERVICE.getService(),
                quotoBondedId,
                bonded.getYear()+"-"+"辅助指标",
                Convert.toStr(SecurityUtils.getUserId()));
        bonded.setApprovalStatus("1");
        return toAjax(cbQuotaBondedService.updateById(bonded));
    }

    @ApiOperation("保税区管理员上报辅助指标")
    @PreAuthorize("@ss.hasPermi('business:QUOTABONDED:report')")
    @Log(title = "保税区管理员上报辅助指标", businessType = BusinessType.UPDATE)
    @PostMapping("report")
    public AjaxResult report(String quotoBondedId,String type) {
        CbQuotaBonded bonded = cbQuotaBondedService.getById(quotoBondedId);
        if(!bonded.getApprovalStatus().equals("2")){
            return AjaxResult.error("当前辅助指标管理员尚未审核审核!");
        }
        if(type.equals("1")){
            bonded.setReportStatus(type);
            bonded.setReportTime(new Date());
            bonded.setReportName(SecurityUtils.getLoginUser().getUser().getNickName());
            return toAjax(cbQuotaBondedService.updateById(bonded));
        }
        return AjaxResult.error("状态有误!");
    }


    /*-------------------------------------------------------------*/
}
