package com.hjy.modlue.gather;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.norm.ICbQuotaBondedFeeService;
import com.hjy.module.vo.norm.CbQuotaBondedFeeVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 辅助指标录入-子表Controller
 *
 * @author Tellsea
 * @date 2023-10-23
 */
@Api(tags = "辅助指标录入-子表Controller")
@RestController
@RequestMapping("/business/QUOTABONDEDFEE")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbQuotaBondedFeeController extends BaseController {

    private final ICbQuotaBondedFeeService cbQuotaBondedFeeService;

    @ApiOperation("查询辅助指标录入-子表列表")
    @GetMapping("/list")
    public TableDataInfo<CbQuotaBondedFeeVo> list(CbQuotaBondedFeeVo entity) {
        return cbQuotaBondedFeeService.queryList(entity);
    }

    @ApiOperation("查询辅助指标录入-子表所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbQuotaBondedFeeVo entity) {
        return AjaxResult.success("查询成功", cbQuotaBondedFeeService.queryAll(entity));
    }

    @ApiOperation("导出辅助指标录入-子表列表")
    @Log(title = "辅助指标录入-子表", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbQuotaBondedFeeVo entity) {
        List<CbQuotaBondedFeeVo> list = cbQuotaBondedFeeService.queryAll(entity);
        ExcelUtil<CbQuotaBondedFeeVo> util = new ExcelUtil<>(CbQuotaBondedFeeVo.class);
        util.exportExcel(response, list, "辅助指标录入-子表数据");
    }

    @ApiOperation("获取辅助指标录入-子表详细信息")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbQuotaBondedFeeService.queryById(id));
    }

    @ApiOperation("新增辅助指标录入-子表")
    @Log(title = "辅助指标录入-子表", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbQuotaBondedFeeVo entity) {
        return toAjax(cbQuotaBondedFeeService.save(entity));
    }

    @ApiOperation("修改辅助指标录入-子表")
    @Log(title = "辅助指标录入-子表", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbQuotaBondedFeeVo entity) {
        return toAjax(cbQuotaBondedFeeService.updateById(entity));
    }

    @ApiOperation("删除辅助指标录入-子表")
    @Log(title = "辅助指标录入-子表", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbQuotaBondedFeeService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
