package com.hjy.modlue.gxzzq;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.gxzzq.ICbGxzzqService;
import com.hjy.module.vo.gxzzq.CbGxzzqVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 广西自治区数据Controller
 *
 * @author Tellsea
 * @date 2023-10-14
 */
@Api(tags = "广西自治区数据Controller")
@RestController
@RequestMapping("/business/GXZZQ")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbGxzzqController extends BaseController {

    private final ICbGxzzqService cbGxzzqService;

    @ApiOperation("查询广西自治区数据列表")
    @PreAuthorize("@ss.hasPermi('business:GXZZQ:list')")
    @GetMapping("/list")
    public TableDataInfo<CbGxzzqVo> list(CbGxzzqVo entity) {
        return cbGxzzqService.queryList(entity);
    }

    @ApiOperation("查询广西自治区数据所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbGxzzqVo entity) {
        return AjaxResult.success("查询成功", cbGxzzqService.queryAll(entity));
    }

    @ApiOperation("导出广西自治区数据列表")
    @PreAuthorize("@ss.hasPermi('business:GXZZQ:export')")
    @Log(title = "广西自治区数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbGxzzqVo entity) {
        List<CbGxzzqVo> list = cbGxzzqService.queryAll(entity);
        ExcelUtil<CbGxzzqVo> util = new ExcelUtil<>(CbGxzzqVo.class);
        util.exportExcel(response, list, "广西自治区数据数据");
    }

    @ApiOperation("获取广西自治区数据详细信息")
    @PreAuthorize("@ss.hasPermi('business:GXZZQ:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbGxzzqService.queryById(id));
    }

    @ApiOperation("新增广西自治区数据")
    @PreAuthorize("@ss.hasPermi('business:GXZZQ:add')")
    @Log(title = "广西自治区数据", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbGxzzqVo entity) {
        return toAjax(cbGxzzqService.save(entity));
    }

    @ApiOperation("修改广西自治区数据")
    @PreAuthorize("@ss.hasPermi('business:GXZZQ:edit')")
    @Log(title = "广西自治区数据", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbGxzzqVo entity) {
        return toAjax(cbGxzzqService.updateById(entity));
    }

    @ApiOperation("删除广西自治区数据")
    @PreAuthorize("@ss.hasPermi('business:GXZZQ:remove')")
    @Log(title = "广西自治区数据", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbGxzzqService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
