package com.hjy.modlue.midwest;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;

import com.hjy.module.service.midwest.ITbMidwestService;
import com.hjy.module.vo.midwest.TbMidwestVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import com.hjy.common.core.page.TableDataInfo;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 中西部综保区Controller
 *
 * @author Tellsea
 * @date 2024-06-24
 */
@Api(tags = "中西部综保区Controller")
@RestController
@RequestMapping("/midwest/MIDWEST")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class TbMidwestController extends BaseController {

    private final ITbMidwestService tbMidwestService;

    @ApiOperation("查询中西部综保区列表")
    @PreAuthorize("@ss.hasPermi('midwest:MIDWEST:list')")
    @GetMapping("/list")
    public TableDataInfo<TbMidwestVo> list(TbMidwestVo entity) {
        return tbMidwestService.queryList(entity);
    }

    @ApiOperation("查询中西部综保区所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(TbMidwestVo entity) {
        return AjaxResult.success("查询成功", tbMidwestService.queryAll(entity));
    }

    @ApiOperation("导出中西部综保区列表")
    @PreAuthorize("@ss.hasPermi('midwest:MIDWEST:export')")
    @Log(title = "中西部综保区", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TbMidwestVo entity) {
        List<TbMidwestVo> list = tbMidwestService.queryAll(entity);
        ExcelUtil<TbMidwestVo> util = new ExcelUtil<>(TbMidwestVo.class);
        util.exportExcel(response, list, "中西部综保区数据");
    }

    @ApiOperation("获取中西部综保区详细信息")
    @PreAuthorize("@ss.hasPermi('midwest:MIDWEST:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", tbMidwestService.queryById(id));
    }

    @ApiOperation("新增中西部综保区")
    @PreAuthorize("@ss.hasPermi('midwest:MIDWEST:add')")
    @Log(title = "中西部综保区", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody TbMidwestVo entity) {
        return toAjax(tbMidwestService.save(entity));
    }

    @ApiOperation("修改中西部综保区")
    @PreAuthorize("@ss.hasPermi('midwest:MIDWEST:edit')")
    @Log(title = "中西部综保区", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody TbMidwestVo entity) {
        return toAjax(tbMidwestService.updateById(entity));
    }

    @ApiOperation("删除中西部综保区")
    @PreAuthorize("@ss.hasPermi('midwest:MIDWEST:remove')")
    @Log(title = "中西部综保区", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(tbMidwestService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
