package com.hjy.modlue.norm;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.norm.ICbContrastConfigService;
import com.hjy.module.vo.norm.CbContrastConfigVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 对比模型参数配置Controller
 *
 * @author Tellsea
 * @date 2023-08-24
 */
@Api(tags = "对比模型参数配置Controller")
@RestController
@RequestMapping("/contrastConfig")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbContrastConfigController extends BaseController {

    private final ICbContrastConfigService cbContrastConfigService;

    @ApiOperation("查询对比模型参数配置列表")
    @PreAuthorize("@ss.hasPermi('modlue:CONTRASTCONFIG:list')")
    @GetMapping("/list")
    public TableDataInfo<CbContrastConfigVo> list(CbContrastConfigVo entity) {
        return cbContrastConfigService.queryList(entity);
    }

    @ApiOperation("查询对比模型参数配置所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbContrastConfigVo entity) {
        return AjaxResult.success("查询成功", cbContrastConfigService.queryAll(entity));
    }

    @ApiOperation("导出对比模型参数配置列表")
    @PreAuthorize("@ss.hasPermi('modlue:CONTRASTCONFIG:export')")
    @Log(title = "对比模型参数配置", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbContrastConfigVo entity) {
        List<CbContrastConfigVo> list = cbContrastConfigService.queryAll(entity);
        ExcelUtil<CbContrastConfigVo> util = new ExcelUtil<>(CbContrastConfigVo.class);
        util.exportExcel(response, list, "对比模型参数配置数据");
    }

    @ApiOperation("获取对比模型参数配置详细信息")
    @PreAuthorize("@ss.hasPermi('modlue:CONTRASTCONFIG:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbContrastConfigService.queryById(id));
    }

    @ApiOperation("新增对比模型参数配置")
    @PreAuthorize("@ss.hasPermi('modlue:CONTRASTCONFIG:add')")
    @Log(title = "对比模型参数配置", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbContrastConfigVo entity) {
        return toAjax(cbContrastConfigService.save(entity));
    }

    @ApiOperation("修改对比模型参数配置")
    @PreAuthorize("@ss.hasPermi('modlue:CONTRASTCONFIG:edit')")
    @Log(title = "对比模型参数配置", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbContrastConfigVo entity) {
        return toAjax(cbContrastConfigService.updateById(entity));
    }

    @ApiOperation("删除对比模型参数配置")
    @PreAuthorize("@ss.hasPermi('modlue:CONTRASTCONFIG:remove')")
    @Log(title = "对比模型参数配置", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbContrastConfigService.removeByIds(Arrays.asList(ids)));
    }
}
