package com.hjy.modlue.norm;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.norm.ICbContrastService;
import com.hjy.module.vo.norm.CbContrastVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 对比模型公式Controller
 *
 * @author Tellsea
 * @date 2023-08-24
 */
@Api(tags = "对比模型公式Controller")
@RestController
@RequestMapping("/contrast")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbContrastController extends BaseController {

    private final ICbContrastService cbContrastService;

    @ApiOperation("查询对比模型公式列表")
    @PreAuthorize("@ss.hasPermi('modlue:CONTRAST:list')")
    @GetMapping("/list")
    public TableDataInfo<CbContrastVo> list(CbContrastVo entity) {
        return cbContrastService.queryList(entity);
    }

    @ApiOperation("查询对比模型公式所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbContrastVo entity) {
        return AjaxResult.success("查询成功", cbContrastService.queryAll(entity));
    }

    @ApiOperation("导出对比模型公式列表")
    @PreAuthorize("@ss.hasPermi('modlue:CONTRAST:export')")
    @Log(title = "对比模型公式", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbContrastVo entity) {
        List<CbContrastVo> list = cbContrastService.queryAll(entity);
        ExcelUtil<CbContrastVo> util = new ExcelUtil<>(CbContrastVo.class);
        util.exportExcel(response, list, "对比模型公式数据");
    }

    @ApiOperation("获取对比模型公式详细信息")
    @PreAuthorize("@ss.hasPermi('modlue:CONTRAST:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbContrastService.queryById(id));
    }

    @ApiOperation("新增对比模型公式")
    @PreAuthorize("@ss.hasPermi('modlue:CONTRAST:add')")
    @Log(title = "对比模型公式", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbContrastVo entity) {
        return toAjax(cbContrastService.saveEntity(entity));
    }

    @ApiOperation("修改对比模型公式")
    @PreAuthorize("@ss.hasPermi('modlue:CONTRAST:edit')")
    @Log(title = "对比模型公式", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbContrastVo entity) {
        return toAjax(cbContrastService.editEntity(entity));
    }

    @ApiOperation("删除对比模型公式")
    @PreAuthorize("@ss.hasPermi('modlue:CONTRAST:remove')")
    @Log(title = "对比模型公式", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbContrastService.removeByIds(Arrays.asList(ids)));
    }
}
