package com.hjy.modlue.norm;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.norm.ICbGiveService;
import com.hjy.module.vo.norm.CbGiveVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 报送数据项配置Controller
 *
 * @author Tellsea
 * @date 2023-08-30
 */
@Api(tags = "报送数据项配置Controller")
@RestController
@RequestMapping("/norm/GIVE")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbGiveController extends BaseController {

    private final ICbGiveService cbGiveService;

    @ApiOperation("查询报送数据项配置列表")
    @PreAuthorize("@ss.hasPermi('norm:GIVE:list')")
    @GetMapping("/list")
    public TableDataInfo<CbGiveVo> list(CbGiveVo entity) {
        return cbGiveService.queryList(entity);
    }

    @ApiOperation("查询报送数据项配置所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbGiveVo entity) {
        return AjaxResult.success("查询成功", cbGiveService.queryAll(entity));
    }

    @ApiOperation("导出报送数据项配置列表")
    @PreAuthorize("@ss.hasPermi('norm:GIVE:export')")
    @Log(title = "报送数据项配置", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbGiveVo entity) {
        List<CbGiveVo> list = cbGiveService.queryAll(entity);
        ExcelUtil<CbGiveVo> util = new ExcelUtil<>(CbGiveVo.class);
        util.exportExcel(response, list, "报送数据项配置数据");
    }

    @ApiOperation("获取报送数据项配置详细信息")
    @PreAuthorize("@ss.hasPermi('norm:GIVE:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbGiveService.queryById(id));
    }

    @ApiOperation("新增报送数据项配置")
    @PreAuthorize("@ss.hasPermi('norm:GIVE:add')")
    @Log(title = "报送数据项配置", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbGiveVo entity) {
        return toAjax(cbGiveService.save(entity));
    }

    @ApiOperation("修改报送数据项配置")
    @PreAuthorize("@ss.hasPermi('norm:GIVE:edit')")
    @Log(title = "报送数据项配置", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbGiveVo entity) {
        return toAjax(cbGiveService.updateById(entity));
    }

    @ApiOperation("删除报送数据项配置")
    @PreAuthorize("@ss.hasPermi('norm:GIVE:remove')")
    @Log(title = "报送数据项配置", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbGiveService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
