package com.hjy.modlue.norm;

import cn.hutool.json.JSONUtil;
import com.hjy.common.annotation.DataScope;
import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.enums.HisModule;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.commom.HisMethods;
import com.hjy.module.service.norm.ICbNormService;
import com.hjy.module.vo.norm.CbNormVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 指标分类Controller
 *
 * @author Tellsea
 * @date 2023-08-21
 */
@Api(tags = "指标分类Controller")
@RestController
@RequestMapping("/norm")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbNormController extends BaseController {

    private final ICbNormService cbNormService;

    @ApiOperation("查询指标分类列表")
    @DataScope(deptAlias="A")
    @PreAuthorize("@ss.hasPermi('business:NORM:list')")
    @GetMapping("/list")
    public TableDataInfo<CbNormVo> list(CbNormVo entity) {
        return cbNormService.queryList(entity);
    }


    @ApiOperation("查询指标分类所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbNormVo entity) {
        return AjaxResult.success("查询成功", cbNormService.queryAll(entity));
    }

    @ApiOperation("导出指标分类列表")
    @PreAuthorize("@ss.hasPermi('business:NORM:export')")
    @Log(title = "指标分类", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbNormVo entity) {
        List<CbNormVo> list = cbNormService.queryAll(entity);
        ExcelUtil<CbNormVo> util = new ExcelUtil<>(CbNormVo.class);
        util.exportExcel(response, list, "指标分类数据");
    }

    @ApiOperation("获取指标分类详细信息")
    @PreAuthorize("@ss.hasPermi('business:NORM:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbNormService.queryById(id));
    }

    @ApiOperation("新增指标分类")
    @PreAuthorize("@ss.hasPermi('business:NORM:add')")
    @Log(title = "指标分类", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbNormVo entity) {

        if(!cbNormService.checkName(entity.getName(), null)) {
            return AjaxResult.error("指标分类名已存在, 请更换后再试~");
        }

        return toAjax(cbNormService.save(entity));
    }

    @ApiOperation("修改指标分类")
    @PreAuthorize("@ss.hasPermi('business:NORM:edit')")
    @Log(title = "指标分类", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbNormVo entity) {

        if(!cbNormService.checkName(entity.getName(), entity.getId())) {
            return AjaxResult.error("指标分类名已存在, 请更换后再试~");
        }

        boolean flag = cbNormService.updateById(entity);
        if (flag){
            HisMethods.saveFromHis(HisModule.MODULE2.getCode(),entity.getId(), JSONUtil.toJsonStr(entity));
        }
        return toAjax(flag);
    }

    @ApiOperation("删除指标分类")
    @PreAuthorize("@ss.hasPermi('business:NORM:remove')")
    @Log(title = "指标分类", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbNormService.removeByIds(Arrays.asList(ids)));
    }
}
