package com.hjy.modlue.norm;

import cn.hutool.json.JSONUtil;
import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.enums.HisModule;
import com.hjy.module.service.commom.HisMethods;
import com.hjy.module.service.norm.ICbNormFeeService;
import com.hjy.module.vo.norm.CbNormFeeVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;

/**
 * 指标权重明细Controller
 *
 * @author Tellsea
 * @date 2023-08-21
 */
@Api(tags = "指标权重明细Controller")
@RestController
@RequestMapping("/normFee")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbNormFeeController extends BaseController {

    private final ICbNormFeeService cbNormFeeService;

    @ApiOperation("查询指标权重明细列表")
    @GetMapping("/list")
    public TableDataInfo<CbNormFeeVo> list(CbNormFeeVo entity) {
        return cbNormFeeService.queryList(entity);
    }

    @ApiOperation("查询指标权重明细所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbNormFeeVo entity) {
        return AjaxResult.success("查询成功", cbNormFeeService.queryAll(entity));
    }


    @ApiOperation("获取指标权重明细详细信息")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbNormFeeService.queryById(id));
    }

    @ApiOperation("新增指标权重明细")
    @Log(title = "指标权重明细", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbNormFeeVo entity) {
        Integer sort = entity.getSort();
        if (sort == null) {
            // 默认为 1
            entity.setSort(1);
        }
        return toAjax(cbNormFeeService.save(entity));
    }

    @ApiOperation("修改指标权重明细")
    @Log(title = "指标权重明细", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbNormFeeVo entity) {
        boolean flag = cbNormFeeService.updateById(entity);
        if (flag) {
            HisMethods.saveFromHis(HisModule.MODULE2.getCode(), entity.getId(), JSONUtil.toJsonStr(entity));
        }
        return toAjax(flag);
    }

    @ApiOperation("修改指标计算公式")
    @Log(title = "指标计算公式", businessType = BusinessType.UPDATE)
    @PostMapping("updateFormulaById")
    public AjaxResult updateFormulaById(@RequestBody CbNormFeeVo entity) {
        boolean flag = cbNormFeeService.updateFormulaById(entity.getFormula(), entity.getId());
        return toAjax(flag);
    }

    @ApiOperation("删除指标权重明细")
    @Log(title = "指标权重明细", businessType = BusinessType.DELETE)
    @GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbNormFeeService.removeByIds(Arrays.asList(ids)));
    }
}
