package com.hjy.modlue.norm;

import cn.hutool.core.collection.CollectionUtil;
import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.DateUtils;
import com.hjy.module.domain.norm.CbNormRole;
import com.hjy.module.mapper.norm.CbNormRoleMapper;
import com.hjy.module.service.norm.ICbNormRoleService;
import com.hjy.module.vo.norm.CbNormRoleVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * 指标采集权限配置Controller
 *
 * @author Tellsea
 * @date 2023-10-17
 */
@Api(tags = "指标采集权限配置Controller")
@RestController
@RequestMapping("/norm/NORMROLE")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbNormRoleController extends BaseController {

    private final ICbNormRoleService cbNormRoleService;

    private final CbNormRoleMapper normRoleMapper;

    @ApiOperation("查询指标采集权限配置列表")
    @PreAuthorize("@ss.hasPermi('norm:NORMROLE:list')")
    @GetMapping("/list")
    public TableDataInfo<CbNormRoleVo> list(CbNormRoleVo entity) {
        return cbNormRoleService.queryList(entity);
    }

    @ApiOperation("查询指标采集权限配置所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbNormRoleVo entity) {
        return AjaxResult.success("查询成功", cbNormRoleService.queryAll(entity));
    }


    @ApiOperation("获取指标采集权限配置详细信息")
    @PreAuthorize("@ss.hasPermi('norm:NORMROLE:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbNormRoleService.queryById(id));
    }

    @ApiOperation("新增指标采集权限配置")
    @PreAuthorize("@ss.hasPermi('norm:NORMROLE:add')")
    @Log(title = "指标采集权限配置", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbNormRoleVo entity) {
        if (!checkdata(entity,false)) return error("该指标已分配，无法再次分配");
        entity.setStartTime(new Date());
        return toAjax(cbNormRoleService.save(entity));
    }

    /**
     * 校验数据
     * @param entity
     * @param isupdate
     * @return
     */
    private boolean checkdata(CbNormRoleVo entity,boolean isupdate){
        if(isupdate){
            if (normRoleMapper.checkdata(entity.getFeeId(),entity.getId(),"2")>0){
                return false;
            }else {
                return true;
            }
        }else{
            if (normRoleMapper.checkdata(entity.getFeeId(),null,"1")>0){
                return false;
            }else {
                return true;
            }
        }
    }

    @ApiOperation("修改指标采集权限配置")
    @PreAuthorize("@ss.hasPermi('norm:NORMROLE:edit')")
    @Log(title = "指标采集权限配置", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbNormRoleVo entity) {
        if (!checkdata(entity,true)) return error("该指标已分配，无法再次分配");

        return toAjax(cbNormRoleService.updateById(entity));
    }

    @ApiOperation("删除指标采集权限配置")
    @PreAuthorize("@ss.hasPermi('norm:NORMROLE:remove')")
    @Log(title = "指标采集权限配置", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbNormRoleService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }

    @ApiOperation("分配项启用")
    @Log(title = "分配项启用", businessType = BusinessType.UPDATE)
    @PostMapping("strta/{id}")
    public AjaxResult strta(@PathVariable String id) {

        CbNormRoleVo roleVo = cbNormRoleService.queryById(id);
        CbNormRoleVo selRoleVo = new CbNormRoleVo();
        selRoleVo.setFeeId(roleVo.getFeeId());
        selRoleVo.setStatus("1");
        List<CbNormRoleVo> cbNormRoleVoList = cbNormRoleService.queryAll(selRoleVo);

        if (CollectionUtil.isNotEmpty(cbNormRoleVoList) && cbNormRoleVoList.size()>0){
            return error("该指标已分配，无法启用");
        }

        CbNormRole cbNormRole = new CbNormRole();
        cbNormRole.setId(id);
        cbNormRole.setStatus("1");
        cbNormRole.setStartTime(DateUtils.getNowDate());
        return toAjax(cbNormRoleService.updateById(cbNormRole));
    }

    @ApiOperation("分配项停用")
    @Log(title = "分配项停用", businessType = BusinessType.UPDATE)
    @PostMapping("stop/{id}")
    public AjaxResult stop(@PathVariable String id) {
       CbNormRole cbNormRole = new CbNormRole();
        cbNormRole.setId(id);
        cbNormRole.setStatus("0");
        cbNormRole.setStopTime(DateUtils.getNowDate());
        return toAjax(cbNormRoleService.updateById(cbNormRole));
    }
}
