package com.hjy.modlue.norm;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.norm.ICbQuotaService;
import com.hjy.module.vo.norm.CbQuotaVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 辅助指标分类Controller
 *
 * @author Tellsea
 * @date 2023-10-09
 */
@Api(tags = "辅助指标分类Controller")
@RestController
@RequestMapping("/business/QUOTA")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbQuotaController extends BaseController {

    private final ICbQuotaService cbQuotaService;

    @ApiOperation("查询辅助指标分类列表")
    @PreAuthorize("@ss.hasPermi('business:QUOTA:list')")
    @GetMapping("/list")
    public TableDataInfo<CbQuotaVo> list(CbQuotaVo entity) {
        return cbQuotaService.queryList(entity);
    }

    @ApiOperation("查询辅助指标分类所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbQuotaVo entity) {
        return AjaxResult.success("查询成功", cbQuotaService.queryAll(entity));
    }

    @ApiOperation("导出辅助指标分类列表")
    @PreAuthorize("@ss.hasPermi('business:QUOTA:export')")
    @Log(title = "辅助指标分类", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbQuotaVo entity) {
        List<CbQuotaVo> list = cbQuotaService.queryAll(entity);
        ExcelUtil<CbQuotaVo> util = new ExcelUtil<>(CbQuotaVo.class);
        util.exportExcel(response, list, "辅助指标分类数据");
    }

    @ApiOperation("获取辅助指标分类详细信息")
    @PreAuthorize("@ss.hasPermi('business:QUOTA:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbQuotaService.queryById(id));
    }

    @ApiOperation("新增辅助指标分类")
    @PreAuthorize("@ss.hasPermi('business:QUOTA:add')")
    @Log(title = "辅助指标分类", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbQuotaVo entity) {
        return toAjax(cbQuotaService.save(entity));
    }

    @ApiOperation("修改辅助指标分类")
    @PreAuthorize("@ss.hasPermi('business:QUOTA:edit')")
    @Log(title = "辅助指标分类", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbQuotaVo entity) {
        return toAjax(cbQuotaService.updateById(entity));
    }

    @ApiOperation("删除辅助指标分类")
    @PreAuthorize("@ss.hasPermi('business:QUOTA:remove')")
    @Log(title = "辅助指标分类", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbQuotaService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
