package com.hjy.modlue.norm;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.core.text.Convert;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.SecurityUtils;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.norm.ICbQuotaFeeService;
import com.hjy.module.vo.norm.CbQuotaFeeVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 辅助指标明细Controller
 *
 * @author lbl
 * @date 2023-10-08
 */
@Api(tags = "辅助指标明细Controller")
@RestController
@RequestMapping("/business/QUOTAFEE")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbQuotaFeeController extends BaseController {

    private final ICbQuotaFeeService cbQuotaFeeService;

    @ApiOperation("查询辅助指标明细列表")
    @GetMapping("/list")
    public TableDataInfo<CbQuotaFeeVo> list(CbQuotaFeeVo entity) {
        return cbQuotaFeeService.queryList(entity);
    }

    @ApiOperation("查询辅助指标明细所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbQuotaFeeVo entity) {
        return AjaxResult.success("查询成功", cbQuotaFeeService.queryAll(entity));
    }


    @ApiOperation("过滤后的指标数据")
    @GetMapping("/listAll/byRole")
    public AjaxResult listAllByRole(CbQuotaFeeVo entity) {
        entity.setUserId(Convert.toStr(SecurityUtils.getUserId()));
        return AjaxResult.success("查询成功", cbQuotaFeeService.queryAll(entity));
    }


    @ApiOperation("导出辅助指标明细列表")
    @Log(title = "辅助指标明细", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbQuotaFeeVo entity) {
        List<CbQuotaFeeVo> list = cbQuotaFeeService.queryAll(entity);
        ExcelUtil<CbQuotaFeeVo> util = new ExcelUtil<>(CbQuotaFeeVo.class);
        util.exportExcel(response, list, "辅助指标明细数据");
    }

    @ApiOperation("获取辅助指标明细详细信息")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbQuotaFeeService.queryById(id));
    }

    @ApiOperation("新增辅助指标明细")
    @Log(title = "辅助指标明细", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbQuotaFeeVo entity) {
        return toAjax(cbQuotaFeeService.add(entity));
    }

    @ApiOperation("修改辅助指标明细")
    @Log(title = "辅助指标明细", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbQuotaFeeVo entity) {
        return toAjax(cbQuotaFeeService.updateById(entity));
    }

    @ApiOperation("删除辅助指标明细")
    @Log(title = "辅助指标明细", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbQuotaFeeService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
