package com.hjy.modlue.portal;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.enums.BusinessType;
import com.hjy.module.service.portal.ICbBondedService;
import com.hjy.system.service.ISysDeptService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 保税区管理Controller
 *
 * @author Tellsea
 * @date 2023-09-08
 */
@Api(tags = "保税区管理Controller")
@RestController
@RequestMapping("/BONDED")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbBondedController extends BaseController {

    private final ICbBondedService cbBondedService;

    private final ISysDeptService deptService;

    @ApiOperation("查询保税区管理列表")
    @PreAuthorize("@ss.hasPermi('business:BONDED:list')")
    @GetMapping("/list")
    public List<SysDept> list() {
        SysDept sysDept = new SysDept();
        sysDept.setType(2);
        return deptService.selectDeptList(sysDept);
    }


    @ApiOperation("获取保税区管理详细信息")
    @PreAuthorize("@ss.hasPermi('business:BONDED:query')")
    @GetMapping(value = "/getInfo/{deptId}")
    public AjaxResult getInfo(@PathVariable("deptId") Long deptId) {
        return AjaxResult.success("查询成功", deptService.selectDeptById(deptId));
    }

    /**
     * 修改部门
     */
    @PreAuthorize("@ss.hasPermi('system:dept:edit')")
    @Log(title = "部门管理", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@Validated @RequestBody SysDept dept) {
        Long deptId = dept.getDeptId();
        double warehouseRate = dept.getWarehouseUsable() / dept.getWarehouse();
        double oilDepotRate = dept.getOilDepotUsable() / dept.getOilDepot();
        double refrigeratoryRate = dept.getRefrigeratoryUsable() / dept.getRefrigeratory();
        double workshopRate = dept.getWorkshopUsable() / dept.getWorkshop();

        dept.setWarehouseRate(warehouseRate);
        dept.setOilDepotRate(oilDepotRate);
        dept.setRefrigeratoryRate(refrigeratoryRate);
        dept.setWorkshopRate(workshopRate);

        double useArea = dept.getWarehouseUsable() + dept.getOilDepotUsable() + dept.getRefrigeratoryUsable() + dept.getWorkshopUsable();
        if (useArea == 0.0d) {
            dept.setUsageRate("0%");
        } else {
            String usageRate = String.format("%.2f", useArea / (dept.getWarehouse() + dept.getOilDepot() + dept.getRefrigeratory() + dept.getWorkshop()) * 100) + "%";
            dept.setUsageRate(usageRate);
        }
        dept.setUseArea(useArea);
        deptService.checkDeptDataScope(deptId);
        deptService.update(dept);
        return AjaxResult.success("修改成功");
    }

}
