package com.hjy.modlue.portal;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.RedisUtil;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.portal.ICbCompanyUserService;
import com.hjy.module.service.wf.IWfInstActionTaskService;
import com.hjy.module.vo.portal.CbCompanyUserVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 门户用户Controller
 *
 * @author Tellsea
 * @date 2023-08-23
 */
@Api(tags = "门户用户Controller")
@RestController
@RequestMapping("/COMPANYUSER")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbCompanyUserController extends BaseController {

    private final ICbCompanyUserService cbCompanyUserService;

    private final IWfInstActionTaskService wfInstActionTaskService;

    private final RedisUtil redisUtil;


    @ApiOperation("查询门户用户列表")
    @PreAuthorize("@ss.hasPermi('business:COMPANYUSER:list')")
    @GetMapping("/list")
    public TableDataInfo<CbCompanyUserVo> list(CbCompanyUserVo entity) {
        return cbCompanyUserService.queryList(entity);
    }

    @ApiOperation("查询门户用户所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbCompanyUserVo entity) {
        return AjaxResult.success("查询成功", cbCompanyUserService.queryAll(entity));
    }

    @ApiOperation("导出门户用户列表")
    @PreAuthorize("@ss.hasPermi('business:COMPANYUSER:export')")
    @Log(title = "门户用户", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbCompanyUserVo entity) {
        List<CbCompanyUserVo> list = cbCompanyUserService.queryAll(entity);
        ExcelUtil<CbCompanyUserVo> util = new ExcelUtil<>(CbCompanyUserVo.class);
        util.exportExcel(response, list, "门户用户数据");
    }

    @ApiOperation("获取门户用户详细信息")
    @PreAuthorize("@ss.hasPermi('business:COMPANYUSER:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功",cbCompanyUserService.queryById(id));
    }

    @ApiOperation("新增门户用户")
    @PreAuthorize("@ss.hasPermi('business:COMPANYUSER:add')")
    @Log(title = "门户用户", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbCompanyUserVo entity) {
        return toAjax(cbCompanyUserService.save(entity));
    }

    @ApiOperation("修改门户用户")
    @PreAuthorize("@ss.hasPermi('business:COMPANYUSER:edit')")
    @Log(title = "门户用户", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbCompanyUserVo entity) {
        cbCompanyUserService.updateById(entity);
        return AjaxResult.success("修改成功");
    }

    @ApiOperation("删除门户用户")
    @PreAuthorize("@ss.hasPermi('business:COMPANYUSER:remove')")
    @Log(title = "门户用户", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbCompanyUserService.removeByIds(Arrays.asList(ids)));
    }

}
