package com.hjy.modlue.portal;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.SecurityUtils;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.portal.ICbConferenceReplyService;
import com.hjy.module.service.portal.ICbJointConferenceService;
import com.hjy.module.vo.portal.CbConferenceReplyVo;
import com.hjy.module.vo.portal.CbJointConferenceVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 联席会议回复Controller
 *
 * @author Tellsea
 * @date 2023-09-27
 */
@Api(tags = "联席会议回复Controller")
@RestController
@RequestMapping("/CONFERENCEREPLY")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbConferenceReplyController extends BaseController {

    private final ICbConferenceReplyService cbConferenceReplyService;

    private final ICbJointConferenceService cbJointConferenceService;

    @ApiOperation("查询联席会议回复列表")
    @PreAuthorize("@ss.hasPermi('business:CONFERENCEREPLY:list')")
    @GetMapping("/list")
    public TableDataInfo<CbConferenceReplyVo> list(CbConferenceReplyVo entity) {
        return cbConferenceReplyService.queryList(entity);
    }

    @ApiOperation("查询联席会议回复所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbConferenceReplyVo entity) {
        return AjaxResult.success("查询成功", cbConferenceReplyService.queryAll(entity));
    }

    @ApiOperation("导出联席会议回复列表")
    @PreAuthorize("@ss.hasPermi('business:CONFERENCEREPLY:export')")
    @Log(title = "联席会议回复", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbConferenceReplyVo entity) {
        List<CbConferenceReplyVo> list = cbConferenceReplyService.queryAll(entity);
        ExcelUtil<CbConferenceReplyVo> util = new ExcelUtil<>(CbConferenceReplyVo.class);
        util.exportExcel(response, list, "联席会议回复数据");
    }

    @ApiOperation("获取联席会议回复详细信息")
    @PreAuthorize("@ss.hasPermi('business:CONFERENCEREPLY:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbConferenceReplyService.queryById(id));
    }

    @ApiOperation("新增联席会议回复")
    @PreAuthorize("@ss.hasPermi('business:CONFERENCEREPLY:add')")
    @Log(title = "联席会议回复", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbConferenceReplyVo entity) {
        // 新增回复信息
        try {
            entity.setDeptName(SecurityUtils.getLoginUser().getUser().getDept().getDeptName());
            cbConferenceReplyService.save(entity);
        } catch (Exception e) {
            return AjaxResult.success("该会议已回复");
        }
        // 按id查询出该会议
        CbJointConferenceVo conference = cbJointConferenceService.queryById(entity.getConferenceId());
        // 给会议的回复数量+1
        conference.setReplied(conference.getReplied()+1);
        // 如果目前回复数量等于参与会议数量，则视为会议完成
        if (conference.getReplied().equals(conference.getDeptNum())){
            conference.setStatus(3);
        }
        cbJointConferenceService.updateById(conference);
        return AjaxResult.success("回复成功");
    }

    @ApiOperation("修改联席会议回复")
    @PreAuthorize("@ss.hasPermi('business:CONFERENCEREPLY:edit')")
    @Log(title = "联席会议回复", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbConferenceReplyVo entity) {
        return toAjax(cbConferenceReplyService.updateById(entity));
    }

    @ApiOperation("删除联席会议回复")
    @PreAuthorize("@ss.hasPermi('business:CONFERENCEREPLY:remove')")
    @Log(title = "联席会议回复", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbConferenceReplyService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
