package com.hjy.modlue.portal;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.portal.ICbContactUsService;
import com.hjy.module.vo.portal.CbContactUsVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 联系我们Controller
 *
 * @author Tellsea
 * @date 2023-08-24
 */
@Api(tags = "联系我们Controller")
@RestController
@RequestMapping("/portal/CONTACTUS")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbContactUsController extends BaseController {

    private final ICbContactUsService cbContactUsService;

    @ApiOperation("查询联系我们列表")
    @PreAuthorize("@ss.hasPermi('business:CONTACTUS:list')")
    @GetMapping("/list")
    public TableDataInfo<CbContactUsVo> list(CbContactUsVo entity) {
        return cbContactUsService.queryList(entity);
    }

    @ApiOperation("导出联系我们列表")
    @PreAuthorize("@ss.hasPermi('business:CONTACTUS:export')")
    @Log(title = "联系我们", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbContactUsVo entity) {
        List<CbContactUsVo> list = cbContactUsService.queryAll(entity);
        ExcelUtil<CbContactUsVo> util = new ExcelUtil<>(CbContactUsVo.class);
        util.exportExcel(response, list, "联系我们数据");
    }

    @ApiOperation("获取联系我们详细信息")
    @PreAuthorize("@ss.hasPermi('business:CONTACTUS:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success("查询成功", cbContactUsService.queryById(id));
    }

    @ApiOperation("新增联系我们")
    @PreAuthorize("@ss.hasPermi('business:CONTACTUS:add')")
    @Log(title = "联系我们", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbContactUsVo entity) {
        return toAjax(cbContactUsService.save(entity));
    }

    @ApiOperation("修改联系我们")
    @PreAuthorize("@ss.hasPermi('business:CONTACTUS:edit')")
    @Log(title = "联系我们", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbContactUsVo entity) {
        return toAjax(cbContactUsService.updateById(entity));
    }

    @ApiOperation("删除联系我们")
    @PreAuthorize("@ss.hasPermi('business:CONTACTUS:remove')")
    @Log(title = "联系我们", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbContactUsService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
