package com.hjy.modlue.portal;

import cn.hutool.core.date.DateUtil;
import com.hjy.common.annotation.DataScope;
import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.enums.CbNewsType;
import com.hjy.common.enums.WorkFlowServiceEnums;
import com.hjy.module.service.portal.ICbCounselingMessageService;
import com.hjy.module.service.portal.ICbSensitiveService;
import com.hjy.module.service.wf.IWfInstActionTaskService;
import com.hjy.module.vo.portal.CbCounselingMessageVo;
import com.hjy.module.vo.portal.CbSensitiveVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

import static com.hjy.modlue.portal.CbSensitiveController.getAjaxResult;

/**
 * 咨询信息Controller
 *
 * @author Tellsea
 * @date 2023-08-31
 */
@Api(tags = "资讯信息Controller")
@RestController
@RequestMapping("/COUNSELINGMESSAGE")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbCounselingMessageController extends BaseController {

    private final ICbCounselingMessageService cbCounselingMessageService;

    private final IWfInstActionTaskService wfInstActionTaskService;

    private final ICbSensitiveService cbSensitiveService;

    @ApiOperation("查询资讯信息列表")
    @PreAuthorize("@ss.hasPermi('business:COUNSELINGMESSAGE:list')")
    @DataScope(deptAlias = "A")
    @GetMapping("/list")
    public TableDataInfo<CbCounselingMessageVo> list(CbCounselingMessageVo entity) {
        return cbCounselingMessageService.queryList(entity);
    }

    @ApiOperation("获取资讯信息详细信息")
    @PreAuthorize("@ss.hasPermi('business:COUNSELINGMESSAGE:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbCounselingMessageService.queryById(id));
    }

    @ApiOperation("删除资讯信息")
    @PreAuthorize("@ss.hasPermi('business:COUNSELINGMESSAGE:remove')")
    @GetMapping(value = "/remove/{id}")
    public AjaxResult remove(@PathVariable("id") String id) {
        cbCounselingMessageService.removeZxById(id);
        return AjaxResult.success("删除成功" );
    }

    @ApiOperation("新增资讯信息")
    @PreAuthorize("@ss.hasPermi('business:COUNSELINGMESSAGE:add')")
    @Log(title = "资讯信息", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbCounselingMessageVo entity) {
        entity.setContent(entity.getContent().replace("/dev-api", "").replace("/hjy-api", ""));
        // 获取敏感集合
        List<CbSensitiveVo> list = cbSensitiveService.queryAll(new CbSensitiveVo());
        AjaxResult trie = getAjaxResult(entity.getContent(),list);
        if (trie != null) return trie;
        cbCounselingMessageService.save(entity);
        if (CbNewsType.AUDIT.getCode().equals(entity.getStatus())){
            wfInstActionTaskService.statrWfqueue(WorkFlowServiceEnums.COUNSELINGMESSAGE.getService(), entity.getId(), entity.getTitle());
            return AjaxResult.success("审核发起");
        }
        return AjaxResult.success("保存成功");
    }

//    @ApiOperation("资讯信息确认")
//    @PreAuthorize("@ss.hasPermi('business:COUNSELINGMESSAGE:confirm')")
//    @Log(title = "资讯信息确认", businessType = BusinessType.UPDATE)
//    @PostMapping("confirm/{id}")
//    public AjaxResult confirm(@PathVariable String id) {
//        CbCounselingMessage counselingMessage = cbCounselingMessageService.getById(id);
//        if (ObjectUtils.isEmpty(counselingMessage)) return error("数据不存在");
//        if (!counselingMessage.getStatus().equals(CbNewsType.DRAFTS.getCode())) return error("当前数据非草稿，无法确认！");
//        counselingMessage.setStatus(CbNewsType.CONFIRM.getCode());
//        return toAjax(cbCounselingMessageService.updateById(counselingMessage));
//    }



    @ApiOperation("修改资讯信息")
    @PreAuthorize("@ss.hasPermi('business:COUNSELINGMESSAGE:edit')")
    @Log(title = "资讯信息", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbCounselingMessageVo entity) {
        return toAjax(cbCounselingMessageService.updateById(entity));
    }

    @ApiOperation("修改提交审核接口")
    @Log(title = "提交审核",businessType = BusinessType.UPDATE)
    @PostMapping("/audit")
    public AjaxResult audit(@RequestBody CbCounselingMessageVo entity){
        entity.setContent(entity.getContent().replace("/dev-api", "").replace("/hjy-api", ""));
        // 获取敏感集合
        List<CbSensitiveVo> list = cbSensitiveService.queryAll(new CbSensitiveVo());
        AjaxResult trie = getAjaxResult(entity.getContent(),list);
        if (trie != null) return trie;
        entity.setAuditTime(DateUtil.formatDateTime(new Date()));
        cbCounselingMessageService.updateById(entity);
        wfInstActionTaskService.statrWfqueue(WorkFlowServiceEnums.MODIFYCOUNSELINGMESSAGE.getService(), entity.getId(),entity.getTitle());
        return AjaxResult.success("审核发起");
    }

}
