package com.hjy.modlue.portal;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.domain.portal.CbDataFilling;
import com.hjy.module.service.article.ICbArticleService;
import com.hjy.module.service.portal.ICbDataFillingService;
import com.hjy.module.vo.article.CbArticleVo;
import com.hjy.module.vo.portal.CbDataFillingVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

/**
 * 数据填报记录Controller
 *
 * @author Tellsea
 * @date 2023-09-26
 */
@Api(tags = "数据填报记录Controller")
@RestController
@RequestMapping("/DATAFILLING")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbDataFillingController extends BaseController {

    private final ICbDataFillingService cbDataFillingService;

    private final ICbArticleService cbArticleService;

    @ApiOperation("查询数据填报记录列表")
    @PreAuthorize("@ss.hasPermi('business:DATAFILLING:list')")
    @GetMapping("/list")
    public TableDataInfo<CbDataFillingVo> list(CbDataFillingVo entity) {
        return cbDataFillingService.queryList(entity);
    }

    @ApiOperation("查询数据填报记录所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbDataFillingVo entity) {
        return AjaxResult.success("查询成功", cbDataFillingService.queryAll(entity));
    }

    @ApiOperation("导出数据填报记录列表")
    @PreAuthorize("@ss.hasPermi('business:DATAFILLING:export')")
    @Log(title = "数据填报记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbDataFillingVo entity) {
        List<CbDataFillingVo> list = cbDataFillingService.queryAll(entity);
        ExcelUtil<CbDataFillingVo> util = new ExcelUtil<>(CbDataFillingVo.class);
        util.exportExcel(response, list, "数据填报记录数据");
    }

    @ApiOperation("获取数据填报记录详细信息")
    @PreAuthorize("@ss.hasPermi('business:DATAFILLING:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbDataFillingService.queryById(id));
    }

    @ApiOperation("新增数据填报记录")
    @PreAuthorize("@ss.hasPermi('business:DATAFILLING:add')")
    @Log(title = "数据填报记录", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbDataFillingVo entity) {
        //查询是否有数据
        LambdaQueryWrapper<CbDataFilling> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(CbDataFilling::getArticleId,entity.getArticleId())
                        .eq(CbDataFilling::getMonth,entity.getMonth())
                                .eq(CbDataFilling::getDeptId,entity.getDeptId());
        CbDataFilling one = cbDataFillingService.getOne(wrapper);
        //如果数据为空则新增数据
        if (Objects.isNull(one)){
            cbDataFillingService.save(entity);
        }else {
            entity.setId(one.getId());
            cbDataFillingService.updateById(entity);
        }
        //修改完后进行计算每月投资金额和统计总共投资金额以及百分比
        CbDataFillingVo cbDataFillingVo = new CbDataFillingVo();
        cbDataFillingVo.setArticleId(entity.getArticleId());
        cbDataFillingVo.setDeptId(entity.getDeptId());
        List<CbDataFillingVo> list = cbDataFillingService.queryAll(cbDataFillingVo);
        //统计实际投资金额
        double reality = list.stream().mapToDouble(CbDataFillingVo::getPrice).sum();
        //计算完成百分比
        double retio = reality / entity.getPlan();
        //获取最新月份的形象进度
        String setSchedule = list.get(list.size() - 1).getSchedule();
        //修改项目表
        CbArticleVo cbArticleVo = new CbArticleVo();
        cbArticleVo.setId(entity.getArticleId());
        cbArticleVo.setRetio(retio);
        cbArticleVo.setReality(reality);
        cbArticleVo.setSchedule(setSchedule);
        cbArticleService.updateById(cbArticleVo);

        return AjaxResult.success("填报成功");
    }

    @ApiOperation("修改数据填报记录")
    @PreAuthorize("@ss.hasPermi('business:DATAFILLING:edit')")
    @Log(title = "数据填报记录", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbDataFillingVo entity) {
        return toAjax(cbDataFillingService.updateById(entity));
    }

    @ApiOperation("删除数据填报记录")
    @PreAuthorize("@ss.hasPermi('business:DATAFILLING:remove')")
    @Log(title = "数据填报记录", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbDataFillingService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
