package com.hjy.modlue.portal;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.portal.ICbExpertServiceService;
import com.hjy.module.vo.portal.CbExpertServiceVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 专家服务配置Controller
 *
 * @author Tellsea
 * @date 2023-08-23
 */
@Api(tags = "专家服务配置Controller")
@RestController
@RequestMapping("/portal/EXPERTSERVICE")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbExpertServiceController extends BaseController {

    private final ICbExpertServiceService cbExpertServiceService;

    @ApiOperation("查询专家服务配置列表")
    @PreAuthorize("@ss.hasPermi('business:EXPERTSERVICE:list')")
    @GetMapping("/list")
    public TableDataInfo<CbExpertServiceVo> list(CbExpertServiceVo entity) {
        return cbExpertServiceService.queryList(entity);
    }

    @ApiOperation("查询专家服务配置所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbExpertServiceVo entity) {
        return AjaxResult.success("查询成功", cbExpertServiceService.queryAll(entity));
    }

    @ApiOperation("导出专家服务配置列表")
    @PreAuthorize("@ss.hasPermi('business:EXPERTSERVICE:export')")
    @Log(title = "专家服务配置", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbExpertServiceVo entity) {
        List<CbExpertServiceVo> list = cbExpertServiceService.queryAll(entity);
        ExcelUtil<CbExpertServiceVo> util = new ExcelUtil<>(CbExpertServiceVo.class);
        util.exportExcel(response, list, "专家服务配置数据");
    }

    @ApiOperation("获取专家服务配置详细信息")
    @PreAuthorize("@ss.hasPermi('business:EXPERTSERVICE:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbExpertServiceService.queryById(id));
    }

    @ApiOperation("新增专家服务配置")
    @PreAuthorize("@ss.hasPermi('business:EXPERTSERVICE:add')")
    @Log(title = "专家服务配置", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbExpertServiceVo entity) {
        return toAjax(cbExpertServiceService.save(entity));
    }

    @ApiOperation("修改专家服务配置")
    @PreAuthorize("@ss.hasPermi('business:EXPERTSERVICE:edit')")
    @Log(title = "专家服务配置", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbExpertServiceVo entity) {
        return toAjax(cbExpertServiceService.updateById(entity));
    }

    @ApiOperation("删除专家服务配置")
    @PreAuthorize("@ss.hasPermi('business:EXPERTSERVICE:remove')")
    @Log(title = "专家服务配置", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbExpertServiceService.removeByIds(Arrays.asList(ids)));
    }

    @ApiOperation("根据用户id查询标签")
    @GetMapping("/{id}")
    public AjaxResult get(@PathVariable String experId){
        return AjaxResult.success("查询成功", cbExpertServiceService.get(experId));
    }
}
