package com.hjy.modlue.portal;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.portal.ICbFriendlyLinksService;
import com.hjy.module.vo.portal.CbFriendlyLinksVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 友情链接Controller
 *
 * @author Tellsea
 * @date 2023-08-23
 */
@Api(tags = "友情链接Controller")
@RestController
@RequestMapping("/portal/FRIENDLYLINKS")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbFriendlyLinksController extends BaseController {

    private final ICbFriendlyLinksService cbFriendlyLinksService;

    @ApiOperation("查询友情链接列表")
    @PreAuthorize("@ss.hasPermi('business:FRIENDLYLINKS:list')")
    @GetMapping("/list")
    public TableDataInfo<CbFriendlyLinksVo> list(CbFriendlyLinksVo entity) {
        return cbFriendlyLinksService.queryList(entity);
    }


    @ApiOperation("导出友情链接列表")
    @PreAuthorize("@ss.hasPermi('business:FRIENDLYLINKS:export')")
    @Log(title = "友情链接", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbFriendlyLinksVo entity) {
        List<CbFriendlyLinksVo> list = cbFriendlyLinksService.queryAll(entity);
        ExcelUtil<CbFriendlyLinksVo> util = new ExcelUtil<>(CbFriendlyLinksVo.class);
        util.exportExcel(response, list, "友情链接数据");
    }

    @ApiOperation("获取友情链接详细信息")
    @PreAuthorize("@ss.hasPermi('business:FRIENDLYLINKS:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success("查询成功", cbFriendlyLinksService.queryById(id));
    }

    @ApiOperation("新增友情链接")
    @PreAuthorize("@ss.hasPermi('business:FRIENDLYLINKS:add')")
    @Log(title = "友情链接", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbFriendlyLinksVo entity) {
        return toAjax(cbFriendlyLinksService.save(entity));
    }

    @ApiOperation("修改友情链接")
    @PreAuthorize("@ss.hasPermi('business:FRIENDLYLINKS:edit')")
    @Log(title = "友情链接", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbFriendlyLinksVo entity) {
        return toAjax(cbFriendlyLinksService.updateById(entity));
    }

    @ApiOperation("删除友情链接")
    @PreAuthorize("@ss.hasPermi('business:FRIENDLYLINKS:remove')")
    @Log(title = "友情链接", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbFriendlyLinksService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
