package com.hjy.modlue.portal;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.portal.ICbConferenceReplyService;
import com.hjy.module.service.portal.ICbJointConferenceService;
import com.hjy.module.vo.portal.CbConferenceReplyVo;
import com.hjy.module.vo.portal.CbJointConferenceVo;
import com.hjy.system.service.ISysDeptService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.*;

/**
 * 联席会议Controller
 *
 * @author Tellsea
 * @date 2023-09-27
 */
@Api(tags = "联席会议Controller")
@RestController
@RequestMapping("/JOINTCONFERENCE")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbJointConferenceController extends BaseController {

    private final ICbJointConferenceService cbJointConferenceService;

    private final ISysDeptService sysDeptService;

    private final ICbConferenceReplyService cbConferenceReplyService;

    @ApiOperation("查询联席会议列表")
    @PreAuthorize("@ss.hasPermi('business:JOINTCONFERENCE:list')")
    @GetMapping("/list")
    public TableDataInfo<CbJointConferenceVo> list(CbJointConferenceVo entity) {
        return cbJointConferenceService.queryList(entity);
    }

    @ApiOperation("查询联席会议所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbJointConferenceVo entity) {
        return AjaxResult.success("查询成功", cbJointConferenceService.queryAll(entity));
    }

    @ApiOperation("导出联席会议列表")
    @PreAuthorize("@ss.hasPermi('business:JOINTCONFERENCE:export')")
    @Log(title = "联席会议", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbJointConferenceVo entity) {
        List<CbJointConferenceVo> list = cbJointConferenceService.queryAll(entity);
        ExcelUtil<CbJointConferenceVo> util = new ExcelUtil<>(CbJointConferenceVo.class);
        util.exportExcel(response, list, "联席会议数据");
    }

    @ApiOperation("获取联席会议详细信息")
    @PreAuthorize("@ss.hasPermi('business:JOINTCONFERENCE:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbJointConferenceService.queryById(id));
    }

    @ApiOperation("获取联席会议详细信息")
    @PreAuthorize("@ss.hasPermi('business:ARTICLE:query')")
    @GetMapping(value = "/getreplyInfo/{id}")
    public AjaxResult getPriceInfo(@PathVariable("id") String id) {
        CbJointConferenceVo conferenceVo = cbJointConferenceService.queryById(id);
        CbConferenceReplyVo conferenceReplyVo = new CbConferenceReplyVo();
        conferenceReplyVo.setConferenceId(id);
        List<CbConferenceReplyVo> list = cbConferenceReplyService.queryAll(conferenceReplyVo);
        Map<String, Object> result = new HashMap<>();
        result.put("conference", conferenceVo);
        result.put("reply", list);
        return AjaxResult.success("查询成功", result);
    }

    @ApiOperation("新增联席会议")
    @PreAuthorize("@ss.hasPermi('business:JOINTCONFERENCE:add')")
    @Log(title = "联席会议", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbJointConferenceVo entity) {
        processEntity(entity);
        if (entity.getDeptIds().isEmpty()){
            entity.setStatus(1);
        }else {
            //设置在进行中
            entity.setStatus(2);
        }
        //默认回复数量设置为0
        entity.setReplied(0);
        return toAjax(cbJointConferenceService.save(entity));
    }

    public void processEntity(@RequestBody CbJointConferenceVo entity) {
        ArrayList<String> deptList = new ArrayList<>();
        for (Integer aLong : entity.getCooperativeUnitId()) {
            long l = aLong.longValue();
            deptList.add(sysDeptService.selectDeptById(l).getDeptName());
        }
        entity.setDeptNum(entity.getCooperativeUnitId().length);
        entity.setDeptIds(Arrays.toString(entity.getCooperativeUnitId()));
        entity.setCooperativeUnit(deptList.toString());
    }

    @ApiOperation("修改联席会议")
    @PreAuthorize("@ss.hasPermi('business:JOINTCONFERENCE:edit')")
    @Log(title = "联席会议", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbJointConferenceVo entity) {
        SysDept sysDept = sysDeptService.selectDeptById(getDeptId());
        if (sysDept.getType() == 1){
            entity.setCondition(1);
        }
        entity.setStatus(2);
        processEntity(entity);
        return toAjax(cbJointConferenceService.updateById(entity));
    }

    @ApiOperation("删除联席会议")
    @PreAuthorize("@ss.hasPermi('business:JOINTCONFERENCE:remove')")
    @Log(title = "联席会议", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbJointConferenceService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
