package com.hjy.modlue.portal;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.portal.ICbLeaveMessageService;
import com.hjy.module.vo.portal.CbLeaveMessageVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 留言反馈Controller
 *
 * @author Tellsea
 * @date 2023-08-24
 */
@Api(tags = "留言反馈Controller")
@RestController
@RequestMapping("/portal/LEAVEMESSAGE")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbLeaveMessageController extends BaseController {

    private final ICbLeaveMessageService cbLeaveMessageService;

    @ApiOperation("查询留言反馈列表")
    @PreAuthorize("@ss.hasPermi('business:LEAVEMESSAGE:list')")
    @GetMapping("/list")
    public TableDataInfo<CbLeaveMessageVo> list(CbLeaveMessageVo entity) {
        return cbLeaveMessageService.queryList(entity);
    }

    @ApiOperation("导出留言反馈列表")
    @PreAuthorize("@ss.hasPermi('business:LEAVEMESSAGE:export')")
    @Log(title = "留言反馈", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbLeaveMessageVo entity) {
        List<CbLeaveMessageVo> list = cbLeaveMessageService.queryAll(entity);
        ExcelUtil<CbLeaveMessageVo> util = new ExcelUtil<>(CbLeaveMessageVo.class);
        util.exportExcel(response, list, "留言反馈数据");
    }

    @ApiOperation("获取留言反馈详细信息")
    @PreAuthorize("@ss.hasPermi('business:LEAVEMESSAGE:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success("查询成功", cbLeaveMessageService.queryById(id));
    }

    @ApiOperation("新增留言反馈")
    @PreAuthorize("@ss.hasPermi('business:LEAVEMESSAGE:add')")
    @Log(title = "留言反馈", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbLeaveMessageVo entity) {
        return toAjax(cbLeaveMessageService.save(entity));
    }

    @ApiOperation("修改留言反馈")
    @PreAuthorize("@ss.hasPermi('business:LEAVEMESSAGE:edit')")
    @Log(title = "留言反馈", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbLeaveMessageVo entity) {
        return toAjax(cbLeaveMessageService.updateById(entity));
    }

    @ApiOperation("删除留言反馈")
    @PreAuthorize("@ss.hasPermi('business:LEAVEMESSAGE:remove')")
    @Log(title = "留言反馈", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbLeaveMessageService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
