package com.hjy.modlue.portal;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.portal.ICbMarkService;
import com.hjy.module.vo.portal.CbMarkVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 咨询标签Controller
 *
 * @author Tellsea
 * @date 2023-08-23
 */
@Api(tags = "咨询标签Controller")
@RestController
@RequestMapping("/MARK")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbMarkController extends BaseController {

    private final ICbMarkService cbMarkService;

    @ApiOperation("查询咨询标签列表")
    @PreAuthorize("@ss.hasPermi('business:MARK:list')")
    @GetMapping("/list")
    public TableDataInfo<CbMarkVo> list(CbMarkVo entity) {
        return cbMarkService.queryList(entity);
    }


    @ApiOperation("查询咨询标签所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbMarkVo entity) {
        return AjaxResult.success("查询成功", cbMarkService.queryAll(entity));
    }

    @ApiOperation("导出咨询标签列表")
    @PreAuthorize("@ss.hasPermi('business:MARK:export')")
    @Log(title = "咨询标签", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbMarkVo entity) {
        List<CbMarkVo> list = cbMarkService.queryAll(entity);
        ExcelUtil<CbMarkVo> util = new ExcelUtil<>(CbMarkVo.class);
        util.exportExcel(response, list, "咨询标签数据");
    }

    @ApiOperation("获取咨询标签详细信息")
    @PreAuthorize("@ss.hasPermi('business:MARK:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbMarkService.queryById(id));
    }

    @ApiOperation("新增咨询标签")
    @PreAuthorize("@ss.hasPermi('business:MARK:add')")
    @Log(title = "咨询标签", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbMarkVo entity) {
        return toAjax(cbMarkService.save(entity));
    }

    @ApiOperation("修改咨询标签")
    @PreAuthorize("@ss.hasPermi('business:MARK:edit')")
    @Log(title = "咨询标签", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbMarkVo entity) {
        return toAjax(cbMarkService.updateById(entity));
    }

    @ApiOperation("删除咨询标签")
    @PreAuthorize("@ss.hasPermi('business:MARK:remove')")
    @Log(title = "咨询标签", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbMarkService.removeByIds(Arrays.asList(ids)));
    }
}
