package com.hjy.modlue.portal;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.portal.ICbMessageService;
import com.hjy.module.vo.portal.CbMessageVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 门户消息通知Controller
 *
 * @author Tellsea
 * @date 2023-08-23
 */
@Api(tags = "门户消息通知Controller")
@RestController
@RequestMapping("/portal/MESSAGE")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbMessageController extends BaseController {

    private final ICbMessageService cbMessageService;

    @ApiOperation("查询门户消息通知列表")
    @PreAuthorize("@ss.hasPermi('business:MESSAGE:list')")
    @GetMapping("/list")
    public TableDataInfo<CbMessageVo> list(CbMessageVo entity) {
        return cbMessageService.queryList(entity);
    }

    @ApiOperation("查询门户消息通知所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbMessageVo entity) {
        return AjaxResult.success("查询成功", cbMessageService.queryAll(entity));
    }

    @ApiOperation("导出门户消息通知列表")
    @PreAuthorize("@ss.hasPermi('business:MESSAGE:export')")
    @Log(title = "门户消息通知", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbMessageVo entity) {
        List<CbMessageVo> list = cbMessageService.queryAll(entity);
        ExcelUtil<CbMessageVo> util = new ExcelUtil<>(CbMessageVo.class);
        util.exportExcel(response, list, "门户消息通知数据");
    }

    @ApiOperation("获取门户消息通知详细信息")
    @PreAuthorize("@ss.hasPermi('business:MESSAGE:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbMessageService.queryById(id));
    }

    @ApiOperation("新增门户消息通知")
    @PreAuthorize("@ss.hasPermi('business:MESSAGE:add')")
    @Log(title = "门户消息通知", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbMessageVo entity) {
        return toAjax(cbMessageService.save(entity));
    }

    @ApiOperation("修改门户消息通知")
    @PreAuthorize("@ss.hasPermi('business:MESSAGE:edit')")
    @Log(title = "门户消息通知", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbMessageVo entity) {
        return toAjax(cbMessageService.updateById(entity));
    }

    @ApiOperation("删除门户消息通知")
    @PreAuthorize("@ss.hasPermi('business:MESSAGE:remove')")
    @Log(title = "门户消息通知", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbMessageService.removeByIds(Arrays.asList(ids)));
    }
}
