package com.hjy.modlue.portal;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.portal.ICbModifyCompanyUserService;
import com.hjy.module.vo.portal.CbModifyCompanyUserVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 门户用户修改临时Controller
 *
 * @author Tellsea
 * @date 2023-10-17
 */
@Api(tags = "门户用户修改临时Controller")
@RestController
@RequestMapping("/business/MODIFYCOMPANYUSER")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbModifyCompanyUserController extends BaseController {

    private final ICbModifyCompanyUserService cbModifyCompanyUserService;

    @ApiOperation("查询门户用户修改临时列表")
    @PreAuthorize("@ss.hasPermi('business:MODIFYCOMPANYUSER:list')")
    @GetMapping("/list")
    public TableDataInfo<CbModifyCompanyUserVo> list(CbModifyCompanyUserVo entity) {
        return cbModifyCompanyUserService.queryList(entity);
    }

    @ApiOperation("查询门户用户修改临时所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbModifyCompanyUserVo entity) {
        return AjaxResult.success("查询成功", cbModifyCompanyUserService.queryAll(entity));
    }

    @ApiOperation("导出门户用户修改临时列表")
    @PreAuthorize("@ss.hasPermi('business:MODIFYCOMPANYUSER:export')")
    @Log(title = "门户用户修改临时", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbModifyCompanyUserVo entity) {
        List<CbModifyCompanyUserVo> list = cbModifyCompanyUserService.queryAll(entity);
        ExcelUtil<CbModifyCompanyUserVo> util = new ExcelUtil<>(CbModifyCompanyUserVo.class);
        util.exportExcel(response, list, "门户用户修改临时数据");
    }

    @ApiOperation("获取门户用户修改临时详细信息")
    @PreAuthorize("@ss.hasPermi('business:MODIFYCOMPANYUSER:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbModifyCompanyUserService.queryById(id));
    }

    @ApiOperation("新增门户用户修改临时")
    @PreAuthorize("@ss.hasPermi('business:MODIFYCOMPANYUSER:add')")
    @Log(title = "门户用户修改临时", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbModifyCompanyUserVo entity) {
        return toAjax(cbModifyCompanyUserService.save(entity));
    }

    @ApiOperation("修改门户用户修改临时")
    @PreAuthorize("@ss.hasPermi('business:MODIFYCOMPANYUSER:edit')")
    @Log(title = "门户用户修改临时", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbModifyCompanyUserVo entity) {
        return toAjax(cbModifyCompanyUserService.updateById(entity));
    }

    @ApiOperation("删除门户用户修改临时")
    @PreAuthorize("@ss.hasPermi('business:MODIFYCOMPANYUSER:remove')")
    @Log(title = "门户用户修改临时", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbModifyCompanyUserService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }

    /**
     * 删除
     * @param id
     * @return
     */
    @PreAuthorize("@ss.hasPermi('business:MODIFYCOMPANYUSER:remove')")
    @Log(title = "门户用户修改临时", businessType = BusinessType.DELETE)
	@DeleteMapping("/remove/{id}")
    public void deleteUser(@PathVariable String id) {
        cbModifyCompanyUserService.deleteUser(id);
    }
}
