package com.hjy.modlue.portal;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.portal.ICbOnlineService;
import com.hjy.module.vo.portal.CbOnlineVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 网上办事Controller
 *
 * @author Tellsea
 * @date 2023-08-24
 */
@Api(tags = "网上办事Controller")
@RestController
@RequestMapping("/portal/ONLINE")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbOnlineController extends BaseController {

    private final ICbOnlineService cbOnlineService;

    @ApiOperation("查询网上办事列表")
    @PreAuthorize("@ss.hasPermi('business:ONLINE:list')")
    @GetMapping("/list")
    public TableDataInfo<CbOnlineVo> list(CbOnlineVo entity) {
        return cbOnlineService.queryList(entity);
    }

    @ApiOperation("导出网上办事列表")
    @PreAuthorize("@ss.hasPermi('business:ONLINE:export')")
    @Log(title = "网上办事", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbOnlineVo entity) {
        List<CbOnlineVo> list = cbOnlineService.queryAll(entity);
        ExcelUtil<CbOnlineVo> util = new ExcelUtil<>(CbOnlineVo.class);
        util.exportExcel(response, list, "网上办事数据");
    }

    @ApiOperation("获取网上办事详细信息")
    @PreAuthorize("@ss.hasPermi('business:ONLINE:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success("查询成功", cbOnlineService.queryById(id));
    }

    @ApiOperation("新增网上办事")
    @PreAuthorize("@ss.hasPermi('business:ONLINE:add')")
    @Log(title = "网上办事", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbOnlineVo entity) {
        return toAjax(cbOnlineService.save(entity));
    }

    @ApiOperation("修改网上办事")
    @PreAuthorize("@ss.hasPermi('business:ONLINE:edit')")
    @Log(title = "网上办事", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbOnlineVo entity) {
        return toAjax(cbOnlineService.updateById(entity));
    }

    @ApiOperation("删除网上办事")
    @PreAuthorize("@ss.hasPermi('business:ONLINE:remove')")
    @Log(title = "网上办事", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbOnlineService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
