package com.hjy.modlue.portal;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.portal.ICbQuestionService;
import com.hjy.module.vo.portal.CbQuestionVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 咨询信息Controller
 *
 * @author Tellsea
 * @date 2023-08-23
 */
@Api(tags = "咨询信息Controller")
@RestController
@RequestMapping("/portal/QUESTION")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbQuestionController extends BaseController {

    private final ICbQuestionService cbQuestionService;

    @ApiOperation("查询咨询信息列表")
    @PreAuthorize("@ss.hasPermi('business:QUESTION:list')")
    @GetMapping("/list")
    public TableDataInfo<CbQuestionVo> list(CbQuestionVo entity) {
        return cbQuestionService.queryList(entity);
    }

    @ApiOperation("查询咨询信息所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbQuestionVo entity) {
        return AjaxResult.success("查询成功", cbQuestionService.queryAll(entity));
    }

    @ApiOperation("导出咨询信息列表")
    @PreAuthorize("@ss.hasPermi('business:QUESTION:export')")
    @Log(title = "咨询信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbQuestionVo entity) {
        List<CbQuestionVo> list = cbQuestionService.queryAll(entity);
        ExcelUtil<CbQuestionVo> util = new ExcelUtil<>(CbQuestionVo.class);
        util.exportExcel(response, list, "咨询信息数据");
    }

    @ApiOperation("获取咨询信息详细信息")
    @PreAuthorize("@ss.hasPermi('business:QUESTION:query')")
    @GetMapping(value = "/getInfo/{answerId}")
    public AjaxResult getInfo(@PathVariable("answerId") String answerId) {
        return AjaxResult.success("查询成功", cbQuestionService.queryById(answerId));
    }

    @ApiOperation("新增咨询信息")
    @PreAuthorize("@ss.hasPermi('business:QUESTION:add')")
    @Log(title = "咨询信息", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbQuestionVo entity) {
        return toAjax(cbQuestionService.save(entity));
    }

    @ApiOperation("修改咨询信息")
    @PreAuthorize("@ss.hasPermi('business:QUESTION:edit')")
    @Log(title = "咨询信息", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbQuestionVo entity) {
        return toAjax(cbQuestionService.updateById(entity));
    }

    @ApiOperation("删除咨询信息")
    @PreAuthorize("@ss.hasPermi('business:QUESTION:remove')")
    @Log(title = "咨询信息", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbQuestionService.removeByIds(Arrays.asList(ids)));
    }
}
