package com.hjy.modlue.portal;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.TrieUtils;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.portal.ICbSensitiveService;
import com.hjy.module.vo.portal.CbSensitiveVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 敏感词管理Controller
 *
 * @author Tellsea
 * @date 2023-08-30
 */
@Api(tags = "敏感词管理Controller")
@RestController
@RequestMapping("/SENSITIVE")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbSensitiveController extends BaseController {

    private final ICbSensitiveService cbSensitiveService;

    @ApiOperation("查询敏感词管理列表")
    @PreAuthorize("@ss.hasPermi('business:SENSITIVE:list')")
    @GetMapping("/list")
    public TableDataInfo<CbSensitiveVo> list(CbSensitiveVo entity) {
        return cbSensitiveService.queryList(entity);
    }

    @ApiOperation("查询敏感词管理所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbSensitiveVo entity) {
        return AjaxResult.success("查询成功", cbSensitiveService.queryAll(entity));
    }

    @ApiOperation("导出敏感词管理列表")
    @PreAuthorize("@ss.hasPermi('business:SENSITIVE:export')")
    @Log(title = "敏感词管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbSensitiveVo entity) {
        List<CbSensitiveVo> list = cbSensitiveService.queryAll(entity);
        ExcelUtil<CbSensitiveVo> util = new ExcelUtil<>(CbSensitiveVo.class);
        util.exportExcel(response, list, "敏感词管理数据");
    }

    @ApiOperation("获取敏感词管理详细信息")
    @PreAuthorize("@ss.hasPermi('business:SENSITIVE:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbSensitiveService.queryById(id));
    }

    @ApiOperation("新增敏感词管理")
    @PreAuthorize("@ss.hasPermi('business:SENSITIVE:add')")
    @Log(title = "敏感词管理", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbSensitiveVo entity) {
        return toAjax(cbSensitiveService.save(entity));
    }

    @ApiOperation("修改敏感词管理")
    @PreAuthorize("@ss.hasPermi('business:SENSITIVE:edit')")
    @Log(title = "敏感词管理", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbSensitiveVo entity) {
        return toAjax(cbSensitiveService.updateById(entity));
    }

    @ApiOperation("删除敏感词管理")
    @PreAuthorize("@ss.hasPermi('business:SENSITIVE:remove')")
    @Log(title = "敏感词管理", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbSensitiveService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }

    /**
     * 敏感词方法
     * @param entity 需要检查的内容
     * @param sensitiveList 敏感词组
     * @return
     */
    public static AjaxResult getAjaxResult(String entity,List<CbSensitiveVo> sensitiveList) {
        List<String> SensitiveList = sensitiveList.stream().map(CbSensitiveVo::getSensitive).collect(Collectors.toList());

        TrieUtils trie = new TrieUtils();
        for (String s : SensitiveList) {
            trie.insert(s);
        }
        if (!Objects.isNull(trie.sensitiveWord(entity))) {
            return AjaxResult.error("包含敏感字符:" + trie.sensitiveWord(entity) + "，请修改");
        }
        return null;
    }
}
