package com.hjy.modlue.portal;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.module.service.portal.ICbSystemService;
import com.hjy.module.vo.portal.CbSystemVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

/**
 * 系统管理Controller
 *
 * @author Tellsea
 * @date 2023-09-08
 */
@Api(tags = "系统管理Controller")
@RestController
@RequestMapping("/SYSTEM")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbSystemController extends BaseController {

    private final ICbSystemService cbSystemService;

    @ApiOperation("查询系统管理列表")
    @PreAuthorize("@ss.hasPermi('business:SYSTEM:list')")
    @GetMapping("/list")
    public TableDataInfo<CbSystemVo> list(CbSystemVo entity) {
        return cbSystemService.queryList(entity);
    }


    @ApiOperation("获取系统管理详细信息")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbSystemService.queryById(id));
    }

    @ApiOperation("修改系统管理")
    @PreAuthorize("@ss.hasPermi('business:SYSTEM:edit')")
    @Log(title = "系统管理", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbSystemVo entity) {
        return toAjax(cbSystemService.updateById(entity));
    }

}
