package com.hjy.modlue.portal;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.module.service.portal.ICbWebsiteService;
import com.hjy.module.vo.portal.CbWebsiteVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

/**
 * 网站管理Controller
 *
 * @author Tellsea
 * @date 2023-08-29
 */
@Api(tags = "网站管理Controller")
@RestController
@RequestMapping("/business/WEBSITE")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbWebsiteController extends BaseController {

    private final ICbWebsiteService cbWebsiteService;

    @ApiOperation("查询网站管理列表")
    @PreAuthorize("@ss.hasPermi('business:WEBSITE:list')")
    @GetMapping("/list")
    public TableDataInfo<CbWebsiteVo> list(CbWebsiteVo entity) {
        return cbWebsiteService.queryList(entity);
    }

    @ApiOperation("获取网站管理详细信息")
    @PreAuthorize("@ss.hasPermi('business:WEBSITE:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success( "查询成功", cbWebsiteService.queryById(id));
    }

    @ApiOperation("修改网站管理")
    @PreAuthorize("@ss.hasPermi('business:WEBSITE:edit')")
    @Log(title = "网站管理", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbWebsiteVo entity) {
        return toAjax(cbWebsiteService.updateById(entity));
    }

}
