package com.hjy.modlue.portalAPI;

import cn.hutool.core.bean.BeanUtil;
import com.hjy.common.annotation.PortalAuth;
import com.hjy.common.constant.Constants;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.CbCompanyUser;
import com.hjy.common.core.redis.RedisCache;
import com.hjy.common.enums.WorkFlowServiceEnums;
import com.hjy.common.utils.RedisUtil;
import com.hjy.common.utils.SecurityUtils;
import com.hjy.framework.web.service.TokenService;
import com.hjy.module.domain.portal.CbModifyCompanyUser;
import com.hjy.module.service.portal.ICbCompanyUserService;
import com.hjy.module.service.portal.ICbModifyCompanyUserService;
import com.hjy.module.service.wf.IWfInstActionTaskService;
import com.hjy.module.vo.portal.CbCompanyUserVo;
import io.jsonwebtoken.Claims;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Objects;

/**
 * 门户用户Controller
 *
 * @author Tellsea
 * @Date 2023-08-23
 */
@Api(tags = "门户用户查询Controller")
@RestController
@RequestMapping("/portal_api/COMPANYUSER")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CompanyUserController {
    /**
     * 专家类型
     */
    private static final String SPECIALIST = "1";
    private final ICbCompanyUserService cbCompanyUserService;

    private final RedisCache redisCache;

    private final TokenService tokenService;

    private final IWfInstActionTaskService wfInstActionTaskService;

    private final RedisUtil redisUtil;

    private final ICbModifyCompanyUserService cbModifyCompanyUserService;


    @PortalAuth
    @ApiOperation("查询门户用户所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbCompanyUserVo entity) {
        return AjaxResult.success("查询成功", cbCompanyUserService.queryAll(entity));
    }

    @PortalAuth
    @ApiOperation("注销用户")
    @GetMapping("/logoff")
    public AjaxResult logoff(HttpServletRequest request) {
        cbCompanyUserService.removeById(SecurityUtils.getPortalUser().getId());
        String token = request.getHeader(Constants.PORTAL_TOKEN);
        Claims claims = tokenService.parseToken(token);
        String uuid = (String) claims.get(Constants.PORTAL_USER_KEY);
        redisCache.deleteObject(Constants.PORTAL_USER_KEY+uuid);
        return AjaxResult.success("注销成功");
    }

    @PortalAuth
    @ApiOperation("退出登录")
    @GetMapping("/logOut")
    public AjaxResult logOut(HttpServletRequest request) {
        String token = request.getHeader(Constants.PORTAL_TOKEN);
        Claims claims = tokenService.parseToken(token);
        String uuid = (String) claims.get(Constants.PORTAL_USER_KEY);
        redisCache.deleteObject(Constants.PORTAL_USER_KEY+uuid);
        return AjaxResult.success("退出成功");
    }

    @PortalAuth
    @ApiOperation("修改门户用户")
    @PostMapping("/edit")
    public AjaxResult edit(@RequestBody CbCompanyUser entity) {
        if (SPECIALIST.equals(entity.getType())){
            cbCompanyUserService.updateById(entity);
            return AjaxResult.success("修改成功");
        }
        if (Objects.nonNull(entity.getAudit()) && entity.getAudit() == 1){
            return AjaxResult.error("审核中的用户不允许修改");
        }
        //设置状态在审核中
        entity.setAudit(1);
        CbModifyCompanyUser cbModifyCompanyUser = BeanUtil.copyProperties(entity, CbModifyCompanyUser.class);
        cbModifyCompanyUserService.save(cbModifyCompanyUser);
        wfInstActionTaskService.statrWfqueue(WorkFlowServiceEnums.MODIFYCOMPANYUSERSERVICE.getService(), entity.getId(), entity.getUserName());
        return AjaxResult.success("审核发起");
    }

    @ApiOperation("获取门户用户详细信息")
    @PortalAuth
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        CbModifyCompanyUser byId = cbModifyCompanyUserService.queryById(id);
        if (Objects.isNull(byId)){
            return AjaxResult.success("查询成功",cbCompanyUserService.queryById(id));
        }
        return AjaxResult.success("查询成功",byId);
    }
}
