package com.hjy.modlue.portalAPI;

import com.hjy.common.annotation.Log;
import com.hjy.common.annotation.PortalAuth;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.modlue.portal.CbJointConferenceController;
import com.hjy.module.service.portal.ICbConferenceReplyService;
import com.hjy.module.service.portal.ICbJointConferenceService;
import com.hjy.module.vo.portal.CbConferenceReplyVo;
import com.hjy.module.vo.portal.CbJointConferenceVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 联席会议Controller
 *
 * @author Tellsea
 * @date 2023-09-27
 */
@Api(tags = "门户联席会议Controller")
@RestController
@RequestMapping("/portal_api/JOINTCONFERENCE")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class JointConferenceController extends BaseController {

    private final ICbJointConferenceService cbJointConferenceService;

    private final ICbConferenceReplyService cbConferenceReplyService;

    private final CbJointConferenceController cbJointConferenceController;


    @ApiOperation("查询联席会议列表")
    @PortalAuth
    @GetMapping("/list")
    public TableDataInfo<CbJointConferenceVo> list(CbJointConferenceVo entity) {
        return cbJointConferenceService.queryList(entity);
    }

    @ApiOperation("查询联席会议所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbJointConferenceVo entity) {
        return AjaxResult.success("查询成功", cbJointConferenceService.queryAll(entity));
    }

    @ApiOperation("获取联席会议详细信息")
    @PortalAuth
    @GetMapping(value = "/getreplyInfo/{id}")
    public AjaxResult getPriceInfo(@PathVariable("id") String id) {
        CbJointConferenceVo conferenceVo = cbJointConferenceService.queryById(id);
        CbConferenceReplyVo conferenceReplyVo = new CbConferenceReplyVo();
        conferenceReplyVo.setConferenceId(id);
        List<CbConferenceReplyVo> list = cbConferenceReplyService.queryAll(conferenceReplyVo);
        Map<String, Object> result = new HashMap<>();
        result.put("conference", conferenceVo);
        result.put("reply", list);
        return AjaxResult.success("查询成功", result);
    }

    @ApiOperation("新增联席会议")
    @PortalAuth
    @Log(title = "联席会议", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbJointConferenceVo entity) {
        cbJointConferenceController.processEntity(entity);
        if (entity.getDeptIds().isEmpty()){
            entity.setStatus(1);
        }else {
            //设置在进行中
            entity.setStatus(2);
        }
        //默认回复数量设置为0
        entity.setReplied(0);
        return toAjax(cbJointConferenceService.save(entity));
    }

    @ApiOperation("获取部门列表")
    @PortalAuth
    @GetMapping("dept")
    public AjaxResult getDept(SysDept dept){
        List<SysDept> sysDepts = cbJointConferenceService.selectDeptList(dept);
        return AjaxResult.success(sysDepts);
    }
}
