package com.hjy.modlue.portalAPI;

import com.hjy.common.config.ZhbsqConfig;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.utils.StringUtils;
import com.hjy.common.utils.file.FileUtils;
import com.hjy.module.service.portal.ICbOpServiceService;
import com.hjy.module.vo.portal.CbOpServiceVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import static com.hjy.framework.datasource.DynamicDataSourceContextHolder.log;

/**
 * 门户运营服务Controller
 *
 * @author Tellsea
 * @date 2023-08-29
 */
@Api(tags = "门户运营服务Controller")
@RestController
@RequestMapping("/portal_api/OPSERVICE")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class OpServiceController {

    private final ICbOpServiceService cbOpServiceService;

    @ApiOperation("查询运营服务所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbOpServiceVo entity) {
        return AjaxResult.success("查询成功", cbOpServiceService.queryAll(entity));
    }

    /**
     * 本地资源通用下载
     */
    @ApiOperation("通用下载请求")
    @GetMapping("/download")
    public void fileDownload(String fileName, Boolean delete, HttpServletResponse response, String id, HttpServletRequest request) {
        try {
            if (!FileUtils.checkAllowDownload(fileName)) {
                throw new Exception(StringUtils.format("文件名称({})非法，不允许下载。 ", fileName));
            }
            String realFileName = System.currentTimeMillis() + fileName.substring(fileName.indexOf("_") + 1);
            String filePath = ZhbsqConfig.getDownloadPath() + fileName;

            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
            FileUtils.setAttachmentResponseHeader(response, realFileName);
            FileUtils.writeBytes(filePath, response.getOutputStream());
            if (delete) {
                FileUtils.deleteFile(filePath);
            }

            CbOpServiceVo cbOpServiceVo = cbOpServiceService.queryById(id);
            CbOpServiceVo opServiceVo = new CbOpServiceVo();
            opServiceVo.setDownload(cbOpServiceVo.getDownload()+1);
            opServiceVo.setId(cbOpServiceVo.getId());
            cbOpServiceService.updateById(opServiceVo);
        } catch (Exception e) {
            log.error("下载文件失败", e);
        }
    }
}
