package com.hjy.modlue.portalAPI;

import com.hjy.common.annotation.Log;
import com.hjy.common.annotation.PortalAuth;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.enums.BusinessType;
import com.hjy.module.domain.portal.CbExpertService;
import com.hjy.module.service.portal.ICbExpertServiceService;
import com.hjy.module.vo.portal.CbExpertServiceVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;

@Api(tags = "专家服务相关接口")
@RestController
@RequestMapping("/portal_api/expert_service")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class PortalExpertServiceController extends BaseController {


    private final ICbExpertServiceService cbExpertServiceService;

    @ApiOperation("查询专家服务配置所有列表")
    @PortalAuth
    @GetMapping("/listAll")
    public AjaxResult listAll(CbExpertServiceVo entity) {
        return AjaxResult.success("查询成功", cbExpertServiceService.queryAll(entity));
    }

    @ApiOperation("新增专家服务配置")
    @PortalAuth
    @Log(title = "专家服务配置", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbExpertServiceVo entity) {
        try {
            CbExpertService existingRecord = cbExpertServiceService.getExper(entity);

            if (existingRecord != null) {
                // 数据已存在，恢复逻辑删除
                existingRecord.setDelFlag("0");
                cbExpertServiceService.updateId(existingRecord);
                return AjaxResult.success("添加成功");
            } else {
                // 数据不存在，保存新记录
                cbExpertServiceService.save(entity);
                return AjaxResult.success();
            }
        } catch (Exception e) {
            return AjaxResult.error("该标签已存在");
        }
    }

    @ApiOperation("修改专家服务配置")
    @PortalAuth
    @Log(title = "专家服务配置", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbExpertServiceVo entity) {
        return toAjax(cbExpertServiceService.updateById(entity));
    }

    @ApiOperation("删除专家服务配置")
    @PortalAuth
    @Log(title = "专家服务配置", businessType = BusinessType.DELETE)
    @GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbExpertServiceService.removeByIds(Arrays.asList(ids)));
    }


    @ApiOperation("根据用户id查询标签")
    @PortalAuth
    @GetMapping("/{id}")
    public AjaxResult get(@PathVariable String id){
        return AjaxResult.success("查询成功", cbExpertServiceService.get(id));
    }
}
