package com.hjy.modlue.portalAPI;

import cn.hutool.core.date.DateTime;
import com.hjy.common.annotation.Log;
import com.hjy.common.annotation.PortalAuth;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.enums.WorkFlowServiceEnums;
import com.hjy.common.utils.SecurityUtils;
import com.hjy.module.service.portal.ICbQuestionService;
import com.hjy.module.service.wf.IWfInstActionTaskService;
import com.hjy.module.vo.portal.CbQuestionVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;

@Api(tags = "门户咨询信息Controller")
@RestController
@RequestMapping("/portal_api/QUESTION")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class PortalQuestionController extends BaseController {

    private final ICbQuestionService cbQuestionService;

    private final IWfInstActionTaskService wfInstActionTaskService;

    @PortalAuth
    @ApiOperation("新增咨询信息")
    @Log(title = "咨询信息", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbQuestionVo entity) {
        entity.setAnswerId(SecurityUtils.getPortalUser().getId());
        entity.setAnsName(SecurityUtils.getPortalUser().getNickName());
        entity.setStatus("0");
        entity.setCheckStatus("0");
        return toAjax(cbQuestionService.save(entity));
    }

    @PortalAuth
    @ApiOperation("修改咨询信息")
    @Log(title = "咨询信息", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbQuestionVo entity) {
        return toAjax(cbQuestionService.updateById(entity));
    }

    @PortalAuth
    @ApiOperation("删除咨询信息")
    @Log(title = "咨询信息", businessType = BusinessType.DELETE)
    @GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {

        return toAjax(cbQuestionService.removeByIds(Arrays.asList(ids)));
    }

    @PortalAuth
    @ApiOperation("查询我的咨询信息列表")
    @GetMapping("/list")
    public TableDataInfo<CbQuestionVo> list(CbQuestionVo entity) {
        return cbQuestionService.queryList(entity);
    }

    @PortalAuth
    @ApiOperation("专家回复接口")
    @PostMapping("/expertAnswer")
    public AjaxResult expertAnswer(@RequestBody CbQuestionVo entity) {
        entity.setReplyTime(DateTime.now());
        cbQuestionService.updateById(entity);
        if (!"0".equals(entity.getCheckStatus())) {
            wfInstActionTaskService.statrWfqueue(WorkFlowServiceEnums.QUESTIONSERVICE.getService(), entity.getId(), entity.getQuestion());
            return AjaxResult.success("审核发起");
        }
        return AjaxResult.success("保存成功");
    }

    @PortalAuth
    @ApiOperation("用户评价接口")
    @PostMapping("/userEvaluation")
    public AjaxResult userEvaluation(@RequestBody CbQuestionVo entity) {
        return AjaxResult.success(cbQuestionService.updateById(entity));
    }

    @PortalAuth
    @ApiOperation("专家评价统计查询接口")
    @GetMapping("/evaluate")
    public AjaxResult evaluate(CbQuestionVo entity) {
        return AjaxResult.success(cbQuestionService.evaluate(entity));
    }


}
