package com.hjy.modlue.rank;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.rank.ICbRankService;
import com.hjy.module.vo.rank.CbRankVo;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 广西保税区排名Controller
 *
 * @author Tellsea
 * @date 2023-10-10
 */
//@Api(tags = "广西保税区排名Controller")
@RestController
@RequestMapping("/rank/RANK")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbRankController extends BaseController {

    private final ICbRankService cbRankService;

    //    @ApiOperation("查询广西保税区排名列表")
//    @PreAuthorize("@ss.hasPermi('rank:RANK:list')")
    @GetMapping("/list")
    public TableDataInfo<CbRankVo> list(CbRankVo entity) {
        return cbRankService.queryList(entity);
    }

    //    @ApiOperation("查询广西保税区排名所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbRankVo entity) {
        return AjaxResult.success("查询成功", cbRankService.queryAll(entity));
    }

    //    @ApiOperation("导出广西保税区排名列表")
    @PreAuthorize("@ss.hasPermi('rank:RANK:export')")
    @Log(title = "广西保税区排名", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbRankVo entity) {
        List<CbRankVo> list = cbRankService.queryAll(entity);
        ExcelUtil<CbRankVo> util = new ExcelUtil<>(CbRankVo.class);
        util.exportExcel(response, list, "广西保税区排名数据");
    }

    //    @ApiOperation("获取广西保税区排名详细信息")
    @PreAuthorize("@ss.hasPermi('rank:RANK:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbRankService.queryById(id));
    }

    //    @ApiOperation("新增广西保税区排名")
    @PreAuthorize("@ss.hasPermi('rank:RANK:add')")
    @Log(title = "广西保税区排名", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbRankVo entity) {
        return toAjax(cbRankService.save(entity));
    }

    //    @ApiOperation("修改广西保税区排名")
    @PreAuthorize("@ss.hasPermi('rank:RANK:edit')")
    @Log(title = "广西保税区排名", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbRankVo entity) {
        return toAjax(cbRankService.updateById(entity));
    }

    //    @ApiOperation("删除广西保税区排名")
    @PreAuthorize("@ss.hasPermi('rank:RANK:remove')")
    @Log(title = "广西保税区排名", businessType = BusinessType.DELETE)
    @GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbRankService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
