package com.hjy.modlue.report;

import cn.hutool.core.util.StrUtil;
import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;

import com.hjy.module.service.report.IReportRecordService;
import com.hjy.module.service.report.IReportWordService;
import com.hjy.module.service.template.IExcelOutInTotalService;
import com.hjy.module.vo.report.ReportRecordVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import com.hjy.common.core.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 辅助决策Controller
 *
 * @author Tellsea
 * @date 2024-04-26
 */
@Api(tags = "辅助决策Controller")
@RestController
@RequestMapping("/report/RECORD")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class ReportRecordController extends BaseController {

    private final IReportRecordService reportRecordService;
    private final IExcelOutInTotalService iExcelOutInTotalService;
    private final IReportWordService reportWordService;

    @ApiOperation("查询辅助决策列表")
    @PreAuthorize("@ss.hasPermi('report:RECORD:list')")
    @GetMapping("/list")
    public TableDataInfo<ReportRecordVo> list(ReportRecordVo entity) {
        return reportRecordService.queryList(entity);
    }

    @ApiOperation("查询辅助决策所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(ReportRecordVo entity) {
        return AjaxResult.success("查询成功", reportRecordService.queryAll(entity));
    }

    @ApiOperation("导出辅助决策列表")
    @PreAuthorize("@ss.hasPermi('report:RECORD:export')")
    @Log(title = "辅助决策", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ReportRecordVo entity) {
        List<ReportRecordVo> list = reportRecordService.queryAll(entity);
        ExcelUtil<ReportRecordVo> util = new ExcelUtil<>(ReportRecordVo.class);
        util.exportExcel(response, list, "辅助决策数据");
    }

    @ApiOperation("获取辅助决策详细信息")
    @PreAuthorize("@ss.hasPermi('report:RECORD:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", reportRecordService.queryById(id));
    }

    @ApiOperation("新增辅助决策")
    @PreAuthorize("@ss.hasPermi('report:RECORD:add')")
    @Log(title = "辅助决策", businessType = BusinessType.INSERT)
    @PostMapping("add")
    @Transactional(rollbackFor = Exception.class)
    public AjaxResult add(@RequestBody ReportRecordVo entity) {
        String paths=entity.getPaths();
        if (StrUtil.isEmpty(paths)){
            return AjaxResult.error("请上传原数据");
        }
        return toAjax(reportRecordService.saveForm(entity));
    }

    @ApiOperation("修改辅助决策")
    @PreAuthorize("@ss.hasPermi('report:RECORD:edit')")
    @Log(title = "辅助决策", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody ReportRecordVo entity) {
        return toAjax(reportRecordService.updateById(entity));
    }
    @ApiOperation("上传辅助决策")
    @PreAuthorize("@ss.hasPermi('report:RECORD:edit')")
    @Log(title = "辅助决策", businessType = BusinessType.UPDATE)
    @PostMapping("upload")
    public AjaxResult upload(@RequestPart MultipartFile file,String id) throws Exception{
        return toAjax(reportRecordService.upload(file,id));
    }

    @ApiOperation("删除辅助决策")
    @PreAuthorize("@ss.hasPermi('report:RECORD:remove')")
    @Log(title = "辅助决策", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    @Transactional(rollbackFor = Exception.class)
    public AjaxResult remove(@PathVariable String[] ids) {
        for (String id : ids) {
            iExcelOutInTotalService.removeByReportId(id);
            reportWordService.removeByReportId(id);
        }
        return toAjax(reportRecordService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }

    @ApiOperation("导入excel、支持批量")
    @PreAuthorize("@ss.hasPermi('business:template:import')")
    @PostMapping("/importExcel")
    public AjaxResult importExcelBatch(@RequestParam(value = "file")MultipartFile file )throws Exception{
        return AjaxResult.success("导入成功", reportRecordService.importExcel(file));
    }
}
