package com.hjy.modlue.report;

import com.hjy.common.annotation.Log;
import com.hjy.common.config.ZhbsqConfig;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.file.FileUploadUtils;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.framework.config.ServerConfig;
import com.hjy.module.domain.report.ReportTemplate;
import com.hjy.module.service.report.IReportTemplateService;
import com.hjy.module.vo.report.ReportTemplateVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import com.hjy.common.core.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.List;

/**
 * 模板管理Controller
 *
 * @author lbl
 * @date 2024-05-07
 */
@Api(tags = "模板管理Controller")
@RestController
@RequestMapping("/report/TEMPLATE")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class ReportTemplateController extends BaseController {

    private final IReportTemplateService reportTemplateService;
    private final ServerConfig serverConfig;

    @ApiOperation("查询模板管理列表")
    @PreAuthorize("@ss.hasPermi('report:TEMPLATE:list')")
    @GetMapping("/list")
    public TableDataInfo<ReportTemplateVo> list(ReportTemplateVo entity) {
        return reportTemplateService.queryList(entity);
    }

    @ApiOperation("查询模板管理所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(ReportTemplateVo entity) {
        return AjaxResult.success("查询成功", reportTemplateService.queryAll(entity));
    }

    @ApiOperation("导出模板管理列表")
    @PreAuthorize("@ss.hasPermi('report:TEMPLATE:export')")
    @Log(title = "模板管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ReportTemplateVo entity) {
        List<ReportTemplateVo> list = reportTemplateService.queryAll(entity);
        ExcelUtil<ReportTemplateVo> util = new ExcelUtil<>(ReportTemplateVo.class);
        util.exportExcel(response, list, "模板管理数据");
    }

    @ApiOperation("获取模板管理详细信息")
    @PreAuthorize("@ss.hasPermi('report:TEMPLATE:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", reportTemplateService.queryById(id));
    }

    @ApiOperation("新增模板管理")
    @PreAuthorize("@ss.hasPermi('report:TEMPLATE:add')")
    @Log(title = "模板管理", businessType = BusinessType.INSERT)
    @PostMapping("/add")
    public AjaxResult add(@RequestBody ReportTemplateVo entity) {
        return toAjax(reportTemplateService.save(entity));
    }

    @ApiOperation("修改模板管理")
    @PreAuthorize("@ss.hasPermi('report:TEMPLATE:edit')")
    @Log(title = "模板管理", businessType = BusinessType.UPDATE)
    @PostMapping("/edit")
    public AjaxResult edit(@RequestBody ReportTemplateVo entity) {
        return toAjax(reportTemplateService.updateById(entity));
    }

    @ApiOperation("删除模板管理")
    @PreAuthorize("@ss.hasPermi('report:TEMPLATE:remove')")
    @Log(title = "模板管理", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(reportTemplateService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }

    @PreAuthorize("@ss.hasPermi('report:TEMPLATE:edit')")
    @Log(title = "修改状态", businessType = BusinessType.UPDATE)
    @PostMapping("/changeTemplateStatus")
    public AjaxResult changeTemplateStatus(@RequestBody ReportTemplate entity) {
        return toAjax(reportTemplateService.updateById(entity));
    }

    @ApiOperation("查询模版启用数量")
    @PreAuthorize("@ss.hasPermi('report:TEMPLATE:list')")
    @GetMapping("/queryStatusNum")
    public int queryStatusNum() {
        return reportTemplateService.queryStatusNum();
    }

    @ApiOperation("导入模版")
    @PreAuthorize("@ss.hasPermi('report:TEMPLATE:edit')")
    @PostMapping("/importTemplate")
    public AjaxResult importTemplate(MultipartFile file) {
        String targetPath = "";
        String fileName = "";
        try {
            InputStream inputStream = file.getInputStream();
            targetPath = ZhbsqConfig.getTemplatePath();
            fileName = FilenameUtils.getBaseName(file.getOriginalFilename()) + "_" + System.currentTimeMillis() + "."+ FileUploadUtils.getExtension(file);
            File targetFile = new File(targetPath, fileName);
            FileUtils.copyInputStreamToFile(inputStream, targetFile);
            fileName = "/profile/template/" + fileName;
        } catch (IOException e) {
            e.printStackTrace();
            return AjaxResult.error("导入失败");
        }
        return AjaxResult.success("导入成功", fileName);
    }
}
