package com.hjy.modlue.report;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;

import com.hjy.framework.config.ServerConfig;
import com.hjy.module.service.report.IReportWordService;
import com.hjy.module.vo.report.ReportWordVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import com.hjy.common.core.page.TableDataInfo;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 模板报告Controller
 *
 * @author Tellsea
 * @date 2024-05-06
 */
@Api(tags = "模板报告Controller")
@RestController
@RequestMapping("/report/WORD")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class ReportWordController extends BaseController {

    private final IReportWordService reportWordService;
    private final ServerConfig serverConfig;

    @ApiOperation("查询模板报告列表")
    @PreAuthorize("@ss.hasPermi('report:WORD:list')")
    @GetMapping("/list")
    public TableDataInfo<ReportWordVo> list(ReportWordVo entity) {
        return reportWordService.queryList(entity);
    }

    @ApiOperation("查询模板报告所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(ReportWordVo entity) {
        List<ReportWordVo>list= reportWordService.queryAll(entity);
        return AjaxResult.success("查询成功",list);
    }

    @ApiOperation("导出模板报告列表")
    @PreAuthorize("@ss.hasPermi('report:WORD:export')")
    @Log(title = "模板报告", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ReportWordVo entity) {
        List<ReportWordVo> list = reportWordService.queryAll(entity);
        ExcelUtil<ReportWordVo> util = new ExcelUtil<>(ReportWordVo.class);
        util.exportExcel(response, list, "模板报告数据");
    }

    @ApiOperation("获取模板报告详细信息")
    @PreAuthorize("@ss.hasPermi('report:WORD:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", reportWordService.queryById(id));
    }

    @ApiOperation("新增模板报告")
    @PreAuthorize("@ss.hasPermi('report:WORD:add')")
    @Log(title = "模板报告", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody ReportWordVo entity) {
        return toAjax(reportWordService.save(entity));
    }

    @ApiOperation("修改模板报告")
    @PreAuthorize("@ss.hasPermi('report:WORD:edit')")
    @Log(title = "模板报告", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody ReportWordVo entity) {
        return toAjax(reportWordService.updateById(entity));
    }

    @ApiOperation("删除模板报告")
    @PreAuthorize("@ss.hasPermi('report:WORD:remove')")
    @Log(title = "模板报告", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(reportWordService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
