package com.hjy.modlue.reportsetting;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;

import com.hjy.module.service.reportsetting.IReportSettingService;
import com.hjy.module.vo.reportsetting.ReportSettingVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import com.hjy.common.core.page.TableDataInfo;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 模板设置Controller
 *
 * @author Tellsea
 * @date 2024-04-25
 */
@Api(tags = "模板设置Controller")
@RestController
@RequestMapping("/report/SETTING")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class ReportSettingController extends BaseController {

    private final IReportSettingService reportSettingService;

    @ApiOperation("查询模板设置列表")
    @PreAuthorize("@ss.hasPermi('business:SETTING:list')")
    @GetMapping("/list")
    public TableDataInfo<ReportSettingVo> list(ReportSettingVo entity) {
        return reportSettingService.queryList(entity);
    }

    @ApiOperation("查询模板设置所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(ReportSettingVo entity) {
        return AjaxResult.success("查询成功", reportSettingService.queryAll(entity));
    }

    @ApiOperation("导出模板设置列表")
    @PreAuthorize("@ss.hasPermi('business:SETTING:export')")
    @Log(title = "模板设置", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ReportSettingVo entity) {
        List<ReportSettingVo> list = reportSettingService.queryAll(entity);
        ExcelUtil<ReportSettingVo> util = new ExcelUtil<>(ReportSettingVo.class);
        util.exportExcel(response, list, "模板设置数据");
    }

    @ApiOperation("获取模板设置详细信息")
    @PreAuthorize("@ss.hasPermi('business:SETTING:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", reportSettingService.queryById(id));
    }

    @ApiOperation("新增模板设置")
    @PreAuthorize("@ss.hasPermi('business:SETTING:add')")
    @Log(title = "模板设置", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody ReportSettingVo entity) {
        return toAjax(reportSettingService.save(entity));
    }

    @ApiOperation("修改模板设置")
    @PreAuthorize("@ss.hasPermi('business:SETTING:edit')")
    @Log(title = "模板设置", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody ReportSettingVo entity) {
        return toAjax(reportSettingService.updateById(entity));
    }

    @ApiOperation("删除模板设置")
    @PreAuthorize("@ss.hasPermi('business:SETTING:remove')")
    @Log(title = "模板设置", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(reportSettingService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
