package com.hjy.modlue.statistics;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.R;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.statistics.ICbIntegralService;
import com.hjy.module.vo.statistics.CbIntegralVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 保税区积分Controller
 *
 * @author Tellsea
 * @date 2023-09-01
 */
@Api(tags = "保税区积分Controller")
@RestController
@RequestMapping("/statistics/INTEGRAL")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbIntegralController extends BaseController {

    private final ICbIntegralService cbIntegralService;

    @ApiOperation("查询保税区积分列表")
    @PreAuthorize("@ss.hasPermi('statistics:INTEGRAL:list')")
    @GetMapping("/list")
    public TableDataInfo<CbIntegralVo> list(CbIntegralVo entity) {
        return cbIntegralService.queryList(entity);
    }

    @ApiOperation("查询保税区积分所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbIntegralVo entity) {
        return AjaxResult.success("查询成功", cbIntegralService.queryAll(entity));
    }

    @ApiOperation("导出保税区积分列表")
    @PreAuthorize("@ss.hasPermi('statistics:INTEGRAL:export')")
    @Log(title = "保税区积分", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbIntegralVo entity) {
        List<CbIntegralVo> list = cbIntegralService.queryAll(entity);
        ExcelUtil<CbIntegralVo> util = new ExcelUtil<>(CbIntegralVo.class);
        util.exportExcel(response, list, "保税区积分数据");
    }

    @ApiOperation("获取保税区积分详细信息")
    @PreAuthorize("@ss.hasPermi('statistics:INTEGRAL:query')")
    @GetMapping(value = "/getInfo/{id}")
    public R<CbIntegralVo> getInfo(@PathVariable("id") String id) {
        return R.ok(cbIntegralService.queryById(id),"查询成功");
    }



}
