package com.hjy.modlue.template.controller;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.template.IExcelOutInTotalService;
import com.hjy.module.vo.template.ExcelOutInTotalVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import com.hjy.common.core.page.TableDataInfo;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 特定地区进出口总值（人民币）Controller
 *
 * @author Tellsea
 * @date 2024-04-23
 */
@Api(tags = "特定地区进出口总值（人民币）Controller")
@RestController
@RequestMapping("/business/OUTINTOTAL")
public class ExcelOutInTotalController extends BaseController {
    @Autowired
    private IExcelOutInTotalService excelOutInTotalService;

    @ApiOperation("查询特定地区进出口总值（人民币）列表")
    @PreAuthorize("@ss.hasPermi('business:OUTINTOTAL:list')")
    @GetMapping("/list")
    public TableDataInfo<ExcelOutInTotalVo> list(ExcelOutInTotalVo entity) {
        return excelOutInTotalService.queryList(entity);
    }

    @ApiOperation("查询特定地区进出口总值（人民币）所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(ExcelOutInTotalVo entity) {
        return AjaxResult.success("查询成功", excelOutInTotalService.queryAll(entity));
    }

    @ApiOperation("导出特定地区进出口总值（人民币）列表")
    @PreAuthorize("@ss.hasPermi('business:OUTINTOTAL:export')")
    @Log(title = "特定地区进出口总值（人民币）", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ExcelOutInTotalVo entity) {
        List<ExcelOutInTotalVo> list = excelOutInTotalService.queryAll(entity);
        ExcelUtil<ExcelOutInTotalVo> util = new ExcelUtil<>(ExcelOutInTotalVo.class);
        util.exportExcel(response, list, "特定地区进出口总值（人民币）数据");
    }

    @ApiOperation("获取特定地区进出口总值（人民币）详细信息")
    @PreAuthorize("@ss.hasPermi('business:OUTINTOTAL:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success("查询成功", excelOutInTotalService.queryById(id));
    }

    @ApiOperation("新增特定地区进出口总值（人民币）")
    @PreAuthorize("@ss.hasPermi('business:OUTINTOTAL:add')")
    @Log(title = "特定地区进出口总值（人民币）", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody ExcelOutInTotalVo entity) {
        return toAjax(excelOutInTotalService.save(entity));
    }

    @ApiOperation("修改特定地区进出口总值（人民币）")
    @PreAuthorize("@ss.hasPermi('business:OUTINTOTAL:edit')")
    @Log(title = "特定地区进出口总值（人民币）", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody ExcelOutInTotalVo entity) {
        return toAjax(excelOutInTotalService.updateById(entity));
    }

    @ApiOperation("删除特定地区进出口总值（人民币）")
    @PreAuthorize("@ss.hasPermi('business:OUTINTOTAL:remove')")
    @Log(title = "特定地区进出口总值（人民币）", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(excelOutInTotalService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
