package com.hjy.modlue.template.controller;

import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.module.service.report.IReportTemplateService;
import com.hjy.module.service.template.IExcelOutInTotalService;
import com.hjy.module.vo.report.ReportTemplateVo;
import com.hjy.module.vo.template.ExcelOutInTotalVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;


/**
 * 模版生成Controller
 *
 * @author lbl
 * @date 2024-04-23
 */
@Api(tags = "模版生成Controller")
@RestController
@RequestMapping("/template")
public class GenerateTemplateController extends BaseController {
    @Autowired
    private IExcelOutInTotalService excelOutInTotalService;
    @Autowired
    private IReportTemplateService reportTemplateService;

    @ApiOperation("查询特定地区进出口总值（人民币）列表")
    @PreAuthorize("@ss.hasPermi('business:OUTINTOTAL:list')")
    @GetMapping("/list")
    public TableDataInfo<ExcelOutInTotalVo> list(ExcelOutInTotalVo entity) {
        return excelOutInTotalService.queryList(entity);
    }

    @ApiOperation("查询特定地区进出口总值（人民币）所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(ExcelOutInTotalVo entity) {
        return AjaxResult.success("查询成功", excelOutInTotalService.queryAll(entity));
    }

    @ApiOperation("导入excel")
    @PreAuthorize("@ss.hasPermi('business:template:import')")
    @PostMapping("/importExcel")
    public AjaxResult importExcel(MultipartFile file,String gxTradeTotalMoney) {
        ReportTemplateVo entity = new ReportTemplateVo();
        entity.setStatus("1");
        List<ReportTemplateVo> voList = reportTemplateService.queryAll(entity);
        if(voList.size() != 1) {
            return AjaxResult.error("没有启用模版或者启用了多个模版，请检查模版管理菜单");
        }
        excelOutInTotalService.analyze(file,gxTradeTotalMoney);
        return AjaxResult.success("导入成功");
    }


}
