package com.hjy.modlue.totexp;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.totexp.ICbTotexpAreaService;
import com.hjy.module.vo.totexp.CbTotexpAreaVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 自治区数据Controller
 *
 * @author Tellsea
 * @date 2023-10-15
 */
@Api(tags = "自治区数据Controller")
@RestController
@RequestMapping("/area/TOTEXPAREA")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbTotexpAreaController extends BaseController {

    private final ICbTotexpAreaService cbTotexpAreaService;

    @ApiOperation("查询自治区数据列表")
    @PreAuthorize("@ss.hasPermi('area:TOTEXPAREA:list')")
    @GetMapping("/list")
    public TableDataInfo<CbTotexpAreaVo> list(CbTotexpAreaVo entity) {
        return cbTotexpAreaService.queryList(entity);
    }

    @ApiOperation("查询自治区数据所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbTotexpAreaVo entity) {
        return AjaxResult.success("查询成功", cbTotexpAreaService.queryAll(entity));
    }

    @ApiOperation("导出自治区数据列表")
    @PreAuthorize("@ss.hasPermi('area:TOTEXPAREA:export')")
    @Log(title = "自治区数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbTotexpAreaVo entity) {
        List<CbTotexpAreaVo> list = cbTotexpAreaService.queryAll(entity);
        ExcelUtil<CbTotexpAreaVo> util = new ExcelUtil<>(CbTotexpAreaVo.class);
        util.exportExcel(response, list, "自治区数据数据");
    }

    @ApiOperation("获取自治区数据详细信息")
    @PreAuthorize("@ss.hasPermi('area:TOTEXPAREA:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success("查询成功", cbTotexpAreaService.queryById(id));
    }

    @ApiOperation("新增自治区数据")
    @PreAuthorize("@ss.hasPermi('area:TOTEXPAREA:add')")
    @Log(title = "自治区数据", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbTotexpAreaVo entity) {
        return toAjax(cbTotexpAreaService.save(entity));
    }

    @ApiOperation("修改自治区数据")
    @PreAuthorize("@ss.hasPermi('area:TOTEXPAREA:edit')")
    @Log(title = "自治区数据", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbTotexpAreaVo entity) {
        return toAjax(cbTotexpAreaService.updateById(entity));
    }

    @ApiOperation("删除自治区数据")
    @PreAuthorize("@ss.hasPermi('area:TOTEXPAREA:remove')")
    @Log(title = "自治区数据", businessType = BusinessType.DELETE)
    @GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(cbTotexpAreaService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
