package com.hjy.modlue.totexp;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.exception.ServiceException;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.domain.totexp.CbTotexp;
import com.hjy.module.service.totexp.ICbTotexpService;
import com.hjy.module.vo.totexp.CbTotexpVo;
import com.hjy.system.service.ISysDeptService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 保税区进出口总值数据Controller
 *
 * @author Tellsea
 * @date 2023-10-10
 */
//@Api(tags = "保税区进出口总值数据Controller")
@RestController
@RequestMapping("/totexp/TOTEXP")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbTotexpController extends BaseController {

    private final ICbTotexpService cbTotexpService;
    private final ISysDeptService sysDeptService;

    //    @ApiOperation("查询保税区进出口总值数据列表")
    @PreAuthorize("@ss.hasPermi('totexp:TOTEXP:list')")
    @GetMapping("/list")
    public TableDataInfo<CbTotexpVo> list(CbTotexpVo entity) {
        return cbTotexpService.queryList(entity);
    }

    //    @ApiOperation("查询保税区进出口总值数据所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(CbTotexpVo entity) {
        return AjaxResult.success("查询成功", cbTotexpService.queryAll(entity));
    }

    //    @ApiOperation("导出保税区进出口总值数据列表")
    @PreAuthorize("@ss.hasPermi('totexp:TOTEXP:export')")
    @Log(title = "保税区进出口总值数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CbTotexpVo entity) {
        List<CbTotexpVo> list = cbTotexpService.queryAll(entity);
        ExcelUtil<CbTotexpVo> util = new ExcelUtil<>(CbTotexpVo.class);
        util.exportExcel(response, list, "保税区进出口总值数据数据");
    }

    //    @ApiOperation("获取保税区进出口总值数据详细信息")
    @PreAuthorize("@ss.hasPermi('totexp:TOTEXP:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbTotexpService.queryById(id));
    }

    //    @ApiOperation("新增保税区进出口总值数据")
    @PreAuthorize("@ss.hasPermi('totexp:TOTEXP:add')")
    @Log(title = "保税区进出口总值数据", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbTotexpVo entity) {
        // 查询对应的dept部门
        SysDept sysDept = sysDeptService.selectDeptById(Long.valueOf(entity.getDeptId()));
        if (sysDept == null) {
            throw new ServiceException("不存在该部门", 404);
        } else {
            if (4 == sysDept.getType()) {
                // 将部门id 赋值给 city_dept_id
//                entity.setCityDeptId(Long.toString(sysDept.getDeptId()));
                entity.setCityDeptId(String.valueOf(sysDept.getParentId()));
            }
        }
        LambdaQueryWrapper<CbTotexp> wrapper = new LambdaQueryWrapper<CbTotexp>()
                .eq(CbTotexp::getDeptId, entity.getDeptId())
                .eq(CbTotexp::getYear, entity.getYear());
        CbTotexp one = cbTotexpService.getOne(wrapper);
        if (one != null) {
            return AjaxResult.error(400, "请勿添加重复数据");
        }

        return toAjax(cbTotexpService.save(entity));
    }

    //    @ApiOperation("修改保税区进出口总值数据")
    @PreAuthorize("@ss.hasPermi('totexp:TOTEXP:edit')")
    @Log(title = "保税区进出口总值数据", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbTotexpVo entity) {
        return toAjax(cbTotexpService.updateById(entity));
    }

    //    @ApiOperation("删除保税区进出口总值数据")
    @PreAuthorize("@ss.hasPermi('totexp:TOTEXP:remove')")
    @Log(title = "保税区进出口总值数据", businessType = BusinessType.DELETE)
    @GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(cbTotexpService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }
}
