package com.hjy.modlue.wf;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.R;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.SecurityUtils;
import com.hjy.common.utils.StringUtils;
import com.hjy.module.domain.wf.dto.WftasksubmitDto;
import com.hjy.module.service.wf.IWfInstActionTaskService;
import com.hjy.module.vo.wf.WfInstActionTaskVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

/**
 * 审批任务实例Controller
 *
 * @author Tellsea
 * @date 2023-08-09
 */
@Api(tags = "审批任务实例Controller")
@RestController
@RequestMapping("/actionTask")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class WfInstActionTaskController extends BaseController {

    private final IWfInstActionTaskService wfInstActionTaskService;

    @ApiOperation("查询审批任务实例列表")
    @PreAuthorize("@ss.hasPermi('business:INSTACTIONTASK:list')")
    @GetMapping("/list")
    public TableDataInfo<WfInstActionTaskVo> list(WfInstActionTaskVo entity) {
        return wfInstActionTaskService.queryList(entity);
    }

    @PreAuthorize("@ss.hasPermi('business:INSTACTIONTASK:byUser')")
    @ApiOperation("查询用户自己的审核任务")
    @GetMapping("/list_byUser")
    public TableDataInfo<WfInstActionTaskVo> listByuser(WfInstActionTaskVo entity) {
        entity.setExecId(StringUtils.obj2str(SecurityUtils.getUserId()));
        return wfInstActionTaskService.queryList(entity);
    }

    @ApiOperation("查询审批任务实例所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(WfInstActionTaskVo entity) {
        return AjaxResult.success("查询成功", wfInstActionTaskService.queryAll(entity));
    }


    @ApiOperation("获取审批任务实例详细信息")
    @PreAuthorize("@ss.hasPermi('business:INSTACTIONTASK:query')")
    @GetMapping(value = "/getInfo/{id}")
    public R<WfInstActionTaskVo> getInfo(@PathVariable("id") String id) {
        return R.ok(wfInstActionTaskService.queryById(id), "查询成功");
    }

    @ApiOperation("任务审核通过")
    @PreAuthorize("@ss.hasPermi('task:instaction:approve')")
    @Log(title = "任务审核通过", businessType = BusinessType.UPDATE)
    @PostMapping("approve")
    public AjaxResult approve(@RequestBody WftasksubmitDto submitDto) {
        return wfInstActionTaskService.approve(submitDto);
    }

    @ApiOperation("任务审核驳回")
    @PreAuthorize("@ss.hasPermi('task:instaction:reject')")
    @Log(title = "任务审核驳回", businessType = BusinessType.UPDATE)
    @PostMapping("reject")
    public AjaxResult reject(@RequestBody WftasksubmitDto submitDto) {
        return wfInstActionTaskService.reject(submitDto);
    }

    @ApiOperation("任务审核退回")
    @PreAuthorize("@ss.hasPermi('task:instaction:goback')")
    @Log(title = "任务审核退回", businessType = BusinessType.UPDATE)
    @PostMapping("goback")
    public AjaxResult goback(@RequestBody WftasksubmitDto submitDto) {
        return wfInstActionTaskService.goback(submitDto);
    }

    @ApiOperation("查询用户自己的审核待办任务数量")
    @GetMapping("/queryBy")
    public Integer listByusers() {
        return wfInstActionTaskService.queryBy();
    }
}
