package com.hjy.modlue.wf;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.service.wf.IWfProDefActionService;
import com.hjy.module.vo.wf.WfProDefActionVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 流程节点定义Controller
 *
 * @author Tellsea
 * @date 2023-08-09
 */
@Api(tags = "流程节点定义Controller")
@RestController
@RequestMapping("/defAction")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class WfProDefActionController extends BaseController {

    private final IWfProDefActionService wfProDefActionService;

    @ApiOperation("查询流程节点定义列表")
    @GetMapping("/list")
    public TableDataInfo<WfProDefActionVo> list(WfProDefActionVo entity) {
        return wfProDefActionService.queryList(entity);
    }

    @ApiOperation("查询流程节点定义所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(WfProDefActionVo entity) {
        return AjaxResult.success("查询成功", wfProDefActionService.queryAll(entity));
    }

    @ApiOperation("导出流程节点定义列表")
    @PreAuthorize("@ss.hasPermi('business:PRODEFACTION:export')")
    @Log(title = "流程节点定义", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, WfProDefActionVo entity) {
        List<WfProDefActionVo> list = wfProDefActionService.queryAll(entity);
        ExcelUtil<WfProDefActionVo> util = new ExcelUtil<>(WfProDefActionVo.class);
        util.exportExcel(response, list, "流程节点定义数据");
    }

    @ApiOperation("获取流程节点定义详细信息")
    @PreAuthorize("@ss.hasPermi('business:PRODEFACTION:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", wfProDefActionService.queryById(id));
    }

    @ApiOperation("新增流程节点定义")
    @PreAuthorize("@ss.hasPermi('business:PRODEFACTION:add')")
    @Log(title = "流程节点定义", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody WfProDefActionVo entity) {
        return toAjax(wfProDefActionService.save(entity));
    }

    @ApiOperation("修改流程节点定义")
    @PreAuthorize("@ss.hasPermi('business:PRODEFACTION:edit')")
    @Log(title = "流程节点定义", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody WfProDefActionVo entity) {
        return toAjax(wfProDefActionService.updateById(entity));
    }

    @ApiOperation("删除流程节点定义")
    @PreAuthorize("@ss.hasPermi('business:PRODEFACTION:remove')")
    @Log(title = "流程节点定义", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(wfProDefActionService.removeByIds(Arrays.asList(ids)));
    }
}
