package com.hjy.modlue.wf;

import cn.hutool.json.JSONUtil;
import com.hjy.common.annotation.Log;
import com.hjy.common.annotation.PortalAuth;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.core.text.Convert;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.enums.HisModule;
import com.hjy.common.utils.StringUtils;
import com.hjy.common.utils.poi.ExcelUtil;
import com.hjy.module.domain.wf.WfProtemplate;
import com.hjy.module.service.commom.HisMethods;
import com.hjy.module.service.wf.IWfInstProService;
import com.hjy.module.service.wf.IWfProtemplateService;
import com.hjy.module.vo.wf.WfProtemplateVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * 流程模板Controller
 *
 * @author Tellsea
 * @date 2023-08-09
 */
@Api(tags = "流程模板Controller")
@RestController
@RequestMapping("/proTemplate")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class WfProtemplateController extends BaseController {

    private final IWfProtemplateService wfProtemplateService;

    private final IWfInstProService wfInstProService;

    @PortalAuth
    @ApiOperation("查询流程模板列表")
    @PreAuthorize("@ss.hasPermi('business:PROTEMPLATE:list')")
    @GetMapping("/list")
    public TableDataInfo<WfProtemplateVo> list(WfProtemplateVo entity) {
        return wfProtemplateService.queryList(entity);
    }

    @ApiOperation("查询流程模板所有列表")
    @GetMapping("/listAll")
    public AjaxResult listAll(WfProtemplateVo entity) {
        return AjaxResult.success("查询成功", wfProtemplateService.queryAll(entity));
    }



    @ApiOperation("导出流程模板列表")
    @PreAuthorize("@ss.hasPermi('business:PROTEMPLATE:export')")
    @Log(title = "流程模板", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, WfProtemplateVo entity) {
        List<WfProtemplateVo> list = wfProtemplateService.queryAll(entity);
        ExcelUtil<WfProtemplateVo> util = new ExcelUtil<>(WfProtemplateVo.class);
        util.exportExcel(response, list, "流程模板数据");
    }


    @ApiOperation("获取流程模板详细信息")
    @PreAuthorize("@ss.hasPermi('business:PROTEMPLATE:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", wfProtemplateService.queryById(id));
    }

    @ApiOperation("新增流程模板")
    @PreAuthorize("@ss.hasPermi('business:PROTEMPLATE:add')")
    @Log(title = "流程模板", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody WfProtemplateVo entity) {
        if (!checkCode(entity,false)){
            return error("流程编号已存在，无法保存");
        }
        if (!checkService(entity,false)){
            return error("业务Service已存在，无法保存");
        }
        entity.setVersion("1");
        return toAjax(wfProtemplateService.save(entity));
    }

    @ApiOperation("修改流程模板")
    @PreAuthorize("@ss.hasPermi('business:PROTEMPLATE:edit')")
    @Log(title = "流程模板", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody WfProtemplateVo entity) {
        if (!checkCode(entity,true)){
            return error("流程编号已存在，无法保存");
        }
        if (!checkService(entity,true)){
            return error("业务Service已存在，无法保存");
        }
        entity.setVersion(Convert.toStr(Convert.toInt(entity.getVersion())+1));
        boolean flag = wfProtemplateService.updateById(entity);
        if (flag){
            HisMethods.saveFromHis(HisModule.MODULE1.getCode(),entity.getId(), JSONUtil.toJsonStr(entity));
        }
        return toAjax(flag);
    }

    private boolean checkCode(WfProtemplateVo entity,boolean isUpdate){
        if (!isUpdate){
            int count = wfProtemplateService.lambdaQuery().eq(WfProtemplate::getProcode,entity.getProcode()).count();
            if (count > 0){
                return false;
            }
            return true;
        }else{
            int count = wfProtemplateService.lambdaQuery()
                    .eq(WfProtemplate::getProcode,entity.getProcode())
                    .notIn(StringUtils.isNotNull(entity.getId()),WfProtemplate::getId,entity.getId())
                    .count();
            if (count > 0){
                return false;
            }
            return true;
        }
    }

    private boolean checkService(WfProtemplateVo entity,boolean isUpdate){
        if (!isUpdate){
            int count = wfProtemplateService.lambdaQuery()
                    .eq(WfProtemplate::getServiceUrl,entity.getServiceUrl())
                    .eq(WfProtemplate::getCbId,entity.getCbId())
                    .count();
            if (count > 0){
                return false;
            }
            return true;
        }else{
            int count = wfProtemplateService.lambdaQuery()
                    .eq(WfProtemplate::getServiceUrl,entity.getServiceUrl())
                    .notIn(StringUtils.isNotNull(entity.getId()),WfProtemplate::getId,entity.getId())
                    .eq(WfProtemplate::getCbId,entity.getCbId())
                    .count();
            if (count > 0){
                return false;
            }
            return true;
        }

    }


    @ApiOperation("删除流程模板")
    @PreAuthorize("@ss.hasPermi('business:PROTEMPLATE:remove')")
    @Log(title = "流程模板", businessType = BusinessType.DELETE)
	@GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(wfProtemplateService.removeByIds(Arrays.asList(ids)));
    }

    @ApiOperation("流程模板启用/禁用")
    @PreAuthorize("@ss.hasPermi('business:PROTEMPLATE:edit')")
    @Log(title = "流程模板", businessType = BusinessType.UPDATE)
    @PostMapping("type")
    public AjaxResult type(@RequestBody WfProtemplateVo entity) {
        if (!entity.getType()) {
            //查询该模板下还有没有在进行中的任务
            Integer integer = wfInstProService.queryType(entity.getId());
            if (integer != 0) {
                return AjaxResult.error("该模板下有进行中的流程");
            }
        }
        return toAjax(wfProtemplateService.updateById(entity));
    }
}
