package com.hjy.web.controller.system;

import com.hjy.common.constant.Constants;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysMenu;
import com.hjy.common.core.domain.entity.SysUser;
import com.hjy.common.core.domain.model.LoginBody;
import com.hjy.common.core.sm2.Sm2;
import com.hjy.common.utils.SecurityUtils;
import com.hjy.common.utils.Sm2Utils;
import com.hjy.framework.web.service.SysLoginService;
import com.hjy.framework.web.service.SysPermissionService;
import com.hjy.system.service.ISysMenuService;
import org.bouncycastle.crypto.InvalidCipherTextException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.Set;

/**
 * 登录验证
 *
 * @author HJY
 */
@RestController
public class SysLoginController {
    @Autowired
    private SysLoginService loginService;

    @Autowired
    private ISysMenuService menuService;

    @Autowired
    private SysPermissionService permissionService;

    private String privateKeyHex = null;

    /**
     * 登录方法
     *
     * @param loginBody 登录信息
     * @return 结果
     */
    @PostMapping("/login")
    public AjaxResult login(@RequestBody LoginBody loginBody) throws InvalidCipherTextException {
        String decrypt = Sm2Utils.decrypt(loginBody.getPassword(), privateKeyHex);
        AjaxResult ajax = AjaxResult.success();
        // 生成令牌
        String token = loginService.login(loginBody.getUsername(), decrypt, loginBody.getCode(),
                loginBody.getUuid());
        ajax.put(Constants.TOKEN, token);
        return ajax;
    }

    /**
     * 获取公密接口
     */
    @PostMapping("/genPUPRkey")
    public AjaxResult genPUPRkey() throws NoSuchAlgorithmException {
        Sm2 sm2 = Sm2Utils.genPUPRkey();
        privateKeyHex = sm2.getPrivateKeyHex();
        return AjaxResult.success(sm2.getPublicKeyHex());
    }

    /**
     * 获取用户信息
     *
     * @return 用户信息
     */
    @GetMapping("getInfo")
    public AjaxResult getInfo() {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        // 角色集合
        Set<String> roles = permissionService.getRolePermission(user);
        // 权限集合
        Set<String> permissions = permissionService.getMenuPermission(user);
        AjaxResult ajax = AjaxResult.success();
        ajax.put("user", user);
        ajax.put("roles", roles);
        ajax.put("permissions", permissions);
        return ajax;
    }

    /**
     * 获取路由信息
     *
     * @return 路由信息
     */
    @GetMapping("getRouters")
    public AjaxResult getRouters() {
        Long userId = SecurityUtils.getUserId();
        List<SysMenu> menus = menuService.selectMenuTreeByUserId(userId);
        return AjaxResult.success(menuService.buildMenus(menus));
    }
}
