package com.hjy.common.utils;

import com.hjy.common.core.text.Convert;
import org.apache.commons.lang3.time.DateFormatUtils;

import java.lang.management.ManagementFactory;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.*;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 时间工具类
 *
 * @author HJY
 */
public class DateUtils extends org.apache.commons.lang3.time.DateUtils {
    private static final String[] parsePatterns = {
            "yyyy-MM-dd", "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd HH:mm", "yyyy-MM",
            "yyyy/MM/dd", "yyyy/MM/dd HH:mm:ss", "yyyy/MM/dd HH:mm", "yyyy/MM",
            "yyyy.MM.dd", "yyyy.MM.dd HH:mm:ss", "yyyy.MM.dd HH:mm", "yyyy.MM",
            "yyyy年MM月dd日 HH时mm分ss秒"};
    public static String YYYY = "yyyy";
    public static String YYYY_MM = "yyyy-MM";
    public static String YYYY_MM_DD = "yyyy-MM-dd";
    public static String YYYYMMDDHHMMSS = "yyyyMMddHHmmss";
    public static String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";
    public static String YYYY_MM_DD_HH_MM_SS_CND = "yyyy年MM月dd日 HH时mm分ss秒";

    /**
     * 获取当前Date型日期
     *
     * @return Date() 当前日期
     */
    public static Date getNowDate() {
        return new Date();
    }

    /**
     * 获取当前日期, 默认格式为yyyy-MM-dd
     *
     * @return String
     */
    public static String getDate() {
        return dateTimeNow(YYYY_MM_DD);
    }


    public static final String getTime() {
        return dateTimeNow(YYYY_MM_DD_HH_MM_SS);
    }

    public static final String dateTimeNow() {
        return dateTimeNow(YYYYMMDDHHMMSS);
    }

    public static final String dateTimeNow(final String format) {
        return parseDateToStr(format, new Date());
    }

    public static final String getYear(Date date) {
        return parseDateToStr(YYYY_MM_DD_HH_MM_SS_CND, date).substring(0, 4);
    }

    public static final String getQuarterByTime(Date date) {
        return getquarter(parseDateToStr(YYYYMMDDHHMMSS, date).substring(4, 6));
    }


    public static final String getmonth(Date date) {
        return parseDateToStr(YYYY_MM_DD_HH_MM_SS_CND, date).substring(5, 8);
    }


    public static final String dateTime(final Date date) {
        return parseDateToStr(YYYY_MM_DD, date);
    }

    public static final String parseDateToStr(final String format, final Date date) {
        return new SimpleDateFormat(format).format(date);
    }

    public static final Date dateTime(final String format, final String ts) {
        try {
            return new SimpleDateFormat(format).parse(ts);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * @return 获取近5年列表
     */
    public static String getRecentYearFive() {
        return getRecentYears(5);
    }

    /**
     * 获取近 numYears 年
     *
     * @param numYears 具体获取多少年
     * @return 返回获取到的年份，拼接好的字符串，包括当前年份
     */
    public static String getRecentYears(int numYears) {
        List<Integer> years = new ArrayList<>();
        int currentYear = Year.now().getValue();
        for (int i = 0; i < numYears; i++) {
            years.add(currentYear - i);
        }
        return years.stream().map(String::valueOf).collect(Collectors.joining(","));
    }


    /**
     * 日期路径 即年/月/日 如2018/08/08
     */
    public static final String datePath() {
        Date now = new Date();
        return DateFormatUtils.format(now, "yyyy/MM/dd");
    }

    /**
     * 日期路径 即年/月/日 如20180808
     */
    public static final String dateTime() {
        Date now = new Date();
        return DateFormatUtils.format(now, "yyyyMMdd");
    }

    /**
     * 日期型字符串转化为日期 格式
     */
    public static Date parseDate(Object str) {
        if (str == null) {
            return null;
        }
        try {
            return parseDate(str.toString(), parsePatterns);
        } catch (ParseException e) {
            return null;
        }
    }

    /**
     * 获取服务器启动时间
     */
    public static Date getServerStartDate() {
        long time = ManagementFactory.getRuntimeMXBean().getStartTime();
        return new Date(time);
    }

    /**
     * 计算相差天数
     */
    public static int differentDaysByMillisecond(Date date1, Date date2) {
        return Math.abs((int) ((date2.getTime() - date1.getTime()) / (1000 * 3600 * 24)));
    }

    /**
     * 计算两个时间差
     */
    public static String getDatePoor(Date endDate, Date nowDate) {
        long nd = 1000 * 24 * 60 * 60;
        long nh = 1000 * 60 * 60;
        long nm = 1000 * 60;
        // long ns = 1000;
        // 获得两个时间的毫秒时间差异
        long diff = endDate.getTime() - nowDate.getTime();
        // 计算差多少天
        long day = diff / nd;
        // 计算差多少小时
        long hour = diff % nd / nh;
        // 计算差多少分钟
        long min = diff % nd % nh / nm;
        // 计算差多少秒//输出结果
        // long sec = diff % nd % nh % nm / ns;
        return day + "天" + hour + "小时" + min + "分钟";
    }

    /**
     * 增加 LocalDateTime ==> Date
     */
    public static Date toDate(LocalDateTime temporalAccessor) {
        ZonedDateTime zdt = temporalAccessor.atZone(ZoneId.systemDefault());
        return Date.from(zdt.toInstant());
    }

    /**
     * 增加 LocalDate ==> Date
     */
    public static Date toDate(LocalDate temporalAccessor) {
        LocalDateTime localDateTime = LocalDateTime.of(temporalAccessor, LocalTime.of(0, 0, 0));
        ZonedDateTime zdt = localDateTime.atZone(ZoneId.systemDefault());
        return Date.from(zdt.toInstant());
    }

    /**
     * 季度
     *
     * @param month
     * @return
     */
    public static String getquarter(String month) {
        Long monthLong = Convert.toLong(month, null);
        if (StringUtils.isNull(monthLong)) {
            return null;
        }
        String result = "";
        if (monthLong <= 3) {
            result = "1";
        } else if (monthLong > 3 && monthLong <= 6) {
            result = "2";
        } else if (monthLong > 6 && monthLong <= 9) {
            result = "3";
        } else if (monthLong > 9 && monthLong <= 12) {
            result = "4";
        }
        return result;
    }

    /**
     * 将 XX月截取 数值月份
     *
     * @param month
     * @return
     */
    public static String subMonth(String month) {
        if (StringUtils.isNull(month)) return null;
        if (month.indexOf("月") == -1) return null;
        return month.substring(0, 2);
    }

    public static void main(String[] args) {
        System.out.println(subMonth("09月"));
    }

}
