package com.hjy.common.utils.excel;

import com.hjy.common.core.text.Convert;
import com.hjy.common.utils.excel.vo.TaxEnterpriseItemVo;
import com.hjy.common.utils.excel.vo.TaxEnterpriseTableVo;
import com.hjy.common.utils.excel.vo.TaxEnterpriseVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFFormulaEvaluator;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFFormulaEvaluator;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;

/**
 * 特殊监管区内企业进出口总值 excel读取
 */
@Slf4j
public class TaxReadEnterpriseExcelUtil {

    public static String fileName = "D:\\out\\特殊监管区内企业进出口总值 .xls";

    private static FormulaEvaluator formulaEvaluator = null;

    public static void main(String[] args) {
        TaxEnterpriseTableVo tableVo = creatWorkBook(fileName);
        //String jsonStr =  JSONUtil.toJsonStr(tableVo);
        log.debug("特殊监管区内企业进出口总值 excel 已读取");
    }


    /**
     * 创建 excel的, 返回 对象, 返回 null时, 未读取成功
     * @param fileName
     * @return
     */
    public static TaxEnterpriseTableVo creatWorkBook(String fileName) {

        log.info("判断文件是否存在...");
        File excelFile = new File(fileName);
        if (!excelFile.exists()) {
            log.warn("指定的Excel文件不存在！");
            return null;
        }

        log.info("创建 WorkBook ...");
        String fileType = fileName.substring(fileName.lastIndexOf(".") + 1);
        Workbook workbook = null;
        try {
            FileInputStream fileInputStream = new FileInputStream(excelFile);
            if (fileType.equalsIgnoreCase("xls")) {
                workbook = new HSSFWorkbook(fileInputStream); //生成.xls的excel
                //后面使用它来执行计算公式
                TaxReadEnterpriseExcelUtil.formulaEvaluator = new HSSFFormulaEvaluator((HSSFWorkbook) workbook);
            } else if (fileType.equalsIgnoreCase("xlsx")) {
                workbook = new XSSFWorkbook(fileInputStream); //生成.xlsx的excel
                TaxReadEnterpriseExcelUtil.formulaEvaluator = new XSSFFormulaEvaluator((XSSFWorkbook) workbook);
            } else {
                log.warn("文件格式不对");
                return null;
            }
            log.info("开始解析 WorkBook...");
            return parseExcel(workbook, 0);

        } catch (FileNotFoundException e) {
            log.error("文件读取异常 [{}] - [{}]", e.getMessage(), e);
        } catch (IOException e) {
            log.error("IO读取异常 [{}] - [{}]", e.getMessage(), e);
        }

        return null;
    }

    /**
     * 文件流
     * @param fileName
     * @return
     */
    public static TaxEnterpriseTableVo creatWorkBook(FileInputStream fileInputStream,String fileName) {

        log.info("判断文件是否存在...");
        if (StringUtils.isBlank(fileName)) {
            log.warn("指定的Excel文件不存在！");
            return null;
        }

        log.info("创建 WorkBook ...");
        String fileType = fileName.substring(fileName.lastIndexOf(".") + 1);
        Workbook workbook = null;
        try {
            //FileInputStream fileInputStream = new FileInputStream(excelFile);
            if (fileType.equalsIgnoreCase("xls")) {
                workbook = new HSSFWorkbook(fileInputStream); //生成.xls的excel
                //后面使用它来执行计算公式
                TaxReadEnterpriseExcelUtil.formulaEvaluator = new HSSFFormulaEvaluator((HSSFWorkbook) workbook);
            } else if (fileType.equalsIgnoreCase("xlsx")) {
                workbook = new XSSFWorkbook(fileInputStream); //生成.xlsx的excel
                TaxReadEnterpriseExcelUtil.formulaEvaluator = new XSSFFormulaEvaluator((XSSFWorkbook) workbook);
            } else {
                log.warn("文件格式不对");
                return null;
            }
            log.info("开始解析 WorkBook...");
            return parseExcel(workbook, 0);

        } catch (FileNotFoundException e) {
            log.error("文件读取异常 [{}] - [{}]", e.getMessage(), e);
        } catch (IOException e) {
            log.error("IO读取异常 [{}] - [{}]", e.getMessage(), e);
        }

        return null;
    }

    /**
     * 解析 读取 excel的数据
     * @param workbook
     * @param sheetNum
     */
    private static TaxEnterpriseTableVo parseExcel(Workbook workbook, int sheetNum) {
        Sheet sheet = workbook.getSheetAt(sheetNum); // 获取表格
        log.info("sheet = {}", sheet.getSheetName());


        //保存的读取对象
        TaxEnterpriseTableVo tableVo = new TaxEnterpriseTableVo();
        TaxEnterpriseVo accountVo = new TaxEnterpriseVo();
        TaxEnterpriseVo numVo = new TaxEnterpriseVo();
        accountVo.setList(new ArrayList<>());
        numVo.setList(new ArrayList<>());
        tableVo.setAccountVo(accountVo);
        tableVo.setNumVo(numVo);


        Row firstRow = sheet.getRow(sheet.getFirstRowNum()); // 获取第一行，一般是标题
        if (null == firstRow) {
            log.warn("解析Excel失败，在第一行没有读取到任何数据！");
        }

        // 解析每一行的数据，构造数据对象
        int rowStart = 0; //标题下面的数据,数据起始行
        //int rowEnd = sheet.getPhysicalNumberOfRows();//获取有记录的行数，即：最后有数据的行是第n行，前面有m行是空行没数据，则返回n-m；
        int rowEndLast = sheet.getLastRowNum();//获取有记录的行数，即：最后有数据的行是第n行，前面有m行是空行没数据，则返回n-m；

        //第一部分企业的读取
        boolean firstEnterpriseFlag = false;
        int firstIndexNum = 0;
        boolean firstEnterpriseItemFlag = false;
        //第一部分已读的标记
        boolean firstEnterpriseReadFlag = false;

        ///第二部分企业的读取
        boolean secondEnterpriseFlag = false;
        int secondIndexNum = 0;
        boolean secondEnterpriseItemFlag = false;
        //第二部分已读的标记
        boolean secondEnterpriseReadFlag = false;


        //开始读取 每行的数据
        for (int rowNum = rowStart; rowNum <= rowEndLast; rowNum++) {
            Row row = sheet.getRow(rowNum);

            //第一部分的企业 读取结束
            if(firstEnterpriseFlag && firstEnterpriseItemFlag && null == row) {
                firstEnterpriseFlag = false;
                firstEnterpriseItemFlag = false;
                firstEnterpriseReadFlag = true;
            }
            //第二部分的企业 读取结束
            if(secondEnterpriseFlag && secondEnterpriseItemFlag && null == row) {
                secondEnterpriseFlag = false;
                secondEnterpriseItemFlag = false;
                secondEnterpriseReadFlag = true;
            }
            if (null == row) continue;
            String f0 = TaxReadEnterpriseExcelUtil.readCell(row, 0, "");
            if (ObjectUtils.isEmpty(f0)) continue;
            if(!firstEnterpriseReadFlag && !firstEnterpriseFlag ) {
                firstEnterpriseFlag = true;
                firstIndexNum = rowNum;
            }
            if(firstEnterpriseFlag && rowNum == (firstIndexNum + 2) ) {
                String title = TaxReadEnterpriseExcelUtil.readCell(row, 2, "");
                accountVo.setTitle(title);
            }

            if(firstEnterpriseFlag && firstEnterpriseItemFlag) {
                //读取 第一部分的 企业的 item
                String name = TaxReadEnterpriseExcelUtil.readCell(row, 1, "");
                String f2 = TaxReadEnterpriseExcelUtil.readCell(row, 2, "");
                String f3 = TaxReadEnterpriseExcelUtil.readCell(row, 3, "");
                TaxEnterpriseItemVo itemVo = new TaxEnterpriseItemVo();
                itemVo.setCode(com.hjy.common.utils.StringUtils.obj2str(Convert.toLong(f0,null)));
                itemVo.setName(name);
                itemVo.setValue(f2);
                itemVo.setCompireCalue(f3);
                accountVo.getList().add(itemVo);
            }

            if(firstEnterpriseFlag && "#合计".equals(f0)) {
                //准备 读取 item
                firstEnterpriseItemFlag = true;
            }

            //第二部分
            if(!secondEnterpriseFlag && firstEnterpriseReadFlag && "特殊监管区内企业进出口总值".equals(f0.trim())) {
                secondEnterpriseFlag = true;
                secondIndexNum = rowNum;
            }

            if(secondEnterpriseFlag && rowNum == (secondIndexNum + 3) ) {
                String title = TaxReadEnterpriseExcelUtil.readCell(row, 2, "");
                numVo.setTitle(title);
            }

            if(secondEnterpriseFlag && secondEnterpriseItemFlag) {
                //读取 第二部分的 企业的 item
                String name = TaxReadEnterpriseExcelUtil.readCell(row, 1, "");
                String f2 = TaxReadEnterpriseExcelUtil.readCell(row, 2, "");
                String f3 = TaxReadEnterpriseExcelUtil.readCell(row, 3, "");
                TaxEnterpriseItemVo itemVo = new TaxEnterpriseItemVo();
                itemVo.setCode(f0);
                itemVo.setName(name);
                BigDecimal f2decimal = BigDecimal.ZERO;
                if(StringUtils.isNotBlank(f2)) {
                    f2decimal = new BigDecimal(f2);
                    itemVo.setValue(String.valueOf(f2decimal.intValue()));
                }
                if(StringUtils.isNotBlank(f3)){
                    f2decimal = new BigDecimal(f3);
                    itemVo.setCompireCalue(String.valueOf(f2decimal.intValue()));
                }
                numVo.getList().add(itemVo);
            }

            if(secondEnterpriseFlag && "#合计".equals(f0)) {
                secondEnterpriseItemFlag = true;
            }
        }

        return tableVo;
    }

    /**
     * 处理单元格类型
     * @param cell
     * @return
     */
    private static String convertCellValueToString(Cell cell) {
        if (cell == null) {
            return null;
        }
        String content = null;
        try {
            switch (cell.getCellType()) {
                case NUMERIC:   //数字或者时间
                    Double doubleValue = cell.getNumericCellValue();
                    // 格式化科学计数法，取一位整数
                    //DecimalFormat df = new DecimalFormat("0");
                    /*DecimalFormat df = new DecimalFormat("#####0.00");
                    content = df.format(doubleValue);*/
                    content = doubleValue.toString();
                    break;
                case STRING:    //字符串
                    content = cell.getStringCellValue();
                    break;
                case BOOLEAN:   //布尔
                    Boolean booleanValue = cell.getBooleanCellValue();
                    content = booleanValue.toString();
                    break;
                case BLANK:     // 空值
                    break;
                case FORMULA:   // 公式
                    //content = cell.getCellFormula();
                    //String str = String.valueOf(cell.getNumericCellValue());
                    //执行公式，此处cell的值就是公式
                    CellValue evaluate = TaxReadEnterpriseExcelUtil.formulaEvaluator.evaluate(cell);
                    Double doubleNumber = Double.parseDouble(evaluate.formatAsString());
                    DecimalFormat df1 = new DecimalFormat("###.####");
                    String formattedResult = df1.format(doubleNumber);
                    content = formattedResult;
                    break;
                case ERROR:     // 故障
                    break;
                default:
                    break;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return content;
    }


    /**
     * 读取 单元格
     * @param row 行数
     * @param cellNum 单元格的 num
     * @return
     */
    private static String readCell(Row row, int cellNum, String defaultValue) {
        Cell cell = row.getCell(cellNum);
        if (cell != null) {
            String content = convertCellValueToString(cell);
            //log.info("第" + row.getRowNum() + "行，第" + cellNum + "列，内容：" + content);
            //  处理数据
            return content;
        }

        //默认值
        if(StringUtils.isNotBlank(defaultValue)) {
            return defaultValue;
        }

        return null;
    }

}
