package com.hjy.framework.aspectj;

import cn.hutool.core.util.ObjectUtil;
import com.hjy.common.annotation.PortalAuth;
import com.hjy.common.constant.Constants;
import com.hjy.common.core.redis.RedisCache;
import com.hjy.common.utils.StringUtils;
import com.hjy.framework.web.service.TokenService;
import io.jsonwebtoken.Claims;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;
import org.springframework.util.ObjectUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;

@Aspect
@Component
@Slf4j
public class PortalAuthAspect {

    @Autowired
    private TokenService tokenService;

    @Autowired
    private RedisCache redisCache;


    /**
     * 定义切点Pointcut
     */
    @Pointcut("execution(public * com.hjy.*.portalAPI.*Controller.*(..))")
    public void excudeService() {
    }

    @Before("excudeService()")
    public void before(JoinPoint point){
        Signature signature = point.getSignature();
        MethodSignature methodSignature = (MethodSignature) signature;
        Method method = methodSignature.getMethod ();
        if (method != null) {
            if (ObjectUtil.isNull(method.getAnnotation(PortalAuth.class))){
                return;
            }
        }
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = requestAttributes.getRequest();
        String token = request.getHeader(Constants.PORTAL_TOKEN);
        if (StringUtils.isNull(token)){
            Assert.isTrue(false,"用户信息不存在，请登录！");
        }
        if (token.equals("null")){
            Assert.isTrue(false,"用户信息不存在，请登录！");
        }
        Claims claims = tokenService.parseToken(token);
        // 解析对应的权限以及用户信息
        String uuid = (String) claims.get(Constants.PORTAL_USER_KEY);
        Object companyUser = redisCache.getCacheObject(Constants.PORTAL_USER_KEY+uuid);
        if (ObjectUtils.isEmpty(companyUser)){
            Assert.isTrue(false,"登录信息已过期");
        }
        UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(companyUser, null, null);
        authenticationToken.setDetails(new WebAuthenticationDetailsSource().buildDetails(request));
        SecurityContextHolder.getContext().setAuthentication(authenticationToken);
    }


}
