package com.hjy.framework.handle;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpStatus;
import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import com.hjy.common.core.domain.BaseEntity;
import com.hjy.common.core.domain.model.LoginUser;
import com.hjy.common.exception.ServiceException;
import com.hjy.common.utils.SecurityUtils;
import com.hjy.common.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.ibatis.reflection.MetaObject;

import java.util.Date;

/**
 * MybatisPlus 字段注入
 *
 * @author Tellsea
 * @date 2022/9/1
 */
@Slf4j
public class CreateAndUpdateMetaObjectHandler implements MetaObjectHandler {

    @Override
    public void insertFill(MetaObject metaObject) {
        try {
            if (ObjectUtils.isNotEmpty(metaObject) && metaObject.getOriginalObject() instanceof BaseEntity) {
                BaseEntity baseEntity = (BaseEntity) metaObject.getOriginalObject();
                Date current = ObjectUtils.isNotEmpty(baseEntity.getCreateTime())
                        ? baseEntity.getCreateTime() : new Date();
                baseEntity.setCreateTime(current);
                baseEntity.setUpdateTime(current);
                String username = StringUtils.isNotBlank(baseEntity.getCreateBy())
                        ? baseEntity.getCreateBy() : getLoginUsername();
                // 当前已登录 且 创建人为空 则填充
                baseEntity.setCreateBy(username);
                // 当前已登录 且 更新人为空 则填充
                baseEntity.setUpdateBy(username);
//                for (Field fld : getAllFields(metaObject)){
//                    if (fld.getName().equals("delFlag")){
//                        this.setFieldValByName("delFlag","0",metaObject);
////                        fld.setAccessible(true);
////                        fld.set(metaObject, "0");
////                        fld.setAccessible(false);
//                    }
//                    if (fld.getName().equals("baId")){
//                        fld.setAccessible(true);
//                        fld.set(metaObject, getLoginUserbzId());
//                        fld.setAccessible(false);
//                    }
//                }

            }

			// 默认设置状态0
        } catch (Exception e) {
            throw new ServiceException("自动注入异常 => " + e.getMessage(), HttpStatus.HTTP_UNAUTHORIZED);
        }
    }

    @Override
    public void updateFill(MetaObject metaObject) {
        try {
            if (ObjectUtil.isNotNull(metaObject) && metaObject.getOriginalObject() instanceof BaseEntity) {
                BaseEntity baseEntity = (BaseEntity) metaObject.getOriginalObject();
                Date current = new Date();
                // 更新时间填充(不管为不为空)
                baseEntity.setUpdateTime(current);
                String username = getLoginUsername();
                // 当前已登录 更新人填充(不管为不为空)
                if (StringUtils.isNotBlank(username)) {
                    baseEntity.setUpdateBy(username);
                }
            }
        } catch (Exception e) {
            throw new ServiceException("自动注入异常 => " + e.getMessage(), HttpStatus.HTTP_UNAUTHORIZED);
        }
    }

    /**
     * 获取登录用户名
     */
    private String getLoginUsername() {
        String userName = null;
        try {
            LoginUser loginUser = SecurityUtils.getLoginUser();
            userName = loginUser.getUsername();
        } catch (Exception e) {
            try {
                userName = SecurityUtils.getPortalUser().getUserName();
            }catch (Exception exception){
                log.warn("自动注入警告 => 用户未登录");
                return null;
            }
        }
        return userName;
    }

}
