package com.hjy.module.aspect;

import cn.hutool.core.convert.Convert;
import cn.hutool.json.JSONException;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.hjy.common.annotation.ResVoDict;
import com.hjy.common.constant.Constants;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDictModel;
import com.hjy.common.utils.reflect.ReflectUtils;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.util.*;

@Aspect
@Component
@Slf4j
public class ResVoDictAspect {

    @Lazy
    @Autowired
    private DictFilterUtils dictFilterUtils;

    @Autowired
    public RedisTemplate redisTemplate;

    @Autowired
    private ObjectMapper objectMapper;

    /**
     * 定义切点Pointcut
     */
    @Pointcut("execution(public * com.hjy..*.*Controller.*(..))")
    public void excudeService() {
    }

    @Around("excudeService()")
    public Object doAround(ProceedingJoinPoint pjp) throws Throwable {
        //long time1=System.currentTimeMillis();
        Object result = pjp.proceed();
        //long time2=System.currentTimeMillis();
        //log.debug("获取JSON数据 耗时："+(time2-time1)+"ms");
        //long start=System.currentTimeMillis();
        try {
            result=this.parseDictText(result);
        }catch (JSONException e){
            log.error("json解析失败");
            return result;
        }

        //long end=System.currentTimeMillis();
        //log.debug("注入字典到JSON数据  耗时"+(end-start)+"ms");
        return result;
    }


    /**
     * 本方法针对返回对象为TableDataInfo 的分页列表数据进行动态字典注入
     * 字典注入实现 通过对实体类添加注解@PageDict 来标识需要的字典内容,字典分为单字典code即可 ，
     * 示例为SysUser   字段为sex 添加了注解@PageDict(dicCode = "sex") 会在字典服务立马查出来对应的text 然后在请求list的时候将这个字典text，已字段名称加_dictText形式返回到前端
     * 例输入当前返回值的就会多出一个sex_dictText字段
     * @param result
     */
    private Object parseDictText(Object result){
        if (result instanceof AjaxResult) {
                //step.1 筛选出加了 Dict 注解的字段列表
                List<Field> dictFieldList = new ArrayList<>();
                // 字典数据列表， key = 字典code，value=数据列表
                Map<String, List<String>> dataListMap = new HashMap<>(5);

                //取出结果集
                Object resultObj = ((AjaxResult) result).get("data");
                if (Objects.isNull(resultObj)) return result;
                //update-begin--Author:zyf -- Date:20220606 ----for：【VUEN-1230】 判断是否含有字典注解,没有注解返回-----
                String json = "{}";
                JSONObject item = null;
            try {
                    //解决@JsonFormat注解解析不了的问题详见SysAnnouncement类的@JsonFormat
                    json = objectMapper.writeValueAsString(resultObj);
                    item = JSONObject.parseObject(json);
                } catch (JSONException | JsonProcessingException | com.alibaba.fastjson2.JSONException e) {
                    log.error("json解析失败:" + e.getMessage());
                    return result;
                }


                // 遍历所有字段，把字典Code取出来，放到 map 里
                for (Field field : ReflectUtils.getAllFields(resultObj)) {
                    String value = item.getString(field.getName());
                    if (StringUtils.isEmpty(value)) {
                        continue;
                    }
                    if (field.getAnnotation(ResVoDict.class) != null) {
                        if (!dictFieldList.contains(field)) {
                            dictFieldList.add(field);
                        }
                        String code = field.getAnnotation(ResVoDict.class).dicCode();
                        String text = field.getAnnotation(ResVoDict.class).dicText();
                        String table = field.getAnnotation(ResVoDict.class).dictTable();

                        List<String> dataList;
                        String dictCode = code;
                        if (!org.springframework.util.StringUtils.isEmpty(table)) {
                            dictCode = String.format("%s,%s,%s", table, text, code);
                        }
                        dataList = dataListMap.computeIfAbsent(dictCode, k -> new ArrayList<>());
                        dictFilterUtils.listAddAllDeduplicate(dataList, Arrays.asList(value.split(",")));
                    }if(field.getType().getTypeName().equals(List.class.getTypeName())){ //判断是否为List

                        java.lang.reflect.Type gt = field.getGenericType(); //得到泛型类型
                        ParameterizedType pt = (ParameterizedType)gt;
                        Class<?> listClazz = (Class<?>)pt.getActualTypeArguments()[0];

                        List<Object> objects = (List<Object>) JSONUtil.parseArray(value);
                        List<Object> temp = new ArrayList<>();
                        for(Object obj : objects) {
                            if (StringUtils.isEmpty(obj)) {
                                continue;
                            }

                            for (Field itemfield : listClazz.getDeclaredFields()) {
                                if (itemfield.getAnnotation(ResVoDict.class) != null) {
                                    String code = itemfield.getAnnotation(ResVoDict.class).dicCode();
                                    String text = itemfield.getAnnotation(ResVoDict.class).dicText();
                                    String table = itemfield.getAnnotation(ResVoDict.class).dictTable();
                                    List<String> dataList;
                                    String dictCode = code;
                                    if (!org.springframework.util.StringUtils.isEmpty(table)) {
                                        dictCode = String.format("%s,%s,%s", table, text, code);
                                    }
                                    dataList = dataListMap.computeIfAbsent(dictCode, k -> new ArrayList<>());
                                    String itemfieStr = (String) JSONObject.parseObject(JSONObject.toJSONString(obj)).get(itemfield.getName());
                                    //String itemfieStr = ReflectUtils.getFieldValue(obj,itemfield.getName());
                                    dictFilterUtils.listAddAllDeduplicate(dataList, Arrays.asList(Convert.toStr(itemfieStr,"").split(",")));

                                    Map<String, List<SysDictModel>> translText = dictFilterUtils.translateAllDict(dataListMap);

                                    List<SysDictModel> dictModels = translText.get(dictCode);
                                    if (dictModels == null || dictModels.size() == 0) {
                                        continue;
                                    }
                                    if (Objects.nonNull(dictModels)){
                                        String textValue = dictModels.get(dictModels.size()-1).getText();
                                        JSONObject jsonobject = JSON.parseObject(JSON.toJSONString(obj));
                                        jsonobject.put(itemfield.getName() + Constants.DICT_TEXT_SUFFIX, textValue);
                                        obj = JSONObject.parseObject(JSON.toJSONString(jsonobject));
                                    }
                                }
                            }
                            temp.add(obj);
                        }
                        item.put(field.getName(),temp);

                    }
                }
                //step.2 调用翻译方法，一次性翻译
                Map<String, List<SysDictModel>> translText = dictFilterUtils.translateAllDict(dataListMap);

                //step.3 将翻译结果填充到返回结果里
                for (Field field : dictFieldList) {
                    String code = field.getAnnotation(ResVoDict.class).dicCode();
                    String text = field.getAnnotation(ResVoDict.class).dicText();
                    String table = field.getAnnotation(ResVoDict.class).dictTable();
                    String fieldDictCode = code;
                    if (!StringUtils.isEmpty(table)) {
                        fieldDictCode = String.format("%s,%s,%s", table, text, code);
                    }
                    String value = item.getString(field.getName());
                    if (Objects.nonNull(value)) {
                        List<SysDictModel> dictModels = translText.get(fieldDictCode);
                        if (dictModels == null || dictModels.size() == 0) {
                            continue;
                        }
                        String textValue = dictFilterUtils.translDictText(dictModels, value);
                        item.put(field.getName() + Constants.DICT_TEXT_SUFFIX, textValue);
                    }
                }
                ((AjaxResult) result).put("data",item);

        }
        return result;
    }

}
