package com.hjy.module.service.garden.impl;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.utils.PageUtils;
import com.hjy.module.domain.garden.CbGardenData;
import com.hjy.module.mapper.garden.CbGardenDataMapper;
import com.hjy.module.service.garden.ICbGardenDataService;
import com.hjy.module.vo.analysis.GardenAnalysisVo;
import com.hjy.module.vo.garden.CbGardenDataVo;
import com.hjy.system.mapper.SysDeptMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 企业园区大屏数据Service业务层处理
 *
 * @author Tellsea
 * @date 2023-10-11
 */
@Service
public class CbGardenDataServiceImpl extends ServiceImpl<CbGardenDataMapper, CbGardenData> implements ICbGardenDataService {

    SysDeptMapper sysDeptMapper;
    private CbGardenDataMapper gardenDataMapper;

    public CbGardenDataServiceImpl(CbGardenDataMapper gardenDataMapper) {
        this.gardenDataMapper = gardenDataMapper;
    }

    @Autowired
    private SysDeptMapper sysDeptMapper(SysDeptMapper sysDeptMapper) {
        return this.sysDeptMapper = sysDeptMapper;
    }

    @Override
    public TableDataInfo<CbGardenDataVo> queryList(CbGardenDataVo entity) {
        return PageUtils.buildDataInfo(this.baseMapper.queryList(PageUtils.buildPage(), entity));
    }

    @Override
    public List<CbGardenDataVo> queryAll(CbGardenDataVo entity) {
        return this.baseMapper.queryList(entity);
    }

    @Override
    public CbGardenDataVo queryById(String id) {
        return this.baseMapper.queryById(id);
    }

    /**
     * @param deptId 部门id
     * @return 返回构造好的数据对象 {@link Map<String, List<Object>>}
     */
    @Override
    public GardenAnalysisVo queryInoutYearTotalValueByDeptId(String deptId) {
        // 一层 key 是 inout 进出口 operate 经营

        // 进出口历年
        HashMap<String, List<Object>> inout = getInout(deptId);
        // 获取经营历年
        HashMap<String, List<Object>> operate = getOperate(deptId);

        return GardenAnalysisVo.builder().Inout(inout).operate(operate).build();
    }

    public HashMap<String, List<Object>> getOperate(String deptId) {
        // 经营历年
        List<Map<String, Object>> operateMap = gardenDataMapper.selectYearTotalValueByDeptIdOne(deptId);
        return buildAnalysis(operateMap);
    }

    public HashMap<String, List<Object>> getInout(String deptId) {
        // 进出口历年
        List<Map<String, Object>> inoutMap = gardenDataMapper.selectYearTotalValueByDeptIdTwo(deptId);

        return buildAnalysis(inoutMap);
    }

    /**
     * 统一方法；将 获取出来的数据转换为前端对应的数据
     */
    private HashMap<String, List<Object>> buildAnalysis(List<Map<String, Object>> map) {
        List<Object> x = new ArrayList<>(); // 存储 x 轴数据
        List<Object> val = new ArrayList<>(); // 存储 x 轴对应的value
        map.forEach(item -> {
            String year = (String) item.get("year");
            Object value = item.get("value");
            x.add(year);
            val.add(value);
        });
        HashMap<String, List<Object>> resultMap = new HashMap<>();
        resultMap.put("x", x);
        resultMap.put("value", val);
        return resultMap;
    }
}
