package com.hjy.module.service.garden.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.utils.PageUtils;
import com.hjy.module.domain.garden.TbBondedScreenData;
import com.hjy.module.mapper.garden.TbBondedScreenDataMapper;
import com.hjy.module.service.garden.ITbBondedScreenDataService;
import com.hjy.module.vo.garden.TbBondedScreenDataVo;
import com.hjy.system.service.ISysDeptService;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;


/**
 * 保税区大屏所需数据Service业务层处理
 *
 * @author Tellsea
 * @date 2023-09-28
 */
@Service
public class TbBondedScreenDataServiceImpl extends ServiceImpl<TbBondedScreenDataMapper, TbBondedScreenData> implements ITbBondedScreenDataService {

    ISysDeptService sysDeptService;
    @Autowired
    private ISysDeptService sysDeptService(ISysDeptService sysDeptService){
        return this.sysDeptService = sysDeptService;
    }

    @Override
    public TableDataInfo<TbBondedScreenDataVo> queryList(TbBondedScreenDataVo entity) {
        return PageUtils.buildDataInfo(this.baseMapper.queryList(PageUtils.buildPage(), entity));
    }

    @Override
    public List<TbBondedScreenDataVo> queryAll(TbBondedScreenDataVo entity) {
        return this.baseMapper.queryList(entity);
    }

    @Override
    public TbBondedScreenDataVo queryById(String id) {
        return this.baseMapper.queryById(id);
    }

    @Override
    public String importData(MultipartFile file) throws IOException {

        System.err.println("\n开始执行文件上传....\n");

        //判空
        if(file.isEmpty()) return "文件为空,无法执行上传...";
        //获取文件上传数据
        XSSFWorkbook wb = new XSSFWorkbook(file.getInputStream());
        //获取第一页sheet
        XSSFSheet sheet = wb.getSheetAt(0);
        //定义计数器
        int count = 0;
        //定义行对象
        XSSFRow row = null;
        //解析数据封装到集合
        //创建接收集合
        List<TbBondedScreenData> list = new ArrayList<>();
        //循环遍历sheet页面(默认只有一页)
        for (int i = 1; i < sheet.getPhysicalNumberOfRows(); i++) {
            //封装数据到行对象
            row = sheet.getRow(i);
            //定义返回值实体
            TbBondedScreenData tbBondedScreenData = new TbBondedScreenData();

            //保税区ID  deptId
            if(row.getCell(1) != null && !row.getCell(1).equals("")){
                String deptName = row.getCell(1).getStringCellValue();
                List<SysDept> deptList = sysDeptService.selectDeptList(null).stream().filter(item -> item.getDeptName().contains(deptName)).collect(Collectors.toList());
                if(!deptList.isEmpty()){
                    tbBondedScreenData.setDeptId(String.valueOf(deptList.get(0).getDeptId()));
                }
            }

            //type  设施类型[1=标仓,2=冷库,3=油房,4=厂房]
            if(row.getCell(2) != null && !row.getCell(2).equals("")){
                String deptId = row.getCell(2).getStringCellValue();
                tbBondedScreenData.setType(deptId.equals("标仓") ? 1 + "" : deptId.equals("冷库") ? 2 + "" : deptId.equals("油房") ? 3 + "" : deptId.equals("厂房") ? 4 + "" : 0 + "");
            }
            //total_area  设施面积
            if(row.getCell(3) != null && !row.getCell(3).equals("")){
                Double totalArea = row.getCell(3).getNumericCellValue();
                tbBondedScreenData.setTotalArea(String.valueOf(totalArea));
            }
            //use_area  已用面积
            if(row.getCell(4) != null && !row.getCell(4).equals("")){
                Double useArea = row.getCell(4).getNumericCellValue();
                tbBondedScreenData.setUseArea(String.valueOf(useArea));
            }

            //创建时间
            tbBondedScreenData.setCreateTime(new Date());

            //添加到集合
            list.add(tbBondedScreenData);
        }

        System.err.println("待处理数据量为: " + list.size() + " 条。");

        int processors = 1;
        if(list.size() > 30){
            //获取当前电脑线程数,使用一半线程
            processors = Runtime.getRuntime().availableProcessors() / 2;
        }
        //创建多线程
        ExecutorService executorService = Executors.newFixedThreadPool(processors);

        try{
            //每次30条
            int bai = 30;
            //计算需要创建的线程数
            int threads = (int) Math.ceil((double) list.size() / bai);
            for (int i = 0; i < threads; i++) {
                int start = i * bai;
                int end = Math.min(start + bai,list.size());

                //截取集合数量
                List<TbBondedScreenData> tbBondedScreenData = list.subList(start, end);

                //多线程执行任务
                executorService.execute(() -> {
                    saveBatch(tbBondedScreenData);
                });
                //计数
                count += tbBondedScreenData.size();
            }
        }finally {
            executorService.shutdown();
            wb.close();
            if(wb != null){
                wb.close();
            }
        }
        System.err.println("\n文件上传完成,共上传 " + count + "条 " + "数据...\n");
        return "上传完成,共上传"  + count + "条"  + "数据。";
    }
}
