package com.hjy.module.service.garden.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.utils.PageUtils;
import com.hjy.module.domain.garden.TbBondedUpply;
import com.hjy.module.mapper.garden.TbBondedUpplyMapper;
import com.hjy.module.service.garden.ITbBondedUpplyService;
import com.hjy.module.vo.garden.TbBondedUpplyVo;
import com.hjy.system.service.ISysDeptService;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;

/**
 * 保税区供应链数据Service业务层处理
 *
 * @author Tellsea
 * @date 2023-09-28
 */
@Service
public class TbBondedUpplyServiceImpl extends ServiceImpl<TbBondedUpplyMapper, TbBondedUpply> implements ITbBondedUpplyService {

    ISysDeptService sysDeptService;
    @Autowired
    private ISysDeptService sysDeptService(ISysDeptService sysDeptService){
        return this.sysDeptService = sysDeptService;
    }

    @Override
    public TableDataInfo<TbBondedUpplyVo> queryList(TbBondedUpplyVo entity) {
        return PageUtils.buildDataInfo(this.baseMapper.queryList(PageUtils.buildPage(), entity));
    }

    @Override
    public List<TbBondedUpplyVo> queryAll(TbBondedUpplyVo entity) {
        return this.baseMapper.queryList(entity);
    }

    @Override
    public TbBondedUpplyVo queryById(String lddwmc) {
        return this.baseMapper.queryById(lddwmc);
    }

    @Override
    public String importData(MultipartFile file) throws IOException {

        System.err.println("\n开始执行文件上传....\n");

        //判空
        if(file.isEmpty()) return "文件为空,无法执行上传...";
        //获取文件上传数据
        XSSFWorkbook wb = new XSSFWorkbook(file.getInputStream());
        //获取第一页sheet
        XSSFSheet sheet = wb.getSheetAt(0);
        //定义计数器
        int count = 0;
        //定义行对象
        XSSFRow row = null;
        //解析数据封装到集合
        //创建接受集合
        List<TbBondedUpply> list = new ArrayList<>();
        //循环遍历sheet页面(默认只有一页)
        for (int i = 2; i < sheet.getPhysicalNumberOfRows(); i++) {
            //封装数据到行对象
            row = sheet.getRow(i);
            //定义返回值实体
            TbBondedUpply tbBondedUpply = new TbBondedUpply();

            //保税区ID  deptId
            if(row.getCell(1) != null && !row.getCell(1).equals("")){
                String deptId = row.getCell(1).getStringCellValue();
                tbBondedUpply.setDeptId(String.valueOf(sysDeptService.selectDeptList(null).stream().filter(item -> item.getDeptName().contains(deptId)).collect(Collectors.toList()).get(0).getDeptId()));
            }

            //bgdbztj  报关单币种统计
            if(row.getCell(2) != null && !row.getCell(2).equals("")){
                String deptId = row.getCell(2).getStringCellValue();
                tbBondedUpply.setBgdbztj(deptId);
            }
            //sbdwmc  申报单位名称
            if(row.getCell(3) != null && !row.getCell(3).equals("")){
                String deptId = row.getCell(3).getStringCellValue();
                tbBondedUpply.setSbdwmc(deptId);
            }
            //lddwbmmc  录单单位编码名称
            if(row.getCell(4) != null && !row.getCell(4).equals("")){
                String deptId = row.getCell(4).getStringCellValue();
                tbBondedUpply.setLddwbmmc(deptId);
            }
            //lddwmc  录单单位名称
            if(row.getCell(5) != null && !row.getCell(5).equals("")){
                String deptId = row.getCell(5).getStringCellValue();
                tbBondedUpply.setLddwmc(deptId);
            }
            //jckbz  进出口标志
            if(row.getCell(6) != null && !row.getCell(6).equals("")){
                String deptId = row.getCell(6).getStringCellValue();
                tbBondedUpply.setJckbz(deptId);
            }
            //jckrq  进出口日期
            if(row.getCell(7) != null && !row.getCell(7).equals("")){
                String deptId = row.getCell(7).getStringCellValue();
                tbBondedUpply.setJckrq(deptId);
            }
            //jz  净重
            if(row.getCell(8) != null && !row.getCell(8).equals("")){
                String deptId = row.getCell(8).getStringCellValue();
                tbBondedUpply.setJz(deptId);
            }
            //jingzhi  净值
            if(row.getCell(9) != null && !row.getCell(9).equals("")){
                String deptId = row.getCell(9).getStringCellValue();
                tbBondedUpply.setJingzhi(deptId);
            }
            //js  件数
            if(row.getCell(10) != null && !row.getCell(10).equals("")){
                String deptId = row.getCell(10).getStringCellValue();
                tbBondedUpply.setJs(deptId);
            }
            //jnsfhrbh  境内收发货人编号
            if(row.getCell(11) != null && !row.getCell(11).equals("")){
                String deptId = row.getCell(11).getStringCellValue();
                tbBondedUpply.setJnsfhrbh(deptId);
            }
            //jnsfhrmc  境内收发货人名称
            if(row.getCell(12) != null && !row.getCell(12).equals("")){
                String deptId = row.getCell(12).getStringCellValue();
                tbBondedUpply.setJnsfhrmc(deptId);
            }
            //jwfhrdm  境外发货人代码
            if(row.getCell(13) != null && !row.getCell(13).equals("")){
                String deptId = row.getCell(13).getStringCellValue();
                tbBondedUpply.setJwfhrdm(deptId);
            }
            //jwsfhrmc  境外收发货人名称
            if(row.getCell(14) != null && !row.getCell(14).equals("")){
                String deptId = row.getCell(14).getStringCellValue();
                tbBondedUpply.setJwsfhrmc(deptId);
            }
            //jwsfhrdz  境外收发货人地址
            if(row.getCell(15) != null && !row.getCell(15).equals("")){
                String deptId = row.getCell(15).getStringCellValue();
                tbBondedUpply.setJwsfhrdz(deptId);
            }
            //qyrq  启运日期
            if(row.getCell(16) != null && !row.getCell(16).equals("")){
                String deptId = row.getCell(16).getStringCellValue();
                tbBondedUpply.setQyrq(deptId);
            }
            //xbrq  卸毕日期
            if(row.getCell(17) != null && !row.getCell(17).equals("")){
                String deptId = row.getCell(17).getStringCellValue();
                tbBondedUpply.setXbrq(deptId);
            }
            //创建时间
            tbBondedUpply.setCreateTime(new Date());

            //添加到集合
            list.add(tbBondedUpply);
        }

        System.err.println("待处理数据量为: " + list.size() + " 条。");

        int processors = 1;
        if(list.size() > 30){
            //获取当前电脑线程数,使用一半线程
            processors = Runtime.getRuntime().availableProcessors() / 2;
        }
        //创建多线程
        ExecutorService executorService = Executors.newFixedThreadPool(processors);

        try{
            //每次30条
            int bai = 30;
            //计算需要创建的线程数
            int threads = (int) Math.ceil((double) list.size() / bai);
            for (int i = 0; i < threads; i++) {
                int start = i * bai;
                int end = Math.min(start + bai,list.size());

                //截取集合数量
                List<TbBondedUpply> tbEconomicsIndexList = list.subList(start, end);

                //多线程执行任务
                executorService.submit(() -> {
                    this.saveBatch(tbEconomicsIndexList);
                });
                //计数
                count += tbEconomicsIndexList.size();
            }
        }finally {
            executorService.shutdown();
            wb.close();
            if(wb != null){
                wb.close();
            }
        }
        System.err.println("\n文件上传完成,共上传 " + count + "条 " + "数据...\n");
        return "上传完成,共上传"  + count + "条"  + "数据。";
    }
}
