package com.hjy.module.service.garden.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.utils.PageUtils;
import com.hjy.module.domain.garden.TbEconomicsIndex;
import com.hjy.module.mapper.garden.TbEconomicsIndexMapper;
import com.hjy.module.service.garden.ITbEconomicsIndexService;
import com.hjy.module.vo.garden.TbEconomicsIndexVo;
import com.hjy.system.service.ISysDeptService;
import org.apache.commons.lang3.StringUtils;

import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.swing.filechooser.FileSystemView;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;

import static org.apache.poi.ss.usermodel.CellType.NUMERIC;
import static org.apache.poi.ss.usermodel.CellType.STRING;

/**
 * 综合保税区经济运行主要指标Service业务层处理
 *
 * @author Mechrevo
 * @date 2023-09-27
 */
@Service
public class TbEconomicsIndexServiceImpl extends ServiceImpl<TbEconomicsIndexMapper, TbEconomicsIndex> implements ITbEconomicsIndexService {

    ISysDeptService sysDeptService;
    @Autowired
    private ISysDeptService sysDeptService(ISysDeptService sysDeptService){
        return this.sysDeptService = sysDeptService;
    }
    @Autowired
    TbEconomicsIndexMapper tbEconomicsIndexMapper;

    @Override
    public TableDataInfo<TbEconomicsIndexVo> queryList(TbEconomicsIndexVo entity) {
        return PageUtils.buildDataInfo(this.baseMapper.queryList(PageUtils.buildPage(), entity));
    }

    @Override
    public List<TbEconomicsIndexVo> queryAll(TbEconomicsIndexVo entity) {
        return this.baseMapper.queryList(entity);
    }

    @Override
    public TbEconomicsIndexVo queryById(String id) {
        return this.baseMapper.queryById(id);
    }

    @Override
    public String importData(MultipartFile file) throws IOException {
        System.err.println("\n开始执行文件上传....\n");

        //判空
        if(file.isEmpty()) return "文件为空,无法执行上传...";
        //获取文件上传数据
        XSSFWorkbook wb = new XSSFWorkbook(file.getInputStream());
        //获取第一页sheet
        XSSFSheet sheet = wb.getSheetAt(0);
        //定义计数器
        int count = 0;
        //定义行对象
        XSSFRow row = null;
        //解析数据封装到集合
        //创建接受集合
        List<TbEconomicsIndex> list = new ArrayList<>();
        //循环遍历sheet页面(默认只有一页)
        for (int i = 2; i < sheet.getPhysicalNumberOfRows(); i++) {
            //封装数据到行对象
            row = sheet.getRow(i);

            //定义返回值实体
            TbEconomicsIndex tbEconomicsIndex = new TbEconomicsIndex();
            //保税区ID  deptId
            if(row.getCell(5) != null && !row.getCell(5).equals("")){
                String deptId = row.getCell(5).getStringCellValue();
                tbEconomicsIndex.setDeptId(String.valueOf(sysDeptService.selectDeptList(null).stream().filter(item -> item.getDeptName().contains(deptId)).collect(Collectors.toList()).get(0).getDeptId()));
            }
            //
            //添加到集合
            list.add(tbEconomicsIndex);
        }

        System.err.println("待处理数据量为: " + list.size() + " 条。");

        int processors = 1;
        if(list.size() > 30){
            //获取当前电脑线程数,使用一半线程
            processors = Runtime.getRuntime().availableProcessors() / 2;
        }
        //创建多线程
        ExecutorService executorService = Executors.newFixedThreadPool(processors);

        try{
            //每次30条
            int bai = 30;
            //计算需要创建的线程数
            int threads = (int) Math.ceil((double) list.size() / bai);
            for (int i = 0; i < threads; i++) {
                int start = i * bai;
                int end = Math.min(start + bai,list.size());

                //截取集合数量
                List<TbEconomicsIndex> tbEconomicsIndexList = list.subList(start, end);

                //多线程执行任务
                executorService.submit(() -> {
                    this.saveBatch(tbEconomicsIndexList);
                });
                //计数
                count += tbEconomicsIndexList.size();
            }
        }finally {
            executorService.shutdown();
            wb.close();
            if(wb != null){
                wb.close();
            }
        }
        System.err.println("\n文件上传完成,共上传 " + count + "条 " + "数据...\n");
        return "上传完成,共上传"  + count + "条"  + "数据。";
    }

    @Override
    public void rowChangeLine(MultipartFile file) throws Exception {
        if (file.isEmpty()) return;

        try (HSSFWorkbook workbook = new HSSFWorkbook(file.getInputStream())) {
            HSSFSheet sheet = workbook.getSheetAt(0);
            Sheet transposedSheet = workbook.createSheet("行列互换");

            int rowCount = sheet.getLastRowNum() + 1;
            int colCount = sheet.getRow(0).getLastCellNum();

            for (int r = 0; r < rowCount; r++) {
                Row row = sheet.getRow(r);
                for (int c = 0; c < colCount; c++) {
                    Cell cell = row.getCell(c);
                    if (cell != null) {
                        Row transposedRow = transposedSheet.getRow(c);
                        if (transposedRow == null) {
                            transposedRow = transposedSheet.createRow(c);
                        }
                        Cell transposedCell = transposedRow.createCell(r);

                        switch (cell.getCellType()) {
                            case STRING:
                                transposedCell.setCellValue(cell.getStringCellValue());
                                break;
                            case NUMERIC:
                                transposedCell.setCellValue(cell.getNumericCellValue());
                                break;
                            case BOOLEAN:
                                transposedCell.setCellValue(cell.getBooleanCellValue());
                                break;
                            case BLANK:
                                transposedCell.setCellValue("");
                                break;
                            default:
                                //将excel中的运算获得的单元格处理成运算后的值
                                FormulaEvaluator evaluator = workbook.getCreationHelper().createFormulaEvaluator();
                                CellValue cellValue1 = evaluator.evaluate(cell);
                                if (cellValue1.getCellType() == CellType.NUMERIC) {
                                    transposedCell.setCellValue(cellValue1.getNumberValue());
                                }
                        }
                    }
                }
            }
            //将处理后的文件保存到桌面
            try (FileOutputStream outputStream = new FileOutputStream(FileSystemView.getFileSystemView().getHomeDirectory().getPath() + "\\综合保税区.xlsx")) {
                workbook.write(outputStream);
                List<TbEconomicsIndex> tbEconomicsIndexList = readTheExcel(transposedSheet);
                for (int i = 0; i < tbEconomicsIndexList.size(); i++) {
                    if(i % 3 == 0){

                    }
                }
                int size = tbEconomicsIndexList.size();
                this.saveBatch(tbEconomicsIndexList);
            }finally {
                if(workbook != null)
                    workbook.close();
            }
        }
    }

    @Override
    public List<TbEconomicsIndex> readLine(MultipartFile file) throws IOException {

        HSSFWorkbook wb = new HSSFWorkbook(file.getInputStream());

        Sheet sheet = wb.getSheetAt(0);

        int firstRowNum = 2;

        int lastRowNum = sheet.getLastRowNum() - 1;

        Row row = null;

        Cell cell_a = null;

        String deptId = null;

        //循环获取列数
        Row row1 = sheet.getRow(5);
        int columnCount = 0;
        for (int i = 0; i < row1.getLastCellNum(); i++) {
            Cell cell = row1.getCell(i);
            if (cell != null) {
                columnCount++;
            }
        }

        int start = 5;

        List<TbEconomicsIndex> result = new ArrayList<>();

        List<SysDept> sysDepts = sysDeptService.selectDeptList(null);

        for (int k = 0; k < columnCount - 5; k++) {

            TbEconomicsIndexVo tbEconomicsIndex = new TbEconomicsIndexVo();

            for (int i = firstRowNum; i <= lastRowNum; i++) {

                if(i == 4 || i == 5)continue;

                row = sheet.getRow(i); //取得第i行

                cell_a = row.getCell(start); //取得i行的第一列

                Object cellValue = null;
                DecimalFormat decimalFormat = new DecimalFormat("0.#########");
                if(cell_a == null)
                    continue;
                switch (cell_a.getCellType()) {
                    case STRING:
                        cellValue = cell_a.getStringCellValue();
                        break;
                    case NUMERIC:
                        cellValue = cell_a.getNumericCellValue();
                        cellValue = decimalFormat.format(cellValue);
                        break;
                    case BOOLEAN:
                        cellValue = cell_a.getBooleanCellValue();
                        cellValue = decimalFormat.format(cellValue);
                        break;
                    case BLANK:
                        break;
                    default:
                        //将excel中的运算获得的单元格处理成运算后的值
                        FormulaEvaluator evaluator = wb.getCreationHelper().createFormulaEvaluator();
                        CellValue cellValue1 = evaluator.evaluate(cell_a);
                        if (cellValue1.getCellType() == CellType.NUMERIC) {
                            cellValue = cellValue1.getNumberValue();
                        }

                }
                System.err.println(cellValue);

//                if(cellValue == null || cellValue.equals(""))continue;
                switch (i){
                    case 2:
                        //deptId  保税区名称
                        if(cellValue != null && !String.valueOf(cellValue).trim().equals("")){
                            deptId = String.valueOf(cellValue);
                        }
                        if(deptId != null && !deptId.trim().equals("")){
                            String finalDeptId = deptId;
                            tbEconomicsIndex.setDeptId(String.valueOf(sysDepts.stream().filter(a -> a.getDeptName().contains(finalDeptId)).collect(Collectors.toList()).get(0).getDeptId()));
                        }
                        break;
                    case 3:
                        //statementtime 录入时间
                        if(cell_a.getCellType() == STRING) {
                            tbEconomicsIndex.setStatementTime(String.valueOf(cellValue));
                        }else {
                            Date setupTime = DateUtil.getJavaDate(Double.valueOf(String.valueOf(cellValue)));
                            tbEconomicsIndex.setStatementTime(new SimpleDateFormat("yyyy-MM-dd").format(setupTime));
                        }
                        break;
                    case 6:
                        //jckzz 进出口总值
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setJckzz(new Double(String.valueOf(cellValue)));
                        break;
                    case 7:
                        //jgmyjckz 加工贸易进出口值
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setJgmyjckz(new Double(String.valueOf(cellValue)));
                        break;
                    case 8:
                        //wlhwjckz 物流货物进出口值
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setWlhwjckz(new Double(String.valueOf(cellValue)));
                        break;
                    case 9:
                        //ybmyjckz 一般贸易进出口值
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setYbmyjckz(new Double(String.valueOf(cellValue)));
                        break;
                    case 10:
                        //jyzsr 经营总收入
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setJyzsr(new Double(String.valueOf(cellValue)));
                        break;
                    case 11:
                        //hgss 海关税收
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setHgss(new Double(String.valueOf(cellValue)));
                        break;
                    case 12:
                        //swbmss 税务部门税收
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setSwbmss(new Double(String.valueOf(cellValue)));
                        break;
                    case 13:
                        //hwmyswszze 货物贸易涉外收支总额
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setHwmyswszze(new Double(String.valueOf(cellValue)));
                        break;
                    case 14:
                        //qmqycyry 期末企业从业人员
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setQmqycyry(new Double(String.valueOf(cellValue)));
                        break;
                    case 15:
                        //hyqys 活跃企业数
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setHyqys(new Double(String.valueOf(cellValue)));
                        break;
                    case 16:
                        //dwmjjckz 单位面积进出口值
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setDwmjjckz(new Double(String.valueOf(cellValue)));
                        break;
                    case 17:
                        //dwmjjyzsr 单位面积经营总收入
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setDwmjjyzsr(new Double(String.valueOf(cellValue)));
                        break;
                    case 18:
                        //dwmjhgss 单位面积海关税收
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setDwmjhgss(new Double(String.valueOf(cellValue)));
                        break;
                    case 19:
                        //dwmjswbmss单位面积税务部门税收
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setDwmjswbmss(new Double(String.valueOf(cellValue)));
                        break;
                    case 20:
                        //dwmjhwmyswsze单位面积货物贸易涉外收支额
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setDwmjhwmyswsze(new Double(String.valueOf(cellValue)));
                        break;
                    case 21:
                        //qmdwmjqycyry期末单位面积企业从业人员
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setQmdwmjqycyry(new Double(String.valueOf(cellValue)));
                        break;
                    case 22:
                        //jckzzzf进出口总值增幅
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setJckzzzf(new Double(String.valueOf(cellValue)));
                        break;
                    case 23:
                        //tqhyqyszf同期活跃企业数增幅
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setTqhyqyszf(new Double(String.valueOf(cellValue)));
                        break;
                    case 24:
                        //qmpzghmjysl期末批准规划面积验收率
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setQmpzghmjysl(new Double(String.valueOf(cellValue)));
                        break;
                    case 25:
                        //qmfbwwmjkfl期末封闭围网面积开发率
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setQmfbwwmjkfl(new Double(String.valueOf(cellValue)));
                        break;
                    case 26:
                        //sjsywzje实际使用外资金额
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setSjsywzje(new Double(String.valueOf(cellValue)));
                        break;
                    case 27:
                        //jcqhwzz进出区货物总值（二线业务）
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setJcqhwzz(new Double(String.valueOf(cellValue)));
                        break;
                    case 28:
                        //jckzzzqszstqjckbz进出口总值占其所在省（区、市）同期进出口比重
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setJckzzzqszstqjckbz(new Double(String.valueOf(cellValue)));
                        break;
                    case 29:
                        //wxywjckz维修业务进出口值
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setWxywjckz(new Double(String.valueOf(cellValue)));
                        break;
                    case 30:
                        //yfywjckz研发业务进出口值
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setYfywjckz(new Double(String.valueOf(cellValue)));
                        break;
                    case 31:
                        //zlmyjckz租赁贸易进出口值
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setZlmyjckz(new Double(String.valueOf(cellValue)));
                        break;
                    case 32:
                        //kjdsywjckz 跨境电商业务进出口值
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setKjdsywjckz(new Double(String.valueOf(cellValue)));
                        break;
                    case 33:
                        //kmysgycz 规模以上工业产值
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setGmysgycz(new Double(String.valueOf(cellValue)));
                        break;
                    case 34:
                        //gdzctz 固定资产投资
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setGdtzcz(new Double(String.valueOf(cellValue)));
                        break;
                    case 35:
                        //ztqszsjckzbz 占同期所在市进出口值比重
                        if(cell_a.getCellType() != STRING && cellValue != null && StringUtils.isNotBlank(String.valueOf(cellValue)))
                            tbEconomicsIndex.setZtqszsjckzbz(new Double(String.valueOf(cellValue)));
                        break;
                }

            }
            result.add(tbEconomicsIndex);
            int size = result.size();
            if(size % 3 == 0){
                result = disposeList(result);
            }
            start++;
        }
//        result.forEach(item -> System.err.println(item.toString()));

        saveBatch(result);
        return result;
    }



    private List<TbEconomicsIndex> readTheExcel(Sheet transposedSheet){
        //定义行对象
        Row row = null;
        int start = 5;
        int end = 8;
        String deptIdName = null;
        boolean flag = false;
        List<TbEconomicsIndex> result = new ArrayList<>();
        //遍历处理信息
        for (int i = start; i < transposedSheet.getPhysicalNumberOfRows(); i++) {
            TbEconomicsIndex tbEconomicsIndex = new TbEconomicsIndex();
            //开始解析数据
            row = transposedSheet.getRow(i);

            //deptId  保税区名称
            if(row.getCell(2) != null && !row.getCell(2).toString().trim().equals("")){
                String stringCellValue = row.getCell(2).getStringCellValue();
                if(deptIdName == null){
                    tbEconomicsIndex.setDeptId(String.valueOf(sysDeptService.selectDeptList(null).stream().filter(item -> item.getDeptName().contains(stringCellValue)).collect(Collectors.toList()).get(0).getDeptId()));
                    deptIdName = stringCellValue;
                }
            }
            //statement_time 时间范围
            if(row.getCell(3) != null && !row.getCell(3).toString().trim().equals("")){
                if(end - start == 1){
                    String stringCellValue = row.getCell(3).getStringCellValue();
                    tbEconomicsIndex.setStatementTime(stringCellValue);
                    flag = true;
                }else {
                    Double stringCellValue = row.getCell(3).getNumericCellValue();
                    Date setupTime = DateUtil.getJavaDate(Double.valueOf(stringCellValue));
                    tbEconomicsIndex.setStatementTime(new SimpleDateFormat("yyyy-MM-dd").format(setupTime));
                }

            }

            //jckzz 进出口总值
            Cell cell3 = row.getCell(6);
            if (cell3 != null && (cell3.getCellType() == NUMERIC || cell3.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setJckzz(cell3.getNumericCellValue());
            //jgmyjckz 加工贸易进出口值
            Cell cell4 = row.getCell(7);
            if (cell4 != null && (cell4.getCellType() == NUMERIC || cell4.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setJgmyjckz(cell4.getNumericCellValue());
            //wlhwjckz 物流货物进出口值
            Cell cell5 = row.getCell(8);
            if (cell5 != null && (cell5.getCellType() == NUMERIC || cell5.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setWlhwjckz(cell5.getNumericCellValue());
            //ybmyjckz 一般贸易进出口值
            Cell cell6 = row.getCell(9);
            if (cell6 != null && (cell6.getCellType() == NUMERIC || cell6.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setYbmyjckz(cell6.getNumericCellValue());
            //jyzsr 经营总收入
            Cell cell7 = row.getCell(10);
            if (cell7 != null && (cell7.getCellType() == NUMERIC || cell7.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setJyzsr(cell7.getNumericCellValue());
            //hgss 海关税收
            Cell cell8 = row.getCell(11);
            if (cell8 != null && (cell8.getCellType() == NUMERIC || cell8.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setHgss(cell8.getNumericCellValue());
            //swbmss 税务部门税收
            Cell cell9 = row.getCell(12);
            if (cell9 != null && (cell9.getCellType() == NUMERIC || cell9.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setSwbmss(cell9.getNumericCellValue());
            //hwmyswszze 货物贸易涉外收支总额
            Cell cell10 = row.getCell(13);
            if (cell10 != null && (cell10.getCellType() == NUMERIC || cell10.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setHwmyswszze(cell10.getNumericCellValue());
            //qmqycyry 期末企业从业人员
            Cell cell11 = row.getCell(14);
            if (cell11 != null && (cell11.getCellType() == NUMERIC || cell11.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setQmqycyry( cell11.getNumericCellValue());
            //hyqys 活跃企业数
            Cell cell12 = row.getCell(15);
            if (cell12 != null && (cell12.getCellType() == NUMERIC || cell12.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setHyqys(cell12.getNumericCellValue());
            //dwmjjckz 单位面积进出口值
            Cell cell13 = row.getCell(16);
            if (cell13 != null && (cell13.getCellType() == NUMERIC || cell13.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setDwmjjckz(cell13.getNumericCellValue());
            //dwmjjyzsr 单位面积经营总收入
            Cell cell14 = row.getCell(17);
            if (cell14 != null && (cell14.getCellType() == NUMERIC || cell14.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setDwmjjyzsr(cell14.getNumericCellValue());
            //dwmjhgss 单位面积海关税收
            Cell cell15 = row.getCell(18);
            if (cell15 != null && (cell15.getCellType() == NUMERIC || cell15.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setDwmjhgss(cell15.getNumericCellValue());
            //dwmjswbmss 单位面积税务部门税收
            Cell cell16 = row.getCell(19);
            if (cell16 != null && (cell16.getCellType() == NUMERIC || cell16.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setDwmjswbmss(cell16.getNumericCellValue());
            //dwmjhwmyswsze 单位面积货物贸易涉外收支额
            Cell cell17 = row.getCell(20);
            if (cell17 != null && (cell17.getCellType() == NUMERIC || cell17.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setDwmjhwmyswsze(cell17.getNumericCellValue());
            //qmdwmjqycyry 期末单位面积企业从业人员
            Cell cell18 = row.getCell(21);
            if (cell18 != null && (cell18.getCellType() == NUMERIC || cell18.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setQmdwmjqycyry(cell18.getNumericCellValue());
            //jckzzzf 进出口总值增幅
            Cell cell19 = row.getCell(22);
            if (cell19 != null && (cell19.getCellType() == NUMERIC || cell19.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setJckzzzf(cell19.getNumericCellValue());
            //tqhyqyszf 同期活跃企业数增幅
            Cell cell20 = row.getCell(23);
            if (cell20 != null && (cell20.getCellType() == NUMERIC || cell20.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setTqhyqyszf(cell20.getNumericCellValue());
            //qmpzghmjysl 期末批准规划面积验收率
            Cell cell21 = row.getCell(24);
            if (cell21 != null && (cell21.getCellType() == NUMERIC || cell21.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setQmpzghmjysl(cell21.getNumericCellValue());
            //qmfbwwmjkfl 期末封闭围网面积开发率
            Cell cell22 = row.getCell(25);
            if (cell22 != null && (cell22.getCellType() == NUMERIC || cell22.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setQmfbwwmjkfl(cell22.getNumericCellValue());
            //sjsywzje 实际使用外资金额
            Cell cell23 = row.getCell(26);
            if (cell23 != null && (cell23.getCellType() == NUMERIC || cell23.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setSjsywzje(cell23.getNumericCellValue());
            //jcqhwzz 进出区货物总值（二线业务）
            Cell cell24 = row.getCell(27);
            if (cell24 != null && (cell24.getCellType() == NUMERIC || cell24.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setJcqhwzz(cell24.getNumericCellValue());
            //jckzzzqszstqjckbz 进出口总值占其所在省（区、市）同期进出口比重
            Cell cell25 = row.getCell(28);
            if (cell25 != null && (cell25.getCellType() == NUMERIC || cell25.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setJckzzzqszstqjckbz(cell25.getNumericCellValue());
            //wxywjckz 维修业务进出口值
            Cell cell26 = row.getCell(29);
            if (cell26 != null && (cell26.getCellType() == NUMERIC || cell26.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setWxywjckz(cell26.getNumericCellValue());
            //yfywjckz 研发业务进出口值
            Cell cell27 = row.getCell(30);
            if (cell27 != null && (cell27.getCellType() == NUMERIC || cell27.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setYfywjckz(cell27.getNumericCellValue());
            //zlmyjckz 租赁贸易进出口值
            Cell cell28 = row.getCell(21);
            if (cell28 != null && (cell28.getCellType() == NUMERIC || cell28.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setZlmyjckz(cell28.getNumericCellValue());
            //kjdsywjckz 跨境电商业务进出口值
            Cell cell29 = row.getCell(32);
            if (cell29 != null && (cell29.getCellType() == NUMERIC || cell29.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setKjdsywjckz(cell29.getNumericCellValue());
            //gmysgycz 规模以上工业产值
            Cell cell30 = row.getCell(33);
            if (cell30 != null && (cell30.getCellType() == NUMERIC || cell30.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setGmysgycz(cell30.getNumericCellValue());
            //gdtzcz 固定资产投资
            Cell cell31 = row.getCell(34);
            if (cell31 != null && (cell31.getCellType() == NUMERIC || cell31.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setGdtzcz(cell31.getNumericCellValue());
            //ztqszsjckzbz 占同期所在市进出口值比重
            Cell cell32 = row.getCell(35);
            if (cell32 != null && (cell32.getCellType() == NUMERIC || cell32.getCellType() == CellType.FORMULA))
                tbEconomicsIndex.setZtqszsjckzbz(cell32.getNumericCellValue());
            /*----------------------------------------------------------------------------------------------------------------------------------*/
            tbEconomicsIndex.setCreateTime(new Date());
            result.add(tbEconomicsIndex);
            //判断当前月份是否存在
            if(result.size() == 1){
                List<TbEconomicsIndex> tbEconomicsIndexList = tbEconomicsIndexMapper.selectList(new LambdaQueryWrapper<TbEconomicsIndex>().eq(TbEconomicsIndex::getStatementTime, tbEconomicsIndex.getStatementTime()));
                if(!tbEconomicsIndexList.isEmpty()){
                    throw new SecurityException("您当前导入的Excel时间 [" + tbEconomicsIndex.getStatementTime() + "] 已存在,请勿重复导入!");
                }
            }
            List<TbEconomicsIndexVo> list = new ArrayList<>();
            if(flag){

                List<TbEconomicsIndex> economicsIndexVos = disposeList(result);

            }
            //start每次加1
            start++;
            //当start追上end时,重置start和end之间的差距,并将部门置空
            if(start == end){
                end = end + 3;
                deptIdName = null;
            }
        }
        return result;
    }
    private List<TbEconomicsIndex> disposeList(List<TbEconomicsIndex> result){

        List<TbEconomicsIndex> tbEconomicsIndexList = tbEconomicsIndexMapper.selectList(new LambdaQueryWrapper<TbEconomicsIndex>().eq(TbEconomicsIndex::getDeptId, result.get(0).getDeptId()).eq(TbEconomicsIndex::getStatementTime, result.get(0).getStatementTime()));
        if(!tbEconomicsIndexList.isEmpty()){
            throw new SecurityException("您当前时间数据已导入,请勿重复导入! [" + result.get(0).getStatementTime() + "]");
        }

        TbEconomicsIndex index = result.get(result.size() - 2); //年
            TbEconomicsIndex index1 = result.get(result.size() - 3); // 年+1
            TbEconomicsIndexVo copy = new TbEconomicsIndexVo();
            BeanUtils.copyProperties(index1,copy);
            copy.setStatementTime(index.getStatementTime());
            // todo: 做算术运算
            if(index.getJckzz() != null && index.getJckzz() != 0){
                copy.setJckzz((index1.getJckzz() == null? 0:index1.getJckzz() / index.getJckzz()) - 1);
            }else {
                copy.setJckzz(index1.getJckzz());
            }
            if(index.getJgmyjckz() != null && index.getJgmyjckz() != 0){
                copy.setJgmyjckz((index1.getJgmyjckz() == null? 0:index1.getJgmyjckz() / index.getJgmyjckz()) - 1);
            }else {
                copy.setJgmyjckz(index1.getJgmyjckz());
            }
            if(index.getWlhwjckz() != null && index.getWlhwjckz() != 0){
                copy.setWlhwjckz((index1.getWlhwjckz() == null? 0:index1.getWlhwjckz() / index.getWlhwjckz()) - 1);
            }else {
                copy.setWlhwjckz(index1.getWlhwjckz());
            }
            if(index.getYbmyjckz() != null && index.getYbmyjckz() != 0){
                copy.setYbmyjckz((index1.getYbmyjckz() == null? 0:index1.getYbmyjckz() / index.getYbmyjckz()) - 1);
            }else {
                copy.setYbmyjckz(index1.getYbmyjckz());
            }
            if(index.getJyzsr() != null && index.getJyzsr() != 0){
                copy.setJyzsr((index1.getJyzsr() == null? 0:index1.getJyzsr() / index.getJyzsr()) - 1);
            }else {
                copy.setJyzsr(index1.getJyzsr());
            }
            if(index.getHgss() != null && index.getHgss() != 0){
                copy.setHgss((index1.getHgss() == null? 0:index1.getHgss() / index.getHgss()) - 1);
            }else {
                copy.setHgss(index1.getHgss());
            }
            if(index.getSwbmss() != null && index.getSwbmss() != 0){
                copy.setSwbmss((index1.getSwbmss() == null? 0:index1.getSwbmss() / index.getSwbmss()) - 1);
            }else {
                copy.setSwbmss(index1.getSwbmss());
            }
            if(index.getHwmyswszze() != null && index.getHwmyswszze() != 0){
                copy.setHwmyswszze((index1.getHwmyswszze() == null? 0:index1.getHwmyswszze() / index.getHwmyswszze()) - 1);
            }else {
                copy.setHwmyswszze(index1.getHwmyswszze());
            }
            if(index.getQmqycyry() != null && index.getQmqycyry() != 0){
                copy.setQmqycyry((index1.getQmqycyry() == null? 0:index1.getQmqycyry() / index.getQmqycyry()) - 1);
            }else {
                copy.setQmqycyry(index1.getQmqycyry());
            }
            if(index.getHyqys() != null && index.getHyqys() != 0){
                copy.setHyqys((index1.getHyqys() == null? 0:index1.getHyqys() / index.getHyqys()) - 1);
            }else {
                copy.setHyqys(index1.getHyqys());
            }
            if(index.getDwmjjckz() != null && index.getDwmjjckz() != 0){
                copy.setDwmjjckz((index1.getDwmjjckz() == null? 0:index1.getDwmjjckz() / index.getDwmjjckz()) - 1);
            }else {
                copy.setDwmjjckz(index1.getDwmjjckz());
            }
            if(index.getDwmjjyzsr() != null && index.getDwmjjyzsr() != 0){
                copy.setDwmjjyzsr((index1.getDwmjjyzsr() == null? 0:index1.getDwmjjyzsr() / index.getDwmjjyzsr()) - 1);
            }else {
                copy.setDwmjjyzsr(index1.getDwmjjyzsr());
            }
            if(index.getDwmjhgss() != null && index.getDwmjhgss() != 0){
                copy.setDwmjhgss((index1.getDwmjhgss() == null? 0:index1.getDwmjhgss() / index.getDwmjhgss()) - 1);
            }else {
                copy.setDwmjhgss(index1.getDwmjhgss());
            }
            if(index.getDwmjswbmss() != null && index.getDwmjswbmss() != 0){
                copy.setDwmjswbmss((index1.getDwmjswbmss() == null? 0:index1.getDwmjswbmss() / index.getDwmjswbmss()) - 1);
            }else {
                copy.setDwmjswbmss(index1.getDwmjswbmss());
            }
            if(index.getDwmjhwmyswsze() != null && index.getDwmjhwmyswsze() != 0){
                copy.setDwmjhwmyswsze((index1.getDwmjhwmyswsze() == null? 0:index1.getDwmjhwmyswsze() / index.getDwmjhwmyswsze()) - 1);
            }else {
                copy.setDwmjhwmyswsze(index1.getDwmjhwmyswsze());
            }
            if(index.getQmdwmjqycyry() != null && index.getQmdwmjqycyry() != 0){
                copy.setQmdwmjqycyry((index1.getQmdwmjqycyry() == null? 0:index1.getQmdwmjqycyry() / index.getQmdwmjqycyry()) - 1);
            }else {
                copy.setQmdwmjqycyry(index1.getQmdwmjqycyry());
            }
            if(index.getJckzzzf() != null && index.getJckzzzf() != 0){
                copy.setJckzzzf((index1.getJckzzzf() == null? 0:index1.getJckzzzf() / index.getJckzzzf()) - 1);
            }else {
                copy.setJckzzzf(index1.getJckzzzf());
            }
            if(index.getTqhyqyszf() != null && index.getTqhyqyszf() != 0){
                copy.setTqhyqyszf((index1.getTqhyqyszf() == null? 0:index1.getTqhyqyszf() / index.getTqhyqyszf()) - 1);
            }else {
                copy.setTqhyqyszf(index1.getTqhyqyszf());
            }
            if(index.getQmpzghmjysl() != null && index.getQmpzghmjysl() != 0){
                copy.setQmpzghmjysl((index1.getQmpzghmjysl() == null? 0:index1.getQmpzghmjysl() / index.getQmpzghmjysl()) - 1);
            }else {
                copy.setQmpzghmjysl(index1.getQmpzghmjysl());
            }
            if(index.getQmfbwwmjkfl() != null && index.getQmfbwwmjkfl() != 0){
                copy.setQmfbwwmjkfl((index1.getQmfbwwmjkfl() == null? 0:index1.getQmfbwwmjkfl() / index.getQmfbwwmjkfl()) - 1);
            }else {
                copy.setQmfbwwmjkfl(index1.getQmfbwwmjkfl());
            }
            if(index.getSjsywzje() != null && index.getSjsywzje() != 0){
                copy.setSjsywzje((index1.getSjsywzje() == null? 0:index1.getSjsywzje() / index.getSjsywzje()) - 1);
            }else {
                copy.setSjsywzje(index1.getSjsywzje());
            }
            if(index.getJcqhwzz() != null && index.getJckzz() != 0){
                copy.setJcqhwzz((index1.getJcqhwzz() == null? 0:index1.getJckzz() / index.getJcqhwzz()) - 1);
            }else {
                copy.setJcqhwzz(index1.getJcqhwzz());
            }
            if(index.getJckzzzqszstqjckbz() != null && index.getJckzzzqszstqjckbz() != 0){
                copy.setJckzzzqszstqjckbz((index1.getJckzzzqszstqjckbz() == null? 0:index1.getJckzzzqszstqjckbz() / index.getJckzzzqszstqjckbz()) - 1);
            }else {
                copy.setJckzzzqszstqjckbz(index1.getJckzzzqszstqjckbz());
            }
            if(index.getWxywjckz() != null && index.getWxywjckz() != 0){
                copy.setWxywjckz((index1.getWxywjckz() == null? 0:index1.getWxywjckz() / index.getWxywjckz()) - 1);
            }else {
                copy.setWxywjckz(index1.getWxywjckz());
            }
            if(index.getYfywjckz() != null && index.getYfywjckz() != 0){
                copy.setYfywjckz((index1.getYfywjckz() == null? 0:index1.getYfywjckz() / index.getYfywjckz()) - 1);
            }else {
                copy.setYfywjckz(index1.getYfywjckz());
            }
            if(index.getZlmyjckz() != null && index.getZlmyjckz() != 0){
                copy.setZlmyjckz((index1.getZlmyjckz() == null? 0:index1.getZlmyjckz() / index.getZlmyjckz()) - 1);
            }else {
                copy.setZlmyjckz(index1.getZlmyjckz());
            }
            if(index.getKjdsywjckz() != null && index.getKjdsywjckz() != 0){
                copy.setKjdsywjckz((index1.getKjdsywjckz() == null? 0:index1.getKjdsywjckz() / index.getKjdsywjckz()) - 1);
            }else {
                copy.setKjdsywjckz(index1.getKjdsywjckz());
            }
            if(index.getGmysgycz() != null && index.getGmysgycz() != 0){
                copy.setGmysgycz((index1.getGmysgycz() == null? 0:index1.getGmysgycz() / index.getGmysgycz()) - 1);
            }else {
                copy.setGmysgycz(index1.getGmysgycz());
            }
            if(index.getGdtzcz() != null && index.getGdtzcz() != 0){
                copy.setGdtzcz((index1.getGdtzcz() == null? 0:index1.getGdtzcz() / index.getGdtzcz()) - 1);
            }else {
                copy.setGdtzcz(index1.getGdtzcz());
            }
            if(index.getZtqszsjckzbz() != null && index.getZtqszsjckzbz() != 0){
                copy.setZtqszsjckzbz((index1.getZtqszsjckzbz() == null? 0:index1.getZtqszsjckzbz() / index.getZtqszsjckzbz()) - 1);
            }else {
                copy.setZtqszsjckzbz(index1.getZtqszsjckzbz());
            }
            //将新设置的对象覆盖集合中最后一个对象
            result.set(result.size() - 1,copy);
        return result;
    }

}
