package com.hjy.module.service.garden.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.annotation.DataScope;
import com.hjy.common.constant.UserConstants;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.core.text.Convert;
import com.hjy.common.utils.DictUtils;
import com.hjy.common.utils.PageUtils;
import com.hjy.common.utils.StringUtils;
import com.hjy.module.domain.garden.TbGardenBaseData;
import com.hjy.module.mapper.garden.TbGardenBaseDataMapper;
import com.hjy.module.service.garden.ITbGardenBaseDataService;
import com.hjy.module.vo.garden.GardenBaseDataEasyexcelVo;
import com.hjy.module.vo.garden.TbGardenBaseDataVo;
import com.hjy.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;

/**
 * 园区内企业基础数据Service业务层处理
 *
 * @author Tellsea
 * @date 2023-10-11
 */
@Service
public class TbGardenBaseDataServiceImpl extends ServiceImpl<TbGardenBaseDataMapper, TbGardenBaseData> implements ITbGardenBaseDataService {

    @Autowired
    private ISysDeptService deptService;

    @DataScope(deptAlias = "A")
    @Override
    public TableDataInfo<TbGardenBaseDataVo> queryList(TbGardenBaseDataVo entity) {
        return PageUtils.buildDataInfo(this.baseMapper.queryList(PageUtils.buildPage(), entity));
    }

    @Override
    public List<TbGardenBaseDataVo> queryAll(TbGardenBaseDataVo entity) {
        return this.baseMapper.queryList(entity);
    }

    @Override
    public TbGardenBaseDataVo queryById(String id) {
        return this.baseMapper.queryById(id);
    }

    private String chechGardenByname(String deptName) {
        if (this.lambdaQuery().eq(TbGardenBaseData::getDeptName, deptName).count() > 0) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 写入导入数据
     *
     * @param cachedDataList
     */
    @Override
    public void saveImport(List<GardenBaseDataEasyexcelVo> cachedDataList, String deptId) {
        SysDept upDept = deptService.getDeptOrderSumMax(deptId); // 上一个部门
        AtomicReference<Integer> orderNum = new AtomicReference<>();
        String ancestors = "";
        if (ObjectUtil.isNull(upDept)){
            orderNum.set(1);
            ancestors ="0,100,"+deptId;
        }else{
            orderNum = new AtomicReference<>(upDept.getOrderNum());
            ancestors = upDept.getAncestors();
        }

        List<TbGardenBaseData> saveBatchList = new ArrayList<>();
        AtomicReference<Integer> finalOrderNum = orderNum;
        String finalAncestors = ancestors;
        cachedDataList.stream().forEach(item -> {
            SysDept dept = new SysDept();
            dept.setDeptName(item.getDeptName());
            dept.setParentId(Convert.toLong(deptId));
            String checkSys = deptService.checkDeptNameUnique(dept); // 校验部门数据是否存在0=不存在，1=存在
            if (checkSys.equals(UserConstants.UNIQUE)) { // 不存在
                dept.setParentId(Convert.toLong(deptId));
                dept.setAncestors(finalAncestors);
                finalOrderNum.set(finalOrderNum.get() + 1);
                dept.setOrderNum(finalOrderNum.get());
                dept.setLeader(item.getFddbr());
                dept.setStatus("0");
                dept.setDelFlag("0");
                dept.setType(3);
                deptService.insertDept(dept); // 写入部门信息
                saveBatchList.add(this.cpmteGardenBaseData(item, dept));// 写入企业基础信息表
            } else if (checkSys.equals(UserConstants.NOT_UNIQUE)) { //存在
                String baseCheck = this.chechGardenByname(item.getDeptName());
                if (baseCheck.equals(UserConstants.UNIQUE)) {
                    saveBatchList.add(this.cpmteGardenBaseData(item, dept));// 写入企业基础信息表
                }
            }
        });
        if (!CollectionUtils.isEmpty(saveBatchList)) this.saveBatch(saveBatchList, 100);
    }

    /**
     * 组装企业基础信息表
     *
     * @param item
     * @param dept
     */
    private TbGardenBaseData cpmteGardenBaseData(GardenBaseDataEasyexcelVo item, SysDept dept) {
        String grade = item.getGrade();
        item.setGrade(null);
        TbGardenBaseData baseData = BeanUtil.toBean(item, TbGardenBaseData.class);
        baseData.setDeptId(StringUtils.obj2str(dept.getParentId()));
        baseData.setParentId(StringUtils.obj2str(dept.getParentId()));
        if (ObjectUtil.isNotNull(item.getEnterpriseStructure())) {
            baseData.setEnterpriseStructure(item.getEnterpriseStructure().replaceAll(";", ","));
            String[] entStrArr = item.getEnterpriseStructure().split(";");
            StringBuilder sb = new StringBuilder();
            Arrays.stream(entStrArr).forEach(entstr -> {
                String dictValue = DictUtils.getDictValue("enterprise_structure", entstr);
                if (ObjectUtil.isNotNull(dictValue)) sb.append(dictValue).append(",");
            });
            if (ObjectUtil.isNotNull(sb)){
                String value = sb.substring(0, sb.length()-1);
                baseData.setEnterpriseStructureValue(value);
                baseData.setEnterpriseStructureSort(value);
            }

        }
        if (ObjectUtil.isNotNull(item.getBusinessEvents())) {
            baseData.setBusinessEvents(item.getBusinessEvents().replaceAll(";", ","));
            String[] busStrArr = item.getBusinessEvents().split(";");
            StringBuilder sb = new StringBuilder();
            Arrays.stream(busStrArr).forEach(busstr -> {
                String dictValue = DictUtils.getDictValue("business_events", busstr);
                if (ObjectUtil.isNotNull(dictValue)) sb.append(dictValue).append(",");
            });
            if (ObjectUtil.isNotNull(sb)){
                String value  = sb.substring(0, sb.length()-1);
                baseData.setBusinessEventsValue(value);
                baseData.setBusinessEventsSort(value);
            }
        }
        if (ObjectUtil.isNotNull(grade)) {
            baseData.setGrade(grade.equals("A") ? 1 : grade.equals("B") ? 2 : grade.equals("C") ? 3 : 4);
        }

        return baseData;
    }
}
