package com.hjy.module.service.garden.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.exception.ServiceException;
import com.hjy.common.utils.PageUtils;
import com.hjy.common.utils.StringUtils;
import com.hjy.common.utils.bean.BeanValidators;
import com.hjy.module.domain.garden.TbGardenTradeData;
import com.hjy.module.mapper.garden.TbGardenTradeDataMapper;
import com.hjy.module.service.garden.TbGardenTradeDataService;
import com.hjy.module.vo.garden.TbGardenTradeDataVo;
import com.hjy.system.service.ISysDeptService;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.DateUtil;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Validator;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 园区贸易数据Service业务层处理
 *
 * @author Mechrevo
 * @date 2023-09-26
 */
@Service
public class TbGardenTradeDataServiceImpl extends ServiceImpl<TbGardenTradeDataMapper, TbGardenTradeData> implements TbGardenTradeDataService {

    @Autowired
    protected Validator validator;
    ISysDeptService sysDeptService;

    @Autowired
    private ISysDeptService sysDeptService(ISysDeptService sysDeptService) {
        return this.sysDeptService = sysDeptService;
    }

    @Override
    public TableDataInfo<TbGardenTradeDataVo> queryList(TbGardenTradeDataVo entity) {
        return PageUtils.buildDataInfo(this.baseMapper.queryList(PageUtils.buildPage(), entity));
    }

    @Override
    public List<TbGardenTradeDataVo> queryAll(TbGardenTradeDataVo entity) {
        List<TbGardenTradeDataVo> tbGardenTradeDataVos = this.baseMapper.queryList(entity);
        return tbGardenTradeDataVos;
    }

    @Override
    public TbGardenTradeDataVo queryById(String gardenId) {
        TbGardenTradeDataVo tbGardenTradeDataVo = this.baseMapper.queryById(gardenId);
        return tbGardenTradeDataVo;
    }


    /**
     * 导入用户数据
     *
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param operName        操作用户
     * @return 结果
     */
    @Override
    public String importTbGardenTradeData(List<TbGardenTradeDataVo> TbGardenTradeDataList, Boolean isUpdateSupport, String operName) {
        if (StringUtils.isNull(TbGardenTradeDataList) || TbGardenTradeDataList.size() == 0) {
            throw new ServiceException("导入用户数据不能为空！");
        }
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        for (TbGardenTradeDataVo user : TbGardenTradeDataList) {
            try {
                if (true) {
                    BeanValidators.validateWithException(validator, user);
                    user.setCreateBy(operName);
                    this.save(user);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、" + user.getGardenName() + "园区贸易数据 导入成功");
                } else {
                    failureNum++;
                    failureMsg.append("<br/>" + failureNum + "、已存在");
                }
            } catch (Exception e) {
                failureNum++;
                String msg = "<br/>" + failureNum + "、 导入失败：";
                failureMsg.append(msg + e.getMessage());
                log.error(msg, e);
            }
        }
        if (failureNum > 0) {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new ServiceException(failureMsg.toString());
        } else {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
        }
        return successMsg.toString();
    }

    @Override
    public String importData(MultipartFile file) throws IOException {
        System.err.println("\n开始执行文件上传....\n");

        //判空
        if (file.isEmpty()) return "文件为空,无法执行上传...";
        //获取文件上传数据
        XSSFWorkbook wb = new XSSFWorkbook(file.getInputStream());
        //获取第一页sheet
        XSSFSheet sheet = wb.getSheetAt(0);
        //定义计数器
        int count = 0;
        //定义行对象
        XSSFRow row = null;
        //解析数据封装到集合
        //创建接受集合
        List<TbGardenTradeData> list = new ArrayList<>();
        //循环遍历sheet页面(默认只有一页)
        for (int i = 1; i < sheet.getPhysicalNumberOfRows(); i++) {
            //封装数据到行对象
            row = sheet.getRow(i);
            //定义返回值实体
            TbGardenTradeDataVo tbGardenTradeDataVo = new TbGardenTradeDataVo();
            //cell 第一个可能是序号,暂时摒除

            //deptId 保税区ID
            if (row.getCell(0) != null && !row.getCell(0).toString().trim().equals("")) {
                String stringCellValue = row.getCell(0).getStringCellValue();
                List<SysDept> sysDepts = sysDeptService.selectDeptList(null);
                sysDepts.forEach(item -> {
                    if (item.getDeptName().contains(stringCellValue)) {
                        tbGardenTradeDataVo.setDeptId(String.valueOf(item.getDeptId()));
                    }
                });
            }
            //companyDrawback 企业出口退税额
            XSSFCell cell3 = row.getCell(1);
            if (cell3 != null && (cell3.getCellType() == CellType.NUMERIC || cell3.getCellType() == CellType.FORMULA))
                tbGardenTradeDataVo.setCompanyDrawback(cell3.getNumericCellValue());
            //gardenDrawback 园区出口退税总额
            XSSFCell cell4 = row.getCell(2);
            if (cell4 != null && (cell4.getCellType() == CellType.NUMERIC || cell4.getCellType() == CellType.FORMULA))
                tbGardenTradeDataVo.setGardenDrawback(cell4.getNumericCellValue());
            //gardenTax 园区运营单位缴税总额
            XSSFCell cell5 = row.getCell(3);
            if (cell5 != null && (cell5.getCellType() == CellType.NUMERIC || cell5.getCellType() == CellType.FORMULA))
                tbGardenTradeDataVo.setGardenTax(cell5.getNumericCellValue());
            //companyLists 企业承接单量
            XSSFCell cell6 = row.getCell(4);
            if (cell6 != null && (cell6.getCellType() == CellType.NUMERIC || cell6.getCellType() == CellType.FORMULA))
                tbGardenTradeDataVo.setCompanyLists((int) cell6.getNumericCellValue());
            //gardenLists 园区承接单总量
            XSSFCell cell7 = row.getCell(5);
            if (cell7 != null && (cell7.getCellType() == CellType.NUMERIC || cell7.getCellType() == CellType.FORMULA))
                tbGardenTradeDataVo.setGardenLits((int) cell7.getNumericCellValue());
            //companyInnovative 企业创新获奖量
            XSSFCell cell8 = row.getCell(6);
            if (cell8 != null && (cell8.getCellType() == CellType.NUMERIC || cell8.getCellType() == CellType.FORMULA))
                tbGardenTradeDataVo.setCompanyInnovative((int) cell8.getNumericCellValue());
            //gardenInnovative 园区内创新获奖总量
            XSSFCell cell9 = row.getCell(7);
            if (cell9 != null && (cell9.getCellType() == CellType.NUMERIC || cell9.getCellType() == CellType.FORMULA))
                tbGardenTradeDataVo.setGardenInnovative((int) cell9.getNumericCellValue());
            //companyWaterElectric 企业工业用水、用电量
            XSSFCell cell10 = row.getCell(8);
            if (cell10 != null && (cell10.getCellType() == CellType.NUMERIC || cell10.getCellType() == CellType.FORMULA))
                tbGardenTradeDataVo.setCompanyWaterElectric(cell10.getNumericCellValue());
            //gardenWaterElectric 园区工业用水、用电总量
            XSSFCell cell11 = row.getCell(9);
            if (cell11 != null && (cell11.getCellType() == CellType.NUMERIC || cell11.getCellType() == CellType.FORMULA))
                tbGardenTradeDataVo.setGardenWaterElectric(cell11.getNumericCellValue());
            //createTime 创建时间
            XSSFCell cell12 = row.getCell(10);
            if (cell12 != null && cell12.getCellType() == CellType.NUMERIC && DateUtil.isCellDateFormatted(cell12)) {
                tbGardenTradeDataVo.setCreateTime(cell12.getDateCellValue());
            }

            //添加到集合
            list.add(tbGardenTradeDataVo);

        }

        System.err.println("待处理数据量为: " + list.size() + " 条。");

        int processors = 1;
        if (list.size() > 30) {
            //获取当前电脑线程数,使用一半线程
            processors = Runtime.getRuntime().availableProcessors() / 2;
        }
        //创建多线程
        ExecutorService executorService = Executors.newFixedThreadPool(processors);

        try {
            //每次30条
            int bai = 30;
            //计算需要创建的线程数
            int threads = (int) Math.ceil((double) list.size() / bai);
            for (int i = 0; i < threads; i++) {
                int start = i * bai;
                int end = Math.min(start + bai, list.size());

                //截取集合数量
                List<TbGardenTradeData> tbGardenTradeDataVos = list.subList(start, end);

                //多线程执行任务
                executorService.submit(() -> {
                    this.saveBatch(tbGardenTradeDataVos);
                });
                //计数
                count += tbGardenTradeDataVos.size();
            }
        } finally {
            executorService.shutdown();
            wb.close();
            if (wb != null) {
                wb.close();
            }
        }
        System.err.println("\n文件上传完成,共上传 " + count + "条 " + "数据...\n");
        return "上传完成,共上传" + count + "条" + "数据。";
    }
}
