package com.hjy.module.service.gather.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.core.text.Convert;
import com.hjy.common.utils.StringUtils;
import com.hjy.module.domain.gether.CbGatherFee;
import com.hjy.module.mapper.gather.CbGatherFeeMapper;
import com.hjy.module.service.gather.ICbGatherFeeService;
import com.hjy.module.service.gather.ICbGatherFormulaService;
import com.hjy.module.service.norm.ICbNormFeeService;
import com.hjy.module.vo.gather.CbGatherFeeVo;
import com.hjy.module.vo.gather.CbGatherVo;
import com.hjy.module.vo.norm.CbNormFeeVo;
import com.hjy.system.mapper.SysDeptMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 指标计算Service业务层处理
 *
 * @author lbl
 * @date 2023-10-13
 */
@Slf4j
@Service
public class CbGatherFormulaServiceImpl extends ServiceImpl<CbGatherFeeMapper, CbGatherFee> implements ICbGatherFormulaService {

    @Autowired
    private ICbGatherFeeService gatherFeeService;
    @Autowired
    private ICbNormFeeService normFeeService;
    @Autowired
    private CbGatherFeeMapper gatherFeeMapper;
    @Autowired
    private SysDeptMapper sysDeptMapper;


    /** 按照公式计算年度指标采集值 */
    public boolean computeCollCalue(CbGatherVo entity) throws ScriptException {
        CbNormFeeVo normFeeVo = new CbNormFeeVo();
        normFeeVo.setDelFlag("0");
        normFeeVo.setStatus("1");

        List<CbGatherFeeVo> feeVoList = gatherFeeMapper.queryByGatherId(entity.getId());
        Map<String,Object> maps = feeVoList.stream().collect(
                Collectors.toMap(
                        e -> e.getNormfeeCode(),
                        e -> e.getCollCalue() == null ? 0 : e.getCollCalue(),
                        (entity1,entity2) -> {
                            return entity1;
                        }
                ));
        //过滤得到所有有公式的指标进行后续计算
        feeVoList = feeVoList.stream().filter(vo -> StringUtils.isNotEmpty(vo.getFormula())).collect(Collectors.toList());

        List<CbGatherFee> gatherFeeList = new ArrayList<>();
        for (CbGatherFeeVo feeVo : feeVoList) {
            String formula = feeVo.getFormula();
            for (Map.Entry<String, Object> map : maps.entrySet()) {
                if (formula.contains(map.getKey())) {
                    formula = formula.replace(map.getKey(), map.getValue().toString());
                }
            }
            BigDecimal bd = caculate(formula);

            //四舍五入保留四位小数
            bd = bd.setScale(4, BigDecimal.ROUND_HALF_UP);
            feeVo.setCollCalue(bd);
            CbGatherFee gatherFee = new CbGatherFee();
            BeanUtils.copyProperties(feeVo,gatherFee);
            gatherFeeList.add(gatherFee);
        }
        boolean result = gatherFeeService.updateBatchById(gatherFeeList);
        return result;
    }

    /** 计算同比值 */
    public void computeCompireCalue(CbGatherVo entity) {
        //取上一年
        int lastYear = Integer.parseInt(entity.getYear()) - 1;
        Map<String,String> map = new HashMap<>();
        map.put("deptId",entity.getDeptId());
        map.put("type",entity.getType());
        map.put("month",entity.getMonth());
        map.put("year", String.valueOf(lastYear));
        String lastGatherId = gatherFeeMapper.getGatherId(map);

        int i = gatherFeeMapper.computeCompireCalue(lastGatherId, entity.getId());
        System.out.println("计算规模效益同比，修改了{"+i+"}条数据");
    }

    /** 单独计算 进出口总值增幅(B007)、同期活跃企业数增幅(B008) 两个指标的同环比 */
    public boolean tongbiToValue(CbGatherVo entity) {
        List<CbGatherFeeVo> feeVoList = gatherFeeMapper.queryByGatherId(entity.getId());
        String A001CompireCalue = "";
        String A010CompireCalue = "";
        for (CbGatherFeeVo cbGatherFeeVo : feeVoList) {
            if (StringUtils.equals("A001", cbGatherFeeVo.getNormfeeCode())) {
                A001CompireCalue = cbGatherFeeVo.getCompireCalue();
            }
            if (StringUtils.equals("A010", cbGatherFeeVo.getNormfeeCode())) {
                A010CompireCalue = cbGatherFeeVo.getCompireCalue();
            }
        }
        List<CbGatherFee> gatherFeeList = new ArrayList<>();
        for (CbGatherFeeVo cbGatherFeeVo : feeVoList) {
            if (StringUtils.equals("B007", cbGatherFeeVo.getNormfeeCode()) && StringUtils.isNotEmpty(A001CompireCalue)) {
                cbGatherFeeVo.setCollCalue(new BigDecimal(A001CompireCalue));
                gatherFeeList.add(cbGatherFeeVo);
            }
            if (StringUtils.equals("B008", cbGatherFeeVo.getNormfeeCode()) && StringUtils.isNotEmpty(A010CompireCalue)) {
                cbGatherFeeVo.setCollCalue(new BigDecimal(A010CompireCalue));
                gatherFeeList.add(cbGatherFeeVo);
            }
        }
        boolean result = gatherFeeService.updateBatchById(gatherFeeList);
        return result;
    }

    /** 计算方法：根据计算公式计算值 */
    public static BigDecimal caculate(String formula) throws ScriptException {
        ScriptEngineManager manager = new ScriptEngineManager();
        ScriptEngine engine = manager.getEngineByName("JavaScript");
        BigDecimal result = new BigDecimal(0);
        try {
            result = Convert.toBigDecimal(engine.eval(formula),BigDecimal.ZERO);
        }catch (Exception e){
            result = BigDecimal.ZERO;
            log.info(e.getMessage());
        }
        return result;
    }

    public static void main(String[] args) {
//        String filters = "(A016/A001)-1";
//        Map<String, String> maps = new HashMap<>();
//        maps.put("A001","11");
//        maps.put("A002","22");
//        maps.put("A016","66");
//        for (Map.Entry<String, String> map : maps.entrySet()) {
//            if (filters.contains(map.getKey())) {
//                filters = filters.replace(map.getKey(), map.getValue());
//            }
//        }
//        System.out.println(filters);
        String formula = "111*100";
        try {
            BigDecimal result = Convert.toBigDecimal(caculate(formula));
            System.out.println("计算结果="+result);
        } catch (ScriptException e) {
            System.out.println("计算错误："+ e.getMessage());
        }
    }


}
