package com.hjy.module.service.gather.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.enums.CellExtraTypeEnum;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.annotation.DataScope;
import com.hjy.common.constant.WorkFlowStatus;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.domain.model.LoginUser;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.core.text.Convert;
import com.hjy.common.enums.WorkFlowServiceEnums;
import com.hjy.common.utils.*;
import com.hjy.common.utils.excel.TaxReadEnterpriseExcelUtil;
import com.hjy.common.utils.excel.vo.TaxEnterpriseItemVo;
import com.hjy.common.utils.excel.vo.TaxEnterpriseTableVo;
import com.hjy.common.utils.file.CustomMergeStrategy;
import com.hjy.common.utils.file.FileUtils;
import com.hjy.module.EasyExcel.GatherDataCustomsListener;
import com.hjy.module.domain.gether.CbGather;
import com.hjy.module.domain.gether.CbGatherFee;
import com.hjy.module.domain.norm.CbNorm;
import com.hjy.module.domain.norm.CbNormFee;
import com.hjy.module.mapper.gather.CbGatherMapper;
import com.hjy.module.service.garden.ITbEconomicsIndexService;
import com.hjy.module.service.gather.ICbGatherFeeService;
import com.hjy.module.service.gather.ICbGatherService;
import com.hjy.module.service.gather.ICbGatherWarningService;
import com.hjy.module.service.norm.ICbGiveService;
import com.hjy.module.service.norm.ICbNormFeeService;
import com.hjy.module.service.norm.ICbNormService;
import com.hjy.module.service.statistics.ICbIntegralService;
import com.hjy.module.service.wf.IWfInstActionTaskService;
import com.hjy.module.service.wf.WorkFlowInterface;
import com.hjy.module.vo.gather.CbGatherFeeVo;
import com.hjy.module.vo.gather.CbGatherVo;
import com.hjy.module.vo.gather.CbGatherWarningVo;
import com.hjy.module.vo.gather.GatherImportVo;
import com.hjy.module.vo.gather.vo.*;
import com.hjy.module.vo.norm.CbNormFeeVo;
import com.hjy.module.vo.statistics.CbIntegralFeeVo;
import com.hjy.module.vo.statistics.CbIntegralVo;
import com.hjy.module.vo.totexp.CbTotexpVo;
import com.hjy.system.service.ISysDeptService;
import org.apache.commons.compress.utils.Lists;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * 数据采集Service业务层处理
 *
 * @author Tellsea
 * @date 2023-08-30
 */
@Transactional(rollbackFor = Exception.class)
@Service("gatherService")
public class CbGatherServiceImpl extends ServiceImpl<CbGatherMapper, CbGather> implements ICbGatherService, WorkFlowInterface {


    ITbEconomicsIndexService tbEconomicsIndexService;
    ICbGatherService gatherService;
    ICbGatherService cbGatherService;
    @Autowired
    private ICbGatherFeeService gatherFeeService;
    @Autowired
    private ICbNormService normService;
    @Autowired
    private ICbNormFeeService normFeeService;
    @Autowired
    private ICbGiveService giveService;
    @Autowired
    private ICbGatherWarningService warningService;
    @Autowired
    private ISysDeptService sysDeptService;
    @Autowired
    private ICbIntegralService integralService;

    @Autowired
    private CbGatherMapper gatherMapper;

    @Resource
    private ISysDeptService deptService;

    @Autowired
    private IWfInstActionTaskService actionTaskService;


    /**
     * 对比上下公差
     *
     * @param collValue  实际值
     * @param msDtoValue 上公差
     * @param msDtuValue 下公差
     * @return
     */
    private static boolean bjdtodtu(BigDecimal collValue, BigDecimal msDtoValue, BigDecimal msDtuValue) {
        if (collValue.compareTo(msDtoValue) != 1 || collValue.compareTo(msDtuValue) != -1) {
            return true;
        }
        return false;
    }

    /**
     * excel表态构造
     *
     * @param lebal
     * @return
     */
    public static List<List<String>> getHead(String lebal, String month) {
        List<List<String>> list = new ArrayList<List<String>>();
        for (int i = 0; i < 4; i++) {
            List<String> row = Lists.newArrayList();
            switch (i) {
                case 0:
                    row.add("指标分类");
                    row.add("指标分类");
                    break;
                case 1:
                    row.add("指标名称");
                    row.add("指标名称");
                    break;
                case 2:
                    row.add("计量单位");
                    row.add("计量单位");
                    break;
                case 3:
                    row.add(lebal);
                    row.add(month);
                    break;
                default:
                    break;
            }
            list.add(row);
        }
        return list;

    }

    @Autowired
    private ICbGatherService gatherService(ICbGatherService gatherService) {
        return this.gatherService = gatherService;
    }

    @Autowired
    private ICbGatherService cbGatherService(ICbGatherService cbGatherService) {
        return this.cbGatherService = cbGatherService;
    }

    @Autowired
    private ITbEconomicsIndexService tbEconomicsIndexService(ITbEconomicsIndexService tbEconomicsIndexService) {
        return this.tbEconomicsIndexService = tbEconomicsIndexService;
    }

    @DataScope(deptAlias = "A")
    @Override
    public TableDataInfo<CbGatherVo> queryList(CbGatherVo entity) {
        entity.setType("0");
        Page<CbGatherVo> page = this.baseMapper.queryList(PageUtils.buildPage(), entity);
        return PageUtils.buildDataInfo(page);
    }

    @Override
    public List<CbGatherVo> queryAll(CbGatherVo entity) {
        // 这里也过滤一下 以防万一
        return this.baseMapper.queryList(entity).stream().filter(item -> "0".equals(item.getType())).collect(Collectors.toList());
    }

    /**
     * 获取数据详情
     *
     * @param id
     * @return
     */
    @Override
    public CbGatherVo queryById(String id) {
        CbGatherVo gatherVo = this.baseMapper.queryById(id);
        // 查询对象
        List<CbGatherFeeVo> feeVos = gatherFeeService.queryByTypeAndGatherId(gatherVo.getId(), null);
        gatherVo.setFeeLists(feeVos);
        gatherVo.setTaskVoList(actionTaskService.queryTasksByid(id, WorkFlowServiceEnums.GATHERSERSERVICE.getService()));
        return gatherVo;
    }

    @Override
    public CbGatherVo queryById(String id, Long userId) {
        CbGatherVo gatherVo = this.baseMapper.queryById(id);
        // 查询对象
        List<CbGatherFeeVo> feeVos = gatherFeeService.queryByTypeAndGatherId(gatherVo.getId(), userId);
        gatherVo.setFeeLists(feeVos);
        return gatherVo;
    }

    /**
     * 获取可采集的数据
     *
     * @param id
     * @return
     */
    @Override
    public CbGatherVo getInfoByColl(String id) {

        List<CbGatherFeeVo> gatherFeeVoList = Lists.newArrayList();
        CbGatherVo gatherVo = this.queryById(id);
        if (CollectionUtil.isEmpty(gatherVo.getFeeLists())) {
            // 构造可采集的数据指标
            buildDataIndex(gatherVo, gatherFeeVoList, id);
            gatherVo.setFeeLists(gatherFeeVoList);
        }
        String deptId=gatherVo.getDeptId();
       SysDept sysDept= sysDeptService.selectDeptById(Long.valueOf(deptId));
       if (sysDept!=null){
           //规划面积默认值
           gatherVo.getFeeLists().stream().filter(fee->fee.getNormfeeCode().equals("AA01"))
                   .findFirst()
                   .ifPresent(fee->fee.setCollCalue(BigDecimal.valueOf(sysDept.getPlanningArea())));
           //验收面积
           gatherVo.getFeeLists().stream().filter(fee->fee.getNormfeeCode().equals("AA02"))
                   .findFirst()
                   .ifPresent(fee->fee.setCollCalue(BigDecimal.valueOf(sysDept.getAcceptanceArea())));
       }
        return gatherVo;
    }

    /**
     * @param gatherVo        数据采集对象
     * @param gatherFeeVoList 数据采集明细 --> vo 里有 数据指标单位字段
     * @param id              数据采集对象的id
     */
    private void buildDataIndex(CbGatherVo gatherVo, List<CbGatherFeeVo> gatherFeeVoList, String id) {
        String userId = Convert.toStr(SecurityUtils.getUserId());
            List<CbNorm> normList = normService.lambdaQuery().eq(CbNorm::getStatus, "1")
                    .eq(CbNorm::getDelFlag, "0")
                    .orderByAsc(CbNorm::getSort)
                    .list();
            for (CbNorm norm : normList) {
                List<CbNormFee> roleFeeList = normFeeService.selnormgatherByRole(norm.getId(), userId); // 已配置权限指标

                List<CbNormFee> allFeeList = normFeeService.lambdaQuery()
                        .eq(CbNormFee::getStatus, "1")
                        .eq(CbNormFee::getDelFlag, "0")
                        .eq(CbNormFee::getNormId, norm.getId())
                        .orderByAsc(CbNormFee::getSort)
                        .list();   // 未配置权限指标 （不可采集）
                for (CbNormFee normFee : allFeeList) {
                    CbGatherFeeVo gatherFeeVo = new CbGatherFeeVo();
                    gatherFeeVo.setNormId(norm.getId());
                    gatherFeeVo.setNormName(norm.getName());
                    gatherFeeVo.setNormfeeId(normFee.getId());
                    gatherFeeVo.setNormfeeName(normFee.getFname());
                    gatherFeeVo.setGatherId(id);
                    gatherFeeVo.setFunit(normFee.getFunit());
                    gatherFeeVo.setNormfeeCode(normFee.getFcode());
                    gatherFeeVo.setDeptId(gatherVo.getDeptId());
                    gatherFeeVo.setSendValue("1");
                    gatherFeeVo.setFunitName(DictUtils.getDictLabel("norm_uint", normFee.getFunit()));
                    if (CollectionUtil.isNotEmpty(roleFeeList) && roleFeeList.size() > 0) {
                        for (CbNormFee roleFee : roleFeeList) {
                            if (roleFee.getId().equals(normFee.getId())) {
                                gatherFeeVo.setSendValue("0");
                                break;
                            }
                        }
                    }
                    gatherFeeVoList.add(gatherFeeVo);
                }
            }
    }


    public boolean saveCustomsEntity(CbGatherVo entity, boolean flag) {
        // entity.setCollStatus("1");
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String nickName = loginUser.getUser().getNickName();
        entity.setCollName(nickName);
        entity.setCollTime(new Date());
        if (flag) {
            this.baseMapper.updateById(entity);
        }
        gatherFeeService.saveOrUpdateBatch(CommomUtils.copyList(entity.getFeeLists(), CbGatherFee.class));
        this.checkWraingData(entity); // 校验数据 生成预警信息
        return true;
    }

    /**
     * 保存采集数据
     *
     * @param entity
     * @return
     */
    @Override
    public boolean saveEntity(CbGatherVo entity) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String nickName = loginUser.getUser().getNickName();
        entity.setCollName(nickName);
        entity.setCollTime(new Date());
        this.baseMapper.updateById(entity);
        gatherFeeService.saveOrUpdateBatch(CommomUtils.copyList(entity.getFeeLists(), CbGatherFee.class));
        this.updateOtherFeeValue(entity); //更改其他保税区指标信息
        this.checkWraingData(entity); // 校验数据 生成预警信息
        return true;
    }

    /**
     * 修改采集数据
     *
     * @param entity
     * @return
     */
    @Override
    public boolean updateEntity(CbGatherVo entity) {
        this.baseMapper.updateById(entity);
        gatherFeeService.updateBatchById(CommomUtils.copyList(entity.getFeeLists(), CbGatherFee.class));
        this.updateOtherFeeValue(entity); //更改其他保税区指标信息
        this.updateWbFeeValue(entity); //更改湾办进出口总值增速指标信息
        this.checkWraingData(entity); // 校验数据 生成预警信息
        return true;
    }

    /**
     *  更新湾办进出口总值增速指标信息
     *
     * @param entity
     */
    private void updateWbFeeValue(CbGatherVo entity) {
        AtomicReference<BigDecimal> collValue = new AtomicReference<>(new BigDecimal(0));
        entity.getFeeLists().stream().forEach(item -> {
            if (ObjectUtil.isNotNull(item.getNormfeeCode()) && item.getNormfeeCode().equals("A011")) {
                collValue.set(item.getCollCalue());
            }
        });
        List<CbGather> gatherList = this.lambdaQuery().eq(CbGather::getYear, entity.getYear())
                .eq(CbGather::getMonth, entity.getMonth())
                .eq(CbGather::getDelFlag, "0").list();
        gatherList.stream().forEach(gather -> {
            CbGatherFee fee = gatherFeeService.lambdaQuery().eq(CbGatherFee::getGatherId, gather.getId())
                    .eq(CbGatherFee::getNormfeeCode, "A011")
                    .eq(CbGatherFee::getDelFlag, "0").one();
            if (ObjectUtil.isNotNull(fee)){
                fee.setCollCalue(collValue.get());
                gatherFeeService.updateById(fee);
            }
        });
    }

    /**
     * 更改其他保税区 区总值数据
     *
     * @param entity
     */
    private void updateOtherFeeValue(CbGatherVo entity) {
        AtomicReference<BigDecimal> collValue = new AtomicReference<>(new BigDecimal(0));
        entity.getFeeLists().stream().forEach(item -> {
            if (ObjectUtil.isNotNull(item.getNormfeeCode()) && item.getNormfeeCode().equals("F004")) {
                collValue.set(item.getCollCalue());
            }
        });
        List<CbGather> gatherList = this.lambdaQuery().eq(CbGather::getYear, entity.getYear())
                .eq(CbGather::getMonth, entity.getMonth())
                .eq(CbGather::getDelFlag, "0").list();
        gatherList.stream().forEach(gather -> {
            CbGatherFee fee = gatherFeeService.lambdaQuery().eq(CbGatherFee::getGatherId, gather.getId())
                    .eq(CbGatherFee::getNormfeeCode, "F004")
                    .eq(CbGatherFee::getDelFlag, "0").one();
            if (ObjectUtil.isNotNull(fee)){
                fee.setCollCalue(collValue.get());
                gatherFeeService.updateById(fee);
            }
        });
    }

    /**
     * 校验数据 生成预警信息
     *
     * @param entity
     */
    public void checkWraingData(CbGatherVo entity) {
        Thread thread = new Thread(new Runnable() {
            public void run() {
                List<CbGatherFeeVo> gatherFeeVos = entity.getFeeLists();
                for (CbGatherFeeVo gatherFee : gatherFeeVos) {
                    CbNormFeeVo feeVo = normFeeService.queryById(gatherFee.getNormfeeId());
                    BigDecimal collValue = gatherFee.getCollCalue(); // 采集值
                    BigDecimal collCK = feeVo.getCollValue(); // 采集参考值
                    BigDecimal collDtoValue = collCK.add(collCK.multiply(comBFB(feeVo.getDfRetio())));  // 采集上公差
                    BigDecimal collDtuValue = collCK.subtract(collCK.multiply(comBFB(feeVo.getDfRetio()))); // 采集下公差
                    if (collValue != null) {
                        if (!bjdtodtu(collValue, collDtoValue, collDtuValue)) {
                            int count = warningService.checkByGatherData(entity.getId(), feeVo.getId()); // 校验预警数据是否已存在
                            if (count > 0) continue;
                            // 数据异常->产生预警信息
                            CbGatherWarningVo warningVo = new CbGatherWarningVo();
                            warningVo.setType(entity.getType());
                            warningVo.setGetherId(entity.getId());
                            warningVo.setGetherfeeId(gatherFee.getId());
                            warningVo.setErrValue(collValue);
                            warningVo.setStatus("0");
                            warningVo.setDeptId(entity.getDeptId());
                            warningService.save(warningVo);
                        }
                    }
                }
            }
        });
    }

    private BigDecimal comBFB(BigDecimal retio) {
        retio = retio.divide(new BigDecimal(100), 2, BigDecimal.ROUND_UP);
        return retio;
    }

    /**
     * 下载导入模板
     *
     * @param response
     * @param id
     */
    @Override
    public void downloadModlue(HttpServletResponse response, String id) throws IOException {
        CbGatherVo gatherVo = this.getInfoByColl(id);
        SysDept dept = sysDeptService.selectDeptById(Convert.toLong(gatherVo.getDeptId()));
        List<GatherImportVo> importVoList = getImportData(gatherVo);
        ServletOutputStream out = response.getOutputStream();
        ExcelWriter writer = EasyExcel.write(out)
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                .registerWriteHandler(new CustomMergeStrategy(importVoList.stream().map(GatherImportVo::getNormName)
                        .collect(Collectors.toList()), 0))
                .build();
        WriteSheet sheet1 = EasyExcel.writerSheet("sheet1").head(getHead(dept.getDeptName(), gatherVo.getYear() + "年-" + gatherVo.getMonth())).build();
        writer.write(importVoList, sheet1);
        writer.close();
        out.close();
    }

    /**
     * 构造导出数据
     *
     * @param gatherVo
     * @return
     */
    private List<GatherImportVo> getImportData(CbGatherVo gatherVo) {
        List<GatherImportVo> resultList = Lists.newArrayList();
        for (CbGatherFeeVo feeVo : gatherVo.getFeeLists()) {
            GatherImportVo importVo = new GatherImportVo();
            importVo.setNormName(feeVo.getNormName());
            importVo.setNormfeeName(feeVo.getNormfeeName());
            if (gatherVo.getType().equals("0")) {
                importVo.setUint(DictUtils.getDictLabel("norm_uint", normFeeService.getById(feeVo.getNormfeeId()).getFunit()));
            } else {
                importVo.setUint(DictUtils.getDictLabel("norm_uint", giveService.getById(feeVo.getNormfeeId()).getFunit()));
            }
            importVo.setCollCalue(feeVo.getCollCalue());
            if (feeVo.getSendValue().equals("0")) {
                resultList.add(importVo);
            }
        }
        return resultList;
    }

    /**
     * 解析导入数据
     *
     * @param headLists
     * @param cachedDataList
     * @param type
     */
    public void saveImportData(List<String> headLists, List<GatherImportVo> cachedDataList, String type) {
        String deptNamee = headLists.get(0);
        String yearMonth = headLists.get(1);
        SysDept dept = sysDeptService.selectDeptByName(deptNamee);
        CbGatherVo gatherVo = new CbGatherVo();
        gatherVo.setYear(yearMonth.split("年")[0]);
        gatherVo.setMonth(yearMonth.split("-")[1].replace("月", "").trim());
        gatherVo.setDeptId(Convert.toStr(dept.getDeptId()));
        gatherVo.setType(type);
        List<CbGatherVo> gatherVoList = this.baseMapper.queryList(gatherVo);
        if (CollectionUtil.isEmpty(gatherVoList)) {
            Assert.isTrue(false, "未找到对应保税区采集数据");
        }
        if (gatherVoList.size() > 1) {
            Assert.isTrue(false, "错误，找到多个保税区采集数据，无法指定");
        }
        CbGatherVo cbGatherVo = getInfoByColl(gatherVoList.get(0).getId());
        if (cbGatherVo.getApproveStatus().equals("2")) {
            Assert.isTrue(false, "该数据已审核通过，无法继续导入！");
        }
        if (cbGatherVo.getReportStatus().equals("1")) {
            Assert.isTrue(false, "该数据已完成上报，无法继续导入！");
        }
        // 写入数据
        saveImportDataTodatabase(cbGatherVo, cachedDataList);
    }

    /**
     * 写入解析数据
     *
     * @param info
     * @param importVoList
     */
    private void saveImportDataTodatabase(CbGatherVo info, List<GatherImportVo> importVoList) {
        for (GatherImportVo imfee : importVoList) {
            getFeeValue(imfee, info.getFeeLists());
        }
        saveEntity(info); // 保存
    }

    private void getFeeValue(GatherImportVo imfee, List<CbGatherFeeVo> feeVoList) {
        boolean flag = false;
        BigDecimal collValue = null;
        for (CbGatherFeeVo feeVo : feeVoList) {
            if (feeVo.getNormName().equals(imfee.getNormName()) && feeVo.getNormfeeName().equals(imfee.getNormfeeName())) {
                flag = true;
                feeVo.setCollCalue(imfee.getCollCalue());
                break;
            }
        }
        if (!flag) {
            Assert.isTrue(false, "数据采集明细错误：" + "指标分类[" + imfee.getNormName() + "] - 指标名称[" + imfee.getNormfeeName() + "] 不存在，请检查文件");
        }
    }

    private void getFeeCustomsValue(CbGatherFeeVo fee, List<GatherImportVo> importVoList) {
        BigDecimal collValue;
        for (GatherImportVo importVo : importVoList) {
            String normfeeName = importVo.getUint().replaceAll("　", "").trim();
            if (fee.getNormfeeName().equals(normfeeName) && StringUtils.equals(fee.getDeptId(), String.valueOf(importVo.getDeptId()))) {
                collValue = importVo.getCollCalue();
                if (collValue != null) {
                    fee.setCollCalue(collValue);
                    fee.setCompireCalue(importVo.getCompireCalue());
                }
                break;
            }
        }
    }

    /**
     * 创建积分数据
     *
     * @param entity
     */
    @Override
    public void insertGatherStatisData(CbGatherVo entity) {
        // TODO: 2023/9/1  写入积分表及积分明细
        CbGatherVo gatherVo = this.queryById(entity.getId()); // 采集数据对象
        CbIntegralVo integralVo = new CbIntegralVo();
        integralVo.setGatherId(gatherVo.getId());
        integralVo.setCbId(gatherVo.getDeptId());
        integralVo.setCbName(sysDeptService.selectDeptById(Convert.toLong(gatherVo.getDeptId())).getDeptName());
        integralVo.setDataType(gatherVo.getType());
        integralVo.setYear(gatherVo.getYear());
        integralVo.setMonth(gatherVo.getMonth());
        integralVo.setQuarter(DateUtils.getQuarterByTime(DateUtils.getNowDate()));
        this.figureIntegralData(gatherVo, integralVo, 0);// 构造分析数据详情(绩效积分)
        integralService.saveEntity(integralVo);

        this.figureIntegralData(gatherVo, integralVo, 1);// 构造分析数据详情(采集积分)
        integralVo.setId(null);
        integralVo.getFeeVoList().forEach(item -> {
            item.setId(null);
        });
        integralService.saveEntity(BeanUtil.copyProperties(integralVo, CbIntegralVo.class));
    }

    /**
     * 海关特殊月报业务处理
     *
     * @param file
     * @param type
     * @param mark 标识类型(1:海关部分贸易报表,2:海关进出口总值报表)
     * @throws IOException
     */
    @Override
    public void customsUpload(MultipartFile file, String type, String mark) throws IOException {
        //海关部分贸易报表
        if (StringUtils.equals(mark, "1")) {
            EasyExcel.read((File) FileUtils.multipartFileToFile(file), GatherImportVo.class, new GatherDataCustomsListener(type)).extraRead(CellExtraTypeEnum.MERGE).sheet().headRowNumber(3).doRead();
        } else {
            //海关进出口总值报表
            String title;
            List<TaxEnterpriseItemVo> listTaxen;
            //筛选
            List<GatherImportVo> cachedDataList = new ArrayList<>();
            //文件流
            FileInputStream fileInputStream = (FileInputStream) file.getInputStream();
            //获取excel解析集合
            TaxEnterpriseTableVo taxEnterpriseTableVo = TaxReadEnterpriseExcelUtil.creatWorkBook(fileInputStream, file.getOriginalFilename());
            List<CbGatherVo> gatherTaxenVoList = null;
            List<CbGatherVo> cbGatherVoList = new ArrayList<>();
            if (taxEnterpriseTableVo != null) {
                //表头
                title = taxEnterpriseTableVo.getAccountVo().getTitle();
                //获取exce第一组数据
                listTaxen = taxEnterpriseTableVo.getAccountVo().getList();
                //判断是否存再相同税保区
                long count = listTaxen.stream().map(TaxEnterpriseItemVo::getName).distinct().count();
                if (listTaxen.size() != count) {
                    Assert.isTrue(false, "找到多个相同保税区采集数据，请检查文件");
                }
                //获取年月条件查询指标数据
                gatherTaxenVoList = this.handCbGatherVoList(taxEnterpriseTableVo, cachedDataList, type, title);
                //数据筛选
                List<GatherImportVo> cachedDataLists = cachedDataList.stream().filter(vo -> StringUtils.equals(vo.getUint(), "进出口总值")).collect(Collectors.toList());
                //业务处理
                gatherTaxenVoList.stream().forEach(cbGather -> {
                    CbGatherVo cbGatherVo = this.getInfoByColl(cbGather.getId());
                    //CbGatherVo重新组装数据(减少代码循环次数)
                    CbGatherVo newCbGatherVo = BeanUtil.toBean(cbGatherVo, CbGatherVo.class);
                    //所有指标集合数据
                    cbGatherVoList.add(newCbGatherVo);
                    //业务判断处理
                    if (cbGatherVo.getApproveStatus().equals("2")) {
                        SysDept dept = sysDeptService.selectDeptById(Long.valueOf(cbGatherVo.getDeptId()));
                        Assert.isTrue(false, dept.getDeptName() + "该数据已审核通过，导入已终止！！");
                    }
                    if (cbGatherVo.getReportStatus().equals("1")) {
                        SysDept dept = sysDeptService.selectDeptById(Long.valueOf(cbGatherVo.getDeptId()));
                        Assert.isTrue(false, dept.getDeptName() + "该数据已完成上报，导入已终止！！");
                    }
                    //入库
                    saveCustomsTodatabase(cbGatherVo, cachedDataLists, false);
                });
            }
            //企业活跃数(区保税值(多组保税区名称相同)判断根据excel第一组数据判断处理，减少代码臃肿，优化性能)
            if (taxEnterpriseTableVo.getNumVo() != null) {
                List<GatherImportVo> cachedDataLists = cachedDataList.stream().filter(vo -> StringUtils.contains(vo.getUint(), "活跃企业数")).collect(Collectors.toList());
                cbGatherVoList.stream().forEach(cbGatherVo -> {
                    cbGatherVo = this.getInfoByColl(cbGatherVo.getId());
                    //入库
                    saveCustomsTodatabase(cbGatherVo, cachedDataLists, true);
                });
            }
        }
    }

    /**
     * 海关报表业务
     *
     * @param id
     * @return
     */
    public CbGatherVo getCustomsInfoByColl(String id) {
        CbGatherVo gatherVo = this.baseMapper.queryById(id);
        List<CbGatherFeeVo> gatherFeeVoList = Lists.newArrayList();
        if (gatherVo.getType().equals("0")) { // 量化指标数据
            List<CbNorm> normList = normService.lambdaQuery().eq(CbNorm::getStatus, "1")
                    .eq(CbNorm::getDelFlag, "0")
                    .orderByAsc(CbNorm::getSort)
                    .list();
            for (CbNorm norm : normList) {
                List<CbNormFee> normFeeList = normFeeService.lambdaQuery()
                        .eq(CbNormFee::getStatus, "1")
                        .eq(CbNormFee::getDelFlag, "0")
                        .eq(CbNormFee::getNormId, norm.getId())
                        .orderByAsc(CbNormFee::getSort)
                        .list();
                for (CbNormFee normFee : normFeeList) {
                    CbGatherFeeVo gatherFeeVo = new CbGatherFeeVo();
                    gatherFeeVo.setDeptId(gatherVo.getDeptId());
                    gatherFeeVo.setNormId(norm.getId());
                    gatherFeeVo.setNormName(norm.getName());
                    gatherFeeVo.setNormfeeId(normFee.getId());
                    gatherFeeVo.setNormfeeName(normFee.getFname());
                    gatherFeeVo.setGatherId(id);
                    gatherFeeVoList.add(gatherFeeVo);
                }
            }
        }
        gatherVo.setFeeLists(gatherFeeVoList);
        return gatherVo;
    }

    /**
     * 进出口总值和活跃企业数
     * exce业务处理封装集合
     *
     * @param taxEnterpriseTableVo
     * @param type
     * @param title
     * @return
     */
    public List<CbGatherVo> handCbGatherVoList(TaxEnterpriseTableVo taxEnterpriseTableVo, List<GatherImportVo> cachedDataList, String type, String title) {
        List<CbGatherVo> gatherVoList;
        StringBuilder sbuilder = new StringBuilder();
        CbGatherVo gatherVo = new CbGatherVo();
        //企业进出口总值集合
        List<TaxEnterpriseItemVo> listTaxen = taxEnterpriseTableVo.getAccountVo().getList();
        //活跃企业数集合
        List<TaxEnterpriseItemVo> listTaxenNum = taxEnterpriseTableVo.getNumVo().getList();
        AtomicInteger index = new AtomicInteger(0);
        //进出口总值及活跃企业数
        listTaxen.stream().forEach(vo -> {
            GatherImportVo inOutVo = new GatherImportVo();
            GatherImportVo firmVo = new GatherImportVo();
            SysDept dept = sysDeptService.selectDeptByEmail(vo.getCode());
            if (index.get() == 0) {
                sbuilder.append(dept.getDeptId());
            } else {
                sbuilder.append(",");
                sbuilder.append(dept.getDeptId());
            }
            inOutVo.setUint("进出口总值");
            inOutVo.setCollCalue(new BigDecimal(vo.getValue()));
            inOutVo.setCompireCalue(vo.getCompireCalue());
            inOutVo.setDeptId(dept.getDeptId());
            cachedDataList.add(inOutVo);
            if (listTaxenNum.size() > index.get()){
                TaxEnterpriseItemVo itemVo = listTaxenNum.get(index.get());
                firmVo.setUint("活跃企业数");
                firmVo.setCollCalue(new BigDecimal(itemVo.getValue()));
                firmVo.setCompireCalue(itemVo.getCompireCalue());
                firmVo.setDeptId(dept.getDeptId());
                cachedDataList.add(firmVo);
            }
            index.getAndIncrement();
        });
        String yearMonth = title.split("-")[1];
        gatherVo.setYear(yearMonth.split("年")[0]);
        gatherVo.setMonth(yearMonth.split("年")[1].substring(0, 2).trim());
        gatherVo.setDeptId(sbuilder.toString());
        gatherVo.setType(type);
        //条件查询
        gatherVoList = this.baseMapper.queryList(gatherVo);
        return gatherVoList;
    }


    public void test(List<TaxEnterpriseItemVo> listTaxen) {
        StringBuilder sbuilder = new StringBuilder();
        AtomicInteger index = new AtomicInteger(0);
        listTaxen.stream().forEach(vo -> {
            SysDept dept = sysDeptService.selectDeptByName(vo.getName());
            if (index.get() == 0) {
                sbuilder.append(dept.getDeptId());
            } else {
                sbuilder.append(",");
                sbuilder.append(dept.getDeptId());
            }
            index.getAndIncrement();
        });
    }

    /**
     * 构造分析数据详情
     *
     * @param gatherVo   数据采集实体
     * @param integralVo 数据分析实体
     * @param type       数据分析类型 0=绩效积分 1=采集积分
     */
    private void figureIntegralData(CbGatherVo gatherVo, CbIntegralVo integralVo, int type) {

        List<CbIntegralFeeVo> integralFeeVoList = new ArrayList<>();
        BigDecimal sumScore = new BigDecimal(0);
        for (CbGatherFeeVo feeVo : gatherVo.getFeeLists()) {
            if (ObjectUtil.isNull(feeVo.getCollCalue())) continue;

            CbIntegralFeeVo integralFeeVo = new CbIntegralFeeVo();
            integralFeeVo.setGatherTime(feeVo.getCreateTime());
            integralFeeVo.setGatherBy(feeVo.getCreateBy());
            integralFeeVo.setNormfeeId(feeVo.getId());
            integralFeeVo.setNormfeeName(feeVo.getNormfeeName());
            integralFeeVo.setWarningFlag("0");

            if (gatherVo.getType().equals("0")) { // 量化指标数据
                CbNormFeeVo normFeeVo = normFeeService.queryById(feeVo.getNormfeeId());
                BigDecimal collValue = feeVo.getCollCalue(); // 采集值
                BigDecimal collCK = normFeeVo.getCollValue(); // 采集参考值
                BigDecimal collDtoValue = collCK.add(collCK.multiply(comBFB(normFeeVo.getDfRetio())));  // 采集上公差
                BigDecimal collDtuValue = collCK.subtract(collCK.multiply(comBFB(normFeeVo.getDfRetio()))); // 采集下公差

                BigDecimal msCK = normFeeVo.getMeritsValue(); // 绩效参考值
                BigDecimal msDtoValue = msCK.add(msCK.multiply(comBFB(normFeeVo.getMsRetio())));  // 绩效上公差
                BigDecimal msDtuValue = msCK.subtract(msCK.multiply(comBFB(normFeeVo.getMsRetio()))); // 绩效下公差
                if (type == 0) { // 绩效积分
                    setintegralFeeSocre(collValue, msDtoValue, msDtuValue, integralFeeVo, normFeeVo.getFnumber());
                } else if (type == 1) { // 采集积分
                    setintegralFeeSocre(collValue, collDtoValue, collDtuValue, integralFeeVo, normFeeVo.getFnumber());
                }
            }
            sumScore = sumScore.add(integralFeeVo.getScore());
            integralFeeVoList.add(integralFeeVo);
        }
        integralVo.setSumScore(sumScore);
        integralVo.setType(String.valueOf(type));
        integralVo.setFeeVoList(integralFeeVoList);
    }

    private void setintegralFeeSocre(BigDecimal msCK, BigDecimal DtoValue, BigDecimal DtuValue, CbIntegralFeeVo
            integralFeeVo, BigDecimal fnumber) {
        integralFeeVo.setCollValue(msCK);
        if (!bjdtodtu(msCK, DtoValue, DtuValue)) { // 异常数据
            integralFeeVo.setScore(BigDecimal.ZERO);
            integralFeeVo.setWarningFlag("1");
        } else {
            integralFeeVo.setScore(fnumber);
        }
    }

    /**
     * 海关一般报表导入
     *
     * @param headLists
     * @param cachedDataList
     * @param type
     */
    public void saveImportCustomsData(List<String> headLists, List<GatherImportVo> cachedDataList, String
            type) {
        //获取部门deptId
        StringBuilder sbuilder = new StringBuilder();
        //excel数据NormName过滤
        List<GatherImportVo> list = cachedDataList.stream().filter(vo -> NumberUtil.isNumber(vo.getNormName())).collect(Collectors.toList());
        //过滤是否存在相同的保税区名称
        long count = list.stream().map(GatherImportVo::getNormfeeName).distinct().count();
        if (list.size() != count) {
            Assert.isTrue(false, "找到多个相同保税区采集数据，请检查文件");
        }
        //过滤Uint数据
        List<GatherImportVo> listAll = cachedDataList.stream().filter(vo -> StringUtils.isNotBlank(vo.getUint())).collect(Collectors.toList());
        //变量
        AtomicInteger index = new AtomicInteger(0);
        //deptId变量
        AtomicReference<Long> deptId = new AtomicReference<>(0L);
        //Uint数据业务处理(每个对象deptId赋值)
        listAll.stream().forEach(vo -> {
            if (StringUtils.isNotBlank(vo.getNormName())) {
                //获取deptId
                SysDept dept = sysDeptService.selectDeptByEmail(vo.getNormName());
                if (index.get() == 0) {
                    sbuilder.append(dept.getDeptId());
                } else {
                    sbuilder.append(",");
                    sbuilder.append(dept.getDeptId());
                }
                deptId.set(dept.getDeptId());//记录deptId
                vo.setDeptId(dept.getDeptId());
            } else {
                vo.setDeptId(deptId.get());
            }
            index.getAndIncrement();
        });
        CbGatherVo gatherVo = new CbGatherVo();
        String yearMonth = headLists.get(1);
        gatherVo.setYear(yearMonth.split("年")[0]);
        gatherVo.setMonth(yearMonth.split("-")[1].substring(5, 7).trim());
        gatherVo.setDeptId(sbuilder.toString());
        gatherVo.setType(type);
        //条件查询
        List<CbGatherVo> gatherVoList = this.baseMapper.queryList(gatherVo);
        // 写入数据
        gatherVoList.stream().forEach(cbGather -> {
            //获取指标分类中包含所有指标明细集合对象
            CbGatherVo cbGatherVo = this.getInfoByColl(cbGather.getId());
            if (cbGatherVo.getApproveStatus().equals("2")) {
                SysDept dept = sysDeptService.selectDeptById(Long.valueOf(cbGatherVo.getDeptId()));
                Assert.isTrue(false, dept.getDeptName() + "该数据已审核通过，无法继续导入！");
            }
            if (cbGatherVo.getReportStatus().equals("1")) {
                SysDept dept = sysDeptService.selectDeptById(Long.valueOf(cbGatherVo.getDeptId()));
                Assert.isTrue(false, dept.getDeptName() + "该数据已完成上报，无法继续导入！");
            }
            //添加数据
            saveCustomsTodatabase(cbGatherVo, listAll, true);
        });

    }


    @Override
    public TaxTableVo readExcelData(MultipartFile file, String type) {
        //获取excel内部数据
        TaxTableVo taxTableVo = TaxReadExcelUtil.creatWorkBook(file);
        //取出数据集合
        List<TaxAreaVo> list = taxTableVo.getList();
        //创建数据保存集合
        List<CbNormFee> normFeeList = normFeeService.list();
        List<CbGather> waringGatherList = Lists.newArrayList();
        List<CbGatherFee> cbGatherFeeList = new ArrayList<>();
        list.forEach(item -> {
            //模板主键
            String deptId = sysDeptService.selectDeptByName(item.getName()).getDeptId().toString();
            for (NormCategoryVo normCategoryVo : item.getNormCategoryList()) {
                //指标分类名称
                String normClass = normCategoryVo.getNormClass();
                //遍历数据
                label:
                for (NormItemVo data : normCategoryVo.getList()) {
                    //今年
                    CbGatherFeeVo cbGatherFeeVoYear = new CbGatherFeeVo();
                    //gather_id  根据保税区和时间获得
                    String[] year1 = item.getYearTitle().split("年");
                    String[] split1 = item.getYearTitle().split("—");
                    String month = null;
                    if (split1.length > 1) {
                        month = split1[1];
                        if (month.length() == 2) {
                            month = "0" + month;
                        }
                    } else {
                        month = year1[1];
                    }

                    List<CbGather> gatherList = gatherService.getBaseMapper()
                            .selectList(
                                    new LambdaQueryWrapper<CbGather>()
                                            .eq(CbGather::getDeptId, deptId)
                                            .eq(CbGather::getYear, year1[0])
                                            .eq(CbGather::getMonth, month)
                                            .eq(CbGather::getType, type));
                    if (gatherList.isEmpty()) {
                        //新增cb_gather数据,当前数据采集的时间,保税区
                        CbGather cbGather = new CbGather();
                        cbGather.setType(type);
                        cbGather.setCreateTime(new Date());
                        cbGather.setCollStatus(0 + "");
                        cbGather.setReportStatus(0 + "");
                        cbGather.setApproveStatus(0 + "");
                        cbGather.setDeptId(deptId);
                        cbGather.setYear(year1[0]);
                        cbGather.setMonth(month);
                        //保存
                        cbGatherService.save(cbGather);
                        //将新保存的对象添加到上述的集合中
                        gatherList.add(cbGather);
                    }
                    waringGatherList.add(gatherList.get(0));
                    cbGatherFeeVoYear.setGatherId(gatherList.get(0).getId());
                    //创建人
                    cbGatherFeeVoYear.setCreateBy(SecurityUtils.getLoginUser().getUser().getNickName());
                    //创建时间
                    cbGatherFeeVoYear.setCreateTime(new Date());
                    //删除标识
                    cbGatherFeeVoYear.setDelFlag(0 + "");
                    //部门ID
                    cbGatherFeeVoYear.setDeptId(deptId);
                    //格子值
                    if (data.getYear().equals("——") || data.getYear().equals("/") || data.getYear().equals("—")) {
                        data.setYear("0");
                    }

                    cbGatherFeeVoYear.setCollCalue(new BigDecimal(data.getYear()));
                    //同比值
                    cbGatherFeeVoYear.setCompireCalue(data.getYearCompare());
                    //指标相关
                    String normName = data.getNormName().trim();
                    List<CbNormFee> cbNormFees = normFeeList.stream().filter(q -> q.getFname().equals(normName)).collect(Collectors.toList());
                    if (cbNormFees.isEmpty()) {
                        String trim = normName.substring(0, 4).trim();
                        cbNormFees = normFeeList.stream().filter(q -> q.getFname().contains(trim)).collect(Collectors.toList());
                        if (cbNormFees.isEmpty()) continue;
                    }
                    CbNormFee cbNormFee = cbNormFees.get(0);
                    cbGatherFeeVoYear.setNormId(cbNormFee.getNormId());
                    String split = normClass.split("\\n")[0].trim();
                    cbGatherFeeVoYear.setNormName(split.trim());
                    cbGatherFeeVoYear.setNormfeeId(cbNormFee.getId());
                    cbGatherFeeVoYear.setNormSort(cbNormFee.getSort());
                    cbGatherFeeVoYear.setNormFeeSort(cbNormFee.getSort());
                    cbGatherFeeVoYear.setNormfeeName(data.getNormName());
                    //保存数据采集
                    cbGatherFeeList.add(cbGatherFeeVoYear);
                }
            }
        });

        List<CbGatherFee> savaBatch = new ArrayList<>();
        //循环保存
        for (CbGatherFee cbGatherFee : cbGatherFeeList) {
            List<CbGatherFee> fees = gatherFeeService.getBaseMapper().selectList(new LambdaQueryWrapper<CbGatherFee>().eq(CbGatherFee::getDeptId, cbGatherFee.getDeptId()).eq(CbGatherFee::getGatherId, cbGatherFee.getGatherId()).eq(CbGatherFee::getNormfeeId, cbGatherFee.getNormfeeId()));
            if (fees.size() > 0) {
                //执行修改
                CbGatherFee cbGatherFee1 = fees.get(0);
                BeanUtils.copyProperties(cbGatherFee, cbGatherFee1);
                gatherFeeService.updateById(cbGatherFee1);
                continue;
            }
            savaBatch.add(cbGatherFee);
        }
        if (CollectionUtil.isNotEmpty(savaBatch)) gatherFeeService.saveBatch(savaBatch);

        List<CbGather> distinctList = waringGatherList.stream()
                .distinct()
                .collect(Collectors.toList());
        this.oldSaveWaringData(distinctList); // 生成预警数据
        return taxTableVo;
    }

    /**
     * 生成预警数据
     *
     * @param gatherList
     */
    private void oldSaveWaringData(List<CbGather> gatherList) {
        Long userId = SecurityUtils.getUserId();
        Thread thread = new Thread(new Runnable() {
            public void run() {
                List<CbGather> newList = Lists.newArrayList();
                for (CbGather gather : gatherList) {
                    boolean falg = true;
                    if (CollectionUtil.isNotEmpty(newList)) {
                        for (CbGather che : newList) {
                            if (gather.getId().equals(che.getId())) {
                                falg = false;
                                break;
                            }
                        }
                    }
                    if (falg == true) newList.add(gather);
                }
                newList.forEach(item -> {
                    // 采集数据状态
                    CbGather gather = new CbGather();
                    gather.setCollStatus("1");
                    gather.setId(item.getId());
                    gatherService.updateById(gather);
                    CbGatherVo gatherVo = queryById(item.getId(), userId);
                    checkWraingData(gatherVo);
                });
            }
        });
        thread.start();
    }

    /**
     * 获取 保税区年度进出口总值
     *
     * @return
     */
    @Override
    public List<CbTotexpVo> queryInoutYearList(String recentYears) {
        return gatherMapper.selectInoutYearList(recentYears);
    }

    @Override
    public void updateTmp(String year, String month, String deptId) {
        gatherMapper.updateTmp(year, month, deptId);
    }

    /**
     * 保税区年度进出口总值 含年-月份
     *
     * @return
     */
    @Override
    public List<CbTotexpVo> queryInoutYearPercentage(String recentYears) {
        return gatherMapper.selectInoutYearPercentage(recentYears);
    }

    /**
     * 海关
     * 写入解析数据
     *
     * @param info
     * @param importVoList
     */
    private void saveCustomsTodatabase(CbGatherVo info, List<GatherImportVo> importVoList, boolean flag) {
        for (CbGatherFeeVo fee : info.getFeeLists()) {
            getFeeCustomsValue(fee, importVoList);
        }
        saveCustomsEntity(info, flag); // 保存
    }

    /**
     * 审核流程获取实体 回调函数
     *
     * @param fromDataId
     * @return
     */
    @Override
    public Object getEntity(String fromDataId) {
        return this.queryById(fromDataId);
    }


    /**
     * 处理节点 回调函数
     *
     * @param fromDataId
     * @param pspStatus
     */
    @Override
    public void cutInServiceFrom(String fromDataId, String pspStatus) {
        CbGather entity = this.gatherMapper.selectById(fromDataId);
        switch (pspStatus) {
            case WorkFlowStatus.APPROVE:  // 通过
                entity.setApproveStatus("2");
                entity.setApproveTime(new Date());
                entity.setApproveName(SecurityUtils.getLoginUser().getUser().getNickName());
                cbGatherService.updateById(entity);
                break;
            case WorkFlowStatus.REJECT:   // 拒绝
                entity.setApproveStatus("3");
                entity.setApproveTime(new Date());
                entity.setApproveName(SecurityUtils.getLoginUser().getUser().getNickName());
                cbGatherService.updateById(entity);
                break;
            case WorkFlowStatus.GOBACK:   // 退回

                break;
            default:

                break;
        }
    }
}
