package com.hjy.module.service.gather.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDictData;
import com.hjy.module.domain.garden.TbGardenBaseData;
import com.hjy.module.domain.gether.CbGather;
import com.hjy.module.mapper.article.CbArticleMapper;
import com.hjy.module.mapper.garden.TbGardenBaseDataMapper;
import com.hjy.module.mapper.gather.CbGatherStatisticsBsqMapper;
import com.hjy.module.mapper.gather.CbGatherStatisticsWbMapper;
import com.hjy.module.service.gather.ICbGatherStatisticsBsqService;
import com.hjy.module.vo.article.CbArticleVo;
import com.hjy.module.vo.gather.statistics.comprehensive.MonthTotalValueVO;
import com.hjy.module.vo.statistics.NormCollValueVo;
import com.hjy.system.mapper.SysDeptMapper;
import com.hjy.system.service.ISysDictDataService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 保税区大屏Service业务层处理
 *
 * @author Tellsea
 * @date 2023-08-30
 */
@Service
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbGatherStatisticsBsqServiceImpl  extends ServiceImpl<CbGatherStatisticsBsqMapper, CbGather> implements ICbGatherStatisticsBsqService {
    private final CbGatherStatisticsWbServiceImpl cbGatherStatisticsWbService;

    private final CbGatherStatisticsWbMapper cbGatherStatisticsWbMapper;

    private final CbArticleMapper cbArticleMapper;

    private final SysDeptMapper sysDeptMapper;

    private final TbGardenBaseDataMapper tbGardenBaseDataMapper;

    private final ISysDictDataService dictDataService;
    @Override
    public List<CbArticleVo> getMajorProject(String deptId) {
        CbArticleVo cbArticleVo = new CbArticleVo();
        cbArticleVo.setDeptId(deptId);
        cbArticleVo.setStatus(3);
        return cbArticleMapper.queryList(cbArticleVo);
    }

    @Override
    public Map<Integer, Map<String, List<NormCollValueVo>>> getTotalValue(Integer deptId, String year, String month) {
        return cbGatherStatisticsWbService.getTotalValue(deptId, year, month);
    }

    @Override
    public List<MonthTotalValueVO> getMonthTotalValue(Integer deptId) {
        return cbGatherStatisticsWbMapper.getMonthTotalValue(String.valueOf(LocalDateTime.now().getYear()),deptId);
    }


    @Override
    public List<TbGardenBaseData> enterpriseDirectories(String deptId) {
        List<TbGardenBaseData> sysDepts = tbGardenBaseDataMapper.selectList(new LambdaQueryWrapper<TbGardenBaseData>().eq(TbGardenBaseData::getParentId, deptId));
        return sysDepts;
    }

    @Override
    public AjaxResult getEnterpriseStructure(String deptId) {
        List<TbGardenBaseData> list = this.baseMapper.getEnterpriseStructure(deptId);
        return AjaxResult.success(countDataOccurrences(list, "enterprise_structure"));
    }

    @Override
    public AjaxResult getBusinessEvents(String deptId) {
        List<TbGardenBaseData> list = this.baseMapper.getBusinessEvents(deptId);
        return AjaxResult.success(countDataOccurrences(list, "business_events"));
    }

    private List<Map<String, Object>> countDataOccurrences(List<TbGardenBaseData> list, String dictType) {
        SysDictData sysDictData = new SysDictData();
        sysDictData.setDictType(dictType);
        List<SysDictData> dictList = dictDataService.selectDictDataList(sysDictData);

        Map<String, Integer> countMap = dictList.stream()
                .collect(Collectors.toMap(SysDictData::getDictLabel, e -> 0, (v1, v2) -> v1, LinkedHashMap::new));

        //获取字典-判断集合中是否包含，如果包含则+1
        list.stream()
                .flatMap(tbGardenBaseData -> {
                    if (dictType.equals("enterprise_structure")) {
                        return Arrays.stream(tbGardenBaseData.getEnterpriseStructure().split(","));
                    } else if (dictType.equals("business_events")) {
                        return Arrays.stream(tbGardenBaseData.getBusinessEvents().split(","));
                    }
                    return null;
                })
                .filter(countMap::containsKey)
                .forEach(s -> countMap.put(s, countMap.get(s) + 1));

        //处理返回格式
        return countMap.keySet().stream()
                .map(s -> {
                    HashMap<String, Object> vo = new HashMap<>();
                    vo.put("key", s);
                    vo.put("value", countMap.get(s));
                    return vo;
                })
                .collect(Collectors.toList());
    }

}
