package com.hjy.module.service.gather.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.exception.ServiceException;
import com.hjy.module.domain.garden.TbEnterpriseScreenData;
import com.hjy.module.domain.garden.TbEnterpriseTradePerformance;
import com.hjy.module.domain.garden.TbGardenBaseData;
import com.hjy.module.domain.gether.CbGather;
import com.hjy.module.domain.gether.CbGatherFee;
import com.hjy.module.mapper.gather.CbGatherStatisticsQyMapper;
import com.hjy.module.service.garden.ITbEnterpriseTradePerformanceService;
import com.hjy.module.service.garden.ITbGardenBaseDataService;
import com.hjy.module.service.gather.ICbGatherFeeService;
import com.hjy.module.service.gather.ICbGatherService;
import com.hjy.module.service.gather.ICbGatherStatisticsQyService;
import com.hjy.module.vo.garden.DeptVo;
import com.hjy.module.vo.garden.EnterpriseMainIndexVo;
import com.hjy.module.vo.gather.EnterpriseDataVo;
import com.hjy.module.vo.gather.statistics.comprehensive.DeptVO;
import com.hjy.module.vo.gather.statistics.comprehensive.GylgxVO;
import com.hjy.module.vo.gather.statistics.comprehensive.JyzsyVO;
import com.hjy.module.vo.gather.statistics.comprehensive.QyzyzbVO;
import com.hjy.system.mapper.SysDeptMapper;
import com.hjy.system.service.ISysDeptService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 企业大屏Service业务层处理
 *
 * @author Tellsea
 * @date 2023-10-11
 */
@Service
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbGatherStatisticsQyServiceImpl extends ServiceImpl<CbGatherStatisticsQyMapper, CbGather> implements ICbGatherStatisticsQyService {

    ITbEnterpriseTradePerformanceService tbEnterpriseTradePerformanceService;
    @Autowired
    private ITbEnterpriseTradePerformanceService tbEnterpriseTradePerformanceService(ITbEnterpriseTradePerformanceService tbEnterpriseTradePerformanceService){
        return this.tbEnterpriseTradePerformanceService = tbEnterpriseTradePerformanceService;
    }

    ISysDeptService sysDeptService;
    @Autowired
    private ISysDeptService sysDeptService(ISysDeptService sysDeptService){
        return this.sysDeptService = sysDeptService;
    }
    SysDeptMapper sysDeptMapper;
    @Autowired
    private SysDeptMapper sysDeptMapper(SysDeptMapper sysDeptMapper){
        return this.sysDeptMapper = sysDeptMapper;
    }
    ITbGardenBaseDataService gardenBaseDataService;
    @Autowired
    ITbGardenBaseDataService gardenBaseDataService(ITbGardenBaseDataService gardenBaseDataService){
        return this.gardenBaseDataService = gardenBaseDataService;
    }
    ICbGatherService cbGatherService;
    @Autowired
    private ICbGatherService cbGatherService(ICbGatherService cbGatherService){
        return this.cbGatherService = cbGatherService;
    }
    ICbGatherFeeService cbGatherFeeService;
    @Autowired
    private ICbGatherFeeService cbGatherFeeService(ICbGatherFeeService cbGatherFeeService){
        return this.cbGatherFeeService = cbGatherFeeService;
    }

    @Override
    public DeptVO getqyjbxx(Integer deptId) {
        return this.baseMapper.getqybxx(deptId);
    }

    @Override
    public List<GylgxVO> getgylgxgs(Integer deptId) {
        return this.baseMapper.getgylgxgy(deptId);
    }
    @Override
    public List<EnterpriseDataVo> jckzzyqjnzb(String companyId) {
       //获取该企业信息
        SysDept sysDept = sysDeptService.selectDeptById(Long.valueOf(companyId));
        if(sysDept == null){
            throw new ServiceException("该企业信息错误!");
        }
        if(sysDept.getType() != 3){
            throw new ServiceException("当前不属于企业");
        }
        //获取企业内保税区信息
        List<String> deptIdList = new ArrayList<>(Arrays.asList(sysDept.getAncestors().split(",")));
        //取出保税区
        SysDept parent = new SysDept();
        for(String item :deptIdList){
            SysDept dept = sysDeptService.selectDeptById(Long.valueOf(item));
            if(dept != null){
                if(dept.getType() == 2){
                    parent = dept;
                    break;
                }
            }
        }
        if(parent == null){
            throw new ServiceException("当前企业未绑定保税区,请先前往进行绑定!");
        }
        int month = LocalDateTime.now().getMonth().getValue() - 1;
        int year = LocalDateTime.now().getYear();
        //获取当前企业的值  根据  年 - 月 - 公司主键
        List<TbEnterpriseTradePerformance> selectList = tbEnterpriseTradePerformanceService.getBaseMapper().selectList(new LambdaQueryWrapper<TbEnterpriseTradePerformance>().eq(TbEnterpriseTradePerformance::getYear, year).eq(TbEnterpriseTradePerformance::getMonth, month).eq(TbEnterpriseTradePerformance::getDeptId, companyId));
        if(selectList.isEmpty()){
            Integer yearS = year;
            while (month != 0 || selectList.isEmpty()){
                selectList = tbEnterpriseTradePerformanceService.getBaseMapper().selectList(new LambdaQueryWrapper<TbEnterpriseTradePerformance>().eq(TbEnterpriseTradePerformance::getYear, year).eq(TbEnterpriseTradePerformance::getMonth, month).eq(TbEnterpriseTradePerformance::getDeptId, companyId));
                month--;
                if(month == 0){
                    month = 12;
                    yearS--;
                    if(year - yearS == 2){
                        break;
                    }
                }
            }
            if(selectList.isEmpty()){
                selectList.add(new TbEnterpriseTradePerformance());
            }
        }
        //获取所有的企业
//        List<TbEnterpriseTradePerformance> tradePerformanceList = tbEnterpriseTradePerformanceService.list().stream().filter(f -> deptList.contains(f.getDeptId()) && f.getYear().equals(LocalDateTime.now().getYear() + "") && f.getMonth().equals(LocalDateTime.now().getMonth().getValue() - 1 + "")).collect(Collectors.toList());
        //获取保税区的数据
        String stringMonth = String.valueOf(month).length() == 1 ? "0" + month + "月" : month + "月";
        List<CbGather> gatherList = cbGatherService.getBaseMapper().selectList(new LambdaQueryWrapper<CbGather>().eq(CbGather::getYear, year).eq(CbGather::getMonth, stringMonth).eq(CbGather::getDeptId,parent.getDeptId()).eq(CbGather::getCollStatus,1));
        if(gatherList.isEmpty()){
            Integer yearS = year;
//            gatherList.add(new CbGatherVo());
            while (month != 0 && gatherList.isEmpty()){
                stringMonth = String.valueOf(month).length() == 1 ? "0" + month + "月" : month + "月";
                gatherList = cbGatherService.getBaseMapper().selectList(new LambdaQueryWrapper<CbGather>().eq(CbGather::getYear, yearS).eq(CbGather::getMonth, stringMonth).eq(CbGather::getDeptId,parent.getDeptId()).eq(CbGather::getCollStatus,1));
                month--;
                if(month == 0){
                    month = 12;
                    yearS--;
                    if(year - yearS == 2){
                        break;
                    }
                }
            }

        }
        //进出口总值
        List<CbGatherFee> list1 = cbGatherFeeService.getBaseMapper().selectList(new LambdaQueryWrapper<CbGatherFee>().eq(CbGatherFee::getGatherId, gatherList.get(0).getId()).eq(CbGatherFee::getNormfeeId, 23));
        //经营总收入
        List<CbGatherFee> list2 = cbGatherFeeService.getBaseMapper().selectList(new LambdaQueryWrapper<CbGatherFee>().eq(CbGatherFee::getGatherId, gatherList.get(0).getId()).eq(CbGatherFee::getNormfeeId, 26));
        if(list1.isEmpty()){
            list1.add(new CbGatherFee());
        }
        if(list2.isEmpty()){
            list2.add(new CbGatherFee());
        }
        //创建返回值
        List<EnterpriseDataVo> result = new ArrayList<>();
        Double jckzz = 0d;
        Double jysr = 0d;
        if(!selectList.isEmpty()){
            TbEnterpriseTradePerformance performance = selectList.get(0);
            if (performance != null) {
                //1. 进出口总值
                if(performance.getJckzz() != null){
                    jckzz = performance.getJckzz();
                }
                //2. 当前公司的经营收入 和 保税区的经营收入
                if(selectList.get(0).getJysr() != null){
                    jysr = selectList.get(0).getJysr();
                }
            }
        }
        //1 进出口总值
        double sum1 = Double.parseDouble(list1.get(0).getCollCalue() == null?"0":String.valueOf(list1.get(0).getCollCalue()));
        //2 经营总收入
        double sum2 = Double.parseDouble(list2.get(0).getCollCalue() == null?"0":String.valueOf(list2.get(0).getCollCalue()));
        //赋值
        EnterpriseDataVo vo1 = new EnterpriseDataVo();
        vo1.setType("进出口总值");
//        vo1.setYear(LocalDateTime.now().getYear() + "");
        vo1.setDeptId(companyId);
        vo1.setDeptName(sysDept.getDeptName());
        vo1.setBsqName(parent.getDeptName());
        vo1.setValue(String.valueOf(jckzz));
        vo1.setSumValue(String.valueOf(sum1));
        if(sum1 != 0){
            vo1.setZb(String.format("%.2f",(jckzz / sum1) * 100) + "%");
        }else {
            vo1.setZb(String.format("%.2f",Double.parseDouble(jckzz * 100 + "")) + "%");
//            vo1.setZb(("100%"));
        }
        EnterpriseDataVo vo2 = new EnterpriseDataVo();
        vo2.setType("经营收入");
//        vo2.setYear(LocalDateTime.now().getYear() + "");
        vo2.setDeptId(companyId);
        vo2.setDeptName(sysDept.getDeptName());
        vo2.setBsqName(parent.getDeptName());
        vo2.setValue(String.valueOf(jysr));
        vo2.setSumValue(String.valueOf(sum2));
        if(sum2 != 0){
            vo2.setZb(String.format("%.2f",(jysr / sum2) * 100) + "%");
        }else {
            vo2.setZb(String.format("%.2f",Double.parseDouble(String.valueOf(jysr * 100) )) + "%");
//            vo2.setZb(("100%"));
        }
        //添加到集合
        result.add(vo1);
        result.add(vo2);
        return result;
    }


//    @Override
//    public List<EnterpriseDataVo> jckzzyqjnzb(String companyId) {
//        //当前园区占当前保税区的进出口总值
//        //获取企业信息
//        SysDept sysDept = sysDeptService.selectDeptById(Long.valueOf(companyId));
//        if(sysDept.getType() != 3){
//            throw new ServiceException("该对象不是企业类型!");
//        }
//        if(sysDept == null){
//            throw new ServiceException("企业信息有误!");
//        }
//        //获取保税区信息
//        SysDept deptById = sysDeptService.selectDeptById(sysDept.getParentId());
//        if(deptById == null){
//            throw new ServiceException("保税区信息有误!");
//        }
//        //创建返回值
//        List<EnterpriseDataVo> result = new ArrayList<>();
//        //获取数据
//        List<TbEnterpriseTradePerformance> performanceList = tbEnterpriseTradePerformanceService.list();
//        //取出本年数据
//        performanceList = performanceList.stream().filter(f -> f.getYear() != null ? f.getYear().equals(LocalDateTime.now().getYear() + "") : "".equals(LocalDateTime.now().getYear() + "") && f.getMonth().equals(LocalDateTime.now().getMonth().getValue() - 1 + "")).collect(Collectors.toList());
//        //获取该保税区下属的所有企业主键
//        List<String> deptIdList = sysDeptMapper.selectList(new LambdaQueryWrapper<SysDept>().eq(SysDept::getParentId, deptById.getDeptId())).stream().map(SysDept::getDeptId).map(Objects::toString).collect(Collectors.toList());
//        //过滤出当前保税区下属企业
//        List<TbEnterpriseTradePerformance> list = performanceList.stream().filter(f -> deptIdList.contains(f.getDeptId())).collect(Collectors.toList());
//        //去空
//        performanceList = performanceList.stream().filter(f -> f.getJysr() != null).collect(Collectors.toList());
//        //获取当前公司,本年的进出口总值信息
//        List<TbEnterpriseTradePerformance> tradePerformanceList = list.stream().filter(f -> f.getDeptId().equals(companyId)).sorted(Comparator.comparing(TbEnterpriseTradePerformance::getMonth).reversed()).limit(1L).collect(Collectors.toList());
//        //空值处理
//        List<TbEnterpriseTradePerformance> trade = tradePerformanceList.stream().filter(f -> f.getJckzz() != null).collect(Collectors.toList());
//        //累加当前公司的进出口总值
//        double jckzzSum = trade.get(0).getJckzz();
//        //累加当前保税区下属所有公司的进出海口总值
//        double jckzzTotalSum = list.stream().filter(f -> f.getJckzz() != null).collect(Collectors.toList()).stream().mapToDouble(TbEnterpriseTradePerformance::getJckzz).sum();
//        //2.经营收入
//        //获取当前企业的本年[经营收入]
//        double jysrSum = performanceList.stream().filter(f -> f.getDeptId().equals(companyId)).mapToDouble(TbEnterpriseTradePerformance::getJysr).sum();
//        //获取当前保税区所有企业的本年的[经营收入]
//        double jysrTotalSum = performanceList.stream().mapToDouble(TbEnterpriseTradePerformance::getJysr).sum();
//        //赋值
//        EnterpriseDataVo vo1 = new EnterpriseDataVo();
//        vo1.setType("进出口总值");
//        vo1.setYear(LocalDateTime.now().getYear() + "");
//        vo1.setDeptId(companyId);
//        vo1.setDeptName(sysDept.getDeptName());
//        vo1.setBsqName(deptById.getDeptName());
//        vo1.setValue(String.valueOf(jckzzSum));
//        vo1.setSumValue(String.valueOf(jckzzTotalSum));
//        vo1.setZb((jckzzSum / jckzzTotalSum) * 100 + "%");
//        EnterpriseDataVo vo2 = new EnterpriseDataVo();
//        vo2.setType("经营收入");
//        vo2.setYear(LocalDateTime.now().getYear() + "");
//        vo2.setDeptId(companyId);
//        vo2.setDeptName(sysDept.getDeptName());
//        vo2.setBsqName(deptById.getDeptName());
//        vo2.setValue(String.valueOf(jysrSum));
//        vo2.setSumValue(String.valueOf(jysrTotalSum));
//        vo2.setZb((jysrSum / jysrTotalSum) * 100 + "%");
//        //添加到集合
//        result.add(vo1);
//        result.add(vo2);
//        //返回
//        return result;
//    }

    @Override
    public void lnzxbj(String companyId) {

        List<TbEnterpriseTradePerformance> selectList = tbEnterpriseTradePerformanceService.getBaseMapper().selectList(new LambdaQueryWrapper<TbEnterpriseTradePerformance>().eq(TbEnterpriseTradePerformance::getDeptId, companyId));


    }

    @Override
    public List<QyzyzbVO> getqyzyzb(Integer deptId) {
        return this.baseMapper.getqyzyzb(deptId);
    }

    @Override
    public JyzsyVO qyjbxxhb(Integer deptId) {
        LocalDate date = LocalDate.now();
        DateTimeFormatter pattern = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        String begin = date.withDayOfMonth(1).format(pattern);
        String end = date.withDayOfMonth(date.lengthOfMonth()).format(pattern);
        String firstDayOfLastMonth = date.minusMonths(1).withDayOfMonth(1).format(pattern);
        String lastDayOfLastMonth = date.minusMonths(1).withDayOfMonth(date.minusMonths(1).lengthOfMonth()).format(pattern);
        JyzsyVO vo = this.baseMapper.getqyjbxxhb(deptId,begin,end);
        JyzsyVO vo1 = this.baseMapper.getqyjbxxhb(deptId,firstDayOfLastMonth,lastDayOfLastMonth);
        Double hb = (vo.getJysr() / vo1.getJysr()) - 1;
        vo.setHb(hb);
        vo.setDeptId(deptId);
        vo.setMaxNum(vo1.getJysr());
        return vo;
    }

    @Override
    public List<EnterpriseMainIndexVo> enterpriseMainIndex(String companyId) {

        SysDept sysDept = sysDeptService.selectDeptById(Long.valueOf(companyId));
        if(sysDept == null){
            throw new ServiceException("当前部门信息错误!");
        }
        //获取数据列表
        List<TbEnterpriseTradePerformance> selectList = tbEnterpriseTradePerformanceService.getBaseMapper().selectList(
                new LambdaQueryWrapper<TbEnterpriseTradePerformance>()
                        .eq(TbEnterpriseTradePerformance::getDeptId, companyId)
                        .orderByDesc(TbEnterpriseTradePerformance::getYear)
                        .orderByDesc(TbEnterpriseTradePerformance::getMonth)
        );
        //如果是空,返回
        if(selectList.isEmpty()){
            return new ArrayList<>();
        }
        //统计
        List<EnterpriseMainIndexVo> result = new ArrayList<>();
        //加工贸易进出口值
        EnterpriseMainIndexVo vo1 = new EnterpriseMainIndexVo();
        double jgmyjckz = selectList.stream().mapToDouble(TbEnterpriseTradePerformance::getJgmyjckz).sum();
        vo1.setKey("加工贸易进出口值");
        vo1.setValue(String.format("%.2f",jgmyjckz));
        vo1.setYear(selectList.get(0).getYear());
        vo1.setMonth(selectList.get(0).getMonth());
        //物流货物进出口值
        EnterpriseMainIndexVo vo2 = new EnterpriseMainIndexVo();
        double wlhwjckz = selectList.stream().mapToDouble(TbEnterpriseTradePerformance::getWlhwjckz).sum();
        vo2.setKey("物流货物进出口值");
        vo2.setValue(String.format("%.2f",wlhwjckz));
        vo2.setYear(selectList.get(0).getYear());
        vo2.setMonth(selectList.get(0).getMonth());
        //一般贸易进出口值
        EnterpriseMainIndexVo vo3 = new EnterpriseMainIndexVo();
        double ybmyjckz = selectList.stream().mapToDouble(TbEnterpriseTradePerformance::getYbmyjckz).sum();
        vo3.setValue(String.format("%.2f",ybmyjckz));
        vo3.setKey("一般贸易进出口值");
        vo3.setYear(selectList.get(0).getYear());
        vo3.setMonth(selectList.get(0).getMonth());
        //装进集合
        result.add(vo1);
        result.add(vo2);
        result.add(vo3);
        return result;
    }

    @Override
    public DeptVo abc(String deptId) {

        List<TbEnterpriseTradePerformance> selectList = tbEnterpriseTradePerformanceService.getBaseMapper().selectList(new LambdaQueryWrapper<TbEnterpriseTradePerformance>().eq(TbEnterpriseTradePerformance::getYear, LocalDateTime.now().getYear()).eq(TbEnterpriseTradePerformance::getMonth, LocalDateTime.now().getMonth().getValue() - 1).eq(TbEnterpriseTradePerformance::getDeptId, deptId));
        List<TbGardenBaseData> tbGardenBaseData = gardenBaseDataService.getBaseMapper().selectList(new LambdaQueryWrapper<TbGardenBaseData>().eq(TbGardenBaseData::getDeptId, deptId));
        if(selectList.isEmpty()){
            selectList.add(new TbEnterpriseTradePerformance());
        }
        if(tbGardenBaseData.isEmpty()){
            throw new ServiceException("当前企业信息有误!");
        }
        DeptVo deptVo = new DeptVo();
        TbEnterpriseTradePerformance performance = selectList.get(0);
        if(performance.getHgss() == null){
            performance.setHgss(0.00);
        }
        deptVo.setHgss(performance.getHgss());
        deptVo.setJyzsr(String.valueOf(performance.getJysr()));
        deptVo.setDetail(tbGardenBaseData.get(0).getJyfw());
        deptVo.setZczj(tbGardenBaseData.get(0).getZczj());
        deptVo.setClrq(tbGardenBaseData.get(0).getClrq());
        return deptVo;
    }

    @Override
    public AjaxResult queryList(String deptId) {
        for (int i = 0; i < 6; i++) {
            TbEnterpriseTradePerformance vo = tbEnterpriseTradePerformanceService.getBaseMapper()
                    .selectOne(new LambdaQueryWrapper<TbEnterpriseTradePerformance>()
                            .eq(TbEnterpriseTradePerformance::getDeptId, deptId)
                            .eq(TbEnterpriseTradePerformance::getYear, String.valueOf(LocalDateTime.now().getYear() - i)));
            if (Objects.nonNull(vo)){
                HashMap<String, Object> vos = new HashMap<>();
                vos.put("gmysgycz", generateData(vo.getGmysgycz(), vo.getYear(), 0.05));
                vos.put("jckzz", generateData(vo.getJckzz(), vo.getYear(), 0.03));
                vos.put("ckzz", generateData(vo.getCkzz(), vo.getYear(), 0.06));
                vos.put("jkzz", generateData(vo.getJkzz(), vo.getYear(), 0.04));
                vos.put("sjsywzje", generateData(vo.getSjsywzje(), vo.getYear(), 0.05));
                vos.put("swbmss", generateData(vo.getSwbmss(), vo.getYear(), 0.03));
                vos.put("hgss", generateData(vo.getHgss(), vo.getYear(), 0.04));

                return AjaxResult.success(vos);
            }
        }

        return AjaxResult.success("企业近六年暂无数据");
    }

    @Override
    public AjaxResult screenData(String deptId) {
        return AjaxResult.success(this.baseMapper.screenData(deptId).stream()
                .collect(Collectors.groupingBy(TbEnterpriseScreenData::getProviderType,
                        Collectors.collectingAndThen(Collectors.toList(), list -> list.stream().limit(2).collect(Collectors.toList())))));
    }

    private List<Object> generateData(double initialValue, String year, double coefficient) {
        List<Object> dataList = new ArrayList<>();
        for (int i = 0; i < 5; i++) {
            Map<String, Object> dataMap = new HashMap<>();
            dataMap.put("key", initialValue * (1 - i * coefficient));
            dataMap.put("value", Long.parseLong(year) - i);
            dataList.add(dataMap);
        }
        return dataList;
    }


    @Override
    public boolean saveOrUpdateBatch(Collection<CbGather> entityList) {
        return super.saveOrUpdateBatch(entityList);
    }
}
