package com.hjy.module.service.gather.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.core.text.Convert;
import com.hjy.common.utils.PageUtils;
import com.hjy.module.domain.gether.CbGather;
import com.hjy.module.mapper.gather.CbGatherStatisticsWbMapper;
import com.hjy.module.service.gather.ICbGatherStatisticsWbService;
import com.hjy.module.statistics.service.GxjckzbnzbService;
import com.hjy.module.vo.gather.CbGatherTmpVo;
import com.hjy.module.vo.gather.CbGatherVo;
import com.hjy.module.vo.gather.statistics.comprehensive.InAndOutVo;
import com.hjy.module.vo.gather.statistics.comprehensive.TotalValueAllVO;
import com.hjy.module.vo.statistics.NormCollValueVo;
import com.hjy.system.mapper.SysDeptMapper;
import com.hjy.system.service.ISysDeptService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;


/**
 * 湾办大屏Service业务层处理
 *
 * @author Tellsea
 * @date 2023-08-30
 */
@Service
@Slf4j
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbGatherStatisticsWbServiceImpl extends ServiceImpl<CbGatherStatisticsWbMapper, CbGather> implements ICbGatherStatisticsWbService {

    private final ISysDeptService sysDeptService;

    private final SysDeptMapper sysDeptMapper;

    private final GxjckzbnzbService gxjckzbnzbService;


    @Override
    public TableDataInfo<CbGatherVo> queryList(CbGatherVo entity) {
        return PageUtils.buildDataInfo(this.baseMapper.queryList(PageUtils.buildPage(), entity));
    }

    @Override
    public List<InAndOutVo> getInOutHalfaYearList() {
        SysDept sysDept = new SysDept();
        sysDept.setType(2);
        List<SysDept> listAll = this.sysDeptService.selectDeptList(sysDept);
        StringBuilder stringBuilder = new StringBuilder();
        AtomicInteger index = new AtomicInteger(0);
        listAll.forEach(vo -> {
            if (index.get() == 0) {
                stringBuilder.append(vo.getDeptId());
            } else {
                stringBuilder.append(",");
                stringBuilder.append(vo.getDeptId());
            }
            index.getAndIncrement();
        });
        return this.baseMapper.getInOutHalfaYearList(stringBuilder.toString());
    }

    @Override
    public Map<Integer, Map<String, List<NormCollValueVo>>> getTotalValue(Integer deptId, String year, String month) {
        Map<Integer, Map<String, List<NormCollValueVo>>> vo = new HashMap<>();
        if (deptId == null){
            // 获取所有部门id
            SysDept sysDept = new SysDept();
            sysDept.setType(2);
            List<SysDept> sysDepts = sysDeptMapper.selectDeptList(sysDept);
            // 遍历部门id
            for (SysDept dept : sysDepts) {
                Map<String, List<NormCollValueVo>> temp = gxjckzbnzbService.normHis(String.valueOf(dept.getDeptId()),
                        org.apache.commons.lang3.StringUtils.split("A002##A003##A004", "##"),
                        Convert.toLong(year),
                        Convert.toLong(month.replace("月", "")),
                        1L,
                        true);
                vo.put(dept.getOrderNum(), temp);
            }
        }else {
            Map<String, List<NormCollValueVo>> temp = gxjckzbnzbService.normHis(String.valueOf(deptId),
                    org.apache.commons.lang3.StringUtils.split("A002##A003##A004", "##"),
                    Convert.toLong(year),
                    Convert.toLong(month.replace("月", "")),
                    1L,
                    true);
            vo.put(deptId, temp);
        }

        return vo;

    }

    @Override
    public Map<Integer, Map<String, List<NormCollValueVo>>> getTotalValueBsq(String year, String month) {


        Map<Integer, Map<String, List<NormCollValueVo>>> vo = new HashMap<>();
        // 获取所有部门id
        SysDept sysDept = new SysDept();
        sysDept.setType(2);
        List<SysDept> sysDepts = sysDeptMapper.selectDeptList(sysDept);
        // 遍历部门id
        for (SysDept dept : sysDepts) {
            Map<String, List<NormCollValueVo>> temp = gxjckzbnzbService.normHis(String.valueOf(dept.getDeptId()),
                    StringUtils.split("A001##A002##A003##A004", "##"),
                    Convert.toLong(year),
                    Convert.toLong(month),
                    1L,
                    true);
            vo.put(dept.getOrderNum(), temp);
        }
        return vo;
    }

    @Override
    public CbGatherTmpVo selectTmp(String year) {
        return this.baseMapper.selectTmp(year);
    }

    @Override
    public List<String> selectCode(String year, String month) {
        return this.baseMapper.selectCode(year, month);
    }

    @Override
    public AjaxResult getAllTotalValue() {

        // 查询今年的进出口总值
        List<TotalValueAllVO> list = this.baseMapper.getTotalValueAll(String.valueOf(LocalDateTime.now().getYear()));
        // 查询去年的进出口总值
        List<TotalValueAllVO> oldList = this.baseMapper.getTotalValueAll(String.valueOf(LocalDateTime.now().getYear() - 1));

        BigDecimal scaledNumber;

        for (TotalValueAllVO obj : list) {
            int index = list.indexOf(obj);
            Double value = obj.getTotalValue();
            Double oldValue = oldList.get(index).getTotalValue();
            if (Objects.nonNull(value) && Objects.nonNull(oldValue)) {
                double growthRate = (value - oldValue) / oldValue;
                BigDecimal decimal = BigDecimal.valueOf(growthRate);
                scaledNumber = decimal.multiply(BigDecimal.valueOf(100)).setScale(2, RoundingMode.HALF_UP);
                obj.setGrowthRate(String.valueOf(scaledNumber));
            } else {
                obj.setTotalValue(0.00);
                obj.setGrowthRate("0");
            }
        }

        //获取广西每月数据
        List<TotalValueAllVO> gxjckz = this.baseMapper.getgxjckz(LocalDateTime.now().getYear());

        Map<String, Object> map = new HashMap<>();
        map.put("综保区本年数据", list);
        map.put("广西本年数据", gxjckz);

        ArrayList<Object> objects = new ArrayList<>();
        for (String s : map.keySet()) {
            HashMap<String, Object> vo = new HashMap<>();
            vo.put("key", s);
            vo.put("value", map.get(s));
            objects.add(vo);
        }

        return AjaxResult.success("查询成功", objects);
    }
}
