package com.hjy.module.service.norm.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.constant.WorkFlowStatus;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.WorkFlowServiceEnums;
import com.hjy.common.utils.PageUtils;
import com.hjy.common.utils.SecurityUtils;
import com.hjy.common.utils.bean.BeanUtils;
import com.hjy.module.domain.norm.CbQuotaBonded;
import com.hjy.module.domain.norm.CbQuotaBondedFee;
import com.hjy.module.mapper.norm.CbQuotaBondedFeeMapper;
import com.hjy.module.mapper.norm.CbQuotaBondedMapper;
import com.hjy.module.service.norm.ICbQuotaBondedFeeService;
import com.hjy.module.service.norm.ICbQuotaBondedService;
import com.hjy.module.service.wf.IWfInstActionTaskService;
import com.hjy.module.service.wf.WorkFlowInterface;
import com.hjy.module.vo.norm.CbQuotaBondedFeeVo;
import com.hjy.module.vo.norm.CbQuotaBondedVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 辅助指标-综保区Service业务层处理
 *
 * @author lbl
 * @date 2023-10-09
 */
@Service("quotaBondedService")
public class CbQuotaBondedServiceImpl extends ServiceImpl<CbQuotaBondedMapper, CbQuotaBonded> implements ICbQuotaBondedService, WorkFlowInterface {

    @Autowired
    private ICbQuotaBondedFeeService bondedFeeService;

    @Autowired
    private CbQuotaBondedFeeMapper quotaBondedFeeMapper;

    @Autowired
    private IWfInstActionTaskService actionTaskService;

    @Override
    public TableDataInfo<CbQuotaBondedVo> queryList(CbQuotaBondedVo entity) {
        return PageUtils.buildDataInfo(this.baseMapper.queryList(PageUtils.buildPage(), entity));
    }

    @Override
    public List<CbQuotaBondedVo> queryAll(CbQuotaBondedVo entity) {
        return this.baseMapper.queryList(entity);
    }

    public boolean updateFeeById(CbQuotaBondedVo entity) {
        int i = this.baseMapper.updateById(entity);
        List<CbQuotaBondedFee> list = new ArrayList<>();
        entity.getBondedFeeList().forEach(item -> {
            CbQuotaBondedFee fee = new CbQuotaBondedFee();
            BeanUtils.copyProperties(item,fee);
            list.add(fee);
        });
        boolean result = bondedFeeService.updateBatchById(list);
        return result;
    }

    @Override
    public CbQuotaBondedVo queryById(String id) {
        CbQuotaBondedVo cbQuotaBondedVo = this.baseMapper.queryById(id);
        List<CbQuotaBondedFeeVo> bondedFeeVoList = quotaBondedFeeMapper.queryByBondedId(id);
        CbQuotaBondedVo result = cbQuotaBondedVo.setBondedFeeList(bondedFeeVoList);
        result.setTaskVoList(actionTaskService.queryTasksByid(id, WorkFlowServiceEnums.QUOTABONDEDSERVICE.getService()));
        return result;
    }


    public boolean saveFee(CbQuotaBondedVo bondedVo){
        CbQuotaBondedVo vo = new CbQuotaBondedVo();
        vo.setYear(bondedVo.getYear());
        vo.setBondedId(bondedVo.getBondedId());
        List<CbQuotaBondedVo> voList = this.baseMapper.queryList(vo);
        if(voList.size()>0) throw new RuntimeException("当前年的综合保税区辅助指标数据已录入，请从主页选择修改");

        //保存辅助指标录入主表
        CbQuotaBonded bonded = new CbQuotaBonded();
        bonded.setYear(bondedVo.getYear());
        bonded.setBondedId(bondedVo.getBondedId());
        bonded.setBondedName(bondedVo.getBondedName());
        boolean save = this.save(bonded);

        List<CbQuotaBondedFee> bondedFeeList = new ArrayList<>();

        List<CbQuotaBondedFeeVo> bondedFeeVoList = bondedVo.getBondedFeeList();
        bondedFeeVoList.forEach(item -> {
            CbQuotaBondedFee fee = new CbQuotaBondedFee();
            BeanUtils.copyProperties(item,fee);
            fee.setId(null);
            fee.setCreateTime(null);
            fee.setCreateBy(null);
            fee.setQuotaBondedId(bonded.getId());
            fee.setEnterType(item.getConfirmWay());
            bondedFeeList.add(fee);
        });
        boolean result = bondedFeeService.saveBatch(bondedFeeList);
        return result;
    }

    @Override
    public Object getEntity(String fromDataId) {
        return this.queryById(fromDataId);
    }

    @Override
    public void cutInServiceFrom(String fromDataId, String pspStatus) {
        CbQuotaBonded entity = this.baseMapper.selectById(fromDataId);
        switch (pspStatus) {
            case WorkFlowStatus.APPROVE:  // 通过
                entity.setApprovalStatus("2");
                entity.setApprovalTime(new Date());
                entity.setApprovalName(SecurityUtils.getLoginUser().getUser().getNickName());
                this.baseMapper.updateById(entity);
                break;
            case WorkFlowStatus.REJECT:   // 拒绝
                entity.setApprovalStatus("3");
                entity.setApprovalTime(new Date());
                entity.setApprovalName(SecurityUtils.getLoginUser().getUser().getNickName());
                this.baseMapper.updateById(entity);
                break;
            case WorkFlowStatus.GOBACK:   // 退回

                break;
            default:

                break;
        }
    }
}
