package com.hjy.module.service.portal.impl;

import cn.hutool.core.date.DateTime;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.constant.WorkFlowStatus;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.utils.PageUtils;
import com.hjy.module.domain.portal.CbQuestion;
import com.hjy.module.mapper.portal.CbQuestionMapper;
import com.hjy.module.service.portal.ICbQuestionService;
import com.hjy.module.service.wf.WorkFlowInterface;
import com.hjy.module.vo.portal.CbJudgeCountVo;
import com.hjy.module.vo.portal.CbQuestionVo;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 咨询信息Service业务层处理
 *
 * @author Tellsea
 * @date 2023-08-23
 */
@Service("questionService")
public class CbQuestionServiceImpl extends ServiceImpl<CbQuestionMapper, CbQuestion> implements ICbQuestionService, WorkFlowInterface {

    @Override
    public TableDataInfo<CbQuestionVo> queryList(CbQuestionVo entity) {
        return PageUtils.buildDataInfo(this.baseMapper.queryList(PageUtils.buildPage(), entity));
    }

    @Override
    public List<CbQuestionVo> queryAll(CbQuestionVo entity) {
        return this.baseMapper.queryList(entity);
    }

    @Override
    public CbQuestionVo queryById(String answerId) {
        return this.baseMapper.queryById(answerId);
    }

    @Override
    public List<CbJudgeCountVo> evaluate(CbQuestionVo entity) {
        return this.baseMapper.evaluate(entity.getExpertId());
    }

    @Override
    public Object getEntity(String fromDataId) {
        return this.baseMapper.queryById(fromDataId);
    }

    /**
     * 节点审核过程中回调业务接口
     *
     * @param fromDataId
     * @param pspStatus
     */
    @Override
    public void cutInServiceFrom(String fromDataId, String pspStatus) {
        CbQuestionVo cbQuestionVo = new CbQuestionVo();
        cbQuestionVo.setId(fromDataId);
        switch (pspStatus) {
            case WorkFlowStatus.APPROVE:  // 通过
                cbQuestionVo.setCheckStatus("2");
                cbQuestionVo.setStatus("1");
                cbQuestionVo.setReplyTime(DateTime.now());
                this.baseMapper.updateById(cbQuestionVo);
                break;
            case WorkFlowStatus.REJECT:   // 拒绝
                cbQuestionVo.setCheckStatus("3");
                cbQuestionVo.setStatus("0");
                this.baseMapper.updateById(cbQuestionVo);
                break;
            case WorkFlowStatus.GOBACK:   // 退回
                cbQuestionVo.setCheckStatus("0");
                this.baseMapper.updateById(cbQuestionVo);
                break;
            case WorkFlowStatus.START:   // 发起
                cbQuestionVo.setCheckStatus("1");
                this.baseMapper.updateById(cbQuestionVo);
        }
    }
}
