package com.hjy.module.service.portal.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.constant.WorkFlowStatus;
import com.hjy.common.core.domain.entity.CbCompanyUser;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.utils.PageUtils;
import com.hjy.common.utils.RedisUtil;
import com.hjy.module.domain.portal.CbModifyCompanyUser;
import com.hjy.module.mapper.portal.CbCompanyUserMapper;
import com.hjy.module.service.portal.ICbModifyCompanyUserService;
import com.hjy.module.service.portal.ModifyICbCompanyUserService;
import com.hjy.module.service.wf.WorkFlowInterface;
import com.hjy.module.vo.portal.CbCompanyUserVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 门户用户Service业务层处理
 *
 * @author Tellsea
 * @date 2023-08-23
 */
@Service("modifycompanyUserService")
public class ModifyCbCompanyUserServiceImpl extends ServiceImpl<CbCompanyUserMapper, CbCompanyUser> implements ModifyICbCompanyUserService, WorkFlowInterface {

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private ICbModifyCompanyUserService cbModifyCompanyUserService;

    /**
     * 根据用户名查询
     * @param userName
     * @return
     */
    @Override
    public CbCompanyUser selectCbCompanyUserByuserName(String userName) {
        CbCompanyUser companyUser = lambdaQuery().eq(CbCompanyUser::getUserName,userName)
                                                .eq(CbCompanyUser::getDelFlag,"0")
                                                .one();
        return companyUser;
    }

    @Override
    public int checkCbCompanyUserByuserName(String userName) {
        return lambdaQuery().eq(CbCompanyUser::getUserName,userName)
                .eq(CbCompanyUser::getDelFlag,"0")
                .count();
    }


    @Override
    public TableDataInfo<CbCompanyUserVo> queryList(CbCompanyUserVo entity) {
        return PageUtils.buildDataInfo(this.baseMapper.queryList(PageUtils.buildPage(), entity));
    }

    @Override
    public List<CbCompanyUserVo> queryAll(CbCompanyUserVo entity) {
        return this.baseMapper.queryList(entity);
    }

    @Override
    public CbCompanyUserVo queryById(String id) {
        return this.baseMapper.queryById(id);
    }

    @Override
    public Object getEntity(String fromDataId) {
        return cbModifyCompanyUserService.queryById(fromDataId);
    }

    /**
     * 节点审核过程中回调业务接口
     * @param fromDataId
     * @param pspStatus
     */
    @Override
    public void cutInServiceFrom(String fromDataId, String pspStatus) {
        CbModifyCompanyUser modifycompanyUser = cbModifyCompanyUserService.queryById(fromDataId);
        CbCompanyUser companyUser = BeanUtil.copyProperties(modifycompanyUser, CbCompanyUser.class);

        CbCompanyUser user = new CbCompanyUser();
        user.setId(fromDataId);
        user.setAudit(2);

        switch (pspStatus) {
            case WorkFlowStatus.APPROVE:
                // 通过
                companyUser.setAudit(3);
                this.baseMapper.updateById(companyUser);
                cbModifyCompanyUserService.deleteUser(companyUser.getId());
                break;
            case WorkFlowStatus.REJECT:
                // 拒绝
                user.setAudit(2);
                this.baseMapper.updateById(user);
                cbModifyCompanyUserService.deleteUser(companyUser.getId());
                break;
            case WorkFlowStatus.GOBACK:
                // 退回
                this.baseMapper.updateById(user);
                break;
            default:
                // 发起
                user.setAudit(1);
                this.baseMapper.updateById(user);
                break;
        }

    }
}
