package com.hjy.module.service.rank.impl;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.utils.PageUtils;
import com.hjy.module.domain.rank.CbRank;
import com.hjy.module.mapper.rank.CbRankMapper;
import com.hjy.module.service.rank.ICbRankService;
import com.hjy.module.vo.rank.CbRankVo;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 广西保税区排名Service业务层处理
 *
 * @author Tellsea
 * @date 2023-10-10
 */
@Service
public class CbRankServiceImpl extends ServiceImpl<CbRankMapper, CbRank> implements ICbRankService {

    @Resource
    private CbRankMapper rankMapper;

    @Override
    public TableDataInfo<CbRankVo> queryList(CbRankVo entity) {
        return PageUtils.buildDataInfo(this.baseMapper.queryList(PageUtils.buildPage(), entity));
    }

    @Override
    public List<CbRankVo> queryAll(CbRankVo entity) {
        return this.baseMapper.queryList(entity);
    }

    @Override
    public CbRankVo queryById(String id) {
        return this.baseMapper.queryById(id);
    }

    /**
     * 返回湾办进出口的近年排名
     * 返回大致的结构
     * <json>
     * "2019": [
     * {
     * "deptId": null,
     * "year": "2019",
     * "ranking": 89,
     * "deptName": "北海综合保税区"
     * },
     * ....
     * ],
     * "2020": [
     * {
     * "year": "2019",
     * "ranking": 89,
     * "deptName": "北海综合保税区"
     * },
     * ...
     * ]
     * </json>
     *
     * @return 返回以年份作为 key 的 {@link Map}, value为 {@linkplain List<CbRankVo>} 的数据
     */
    @Override
    public Map<String, List<CbRankVo>> queryInOutRanking() {
        Map<String, List<CbRankVo>> listMap = new HashMap<>();
        // 假设这里是查询最近6年的数据
        int year = LocalDate.now().getYear();
        for (int i = 0; i < 5; i++) {
            year = year - 1;
            // 可以根据 year 和 deptId 去查询对应的数据
            List<CbRankVo> cbRankVo = rankMapper.selectInOutRanking(String.valueOf(year), null);
            // 构造数据，以年份作为key
            if (cbRankVo.isEmpty()){
                year = year - 1;
                cbRankVo = rankMapper.selectInOutRanking(String.valueOf(year), null);
            }
            listMap.put(String.valueOf(year), cbRankVo);
        }
        return listMap;
    }
}
