package com.hjy.module.service.template.impl;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import cn.hutool.poi.excel.ExcelReader;
import cn.hutool.poi.excel.ExcelUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.config.ZhbsqConfig;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.utils.PageUtils;
import com.hjy.common.utils.file.FileUploadUtils;
import com.hjy.common.utils.file.FileUtils;
import com.hjy.module.domain.midwest.NeighborProvince;
import com.hjy.module.domain.midwest.TbMidwest;
import com.hjy.module.domain.report.ReportDTO;
import com.hjy.module.domain.report.ReportRecord;
import com.hjy.module.domain.report.ReportWord;
import com.hjy.module.domain.reportsetting.ReportSetting;
import com.hjy.module.domain.template.ExcelOutInTotal;
import com.hjy.module.mapper.template.ExcelOutInTotalMapper;
import com.hjy.module.service.midwest.ITbMidwestService;
import com.hjy.module.service.report.IReportRecordService;
import com.hjy.module.service.report.IReportTemplateService;
import com.hjy.module.service.report.IReportWordService;
import com.hjy.module.service.reportsetting.IReportSettingService;
import com.hjy.module.service.template.IExcelOutInTotalService;
import com.hjy.module.vo.midwest.TbMidwestVo;
import com.hjy.module.vo.report.ReportRecordVo;
import com.hjy.module.vo.report.ReportTemplateVo;
import com.hjy.module.vo.template.ExcelOutInTotalVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.xwpf.usermodel.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.*;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.nio.file.Paths;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 特定地区进出口总值（人民币）Service业务层处理
 *
 * @author Tellsea
 * @date 2024-04-23
 */
@Service
@Slf4j
public class ExcelOutInTotalServiceImpl extends ServiceImpl<ExcelOutInTotalMapper, ExcelOutInTotal> implements IExcelOutInTotalService {

    @Resource
    private IReportRecordService reportRecordService;

    @Resource
    private IReportSettingService reportSettingService;

    @Resource
    private ITbMidwestService tbMidwestService;


    @Override
    public TableDataInfo<ExcelOutInTotalVo> queryList(ExcelOutInTotalVo entity) {
        return PageUtils.buildDataInfo(this.baseMapper.queryList(PageUtils.buildPage(), entity));
    }

    @Override
    public List<ExcelOutInTotalVo> queryAll(ExcelOutInTotalVo entity) {
        return this.baseMapper.queryList(entity);
    }

    @Override
    public ExcelOutInTotalVo queryById(Long id) {
        return this.baseMapper.queryById(id);
    }

    @Override
    public boolean analyze(MultipartFile file, String gxTradeTotalMoney) {

        try (InputStream inputStream = file.getInputStream()) {
            ExcelReader reader = ExcelUtil.getReader(inputStream);
            return analyze(reader, gxTradeTotalMoney, "");
        } catch (IOException e) {
            e.printStackTrace();
        }
        return false;
    }


    @Override
    public boolean analyze(ExcelReader reader, String gxTradeTotalMoney, String preface) {
        final String nn = "南宁综";
        final String wz = "梧州综";
        final String qz = "钦州综";
        final String bh = "北海综";
        final String px = "凭祥综";
        BigDecimal tenThousand = BigDecimal.valueOf(10000);
        ReportDTO reportDTO = new ReportDTO();
        reportDTO.setGxTradeTotalMoney(gxTradeTotalMoney + "").setPreface(preface);
        String text = reader.readAsText(false);
        List<String> arrayList = StrUtil.split(StrUtil.trim(StrUtil.replace(text, ",", "")), "\n");
        String monthStr = arrayList.get(3);
        List<String> monthList = StrUtil.split(monthStr, "\t");
        String currentMonth = monthList.get(0);
        String cumulativeMonth = monthList.get(1);
        String title = arrayList.get(0);
        String time = StrUtil.subBefore(StrUtil.subAfter(title, "20", true), "特定地区进出口总值", true);
        String year = "20" + StrUtil.subBefore(time, "年", true);
        String month = StrUtil.subBetween(time, "年", "月");
        reportDTO.setYear(year).setMonth(month);
        ReportRecord reportRecord = reportRecordService.findReport(year, month);
        if (reportRecord == null) {
            reportRecord = new ReportRecord();
            reportRecord.setReportYear(year)
                    .setReportMonth(month);
            reportRecordService.save(reportRecord);
        }
        String reportId = reportRecord.getId();
        this.removeByReportId(reportId);

        List<String> zongbaoList = arrayList.stream().filter(str -> str.contains("综合保税区")).collect(Collectors.toList());
        List<ExcelOutInTotal> excelDTOList = buildDTO(zongbaoList, currentMonth, cumulativeMonth);
        //全国综保区数据
        ExcelOutInTotal totalDTO = excelDTOList.get(0);
        String allTotalInc = totalDTO.getTotalInc();
        double inc = new BigDecimal(allTotalInc).doubleValue();
        String incDesc = inc > 0 ? "增长" : "下降";
        reportDTO.setAllAsyncDesc("全国综保区进出口总值为" + totalDTO.getTotalCumulativeMonth() + "亿元，同比" + incDesc + Math.abs(inc) + "%。");
        reportRecord.setNactionForeignTrade(totalDTO.getTotalCumulativeMonth())
                .setNactionForeignTradeInc(totalDTO.getTotalInc());
        //中西部名单
        List<TbMidwestVo> tbMidwestVos = tbMidwestService.queryAll(new TbMidwestVo());
        List<String> midWestNameList = tbMidwestVos.stream().map(TbMidwest::getName).collect(Collectors.toList());
        //中西部进出口总值列表
        List<ExcelOutInTotal> midWestList = excelDTOList.stream()
                .filter(m->midWestNameList.contains(m.getName())).collect(Collectors.toList())
                .stream().sorted(Comparator.comparingDouble(e -> Double.parseDouble(e.getTotalCumulativeMonth())))
                .collect(Collectors.toList());
        //计算排名
        int i = midWestNameList.size();
        for (ExcelOutInTotal excelOutInTotal : midWestList) {
            excelOutInTotal.setIndex(i);
            i--;
        }
        //挑出广西在中西部排名
        StringBuilder midwestSortDesc=new StringBuilder("中西部综保区排名情况：");
        midWestList.stream().filter(str->{
            String name = str.getName();
            return name.contains(nn) || name.contains(wz) || name.contains(qz) || name.contains(bh) || name.contains(px);
        }).forEach(excelOutInTotal -> {
            midwestSortDesc.append(excelOutInTotal.getName())
                    .append("进出口总值").append(excelOutInTotal.getTotalCumulativeMonth())
                    .append("亿元，排名第").append(excelOutInTotal.getIndex()).append("；");
        });
        reportDTO.setMidwestSortDesc(midwestSortDesc.toString());
        //临省
        List<NeighborProvince> neighborProvinces = new ArrayList<>();
        tbMidwestVos.stream()
                .filter(w -> w.getNeighbor() == 1)
                .collect(Collectors.groupingBy(TbMidwest::getProvince))
                .forEach((province, provinceList) -> {
                    List<String> nameList = provinceList.stream().map(TbMidwest::getName).collect(Collectors.toList());
                    List<ExcelOutInTotal> neighborList = midWestList.stream().filter(m->nameList.contains(m.getName())).collect(Collectors.toList());
                    Long provinceSum = neighborList.stream()
                            .collect(Collectors.summarizingLong(e -> Long.parseLong(e.getTotalCumulativeMonth())))
                            .getSum();
                    NeighborProvince neighborProvince = new NeighborProvince();
                    neighborProvince.setProvinceName(province).setTotal(provinceSum);
                    neighborProvinces.add(neighborProvince);
                });
        List<NeighborProvince> sortList = neighborProvinces.stream()
                .sorted(Comparator.comparingLong(NeighborProvince::getTotal))
                .collect(Collectors.toList());
        i = sortList.size();
        StringBuilder sb = new StringBuilder("相邻省份排名情况：");
        for (NeighborProvince neighborProvince : sortList) {
            sb.append(neighborProvince.getProvinceName())
                    .append("进出口总值").append(neighborProvince.getTotal())
                    .append("亿元，排名").append(i).append("；");
            neighborProvince.setIndex(i);
            i--;
        }
        reportDTO.setNeighborProvinceDesc(sb.toString());
        List<ExcelOutInTotal> gxList = excelDTOList.stream().filter(str -> {
            String name = str.getName();
            return name.contains(nn) || name.contains(wz) || name.contains(qz) || name.contains(bh) || name.contains(px);
        }).collect(Collectors.toList());
        gxList.forEach(excelOutInTotal -> {
            excelOutInTotal.setReportId(reportId);
            String name = excelOutInTotal.getName();
            int index = excelOutInTotal.getIndex();
            String totalInc = excelOutInTotal.getTotalInc() + "%";
            totalInc = Double.parseDouble(excelOutInTotal.getTotalInc()) == 0 ? "-" : totalInc;
            String totalMoney = new BigDecimal(excelOutInTotal.getTotalCumulativeMonth()).divide(tenThousand, 1, RoundingMode.HALF_UP).toString();
            if (name.contains(nn)) {
                reportDTO.setNnIndex(index)
                        .setNnInc(totalInc)
                        .setNnTotalMoney(totalMoney);
            } else if (name.contains(px)) {
                reportDTO.setPxIndex(index).setPxInc(totalInc).setPxTotalMoney(totalMoney);
            } else if (name.contains(wz)) {
                reportDTO.setWzIndex(index).setWzInc(totalInc).setWzTotalMoney(totalMoney);
            } else if (name.contains(bh)) {
                reportDTO.setBhIndex(index).setBhInc(totalInc).setBhTotalMoney(totalMoney);
            } else if (name.contains(qz)) {
                reportDTO.setQzIndex(index).setQzInc(totalInc).setQzTotalMoney(totalMoney);
            }
        });
        this.saveBatch(gxList);
        generateReport(reportRecord, gxList, excelDTOList.size(), reportDTO);
        reportRecordService.updateById(reportRecord);
        return true;
    }


    private void generateReport(ReportRecord reportRecord, List<ExcelOutInTotal> gxList, int total, ReportDTO reportDTO) {
        Double beforeTotalCumulativeMonth = 0D;
        BigDecimal hundred = BigDecimal.valueOf(100);
        BigDecimal tenThousand = BigDecimal.valueOf(10000);
        gxList.sort(Comparator.comparing(excelOutInTotal -> Double.parseDouble(excelOutInTotal.getTotalInc())));
        List<ExcelOutInTotal> incList = new ArrayList<>();
        List<ExcelOutInTotal> descList = new ArrayList<>();
        StringBuilder tradeTotalMoneyIndexDesc = new StringBuilder();
        for (ExcelOutInTotal excelOutInTotal : gxList) {
            Double totalCumulativeMonth = Double.parseDouble(excelOutInTotal.getTotalCumulativeMonth());
            BigDecimal totalIncD = new BigDecimal(excelOutInTotal.getTotalInc());
            Double totalInc = totalIncD.add(hundred)
                    .divide(hundred, 5, RoundingMode.HALF_UP).doubleValue();
            if (totalIncD.doubleValue() > 0) {
                incList.add(excelOutInTotal);
            } else if (totalIncD.doubleValue() < 0) {
                descList.add(excelOutInTotal);
            }
            Double t = NumberUtil.div(totalCumulativeMonth, totalInc, 1, RoundingMode.HALF_UP);
            //log.info("t==============:{},{},{}", totalCumulativeMonth, totalInc, t);
            beforeTotalCumulativeMonth = NumberUtil.add(t, beforeTotalCumulativeMonth);
            tradeTotalMoneyIndexDesc.append(excelOutInTotal.getName()).append("排名").append(excelOutInTotal.getIndex()).append("、");
        }
        reportDTO.setTradeTotalMoneyIndexDesc(tradeTotalMoneyIndexDesc.toString());
        String gxTradeTotalMoney = reportDTO.getGxTradeTotalMoney();
        long tradeTotalMoney = gxList.stream().collect(Collectors.summarizingLong(e -> Long.parseLong(e.getTotalCumulativeMonth()))).getSum();
        if (StrUtil.isNotEmpty(gxTradeTotalMoney) && !StrUtil.equals(gxTradeTotalMoney, "undefined") && !StrUtil.equals(gxTradeTotalMoney, "null")) {
            BigDecimal tradeTotalMoneyPercent = new BigDecimal(tradeTotalMoney)
                    .divide(tenThousand, 1, RoundingMode.UP)
                    .divide(new BigDecimal(gxTradeTotalMoney), 3, RoundingMode.UP);
            reportDTO.setTradeTotalMoneyPercent(tradeTotalMoneyPercent.multiply(hundred) + "%");
        }
        //同比增长
        double tradeTotalMoneyInc = (BigDecimal.valueOf(tradeTotalMoney).subtract(BigDecimal.valueOf(beforeTotalCumulativeMonth)))
                .divide(BigDecimal.valueOf(beforeTotalCumulativeMonth), 3, RoundingMode.UP)
                .multiply(hundred).doubleValue();
        String tradeTotalMoneyIncDirection = tradeTotalMoneyInc > 0 ? "增长" : "下降";
        reportDTO.setTradeTotalMoney(new BigDecimal(tradeTotalMoney).divide(tenThousand, 1, RoundingMode.HALF_UP).toString())
                .setTradeTotalMoneyInc(Math.abs(tradeTotalMoneyInc) + "%")
                .setTradeTotalMoneyIncDirection(tradeTotalMoneyIncDirection);
        reportRecord.setForeignTrade(tradeTotalMoney + "").setForeignTradeInc(tradeTotalMoneyInc + "%");
        //log.info("total:{},{},{}", beforeTotalCumulativeMonth, tradeTotalMoney, tradeTotalMoneyInc);
        List<ReportSetting> reportSettings = reportSettingService.list();
        reportSettings.stream()
                .filter(reportSetting -> Double.parseDouble(reportSetting.getMinNum()) <= tradeTotalMoneyInc
                        && Double.parseDouble(reportSetting.getMaxNum()) > tradeTotalMoneyInc)
                .findFirst().ifPresent(reportSetting -> reportDTO.setTradeSituation(reportSetting.getDescription()));
        String incName = incList.stream().map(ExcelOutInTotal::getName).collect(Collectors.joining("、"));
        String incPercent = incList.stream().map(excelOutInTotal -> excelOutInTotal.getTotalInc() + "%").collect(Collectors.joining("、"));
        String descName = descList.stream().map(ExcelOutInTotal::getName).collect(Collectors.joining("、"));
        String descPercent = descList.stream().map(excelOutInTotal -> Math.abs(Double.parseDouble(excelOutInTotal.getTotalInc())) + "%").collect(Collectors.joining("、"));
        StringBuilder bondedIncAnalyse = new StringBuilder("其中，");
        if (incList.size() > 0) {
            bondedIncAnalyse.append(incName).append("同比增长了").append(incPercent).append("；");
        }
        if (descList.size() > 0) {
            bondedIncAnalyse.append(descName).append("同比下降了").append(descPercent);
        }

        reportDTO.setBondedIncAnalyse(bondedIncAnalyse.toString());
        log.info("report:{}", JSONUtil.toJsonStr(reportDTO));
        reportRecordService.createWord(reportDTO, reportRecord);
    }


    private List<ExcelOutInTotal> buildDTO(List<String> list, String currentMonth, String cumulativeMonth) {
        List<ExcelOutInTotal> excelDTOList = new ArrayList<>();
        list.forEach(str -> {
            String[] split = str.split("\t");
            ExcelOutInTotal excelDTO = new ExcelOutInTotal();
            excelDTO.setName(split[0]);
            excelDTO.setCurrentMonth(currentMonth);
            excelDTO.setCumulativeMonth(cumulativeMonth);
            excelDTO.setTotalCurrentMonth("-".equals(split[1]) ? "0.0" : split[1]);
            excelDTO.setTotalCumulativeMonth("-".equals(split[2]) ? "0.0" : new BigDecimal(split[2]).divide(BigDecimal.valueOf(10000),2,RoundingMode.HALF_UP)+"");
            excelDTO.setOutCurrentMonth("-".equals(split[3]) ? "0.0" : split[3]);
            excelDTO.setOutCumulativeMonth("-".equals(split[4]) ? "0.0" : split[4]);
            excelDTO.setInCurrentMonth("-".equals(split[5]) ? "0.0" : split[5]);
            excelDTO.setInCumulativeMonth("-".equals(split[6]) ? "0.0" : split[6]);
            excelDTO.setTotalInc("-".equals(split[7]) ? "0.0" : split[7]);
            excelDTO.setOutInc("-".equals(split[8]) ? "0.0" : split[8]);
            excelDTO.setInInc("-".equals(split[9]) ? "0.0" : split[9]);
            excelDTOList.add(excelDTO);

        });
        excelDTOList.sort(Comparator.comparingDouble(e -> Double.parseDouble(e.getTotalCumulativeMonth())));
        Collections.reverse(excelDTOList);
        int index = 0;
        for (ExcelOutInTotal e : excelDTOList) {
            e.setIndex(index);

            index++;
        }
        return excelDTOList;
    }

    @Override
    public void removeByReportId(String reportId) {
        this.baseMapper.removeByReportId(reportId);
    }


}
