package com.hjy.module.service.totexp.impl;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.core.text.Convert;
import com.hjy.common.utils.DateUtils;
import com.hjy.common.utils.PageUtils;
import com.hjy.module.domain.totexp.CbTotexp;
import com.hjy.module.mapper.totexp.CbTotexpAreaMapper;
import com.hjy.module.mapper.totexp.CbTotexpMapper;
import com.hjy.module.service.totexp.ICbTotexpService;
import com.hjy.module.statistics.service.GxjckzbnzbService;
import com.hjy.module.vo.analysis.YearPercentage;
import com.hjy.module.vo.statistics.NormCollValueVo;
import com.hjy.module.vo.totexp.CbTotexpVo;
import com.hjy.system.mapper.SysDeptMapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * 保税区进出口总值数据Service业务层处理
 *
 * @author Tellsea
 * @date 2023-10-10
 */
@Service
public class CbTotexpServiceImpl extends ServiceImpl<CbTotexpMapper, CbTotexp> implements ICbTotexpService {

    @Resource
    private CbTotexpMapper totexpMapper;
    @Resource
    private CbTotexpAreaMapper areaMapper;

    @Resource
    private SysDeptMapper sysDeptMapper;

    @Resource
    private GxjckzbnzbService gxjckzbnzbService;


    @Override
    public TableDataInfo<CbTotexpVo> queryList(CbTotexpVo entity) {
        return PageUtils.buildDataInfo(this.baseMapper.queryList(PageUtils.buildPage(), entity));
    }

    @Override
    public List<CbTotexpVo> queryAll(CbTotexpVo entity) {
        return this.baseMapper.queryList(entity);
    }

    @Override
    public CbTotexpVo queryById(String id) {
        return this.baseMapper.queryById(id);
    }

    /**
     * 查询近年的 湾办进出口总值的 数据分析集合
     *
     * @return 返回对象是 {@link Map<String, List<CbTotexpVo>}; key是 年份(year)，value是对应的数据{@link CbTotexpVo}
     */
    @Override
    public HashMap<String, List<BigDecimal>> queryInOutYear(String year, String month) {
        HashMap<String, List<BigDecimal>> objectObjectHashMap = new HashMap<>();
        // 获取所有部门id
        SysDept sysDept = new SysDept();
        sysDept.setType(2);
        List<SysDept> sysDepts = sysDeptMapper.selectDeptList(sysDept);
        // 循环五年
        for (int i = 0; i < 5; i++) {
            List<BigDecimal> objects = new ArrayList<>();
            // 遍历部门id
            for (SysDept dept : sysDepts) {
                Map<String, List<NormCollValueVo>> temp = gxjckzbnzbService.normHis(String.valueOf(dept.getDeptId()),
                        StringUtils.split("A001", "##"),
                        Convert.toLong(year),
                        Convert.toLong(month),
                        1L,
                        true);
                if (Objects.isNull(temp.get(month.replaceAll("^0+", "")))) {
                    objects.add(BigDecimal.ZERO);
                } else {
                    objects.add(temp.get(month.replaceAll("^0+", "")).get(0).getCollValue());
                }
            }
            objectObjectHashMap.put(year,objects);
            year = String.valueOf(Integer.parseInt(year) - 1);
            month = "12";

        }
        return objectObjectHashMap;
    }

    /**
     * 获取广西综合保税区
     * 年度增长率 = （本期数-上期数）/上期数 × 100%
     * 同比增长计算公式
     * （本期数 / 同期数 - 1） * 100
     * <p>
     * 同比增长率 =（本期的指标值 - 同期的指标值）/ 同期的指标值*100%
     *
     * @return 返回 Map集合的 key 为 year(年份)，value 为 {@link YearPercentage} 对象
     */
    @Override
    public Map<String, YearPercentage> queryInoutYearPercentage() {
        int nowYear = Convert.toInt(DateUtils.dateTime().substring(0,4));
        int nowMonth = Convert.toInt(DateUtils.dateTime().substring(4,6));
        for (int i = 4; i >=0 ; i--) {
            // todo 待完善
        }



        return null;
    }

    /**
     * 构造同比增长
     *
     * @param curYear  当前年份
     * @param lastYear 去年
     * @return 返回计算后的结果的绝对值
     */
    private BigDecimal calculateYearOnYearGrowth(CbTotexpVo curYear, CbTotexpVo lastYear) {
        BigDecimal curCityTotal = new BigDecimal(curYear.getCityTotal());
        BigDecimal lastCityTotal = new BigDecimal(lastYear.getCityTotal());

        PercentageCalculator calculator = (curValue, lastValue) ->
                curValue.divide(lastValue, 4, RoundingMode.HALF_UP).subtract(BigDecimal.ONE).multiply(new BigDecimal("100"));

        return calculatePercentage(curCityTotal, lastCityTotal, calculator);

    }

    /**
     * 构造年增长率
     *
     * @param curYear  当前年份
     * @param lastYear 去年
     * @return 返回计算后的结果的绝对值
     */
    private BigDecimal calculateYearPercentage(CbTotexpVo curYear, CbTotexpVo lastYear) {
        BigDecimal curImExport = new BigDecimal(curYear.getImExport());
        BigDecimal lastImExport = new BigDecimal(lastYear.getImExport());

        PercentageCalculator calculator = (curValue, lastValue) ->
                curValue.subtract(lastValue).divide(lastValue, 4, RoundingMode.HALF_UP).multiply(new BigDecimal("100"));

        return calculatePercentage(curImExport, lastImExport, calculator);
    }

    private BigDecimal calculatePercentage(BigDecimal curValue, BigDecimal lastValue, PercentageCalculator calculator) {
        return calculator.calculate(curValue, lastValue).abs();
    }

    @FunctionalInterface
    public interface PercentageCalculator {
        /**
         * 计算增长率或者同比率
         *
         * @param curValue  本期值
         * @param lastValue 上一期值 / 同期值
         * @return 返回计算得出结果得绝对值
         */
        BigDecimal calculate(BigDecimal curValue, BigDecimal lastValue);
    }


}
