package com.hjy.module.service.wf.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.constant.WorkFlowStatus;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.utils.DateUtils;
import com.hjy.common.utils.PageUtils;
import com.hjy.common.utils.SecurityUtils;
import com.hjy.common.utils.StringUtils;
import com.hjy.common.utils.spring.SpringUtils;
import com.hjy.module.domain.wf.WfInstAction;
import com.hjy.module.domain.wf.WfInstActionTask;
import com.hjy.module.domain.wf.WfInstPro;
import com.hjy.module.domain.wf.dto.WftasksubmitDto;
import com.hjy.module.mapper.wf.WfInstActionTaskMapper;
import com.hjy.module.service.wf.*;
import com.hjy.module.vo.wf.WfInstActionTaskVo;
import com.hjy.module.vo.wf.WfInstActionVo;
import com.hjy.module.vo.wf.WfProtemplateVo;
import com.hjy.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.List;

/**
 * 审批任务实例Service业务层处理
 *
 * @author Tellsea
 * @date 2023-08-09
 */
@Transactional(rollbackFor = Exception.class)
@Service
public class WfInstActionTaskServiceImpl extends ServiceImpl<WfInstActionTaskMapper, WfInstActionTask> implements IWfInstActionTaskService {

    @Autowired
    private IWfProtemplateService protemplateService;

    @Autowired
    private IWfInstProService instProService;

    @Autowired
    private IWfInstActionService instActionService;

    @Autowired
    private ISysDeptService deptService;


    @Override
    public TableDataInfo<WfInstActionTaskVo> queryList(WfInstActionTaskVo entity) {
        return PageUtils.buildDataInfo(this.baseMapper.queryList(PageUtils.buildPage(), entity));
    }

    @Override
    public List<WfInstActionTaskVo> queryAll(WfInstActionTaskVo entity) {
        return this.baseMapper.queryList(entity);
    }

    /**
     * 根据数据id查询任务节点
     * @param fromDataId
     * @return
     */
    @Override
    public List<WfInstActionTaskVo> queryTasksByid(String fromDataId,String serviceUrl){
        List<WfInstActionTaskVo> list = this.baseMapper.queryTasksByid(fromDataId,serviceUrl);
        return list;
    }

    @Override
    public WfInstActionTaskVo queryById(String id) {
        WfInstActionTaskVo taskVo = this.baseMapper.queryById(id);

        WfInstActionTaskVo taskSrq = new WfInstActionTaskVo();
        taskSrq.setInstProId(taskVo.getInstProId());
        List<WfInstActionTaskVo> taskHisList = this.baseMapper.queryList(taskSrq);
        taskVo.setTaskHisAction(taskHisList);

        WfInstActionVo currentAction = instActionService.getcurrentAction(taskVo.getInstAcId()); // 获取节点Service信息
        // 回调业务接口
        WorkFlowInterface workFlowInterface = (WorkFlowInterface) SpringUtils.getBean(currentAction.getServiceUrl());
        Object obj = workFlowInterface.getEntity(currentAction.getFromDataId()); // 回调业务接口
        if(ObjectUtil.isNull(obj)) Assert.isTrue(false,"业务数据不存在！");
        taskVo.setFormobj(obj);
        taskVo.setInfoUrl(currentAction.getInfoUrl());
        return taskVo;
    }

    /**
     * 启动审核流程
     *
     * @param serviceName service服务名称
     * @param fromDataId  业务id
     * @param keyword     业务备注
     * @param userId      用户id（指定保税区审核流程）
     */
    public void statrWfqueue(String serviceName, String fromDataId, String keyword, String userId) {
        failStartWf(serviceName, fromDataId, keyword, userId);
    }

    /**
     * 启动审核流程
     *
     * @param serviceName service服务名称
     * @param fromDataId  业务id
     * @param keyword     业务备注
     */
    public void statrWfqueue(String serviceName, String fromDataId, String keyword) {
        failStartWf(serviceName, fromDataId, keyword, null);
    }

    /**
     * 审核流程代码逻辑
     *
     * @param serviceName
     * @param fromDataId
     * @param keyword
     */
    private void failStartWf(String serviceName, String fromDataId, String keyword, String userId) {

        int fromDataIdFlag = baseMapper.chekFromDataId(fromDataId,serviceName);
        if (fromDataIdFlag > 0) Assert.isTrue(false,"该数据流程信息已存在，无法再次提交");

        Long cbId = null;
        if (ObjectUtil.isNotNull(userId)) {
            cbId = deptService.getCbidByuserId(userId);
        }

        WfProtemplateVo protemplateVo = protemplateService.queryByService(serviceName, cbId);
        if (CollectionUtil.isEmpty(protemplateVo.getActionList())) {
            Assert.isTrue(false, "流程节点获取失败/为空，请检查节点配置");
        }
        WfInstActionTask task = saveInstPor(protemplateVo, fromDataId, keyword); // 创建任务队列实例
        task.setStatus("0");
        Long deptId = null;
        try {
            deptId = SecurityUtils.getDeptId();
        } catch (Exception e) {
            deptId = 100l;
        }
        task.setDeptId(deptId);
        this.save(task); // 发起当前任务节点

        WfInstActionVo currentAction = instActionService.getcurrentAction(task.getInstAcId()); // 获取节点Service信息
        try{
            // 回调业务接口
            WorkFlowInterface workFlowInterface = (WorkFlowInterface) SpringUtils.getBean(currentAction.getServiceUrl());
            workFlowInterface.cutInServiceFrom(currentAction.getFromDataId(), WorkFlowStatus.START); // 回调业务接口
        }catch (RuntimeException e){
            log.error("流程处理业务回调失败 错误信息：" + e.getMessage());
        }

    }

    /**
     * 创建任务队列实例
     *
     * @param templateVo
     * @param fromDataId
     * @param keyword
     * @return
     */
    private WfInstActionTask saveInstPor(WfProtemplateVo templateVo, String fromDataId, String keyword) {
        WfInstPro instPro = new WfInstPro();
        instPro.setKeywords(keyword);
        instPro.setProId(templateVo.getId());
        instPro.setProName(templateVo.getProname());
        instPro.setBusinessId(fromDataId);
        instPro.setLinkurl(templateVo.getLinkurl());
        instPro.setStatus("1");
        instProService.save(instPro);
        List<WfInstAction> actions = new ArrayList<>();
        final int[] index = {0};
        templateVo.getActionList().forEach(item -> {
            WfInstAction action = new WfInstAction();
            action.setInstAcId(instPro.getId());
            action.setAcname(item.getAcname());
            action.setSeqno(item.getSeqno());
            action.setExecId(item.getExelIds());
            if (index[0] == 0) {
                action.setStatus("2");
            } else {
                action.setStatus("1");
            }
            actions.add(action);
            index[0] = index[0] + 1;

        });
        instActionService.saveBatch(actions);
        WfInstAction action = actions.get(0);
        WfInstActionTask task = new WfInstActionTask();
        transformTask(action, task);
        task.setBeginTime(DateUtils.getNowDate());
        action.setStatus("2");
        instActionService.updateById(action); // 更新任务实例队列状态
        return task;
    }


    /**
     * 任务审核通过
     *
     * @param submitDto
     * @return
     */
    @Override
    public AjaxResult approve(WftasksubmitDto submitDto) {
        WfInstActionTask task = this.getById(submitDto.getTaskId());
        task.setId(submitDto.getTaskId());
        task.setStatus("1");
        task.setEndTime(DateUtils.getNowDate());
        task.setComments(submitDto.getComments());
        this.updateById(task);
        WfInstAction actionUp = instActionService.getById(task.getInstAcId());
        actionUp.setStatus("3");
        instActionService.updateById(actionUp); // 更新任务实例队列状态

        WfInstActionVo nextaction = instActionService.getNextAction(task.getInstAcId()); // 获取下一节点
        if (StringUtils.isNull(nextaction)) { // 没有下一节点 流程已结束
            this.endTaskqueue(task.getId(), task.getInstAcId(), WorkFlowStatus.APPROVE); // 结束队列
            WfInstActionVo actionVo = instActionService.getcurrentAction(task.getInstAcId());
            WorkFlowInterface workFlowInterface = (WorkFlowInterface) SpringUtils.getBean(actionVo.getServiceUrl());
            workFlowInterface.cutInServiceFrom(actionVo.getFromDataId(), WorkFlowStatus.APPROVE); // 回调业务接口
            return AjaxResult.success();
        }
        WfInstActionTask nextTask = new WfInstActionTask();
        transformTask(nextaction, nextTask);
        nextTask.setBeginTime(DateUtils.getNowDate());
        nextTask.setStatus("0");
        nextTask.setDeptId(SecurityUtils.getDeptId());
        this.save(nextTask);
        instActionService.updateById(BeanUtil.toBean(nextaction, WfInstActionVo.class).setStatus("2")); // 更新任务实例队列状态
        return AjaxResult.success();

    }

    /**
     * 任务审核驳回
     *
     * @param submitDto
     * @return
     */
    @Override
    public AjaxResult reject(WftasksubmitDto submitDto) {
        WfInstActionTask task = this.getById(submitDto.getTaskId());
        task.setStatus("2");
        task.setEndTime(DateUtils.getNowDate());
        task.setComments(submitDto.getComments());
        this.updateById(task);
        WfInstAction actionUp = instActionService.getById(task.getInstAcId());
        actionUp.setStatus("4");
        instActionService.updateById(actionUp); // 更新任务实例队列状态
        WfInstActionVo currentAction = instActionService.getcurrentAction(task.getInstAcId()); // 获取节点Service信息
        this.endTaskqueue(task.getId(), currentAction.getId(), WorkFlowStatus.REJECT); // 结束队列

        try {
            // 回调业务接口
            WorkFlowInterface workFlowInterface = (WorkFlowInterface) SpringUtils.getBean(currentAction.getServiceUrl());
            workFlowInterface.cutInServiceFrom(currentAction.getFromDataId(), WorkFlowStatus.REJECT); // 回调业务接口
        }catch (RuntimeException e){
            log.error("流程处理业务回调失败 错误信息：" + e.getMessage());
        }
        return AjaxResult.success();
    }

    /**
     * 任务审核退回
     *
     * @param submitDto
     * @return
     */
    @Override
    public AjaxResult goback(WftasksubmitDto submitDto) {
        WfInstActionTask task = this.getById(submitDto.getTaskId());
        WfInstActionVo backAction = instActionService.getBackAction(task.getInstAcId());
        if (ObjectUtil.isNull(backAction)) {
            return AjaxResult.error("没有上一节点、无法退回");
        }
        task.setStatus("3");
        task.setEndTime(DateUtils.getNowDate());
        task.setComments(submitDto.getComments());
        this.updateById(task);

        // 创建新任务
        WfInstActionTask goBackTask = new WfInstActionTask();
        transformTask(backAction, goBackTask);
        goBackTask.setBeginTime(DateUtils.getNowDate());
        goBackTask.setStatus("0");
        this.save(goBackTask);
        try {
            // 回调业务接口
            WorkFlowInterface workFlowInterface = (WorkFlowInterface) SpringUtils.getBean(backAction.getServiceUrl());
            workFlowInterface.cutInServiceFrom(backAction.getFromDataId(), WorkFlowStatus.GOBACK); // 回调业务接口
        }catch (RuntimeException e){
            log.error("流程处理业务回调失败 错误信息：" + e.getMessage());
        }
        return AjaxResult.success();
    }

    /**
     * 根据id查询待办数量
     *
     * @return
     */
    @Override
    public Integer queryBy() {
        LambdaQueryWrapper<WfInstActionTask> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(WfInstActionTask::getExecId, StringUtils.obj2str(SecurityUtils.getUserId()))
                .eq(WfInstActionTask::getStatus, 0);
        return this.baseMapper.selectCount(wrapper);
    }

    @Transactional
    @Override
    public void delete(String fromDataId, String serviceName) {
        this.baseMapper.deletePROTASK(fromDataId,serviceName);
        this.baseMapper.deleteACTION(fromDataId,serviceName);
        this.baseMapper.deletePRO(fromDataId,serviceName);
    }

    /**
     * 结束审核队列
     *
     * @param taskId
     * @param status
     */
    private void endTaskqueue(String taskId, String atcionId, String status) {
        WfInstActionVo actionVo = instActionService.getcurrentAction(atcionId);
        WfInstPro pro = new WfInstPro();
        pro.setId(actionVo.getProId());
        if (WorkFlowStatus.APPROVE.equals(status)) { // 任务通过
            pro.setStatus("3");
            instProService.updateById(pro); // 结束任务队列、变更状态
        } else if (WorkFlowStatus.REJECT.equals(status)) { // 任务驳回
            pro.setStatus("2");
            instProService.updateById(pro); // 结束任务队列、变更状态
        } else if (WorkFlowStatus.GOBACK.equals(status)) { // 任务退回

        }

    }

    private WfInstActionTask transformTask(WfInstAction action, WfInstActionTask actionTask) {
        actionTask.setInstAcId(action.getId());
        actionTask.setInstProId(action.getInstAcId());
        actionTask.setAcname(action.getAcname());
        actionTask.setExecId(action.getExecId());
        return actionTask;
    }

}
