package com.hjy.module.service.wf.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hjy.common.annotation.DataScope;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.utils.CommomUtils;
import com.hjy.common.utils.PageUtils;
import com.hjy.module.domain.wf.WfProDefAction;
import com.hjy.module.domain.wf.WfProtemplate;
import com.hjy.module.mapper.wf.WfProtemplateMapper;
import com.hjy.module.service.wf.IWfProDefActionService;
import com.hjy.module.service.wf.IWfProtemplateService;
import com.hjy.module.vo.wf.WfProDefActionVo;
import com.hjy.module.vo.wf.WfProtemplateVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.List;

/**
 * 流程模板Service业务层处理
 *
 * @author Tellsea
 * @date 2023-08-09
 */
@Service
public class WfProtemplateServiceImpl extends ServiceImpl<WfProtemplateMapper, WfProtemplate> implements IWfProtemplateService {


    @Autowired
    private IWfProDefActionService wfProDefActionService;

    @Override
    @DataScope(deptAlias = "A")
    public TableDataInfo<WfProtemplateVo> queryList(WfProtemplateVo entity) {
        return PageUtils.buildDataInfo(this.baseMapper.queryList(PageUtils.buildPage(), entity));
    }

    @Override
    public List<WfProtemplateVo> queryAll(WfProtemplateVo entity) {
        return this.baseMapper.queryList(entity);
    }


    @Override
    public WfProtemplateVo queryById(String id) {
        WfProtemplateVo vo = this.baseMapper.queryById(id);
        return vo;
    }

    @Override
    public WfProtemplateVo queryByService(String serviceName, Long cbId) {
        List<WfProtemplate> wfProtemplates = lambdaQuery().eq(WfProtemplate::getServiceUrl, serviceName)
                .eq(WfProtemplate::getDelFlag, "0")
                .eq(ObjectUtil.isNotNull(cbId), WfProtemplate::getCbId, cbId).list();
        if (CollectionUtil.isEmpty(wfProtemplates) || wfProtemplates.size() != 1) {
            Assert.isTrue(false, "相关服务信息查询错误，无法发起流程。请确认用户权限");
        }
        WfProtemplateVo temVo = BeanUtil.toBean(wfProtemplates.get(0), WfProtemplateVo.class);
        List<WfProDefAction> actionList = wfProDefActionService.lambdaQuery()
                .eq(WfProDefAction::getProId, temVo.getId())
                .eq(WfProDefAction::getDelFlag, "0")
                .orderByAsc(WfProDefAction::getSeqno)
                .list();
        temVo.setActionList(CommomUtils.copyList(actionList, WfProDefActionVo.class));
        return temVo;
    }
}
