package com.hjy.module.vo.gather.vo;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFFormulaEvaluator;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFFormulaEvaluator;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * 发展绩效指标汇总统计表 2022年和 2023年的 列不一样, 2023的有分值列
 */
@Slf4j
public class TaxReadExcelUtil {

    public static String fileName = "D:\\out\\广西综合保税区2023年6月发展绩效指标汇总统计表（估算数）0810.xls";
    public static String fileName2 = "D:\\out\\2023测试.xls";
    public static String fileName3 = "D:\\out\\广西综合保税区2022年4月发展绩效指标汇总统计表（估算数）0526.xls";

    private static FormulaEvaluator formulaEvaluator = null;

    public static void main(String[] args) {
//        TaxTableVo tableVo = creatWorkBook(fileName2);
        log.debug("广西综合保税区2023年6月发展绩效指标汇总统计表 已读取结束");
    }


    /**
     * 创建 excel的, 返回 对象, 返回 null时, 未读取成功
     * @param file
     * @return
     */
    public static TaxTableVo creatWorkBook(MultipartFile file) {

        log.info("创建 WorkBook ...");
        String fileType = file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf(".") + 1);
        Workbook workbook = null;
        try {
            if (fileType.equalsIgnoreCase("xls")) {
                workbook = new HSSFWorkbook(file.getInputStream()); //生成.xls的excel
                //后面使用它来执行计算公式
                TaxReadExcelUtil.formulaEvaluator = new HSSFFormulaEvaluator((HSSFWorkbook) workbook);
            } else if (fileType.equalsIgnoreCase("xlsx")) {
                workbook = new XSSFWorkbook(file.getInputStream()); //生成.xlsx的excel
                TaxReadExcelUtil.formulaEvaluator = new XSSFFormulaEvaluator((XSSFWorkbook) workbook);
            } else {
                log.warn("文件格式不对");
                return null;
            }
            log.info("开始解析 WorkBook...");
            return parseExcel(workbook, 0);

        } catch (FileNotFoundException e) {
            log.error("文件读取异常 [{}] - [{}]", e.getMessage(), e);
        } catch (IOException e) {
            log.error("IO读取异常 [{}] - [{}]", e.getMessage(), e);
        }

        return null;
    }

    /**
     * 解析 读取 excel的数据
     * @param workbook
     * @param sheetNum
     */
    private static TaxTableVo parseExcel(Workbook workbook, int sheetNum) {
        Sheet sheet = workbook.getSheetAt(sheetNum); // 获取表格
        log.info("sheet = {}", sheet.getSheetName());


        //保存的读取对象
        TaxTableVo tableVo = new TaxTableVo();
        List<TaxAreaVo> areaVoList = new ArrayList<>();
        tableVo.setList(areaVoList);

        Row firstRow = sheet.getRow(sheet.getFirstRowNum()); // 获取第一行，一般是标题
        if (null == firstRow) {
            log.warn("解析Excel失败，在第一行没有读取到任何数据！");
        }

        //int firstRowNum = 1;
        // 解析每一行的数据，构造数据对象
        int rowStart = 1; //标题下面的数据,数据起始行
        int rowEnd = sheet.getPhysicalNumberOfRows();//获取有记录的行数，即：最后有数据的行是第n行，前面有m行是空行没数据，则返回n-m；
        int rowEndLast = sheet.getLastRowNum();//获取有记录的行数，即：最后有数据的行是第n行，前面有m行是空行没数据，则返回n-m；


        int maxCellColumn = 0;

        //没指标类的指标项的读取
        boolean outNormFlag = false;
        ///指标类的读取
        boolean normClassFlag = false;
        boolean normFlag = false;

        String normClassRecord = "";
        //是否有 分值列
        boolean scoreTitleColumnFlag = false;
        //计量单位的列
        int unitTitleColumn = 4;

        //开始读取 每行的数据
        for (int rowNum = rowStart; rowNum < rowEnd; rowNum++) {
            Row row = sheet.getRow(rowNum);
            //log.info("第 {} 行, {} 列", row.getRowNum(), row.getLastCellNum());
            if (null == row) {
                continue;
            }

            //判断是否有 标题头是否有 分值
            if( 1 == rowNum ) {
                String titleF3 = TaxReadExcelUtil.readCell(row, 3, "");
                if(StringUtils.isNotBlank(titleF3) && titleF3.equals("分值")) {
                    scoreTitleColumnFlag = true;
                }
                if(StringUtils.isNotBlank(titleF3) && titleF3.equals("计量单位")) {
                    unitTitleColumn = 3;
                }

                String titleF4 = TaxReadExcelUtil.readCell(row, 4, "");
                if(StringUtils.isNotBlank(titleF4) && titleF4.equals("计量单位")) {
                    unitTitleColumn = 4;
                }
            }


            if(outNormFlag) {
                String normClass = TaxReadExcelUtil.readCell(row, 0, "");
                if(StringUtils.isNotBlank(normClass)) {
                    normClassFlag = true;
                    outNormFlag = false;
                }
            }

            //结束
            if(normClassFlag) {
                String normClass = TaxReadExcelUtil.readCell(row, 0, "");
                if(StringUtils.isNotBlank(normClass) && "说明".equals(normClass)) {
                    normClassFlag = false;
                }
            }

            //指标分类的 读取
            if(normClassFlag) {
                //指标分类
                String normClass = TaxReadExcelUtil.readCell(row, 0, "");
                //项次
                String rowIndex = TaxReadExcelUtil.readCell(row, 1, "");
                //指标名称
                String normName = TaxReadExcelUtil.readCell(row, 2, "");
                //分值
                String score = scoreTitleColumnFlag ? TaxReadExcelUtil.readCell(row, 3, "") : "0";
                //计量单位
                String unit  = TaxReadExcelUtil.readCell(row, unitTitleColumn, "");
                if(StringUtils.isNotBlank(normClass)) {
                    normClassRecord = normClass;
                }

                //各个区域的值
                TaxAreaVo taxAreaVo = null;
                for (int cellNum = unitTitleColumn + 1; cellNum < row.getLastCellNum(); cellNum = cellNum+3) {
                    String yearValue = TaxReadExcelUtil.readCell(row, cellNum, null);
                    String yearPreValue = TaxReadExcelUtil.readCell(row, cellNum+1, null);
                    String yearCompareValue = TaxReadExcelUtil.readCell(row, cellNum+2, null);
                    //log.debug("areaYearValue 区域值 [{}]", yearValue);
                    if(null == yearValue) {
                        break;
                    }

                    //区域
                    int finalNextAreaCellNum = cellNum;
                    taxAreaVo = areaVoList.stream().filter(it-> it.getStartCellColumn().equals(String.valueOf(finalNextAreaCellNum))).findFirst().orElse(null);
                    //查找 指标分类
                    String finalNormClassRecord = normClassRecord;
                    NormCategoryVo normCategoryVo = taxAreaVo.getNormCategoryList().stream().filter(i->i.getNormClass().equals(finalNormClassRecord)).findFirst().orElse(null);
                    if(null == normCategoryVo) {
                        normCategoryVo = new NormCategoryVo();
                        normCategoryVo.setNormClass(normClass);
                        normCategoryVo.setList(new ArrayList<>());
                        taxAreaVo.getNormCategoryList().add(normCategoryVo);
                    }

                    NormItemVo normItemVo = new NormItemVo();
                    normItemVo.setYear(yearValue);
                    normItemVo.setYearPre(yearPreValue);
                    normItemVo.setYearCompare(yearCompareValue);
                    normItemVo.setNormName(normName);
                    normItemVo.setRowIndex(rowIndex);
                    normItemVo.setScore(score);
                    normItemVo.setUnit(unit);
                    normCategoryVo.getList().add(normItemVo);
                }
            }

            //没有指标分类的指标名称 的读取
            if(outNormFlag) {
                //指标分类
                String normClass = TaxReadExcelUtil.readCell(row, 0, "");
                //项次
                String itemIndex = TaxReadExcelUtil.readCell(row, 1, "");
                //指标名称
                String normName = TaxReadExcelUtil.readCell(row, 2, "");
                //分值
                String score = scoreTitleColumnFlag ? TaxReadExcelUtil.readCell(row, 3, "") : "0";
                //计量单位
                String unit  = TaxReadExcelUtil.readCell(row, unitTitleColumn, "");

                //各个区域的值
                TaxAreaVo taxAreaVo = null;
                for (int cellNum = unitTitleColumn + 1; cellNum < row.getLastCellNum(); cellNum = cellNum+3) {
                    String yearValue = TaxReadExcelUtil.readCell(row, cellNum, null);
                    String yearPreValue = TaxReadExcelUtil.readCell(row, cellNum+1, null);
                    String yearCompareValue = TaxReadExcelUtil.readCell(row, cellNum+2, null);
                    //log.debug("areaYearValue 区域值 [{}]", yearValue);
                    if(null == yearValue) {
                        break;
                    }

                    //区域
                    int finalNextAreaCellNum = cellNum;
                    taxAreaVo = areaVoList.stream().filter(it-> it.getStartCellColumn().equals(String.valueOf(finalNextAreaCellNum))).findFirst().orElse(null);
                    if (ObjectUtils.isEmpty(taxAreaVo)) continue;

                    NormItemVo normItemVo = new NormItemVo();
                    normItemVo.setYear(decimalFormat2Point(yearValue));
                    normItemVo.setYearPre(yearPreValue);
                    normItemVo.setYearCompare(yearCompareValue);
                    normItemVo.setNormName(normName);
                    normItemVo.setUnit(unit);
                    taxAreaVo.getOutNormList().add(normItemVo);
                }
            }


            //表头读取 区域名
            if(row.getRowNum() == 1) {
                //处理Cell 每个区域名占 3列
                for (int cellNum = unitTitleColumn + 1; cellNum < row.getLastCellNum(); cellNum = cellNum+3) {
                    String area = TaxReadExcelUtil.readCell(row, cellNum, null);
//                    log.debug("area [{}]", area);
                    if(null == area) {
                        break;
                    }

                    TaxAreaVo taxAreaVo = new TaxAreaVo();
                    taxAreaVo.setName(area);
                    taxAreaVo.setStartCellColumn(String.valueOf(cellNum));
                    taxAreaVo.setOutNormList(new ArrayList<>());
                    taxAreaVo.setNormCategoryList(new ArrayList<>());
                    areaVoList.add(taxAreaVo);


                    maxCellColumn = cellNum + 2;
                }
            }

            //时间 本年 去年 同比
            if(row.getRowNum() == 2) {
                String yearDateTitle = TaxReadExcelUtil.readCell(row, unitTitleColumn + 1, null);
                String yearPreDateTitle = TaxReadExcelUtil.readCell(row, unitTitleColumn + 2, null);
                String yearCompareTitle = TaxReadExcelUtil.readCell(row, unitTitleColumn + 3, null);
                outNormFlag = true;

                areaVoList.stream().forEach(it-> {
                    it.setYearTitle(yearDateTitle);
                    it.setYearPreTitle(yearPreDateTitle);
                    it.setYearCompareTitle(yearCompareTitle);
                });
            }
        }

        return tableVo;
    }

    /**
     * 处理单元格类型
     * @param cell
     * @return
     */
    private static String convertCellValueToString(Cell cell) {
        if (cell == null) {
            return null;
        }
        String content = null;
        try {
            switch (cell.getCellType()) {
                case NUMERIC:   //数字或者时间
                    Double doubleValue = cell.getNumericCellValue();
                    // 格式化科学计数法，取一位整数
                    //DecimalFormat df = new DecimalFormat("0");
                    /*DecimalFormat df = new DecimalFormat("#####0.00");
                    content = df.format(doubleValue);*/
                    DecimalFormat decimalFormat = new DecimalFormat("0.#########");
                    content = decimalFormat.format(cell.getNumericCellValue());
                    break;
                case STRING:    //字符串
                    content = cell.getStringCellValue();
                    break;
                case BOOLEAN:   //布尔
                    Boolean booleanValue = cell.getBooleanCellValue();
                    content = booleanValue.toString();
                    break;
                case BLANK:     // 空值
                    break;
                case FORMULA:   // 公式
                    //content = cell.getCellFormula();
                    //String str = String.valueOf(cell.getNumericCellValue());
                    //执行公式，此处cell的值就是公式
                    CellValue evaluate = TaxReadExcelUtil.formulaEvaluator.evaluate(cell);

                    String evaluateStr = evaluate.formatAsString();
                    if(!isNumeric(evaluateStr)) {
                        evaluateStr = "0";
                    }
                    Double doubleNumber = Double.parseDouble(evaluateStr);
                    DecimalFormat df1 = new DecimalFormat("###.####");
                    String formattedResult = df1.format(doubleNumber);
                    content = formattedResult;
                    break;
                case ERROR:     // 故障
                    break;
                default:
                    break;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return content;
    }


    /**
     * 读取 单元格
     * @param row 行数
     * @param cellNum 单元格的 num
     * @return
     */
    private static String readCell(Row row, int cellNum, String defaultValue) {
        Cell cell = row.getCell(cellNum);
        if (cell != null) {
            String content = convertCellValueToString(cell);
            log.info("第" + row.getRowNum() + "行，第" + cellNum + "列，内容：" + content);
            //  处理数据
            return content;
        }

        //默认值
        if(StringUtils.isNotBlank(defaultValue)) {
            return defaultValue;
        }

        return null;
    }

    /**
     *
     * @param doubleNumberStr
     * @return
     */
    private static String decimalFormat2Point(String doubleNumberStr) {
        Double doubleNumber = Double.parseDouble(doubleNumberStr);
        DecimalFormat df1 = new DecimalFormat("###.##");
        String formattedResult = df1.format(doubleNumber);
        return formattedResult;
    }

    private static String decimalFormat4Point(String doubleNumberStr) {
        Double doubleNumber = Double.parseDouble(doubleNumberStr);
        DecimalFormat df1 = new DecimalFormat("###.####");
        String formattedResult = df1.format(doubleNumber);
        return formattedResult;
    }

    /**
     * 判断是否为数字
     * @param str
     * @return
     */
    public static boolean isNumeric(String str) {
        String bigStr;
        try {
            bigStr = new BigDecimal(str).toString();
        } catch (Exception e) {
            return false;//异常 说明包含非数字。
        }
        return true;
    }

}
