package com.hjy.quartz.task;

import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.text.Convert;
import com.hjy.common.utils.CommomUtils;
import com.hjy.common.utils.DateUtils;
import com.hjy.common.utils.DictUtils;
import com.hjy.module.domain.gether.CbGather;
import com.hjy.module.domain.gether.CbGatherFee;
import com.hjy.module.domain.norm.CbNorm;
import com.hjy.module.domain.norm.CbNormFee;
import com.hjy.module.mapper.gather.CbGatherFeeMapper;
import com.hjy.module.service.gather.ICbGatherFeeService;
import com.hjy.module.service.gather.ICbGatherService;
import com.hjy.module.service.norm.ICbNormFeeService;
import com.hjy.module.service.norm.ICbNormService;
import com.hjy.module.vo.gather.CbGatherFeeVo;
import com.hjy.system.mapper.SysDeptMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 定时任务调度测试
 *
 * @author HJY
 */
@Slf4j
@Component("geneTask")
public class ZhbsqTask {

    @Autowired
    private ICbGatherService gatherService;

    @Autowired
    private ICbGatherFeeService gatherFeeService;

    @Autowired
    private SysDeptMapper deptMapper;

    @Autowired
    private CbGatherFeeMapper gatherFeeMapper;

    @Autowired
    private ICbNormService normService;

    @Autowired
    private ICbNormFeeService normFeeService;

    /**
     * 生成数据采集列表数据
     */
    public void generateGatherData(){
        SysDept dept = new SysDept();
        dept.setType(2);
        List<SysDept> sysDeptList = deptMapper.selectDeptList(dept);
        List<CbGather> saveEntity = Lists.newArrayList();
        for (SysDept sysDept : sysDeptList) {
            CbGather gather = new CbGather();
            gather.setMonth(DateUtils.getmonth(DateUtils.getNowDate()));
//            gather.setMonth("12月");
            gather.setCollStatus("0");
            gather.setReportStatus("0");
            gather.setApproveStatus("0");
            gather.setType("0");
            gather.setCreateBy("admin");
            gather.setDeptId(Convert.toStr(sysDept.getDeptId()));
            gather.setYear(DateUtils.getYear(DateUtils.getNowDate()));
            saveEntity.add(gather);
        }
        gatherService.saveOrUpdateBatch(saveEntity);
        for (CbGather gather:saveEntity){
            List<CbGatherFeeVo> allFeeList = getNormFeeList(gather);
            gatherFeeService.saveOrUpdateBatch(CommomUtils.copyList(allFeeList, CbGatherFee.class));
        }
        log.info("自动创建数据采集列表数据");
    }

    private List<CbGatherFeeVo>  getNormFeeList(CbGather gather){
        List<CbGatherFeeVo> gatherFeeVoList = Lists.newArrayList();
        List<CbNorm> normList = normService.lambdaQuery().eq(CbNorm::getStatus, "1")
                .eq(CbNorm::getDelFlag, "0")
                .orderByAsc(CbNorm::getSort)
                .list();
        for (CbNorm norm : normList) {
            List<CbNormFee> allFeeList = normFeeService.lambdaQuery()
                    .eq(CbNormFee::getStatus, "1")
                    .eq(CbNormFee::getDelFlag, "0")
                    .eq(CbNormFee::getNormId, norm.getId())
                    .orderByAsc(CbNormFee::getSort)
                    .list();   // 未配置权限指标 （不可采集）
            for (CbNormFee normFee : allFeeList) {
                CbGatherFeeVo gatherFeeVo = new CbGatherFeeVo();
                gatherFeeVo.setNormId(norm.getId());
                gatherFeeVo.setNormName(norm.getName());
                gatherFeeVo.setNormfeeId(normFee.getId());
                gatherFeeVo.setNormfeeName(normFee.getFname());
                gatherFeeVo.setGatherId(gather.getId());
                gatherFeeVo.setFunit(normFee.getFunit());
                gatherFeeVo.setNormfeeCode(normFee.getFcode());
                gatherFeeVo.setDeptId(gather.getDeptId());
                gatherFeeVo.setSendValue("1");
                gatherFeeVo.setFunitName(DictUtils.getDictLabel("norm_uint", normFee.getFunit()));
                gatherFeeVoList.add(gatherFeeVo);
            }
        }
        return gatherFeeVoList;
    }
}

