package com.hjy.system.service.impl;

import com.hjy.common.core.domain.entity.SysDictData;
import com.hjy.common.core.domain.entity.SysDictModel;
import com.hjy.common.utils.DictUtils;
import com.hjy.common.utils.sql.SqlUtil;
import com.hjy.system.mapper.SysDictDataMapper;
import com.hjy.system.service.ISysDictDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 字典 业务层处理
 *
 * @author HJY
 */
@Service
public class SysDictDataServiceImpl implements ISysDictDataService {
    @Autowired
    private SysDictDataMapper dictDataMapper;



    /**
     * 根据条件分页查询字典数据
     *
     * @param dictData 字典数据信息
     * @return 字典数据集合信息
     */
    @Override
    public List<SysDictData> selectDictDataList(SysDictData dictData) {
        return dictDataMapper.selectDictDataList(dictData);
    }

    /**
     * 根据字典类型和字典键值查询字典数据信息
     *
     * @param dictType  字典类型
     * @param dictValue 字典键值
     * @return 字典标签
     */
    @Override
    public String selectDictLabel(String dictType, String dictValue) {
        return dictDataMapper.selectDictLabel(dictType, dictValue);
    }

    /**
     * 根据字典数据ID查询信息
     *
     * @param dictCode 字典数据ID
     * @return 字典数据
     */
    @Override
    public SysDictData selectDictDataById(Long dictCode) {
        return dictDataMapper.selectDictDataById(dictCode);
    }

    /**
     * 批量删除字典数据信息
     *
     * @param dictCodes 需要删除的字典数据ID
     */
    @Override
    public void deleteDictDataByIds(Long[] dictCodes) {
        for (Long dictCode : dictCodes) {
            SysDictData data = selectDictDataById(dictCode);
            dictDataMapper.deleteDictDataById(dictCode);
            List<SysDictData> dictDatas = dictDataMapper.selectDictDataByType(data.getDictType());
            DictUtils.setDictCache(data.getDictType(), dictDatas);
        }
    }

    /**
     * 新增保存字典数据信息
     *
     * @param data 字典数据信息
     * @return 结果
     */
    @Override
    public int insertDictData(SysDictData data) {
        int row = dictDataMapper.insertDictData(data);
        if (row > 0) {
            List<SysDictData> dictDatas = dictDataMapper.selectDictDataByType(data.getDictType());
            DictUtils.setDictCache(data.getDictType(), dictDatas);
        }
        return row;
    }

    /**
     * 修改保存字典数据信息
     *
     * @param data 字典数据信息
     * @return 结果
     */
    @Override
    public int updateDictData(SysDictData data) {
        int row = dictDataMapper.updateDictData(data);
        if (row > 0) {
            List<SysDictData> dictDatas = dictDataMapper.selectDictDataByType(data.getDictType());
            DictUtils.setDictCache(data.getDictType(), dictDatas);
        }
        return row;
    }

    @Override
    public List<SysDictModel> translateDictFromTableByKeys(String table, String text, String code, List<String> keys) {
        //update-begin-author:taoyan date:20220113 for: @dict注解支持 dicttable 设置where条件
        String filterSql = null;
        if(table.toLowerCase().indexOf("where")>0){
            String[] arr = table.split(" (?i)where ");
            table = arr[0];
            filterSql = arr[1];
        }
        String[] tableAndFields = new String[]{table, text, code};
        SqlUtil.filterContent(tableAndFields,null);
        SqlUtil.specialFilterContentForDictSql(filterSql);
        return dictDataMapper.queryTableDictByKeysAndFilterSql(table, text, code, filterSql, keys);
    }

    @Override
    public Map<String, List<SysDictModel>> translateManyDict(String dictCodes, String values) {
        List<SysDictModel> list = dictDataMapper.queryManyDictByKeys(Arrays.asList(dictCodes.split(",")), Arrays.asList(values.split(",")));
        Map<String, List<SysDictModel>> dictMap = new HashMap(5);
        for (SysDictModel dict : list) {
            List<SysDictModel> dictItemList = dictMap.computeIfAbsent(dict.getDictcode(), i -> new ArrayList<>());
            dictItemList.add(new SysDictModel(dict.getDictcode(),dict.getValue(), dict.getText()));
        }
        return dictMap;
    }

    @Override
    public String translateDictFromTable(String table, String text, String code, String key) {
        return dictDataMapper.queryTableDictTextByKey(table,text,code,key);
    }

    @Override
    public String translateDict(String code, String key) {
        return dictDataMapper.selectDictLabel(code,key);
    }

}
